// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: cri-v1alpha2.proto
// Protobuf C++ Version: 6.33.1

#ifndef cri_2dv1alpha2_2eproto_2epb_2eh
#define cri_2dv1alpha2_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 6033001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_bases.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/map.h"  // IWYU pragma: export
#include "google/protobuf/map_type_handler.h"  // IWYU pragma: export
#include "google/protobuf/map_entry.h"
#include "google/protobuf/map_field.h"
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_cri_2dv1alpha2_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_cri_2dv1alpha2_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_cri_2dv1alpha2_2eproto;
}  // extern "C"
namespace runtime {
namespace v1alpha2 {
enum ContainerState : int;
extern const uint32_t ContainerState_internal_data_[];
enum MountPropagation : int;
extern const uint32_t MountPropagation_internal_data_[];
enum NamespaceMode : int;
extern const uint32_t NamespaceMode_internal_data_[];
enum PodSandboxState : int;
extern const uint32_t PodSandboxState_internal_data_[];
enum Protocol : int;
extern const uint32_t Protocol_internal_data_[];
enum SecurityProfile_ProfileType : int;
extern const uint32_t SecurityProfile_ProfileType_internal_data_[];
class AttachRequest;
struct AttachRequestDefaultTypeInternal;
extern AttachRequestDefaultTypeInternal _AttachRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull AttachRequest_class_data_;
class AttachResponse;
struct AttachResponseDefaultTypeInternal;
extern AttachResponseDefaultTypeInternal _AttachResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull AttachResponse_class_data_;
class AuthConfig;
struct AuthConfigDefaultTypeInternal;
extern AuthConfigDefaultTypeInternal _AuthConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull AuthConfig_class_data_;
class Capability;
struct CapabilityDefaultTypeInternal;
extern CapabilityDefaultTypeInternal _Capability_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Capability_class_data_;
class Container;
struct ContainerDefaultTypeInternal;
extern ContainerDefaultTypeInternal _Container_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Container_class_data_;
class ContainerAttributes;
struct ContainerAttributesDefaultTypeInternal;
extern ContainerAttributesDefaultTypeInternal _ContainerAttributes_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerAttributes_class_data_;
class ContainerAttributes_AnnotationsEntry_DoNotUse;
struct ContainerAttributes_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern ContainerAttributes_AnnotationsEntry_DoNotUseDefaultTypeInternal _ContainerAttributes_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerAttributes_AnnotationsEntry_DoNotUse_class_data_;
class ContainerAttributes_LabelsEntry_DoNotUse;
struct ContainerAttributes_LabelsEntry_DoNotUseDefaultTypeInternal;
extern ContainerAttributes_LabelsEntry_DoNotUseDefaultTypeInternal _ContainerAttributes_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerAttributes_LabelsEntry_DoNotUse_class_data_;
class ContainerConfig;
struct ContainerConfigDefaultTypeInternal;
extern ContainerConfigDefaultTypeInternal _ContainerConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerConfig_class_data_;
class ContainerConfig_AnnotationsEntry_DoNotUse;
struct ContainerConfig_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern ContainerConfig_AnnotationsEntry_DoNotUseDefaultTypeInternal _ContainerConfig_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerConfig_AnnotationsEntry_DoNotUse_class_data_;
class ContainerConfig_LabelsEntry_DoNotUse;
struct ContainerConfig_LabelsEntry_DoNotUseDefaultTypeInternal;
extern ContainerConfig_LabelsEntry_DoNotUseDefaultTypeInternal _ContainerConfig_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerConfig_LabelsEntry_DoNotUse_class_data_;
class ContainerFilter;
struct ContainerFilterDefaultTypeInternal;
extern ContainerFilterDefaultTypeInternal _ContainerFilter_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerFilter_class_data_;
class ContainerFilter_LabelSelectorEntry_DoNotUse;
struct ContainerFilter_LabelSelectorEntry_DoNotUseDefaultTypeInternal;
extern ContainerFilter_LabelSelectorEntry_DoNotUseDefaultTypeInternal _ContainerFilter_LabelSelectorEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerFilter_LabelSelectorEntry_DoNotUse_class_data_;
class ContainerMetadata;
struct ContainerMetadataDefaultTypeInternal;
extern ContainerMetadataDefaultTypeInternal _ContainerMetadata_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerMetadata_class_data_;
class ContainerStateValue;
struct ContainerStateValueDefaultTypeInternal;
extern ContainerStateValueDefaultTypeInternal _ContainerStateValue_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStateValue_class_data_;
class ContainerStats;
struct ContainerStatsDefaultTypeInternal;
extern ContainerStatsDefaultTypeInternal _ContainerStats_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStats_class_data_;
class ContainerStatsFilter;
struct ContainerStatsFilterDefaultTypeInternal;
extern ContainerStatsFilterDefaultTypeInternal _ContainerStatsFilter_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatsFilter_class_data_;
class ContainerStatsFilter_LabelSelectorEntry_DoNotUse;
struct ContainerStatsFilter_LabelSelectorEntry_DoNotUseDefaultTypeInternal;
extern ContainerStatsFilter_LabelSelectorEntry_DoNotUseDefaultTypeInternal _ContainerStatsFilter_LabelSelectorEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatsFilter_LabelSelectorEntry_DoNotUse_class_data_;
class ContainerStatsRequest;
struct ContainerStatsRequestDefaultTypeInternal;
extern ContainerStatsRequestDefaultTypeInternal _ContainerStatsRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatsRequest_class_data_;
class ContainerStatsResponse;
struct ContainerStatsResponseDefaultTypeInternal;
extern ContainerStatsResponseDefaultTypeInternal _ContainerStatsResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatsResponse_class_data_;
class ContainerStatus;
struct ContainerStatusDefaultTypeInternal;
extern ContainerStatusDefaultTypeInternal _ContainerStatus_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatus_class_data_;
class ContainerStatusRequest;
struct ContainerStatusRequestDefaultTypeInternal;
extern ContainerStatusRequestDefaultTypeInternal _ContainerStatusRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatusRequest_class_data_;
class ContainerStatusResponse;
struct ContainerStatusResponseDefaultTypeInternal;
extern ContainerStatusResponseDefaultTypeInternal _ContainerStatusResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatusResponse_class_data_;
class ContainerStatusResponse_InfoEntry_DoNotUse;
struct ContainerStatusResponse_InfoEntry_DoNotUseDefaultTypeInternal;
extern ContainerStatusResponse_InfoEntry_DoNotUseDefaultTypeInternal _ContainerStatusResponse_InfoEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatusResponse_InfoEntry_DoNotUse_class_data_;
class ContainerStatus_AnnotationsEntry_DoNotUse;
struct ContainerStatus_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern ContainerStatus_AnnotationsEntry_DoNotUseDefaultTypeInternal _ContainerStatus_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatus_AnnotationsEntry_DoNotUse_class_data_;
class ContainerStatus_LabelsEntry_DoNotUse;
struct ContainerStatus_LabelsEntry_DoNotUseDefaultTypeInternal;
extern ContainerStatus_LabelsEntry_DoNotUseDefaultTypeInternal _ContainerStatus_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ContainerStatus_LabelsEntry_DoNotUse_class_data_;
class Container_AnnotationsEntry_DoNotUse;
struct Container_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern Container_AnnotationsEntry_DoNotUseDefaultTypeInternal _Container_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Container_AnnotationsEntry_DoNotUse_class_data_;
class Container_LabelsEntry_DoNotUse;
struct Container_LabelsEntry_DoNotUseDefaultTypeInternal;
extern Container_LabelsEntry_DoNotUseDefaultTypeInternal _Container_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Container_LabelsEntry_DoNotUse_class_data_;
class CpuUsage;
struct CpuUsageDefaultTypeInternal;
extern CpuUsageDefaultTypeInternal _CpuUsage_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull CpuUsage_class_data_;
class CreateContainerRequest;
struct CreateContainerRequestDefaultTypeInternal;
extern CreateContainerRequestDefaultTypeInternal _CreateContainerRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull CreateContainerRequest_class_data_;
class CreateContainerResponse;
struct CreateContainerResponseDefaultTypeInternal;
extern CreateContainerResponseDefaultTypeInternal _CreateContainerResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull CreateContainerResponse_class_data_;
class DNSConfig;
struct DNSConfigDefaultTypeInternal;
extern DNSConfigDefaultTypeInternal _DNSConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull DNSConfig_class_data_;
class Device;
struct DeviceDefaultTypeInternal;
extern DeviceDefaultTypeInternal _Device_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Device_class_data_;
class ExecRequest;
struct ExecRequestDefaultTypeInternal;
extern ExecRequestDefaultTypeInternal _ExecRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ExecRequest_class_data_;
class ExecResponse;
struct ExecResponseDefaultTypeInternal;
extern ExecResponseDefaultTypeInternal _ExecResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ExecResponse_class_data_;
class ExecSyncRequest;
struct ExecSyncRequestDefaultTypeInternal;
extern ExecSyncRequestDefaultTypeInternal _ExecSyncRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ExecSyncRequest_class_data_;
class ExecSyncResponse;
struct ExecSyncResponseDefaultTypeInternal;
extern ExecSyncResponseDefaultTypeInternal _ExecSyncResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ExecSyncResponse_class_data_;
class FilesystemIdentifier;
struct FilesystemIdentifierDefaultTypeInternal;
extern FilesystemIdentifierDefaultTypeInternal _FilesystemIdentifier_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FilesystemIdentifier_class_data_;
class FilesystemUsage;
struct FilesystemUsageDefaultTypeInternal;
extern FilesystemUsageDefaultTypeInternal _FilesystemUsage_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull FilesystemUsage_class_data_;
class HugepageLimit;
struct HugepageLimitDefaultTypeInternal;
extern HugepageLimitDefaultTypeInternal _HugepageLimit_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull HugepageLimit_class_data_;
class Image;
struct ImageDefaultTypeInternal;
extern ImageDefaultTypeInternal _Image_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Image_class_data_;
class ImageFilter;
struct ImageFilterDefaultTypeInternal;
extern ImageFilterDefaultTypeInternal _ImageFilter_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageFilter_class_data_;
class ImageFsInfoRequest;
struct ImageFsInfoRequestDefaultTypeInternal;
extern ImageFsInfoRequestDefaultTypeInternal _ImageFsInfoRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageFsInfoRequest_class_data_;
class ImageFsInfoResponse;
struct ImageFsInfoResponseDefaultTypeInternal;
extern ImageFsInfoResponseDefaultTypeInternal _ImageFsInfoResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageFsInfoResponse_class_data_;
class ImageSpec;
struct ImageSpecDefaultTypeInternal;
extern ImageSpecDefaultTypeInternal _ImageSpec_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageSpec_class_data_;
class ImageSpec_AnnotationsEntry_DoNotUse;
struct ImageSpec_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern ImageSpec_AnnotationsEntry_DoNotUseDefaultTypeInternal _ImageSpec_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageSpec_AnnotationsEntry_DoNotUse_class_data_;
class ImageStatusRequest;
struct ImageStatusRequestDefaultTypeInternal;
extern ImageStatusRequestDefaultTypeInternal _ImageStatusRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageStatusRequest_class_data_;
class ImageStatusResponse;
struct ImageStatusResponseDefaultTypeInternal;
extern ImageStatusResponseDefaultTypeInternal _ImageStatusResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageStatusResponse_class_data_;
class ImageStatusResponse_InfoEntry_DoNotUse;
struct ImageStatusResponse_InfoEntry_DoNotUseDefaultTypeInternal;
extern ImageStatusResponse_InfoEntry_DoNotUseDefaultTypeInternal _ImageStatusResponse_InfoEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ImageStatusResponse_InfoEntry_DoNotUse_class_data_;
class Int64Value;
struct Int64ValueDefaultTypeInternal;
extern Int64ValueDefaultTypeInternal _Int64Value_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Int64Value_class_data_;
class KeyValue;
struct KeyValueDefaultTypeInternal;
extern KeyValueDefaultTypeInternal _KeyValue_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull KeyValue_class_data_;
class LinuxContainerConfig;
struct LinuxContainerConfigDefaultTypeInternal;
extern LinuxContainerConfigDefaultTypeInternal _LinuxContainerConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxContainerConfig_class_data_;
class LinuxContainerResources;
struct LinuxContainerResourcesDefaultTypeInternal;
extern LinuxContainerResourcesDefaultTypeInternal _LinuxContainerResources_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxContainerResources_class_data_;
class LinuxContainerSecurityContext;
struct LinuxContainerSecurityContextDefaultTypeInternal;
extern LinuxContainerSecurityContextDefaultTypeInternal _LinuxContainerSecurityContext_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxContainerSecurityContext_class_data_;
class LinuxPodSandboxConfig;
struct LinuxPodSandboxConfigDefaultTypeInternal;
extern LinuxPodSandboxConfigDefaultTypeInternal _LinuxPodSandboxConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxPodSandboxConfig_class_data_;
class LinuxPodSandboxConfig_SysctlsEntry_DoNotUse;
struct LinuxPodSandboxConfig_SysctlsEntry_DoNotUseDefaultTypeInternal;
extern LinuxPodSandboxConfig_SysctlsEntry_DoNotUseDefaultTypeInternal _LinuxPodSandboxConfig_SysctlsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxPodSandboxConfig_SysctlsEntry_DoNotUse_class_data_;
class LinuxPodSandboxStatus;
struct LinuxPodSandboxStatusDefaultTypeInternal;
extern LinuxPodSandboxStatusDefaultTypeInternal _LinuxPodSandboxStatus_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxPodSandboxStatus_class_data_;
class LinuxSandboxSecurityContext;
struct LinuxSandboxSecurityContextDefaultTypeInternal;
extern LinuxSandboxSecurityContextDefaultTypeInternal _LinuxSandboxSecurityContext_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull LinuxSandboxSecurityContext_class_data_;
class ListContainerStatsRequest;
struct ListContainerStatsRequestDefaultTypeInternal;
extern ListContainerStatsRequestDefaultTypeInternal _ListContainerStatsRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListContainerStatsRequest_class_data_;
class ListContainerStatsResponse;
struct ListContainerStatsResponseDefaultTypeInternal;
extern ListContainerStatsResponseDefaultTypeInternal _ListContainerStatsResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListContainerStatsResponse_class_data_;
class ListContainersRequest;
struct ListContainersRequestDefaultTypeInternal;
extern ListContainersRequestDefaultTypeInternal _ListContainersRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListContainersRequest_class_data_;
class ListContainersResponse;
struct ListContainersResponseDefaultTypeInternal;
extern ListContainersResponseDefaultTypeInternal _ListContainersResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListContainersResponse_class_data_;
class ListImagesRequest;
struct ListImagesRequestDefaultTypeInternal;
extern ListImagesRequestDefaultTypeInternal _ListImagesRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListImagesRequest_class_data_;
class ListImagesResponse;
struct ListImagesResponseDefaultTypeInternal;
extern ListImagesResponseDefaultTypeInternal _ListImagesResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListImagesResponse_class_data_;
class ListPodSandboxRequest;
struct ListPodSandboxRequestDefaultTypeInternal;
extern ListPodSandboxRequestDefaultTypeInternal _ListPodSandboxRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListPodSandboxRequest_class_data_;
class ListPodSandboxResponse;
struct ListPodSandboxResponseDefaultTypeInternal;
extern ListPodSandboxResponseDefaultTypeInternal _ListPodSandboxResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ListPodSandboxResponse_class_data_;
class MemoryUsage;
struct MemoryUsageDefaultTypeInternal;
extern MemoryUsageDefaultTypeInternal _MemoryUsage_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull MemoryUsage_class_data_;
class Mount;
struct MountDefaultTypeInternal;
extern MountDefaultTypeInternal _Mount_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Mount_class_data_;
class Namespace;
struct NamespaceDefaultTypeInternal;
extern NamespaceDefaultTypeInternal _Namespace_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Namespace_class_data_;
class NamespaceOption;
struct NamespaceOptionDefaultTypeInternal;
extern NamespaceOptionDefaultTypeInternal _NamespaceOption_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull NamespaceOption_class_data_;
class NetworkConfig;
struct NetworkConfigDefaultTypeInternal;
extern NetworkConfigDefaultTypeInternal _NetworkConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull NetworkConfig_class_data_;
class PodIP;
struct PodIPDefaultTypeInternal;
extern PodIPDefaultTypeInternal _PodIP_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodIP_class_data_;
class PodSandbox;
struct PodSandboxDefaultTypeInternal;
extern PodSandboxDefaultTypeInternal _PodSandbox_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandbox_class_data_;
class PodSandboxConfig;
struct PodSandboxConfigDefaultTypeInternal;
extern PodSandboxConfigDefaultTypeInternal _PodSandboxConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxConfig_class_data_;
class PodSandboxConfig_AnnotationsEntry_DoNotUse;
struct PodSandboxConfig_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern PodSandboxConfig_AnnotationsEntry_DoNotUseDefaultTypeInternal _PodSandboxConfig_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxConfig_AnnotationsEntry_DoNotUse_class_data_;
class PodSandboxConfig_LabelsEntry_DoNotUse;
struct PodSandboxConfig_LabelsEntry_DoNotUseDefaultTypeInternal;
extern PodSandboxConfig_LabelsEntry_DoNotUseDefaultTypeInternal _PodSandboxConfig_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxConfig_LabelsEntry_DoNotUse_class_data_;
class PodSandboxFilter;
struct PodSandboxFilterDefaultTypeInternal;
extern PodSandboxFilterDefaultTypeInternal _PodSandboxFilter_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxFilter_class_data_;
class PodSandboxFilter_LabelSelectorEntry_DoNotUse;
struct PodSandboxFilter_LabelSelectorEntry_DoNotUseDefaultTypeInternal;
extern PodSandboxFilter_LabelSelectorEntry_DoNotUseDefaultTypeInternal _PodSandboxFilter_LabelSelectorEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxFilter_LabelSelectorEntry_DoNotUse_class_data_;
class PodSandboxMetadata;
struct PodSandboxMetadataDefaultTypeInternal;
extern PodSandboxMetadataDefaultTypeInternal _PodSandboxMetadata_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxMetadata_class_data_;
class PodSandboxNetworkStatus;
struct PodSandboxNetworkStatusDefaultTypeInternal;
extern PodSandboxNetworkStatusDefaultTypeInternal _PodSandboxNetworkStatus_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxNetworkStatus_class_data_;
class PodSandboxStateValue;
struct PodSandboxStateValueDefaultTypeInternal;
extern PodSandboxStateValueDefaultTypeInternal _PodSandboxStateValue_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStateValue_class_data_;
class PodSandboxStatus;
struct PodSandboxStatusDefaultTypeInternal;
extern PodSandboxStatusDefaultTypeInternal _PodSandboxStatus_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatus_class_data_;
class PodSandboxStatusRequest;
struct PodSandboxStatusRequestDefaultTypeInternal;
extern PodSandboxStatusRequestDefaultTypeInternal _PodSandboxStatusRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatusRequest_class_data_;
class PodSandboxStatusResponse;
struct PodSandboxStatusResponseDefaultTypeInternal;
extern PodSandboxStatusResponseDefaultTypeInternal _PodSandboxStatusResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatusResponse_class_data_;
class PodSandboxStatusResponse_InfoEntry_DoNotUse;
struct PodSandboxStatusResponse_InfoEntry_DoNotUseDefaultTypeInternal;
extern PodSandboxStatusResponse_InfoEntry_DoNotUseDefaultTypeInternal _PodSandboxStatusResponse_InfoEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatusResponse_InfoEntry_DoNotUse_class_data_;
class PodSandboxStatus_AnnotationsEntry_DoNotUse;
struct PodSandboxStatus_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern PodSandboxStatus_AnnotationsEntry_DoNotUseDefaultTypeInternal _PodSandboxStatus_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatus_AnnotationsEntry_DoNotUse_class_data_;
class PodSandboxStatus_LabelsEntry_DoNotUse;
struct PodSandboxStatus_LabelsEntry_DoNotUseDefaultTypeInternal;
extern PodSandboxStatus_LabelsEntry_DoNotUseDefaultTypeInternal _PodSandboxStatus_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatus_LabelsEntry_DoNotUse_class_data_;
class PodSandbox_AnnotationsEntry_DoNotUse;
struct PodSandbox_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern PodSandbox_AnnotationsEntry_DoNotUseDefaultTypeInternal _PodSandbox_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandbox_AnnotationsEntry_DoNotUse_class_data_;
class PodSandbox_LabelsEntry_DoNotUse;
struct PodSandbox_LabelsEntry_DoNotUseDefaultTypeInternal;
extern PodSandbox_LabelsEntry_DoNotUseDefaultTypeInternal _PodSandbox_LabelsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PodSandbox_LabelsEntry_DoNotUse_class_data_;
class PortForwardRequest;
struct PortForwardRequestDefaultTypeInternal;
extern PortForwardRequestDefaultTypeInternal _PortForwardRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PortForwardRequest_class_data_;
class PortForwardResponse;
struct PortForwardResponseDefaultTypeInternal;
extern PortForwardResponseDefaultTypeInternal _PortForwardResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PortForwardResponse_class_data_;
class PortMapping;
struct PortMappingDefaultTypeInternal;
extern PortMappingDefaultTypeInternal _PortMapping_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PortMapping_class_data_;
class PullImageRequest;
struct PullImageRequestDefaultTypeInternal;
extern PullImageRequestDefaultTypeInternal _PullImageRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PullImageRequest_class_data_;
class PullImageResponse;
struct PullImageResponseDefaultTypeInternal;
extern PullImageResponseDefaultTypeInternal _PullImageResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull PullImageResponse_class_data_;
class RemoveContainerRequest;
struct RemoveContainerRequestDefaultTypeInternal;
extern RemoveContainerRequestDefaultTypeInternal _RemoveContainerRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RemoveContainerRequest_class_data_;
class RemoveContainerResponse;
struct RemoveContainerResponseDefaultTypeInternal;
extern RemoveContainerResponseDefaultTypeInternal _RemoveContainerResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RemoveContainerResponse_class_data_;
class RemoveImageRequest;
struct RemoveImageRequestDefaultTypeInternal;
extern RemoveImageRequestDefaultTypeInternal _RemoveImageRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RemoveImageRequest_class_data_;
class RemoveImageResponse;
struct RemoveImageResponseDefaultTypeInternal;
extern RemoveImageResponseDefaultTypeInternal _RemoveImageResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RemoveImageResponse_class_data_;
class RemovePodSandboxRequest;
struct RemovePodSandboxRequestDefaultTypeInternal;
extern RemovePodSandboxRequestDefaultTypeInternal _RemovePodSandboxRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RemovePodSandboxRequest_class_data_;
class RemovePodSandboxResponse;
struct RemovePodSandboxResponseDefaultTypeInternal;
extern RemovePodSandboxResponseDefaultTypeInternal _RemovePodSandboxResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RemovePodSandboxResponse_class_data_;
class ReopenContainerLogRequest;
struct ReopenContainerLogRequestDefaultTypeInternal;
extern ReopenContainerLogRequestDefaultTypeInternal _ReopenContainerLogRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ReopenContainerLogRequest_class_data_;
class ReopenContainerLogResponse;
struct ReopenContainerLogResponseDefaultTypeInternal;
extern ReopenContainerLogResponseDefaultTypeInternal _ReopenContainerLogResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull ReopenContainerLogResponse_class_data_;
class RunPodSandboxRequest;
struct RunPodSandboxRequestDefaultTypeInternal;
extern RunPodSandboxRequestDefaultTypeInternal _RunPodSandboxRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RunPodSandboxRequest_class_data_;
class RunPodSandboxResponse;
struct RunPodSandboxResponseDefaultTypeInternal;
extern RunPodSandboxResponseDefaultTypeInternal _RunPodSandboxResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RunPodSandboxResponse_class_data_;
class RuntimeCondition;
struct RuntimeConditionDefaultTypeInternal;
extern RuntimeConditionDefaultTypeInternal _RuntimeCondition_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RuntimeCondition_class_data_;
class RuntimeConfig;
struct RuntimeConfigDefaultTypeInternal;
extern RuntimeConfigDefaultTypeInternal _RuntimeConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RuntimeConfig_class_data_;
class RuntimeStatus;
struct RuntimeStatusDefaultTypeInternal;
extern RuntimeStatusDefaultTypeInternal _RuntimeStatus_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull RuntimeStatus_class_data_;
class SELinuxOption;
struct SELinuxOptionDefaultTypeInternal;
extern SELinuxOptionDefaultTypeInternal _SELinuxOption_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull SELinuxOption_class_data_;
class SecurityProfile;
struct SecurityProfileDefaultTypeInternal;
extern SecurityProfileDefaultTypeInternal _SecurityProfile_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull SecurityProfile_class_data_;
class StartContainerRequest;
struct StartContainerRequestDefaultTypeInternal;
extern StartContainerRequestDefaultTypeInternal _StartContainerRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StartContainerRequest_class_data_;
class StartContainerResponse;
struct StartContainerResponseDefaultTypeInternal;
extern StartContainerResponseDefaultTypeInternal _StartContainerResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StartContainerResponse_class_data_;
class StatusRequest;
struct StatusRequestDefaultTypeInternal;
extern StatusRequestDefaultTypeInternal _StatusRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StatusRequest_class_data_;
class StatusResponse;
struct StatusResponseDefaultTypeInternal;
extern StatusResponseDefaultTypeInternal _StatusResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StatusResponse_class_data_;
class StatusResponse_InfoEntry_DoNotUse;
struct StatusResponse_InfoEntry_DoNotUseDefaultTypeInternal;
extern StatusResponse_InfoEntry_DoNotUseDefaultTypeInternal _StatusResponse_InfoEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StatusResponse_InfoEntry_DoNotUse_class_data_;
class StopContainerRequest;
struct StopContainerRequestDefaultTypeInternal;
extern StopContainerRequestDefaultTypeInternal _StopContainerRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StopContainerRequest_class_data_;
class StopContainerResponse;
struct StopContainerResponseDefaultTypeInternal;
extern StopContainerResponseDefaultTypeInternal _StopContainerResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StopContainerResponse_class_data_;
class StopPodSandboxRequest;
struct StopPodSandboxRequestDefaultTypeInternal;
extern StopPodSandboxRequestDefaultTypeInternal _StopPodSandboxRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StopPodSandboxRequest_class_data_;
class StopPodSandboxResponse;
struct StopPodSandboxResponseDefaultTypeInternal;
extern StopPodSandboxResponseDefaultTypeInternal _StopPodSandboxResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull StopPodSandboxResponse_class_data_;
class UInt64Value;
struct UInt64ValueDefaultTypeInternal;
extern UInt64ValueDefaultTypeInternal _UInt64Value_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull UInt64Value_class_data_;
class UpdateContainerResourcesRequest;
struct UpdateContainerResourcesRequestDefaultTypeInternal;
extern UpdateContainerResourcesRequestDefaultTypeInternal _UpdateContainerResourcesRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull UpdateContainerResourcesRequest_class_data_;
class UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse;
struct UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUseDefaultTypeInternal;
extern UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUseDefaultTypeInternal _UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse_class_data_;
class UpdateContainerResourcesResponse;
struct UpdateContainerResourcesResponseDefaultTypeInternal;
extern UpdateContainerResourcesResponseDefaultTypeInternal _UpdateContainerResourcesResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull UpdateContainerResourcesResponse_class_data_;
class UpdateRuntimeConfigRequest;
struct UpdateRuntimeConfigRequestDefaultTypeInternal;
extern UpdateRuntimeConfigRequestDefaultTypeInternal _UpdateRuntimeConfigRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull UpdateRuntimeConfigRequest_class_data_;
class UpdateRuntimeConfigResponse;
struct UpdateRuntimeConfigResponseDefaultTypeInternal;
extern UpdateRuntimeConfigResponseDefaultTypeInternal _UpdateRuntimeConfigResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull UpdateRuntimeConfigResponse_class_data_;
class VersionRequest;
struct VersionRequestDefaultTypeInternal;
extern VersionRequestDefaultTypeInternal _VersionRequest_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull VersionRequest_class_data_;
class VersionResponse;
struct VersionResponseDefaultTypeInternal;
extern VersionResponseDefaultTypeInternal _VersionResponse_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull VersionResponse_class_data_;
class WindowsContainerConfig;
struct WindowsContainerConfigDefaultTypeInternal;
extern WindowsContainerConfigDefaultTypeInternal _WindowsContainerConfig_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull WindowsContainerConfig_class_data_;
class WindowsContainerResources;
struct WindowsContainerResourcesDefaultTypeInternal;
extern WindowsContainerResourcesDefaultTypeInternal _WindowsContainerResources_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull WindowsContainerResources_class_data_;
class WindowsContainerSecurityContext;
struct WindowsContainerSecurityContextDefaultTypeInternal;
extern WindowsContainerSecurityContextDefaultTypeInternal _WindowsContainerSecurityContext_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull WindowsContainerSecurityContext_class_data_;
}  // namespace v1alpha2
}  // namespace runtime
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::runtime::v1alpha2::ContainerState_internal_data_>
    internal::EnumTraitsImpl::value<::runtime::v1alpha2::ContainerState>;
template <>
internal::EnumTraitsT<::runtime::v1alpha2::MountPropagation_internal_data_>
    internal::EnumTraitsImpl::value<::runtime::v1alpha2::MountPropagation>;
template <>
internal::EnumTraitsT<::runtime::v1alpha2::NamespaceMode_internal_data_>
    internal::EnumTraitsImpl::value<::runtime::v1alpha2::NamespaceMode>;
template <>
internal::EnumTraitsT<::runtime::v1alpha2::PodSandboxState_internal_data_>
    internal::EnumTraitsImpl::value<::runtime::v1alpha2::PodSandboxState>;
template <>
internal::EnumTraitsT<::runtime::v1alpha2::Protocol_internal_data_>
    internal::EnumTraitsImpl::value<::runtime::v1alpha2::Protocol>;
template <>
internal::EnumTraitsT<::runtime::v1alpha2::SecurityProfile_ProfileType_internal_data_>
    internal::EnumTraitsImpl::value<::runtime::v1alpha2::SecurityProfile_ProfileType>;
}  // namespace protobuf
}  // namespace google

namespace runtime {
namespace v1alpha2 {
enum SecurityProfile_ProfileType : int {
  SecurityProfile_ProfileType_RuntimeDefault = 0,
  SecurityProfile_ProfileType_Unconfined = 1,
  SecurityProfile_ProfileType_Localhost = 2,
  SecurityProfile_ProfileType_SecurityProfile_ProfileType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  SecurityProfile_ProfileType_SecurityProfile_ProfileType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t SecurityProfile_ProfileType_internal_data_[];
inline constexpr SecurityProfile_ProfileType SecurityProfile_ProfileType_ProfileType_MIN =
    static_cast<SecurityProfile_ProfileType>(0);
inline constexpr SecurityProfile_ProfileType SecurityProfile_ProfileType_ProfileType_MAX =
    static_cast<SecurityProfile_ProfileType>(2);
inline bool SecurityProfile_ProfileType_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int SecurityProfile_ProfileType_ProfileType_ARRAYSIZE = 2 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL SecurityProfile_ProfileType_descriptor();
template <typename T>
const ::std::string& SecurityProfile_ProfileType_Name(T value) {
  static_assert(::std::is_same<T, SecurityProfile_ProfileType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to ProfileType_Name().");
  return SecurityProfile_ProfileType_Name(static_cast<SecurityProfile_ProfileType>(value));
}
template <>
inline const ::std::string& SecurityProfile_ProfileType_Name(SecurityProfile_ProfileType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<SecurityProfile_ProfileType_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool SecurityProfile_ProfileType_Parse(
    ::absl::string_view name, SecurityProfile_ProfileType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<SecurityProfile_ProfileType>(SecurityProfile_ProfileType_descriptor(), name,
                                           value);
}
enum Protocol : int {
  TCP = 0,
  UDP = 1,
  SCTP = 2,
  Protocol_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  Protocol_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t Protocol_internal_data_[];
inline constexpr Protocol Protocol_MIN =
    static_cast<Protocol>(0);
inline constexpr Protocol Protocol_MAX =
    static_cast<Protocol>(2);
inline bool Protocol_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int Protocol_ARRAYSIZE = 2 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Protocol_descriptor();
template <typename T>
const ::std::string& Protocol_Name(T value) {
  static_assert(::std::is_same<T, Protocol>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Protocol_Name().");
  return Protocol_Name(static_cast<Protocol>(value));
}
template <>
inline const ::std::string& Protocol_Name(Protocol value) {
  return ::google::protobuf::internal::NameOfDenseEnum<Protocol_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool Protocol_Parse(
    ::absl::string_view name, Protocol* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<Protocol>(Protocol_descriptor(), name,
                                           value);
}
enum MountPropagation : int {
  PROPAGATION_PRIVATE = 0,
  PROPAGATION_HOST_TO_CONTAINER = 1,
  PROPAGATION_BIDIRECTIONAL = 2,
  MountPropagation_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  MountPropagation_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t MountPropagation_internal_data_[];
inline constexpr MountPropagation MountPropagation_MIN =
    static_cast<MountPropagation>(0);
inline constexpr MountPropagation MountPropagation_MAX =
    static_cast<MountPropagation>(2);
inline bool MountPropagation_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int MountPropagation_ARRAYSIZE = 2 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL MountPropagation_descriptor();
template <typename T>
const ::std::string& MountPropagation_Name(T value) {
  static_assert(::std::is_same<T, MountPropagation>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to MountPropagation_Name().");
  return MountPropagation_Name(static_cast<MountPropagation>(value));
}
template <>
inline const ::std::string& MountPropagation_Name(MountPropagation value) {
  return ::google::protobuf::internal::NameOfDenseEnum<MountPropagation_descriptor, 0, 2>(
      static_cast<int>(value));
}
inline bool MountPropagation_Parse(
    ::absl::string_view name, MountPropagation* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<MountPropagation>(MountPropagation_descriptor(), name,
                                           value);
}
enum NamespaceMode : int {
  POD = 0,
  CONTAINER = 1,
  NODE = 2,
  TARGET = 3,
  NamespaceMode_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  NamespaceMode_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t NamespaceMode_internal_data_[];
inline constexpr NamespaceMode NamespaceMode_MIN =
    static_cast<NamespaceMode>(0);
inline constexpr NamespaceMode NamespaceMode_MAX =
    static_cast<NamespaceMode>(3);
inline bool NamespaceMode_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int NamespaceMode_ARRAYSIZE = 3 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL NamespaceMode_descriptor();
template <typename T>
const ::std::string& NamespaceMode_Name(T value) {
  static_assert(::std::is_same<T, NamespaceMode>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to NamespaceMode_Name().");
  return NamespaceMode_Name(static_cast<NamespaceMode>(value));
}
template <>
inline const ::std::string& NamespaceMode_Name(NamespaceMode value) {
  return ::google::protobuf::internal::NameOfDenseEnum<NamespaceMode_descriptor, 0, 3>(
      static_cast<int>(value));
}
inline bool NamespaceMode_Parse(
    ::absl::string_view name, NamespaceMode* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<NamespaceMode>(NamespaceMode_descriptor(), name,
                                           value);
}
enum PodSandboxState : int {
  SANDBOX_READY = 0,
  SANDBOX_NOTREADY = 1,
  PodSandboxState_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  PodSandboxState_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t PodSandboxState_internal_data_[];
inline constexpr PodSandboxState PodSandboxState_MIN =
    static_cast<PodSandboxState>(0);
inline constexpr PodSandboxState PodSandboxState_MAX =
    static_cast<PodSandboxState>(1);
inline bool PodSandboxState_IsValid(int value) {
  return 0 <= value && value <= 1;
}
inline constexpr int PodSandboxState_ARRAYSIZE = 1 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL PodSandboxState_descriptor();
template <typename T>
const ::std::string& PodSandboxState_Name(T value) {
  static_assert(::std::is_same<T, PodSandboxState>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to PodSandboxState_Name().");
  return PodSandboxState_Name(static_cast<PodSandboxState>(value));
}
template <>
inline const ::std::string& PodSandboxState_Name(PodSandboxState value) {
  return ::google::protobuf::internal::NameOfDenseEnum<PodSandboxState_descriptor, 0, 1>(
      static_cast<int>(value));
}
inline bool PodSandboxState_Parse(
    ::absl::string_view name, PodSandboxState* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<PodSandboxState>(PodSandboxState_descriptor(), name,
                                           value);
}
enum ContainerState : int {
  CONTAINER_CREATED = 0,
  CONTAINER_RUNNING = 1,
  CONTAINER_EXITED = 2,
  CONTAINER_UNKNOWN = 3,
  ContainerState_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  ContainerState_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t ContainerState_internal_data_[];
inline constexpr ContainerState ContainerState_MIN =
    static_cast<ContainerState>(0);
inline constexpr ContainerState ContainerState_MAX =
    static_cast<ContainerState>(3);
inline bool ContainerState_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int ContainerState_ARRAYSIZE = 3 + 1;
const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL ContainerState_descriptor();
template <typename T>
const ::std::string& ContainerState_Name(T value) {
  static_assert(::std::is_same<T, ContainerState>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to ContainerState_Name().");
  return ContainerState_Name(static_cast<ContainerState>(value));
}
template <>
inline const ::std::string& ContainerState_Name(ContainerState value) {
  return ::google::protobuf::internal::NameOfDenseEnum<ContainerState_descriptor, 0, 3>(
      static_cast<int>(value));
}
inline bool ContainerState_Parse(
    ::absl::string_view name, ContainerState* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<ContainerState>(ContainerState_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class WindowsContainerSecurityContext final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.WindowsContainerSecurityContext) */ {
 public:
  inline WindowsContainerSecurityContext() : WindowsContainerSecurityContext(nullptr) {}
  ~WindowsContainerSecurityContext() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(WindowsContainerSecurityContext* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(WindowsContainerSecurityContext));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR WindowsContainerSecurityContext(::google::protobuf::internal::ConstantInitialized);

  inline WindowsContainerSecurityContext(const WindowsContainerSecurityContext& from) : WindowsContainerSecurityContext(nullptr, from) {}
  inline WindowsContainerSecurityContext(WindowsContainerSecurityContext&& from) noexcept
      : WindowsContainerSecurityContext(nullptr, ::std::move(from)) {}
  inline WindowsContainerSecurityContext& operator=(const WindowsContainerSecurityContext& from) {
    CopyFrom(from);
    return *this;
  }
  inline WindowsContainerSecurityContext& operator=(WindowsContainerSecurityContext&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const WindowsContainerSecurityContext& default_instance() {
    return *reinterpret_cast<const WindowsContainerSecurityContext*>(
        &_WindowsContainerSecurityContext_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 48;
  friend void swap(WindowsContainerSecurityContext& a, WindowsContainerSecurityContext& b) { a.Swap(&b); }
  inline void Swap(WindowsContainerSecurityContext* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(WindowsContainerSecurityContext* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  WindowsContainerSecurityContext* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<WindowsContainerSecurityContext>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const WindowsContainerSecurityContext& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const WindowsContainerSecurityContext& from) { WindowsContainerSecurityContext::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(WindowsContainerSecurityContext* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.WindowsContainerSecurityContext"; }

  explicit WindowsContainerSecurityContext(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  WindowsContainerSecurityContext(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const WindowsContainerSecurityContext& from);
  WindowsContainerSecurityContext(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, WindowsContainerSecurityContext&& from) noexcept
      : WindowsContainerSecurityContext(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRunAsUsernameFieldNumber = 1,
    kCredentialSpecFieldNumber = 2,
  };
  // string run_as_username = 1;
  void clear_run_as_username() ;
  const ::std::string& run_as_username() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_run_as_username(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_run_as_username();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_run_as_username();
  void set_allocated_run_as_username(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_run_as_username() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_run_as_username(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_run_as_username();

  public:
  // string credential_spec = 2;
  void clear_credential_spec() ;
  const ::std::string& credential_spec() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_credential_spec(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_credential_spec();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_credential_spec();
  void set_allocated_credential_spec(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_credential_spec() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_credential_spec(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_credential_spec();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.WindowsContainerSecurityContext)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 87,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const WindowsContainerSecurityContext& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr run_as_username_;
    ::google::protobuf::internal::ArenaStringPtr credential_spec_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull WindowsContainerSecurityContext_class_data_;
// -------------------------------------------------------------------

class WindowsContainerResources final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.WindowsContainerResources) */ {
 public:
  inline WindowsContainerResources() : WindowsContainerResources(nullptr) {}
  ~WindowsContainerResources() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(WindowsContainerResources* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(WindowsContainerResources));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR WindowsContainerResources(::google::protobuf::internal::ConstantInitialized);

  inline WindowsContainerResources(const WindowsContainerResources& from) : WindowsContainerResources(nullptr, from) {}
  inline WindowsContainerResources(WindowsContainerResources&& from) noexcept
      : WindowsContainerResources(nullptr, ::std::move(from)) {}
  inline WindowsContainerResources& operator=(const WindowsContainerResources& from) {
    CopyFrom(from);
    return *this;
  }
  inline WindowsContainerResources& operator=(WindowsContainerResources&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const WindowsContainerResources& default_instance() {
    return *reinterpret_cast<const WindowsContainerResources*>(
        &_WindowsContainerResources_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 50;
  friend void swap(WindowsContainerResources& a, WindowsContainerResources& b) { a.Swap(&b); }
  inline void Swap(WindowsContainerResources* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(WindowsContainerResources* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  WindowsContainerResources* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<WindowsContainerResources>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const WindowsContainerResources& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const WindowsContainerResources& from) { WindowsContainerResources::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(WindowsContainerResources* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.WindowsContainerResources"; }

  explicit WindowsContainerResources(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  WindowsContainerResources(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const WindowsContainerResources& from);
  WindowsContainerResources(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, WindowsContainerResources&& from) noexcept
      : WindowsContainerResources(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCpuSharesFieldNumber = 1,
    kCpuCountFieldNumber = 2,
    kCpuMaximumFieldNumber = 3,
    kMemoryLimitInBytesFieldNumber = 4,
  };
  // int64 cpu_shares = 1;
  void clear_cpu_shares() ;
  ::int64_t cpu_shares() const;
  void set_cpu_shares(::int64_t value);

  private:
  ::int64_t _internal_cpu_shares() const;
  void _internal_set_cpu_shares(::int64_t value);

  public:
  // int64 cpu_count = 2;
  void clear_cpu_count() ;
  ::int64_t cpu_count() const;
  void set_cpu_count(::int64_t value);

  private:
  ::int64_t _internal_cpu_count() const;
  void _internal_set_cpu_count(::int64_t value);

  public:
  // int64 cpu_maximum = 3;
  void clear_cpu_maximum() ;
  ::int64_t cpu_maximum() const;
  void set_cpu_maximum(::int64_t value);

  private:
  ::int64_t _internal_cpu_maximum() const;
  void _internal_set_cpu_maximum(::int64_t value);

  public:
  // int64 memory_limit_in_bytes = 4;
  void clear_memory_limit_in_bytes() ;
  ::int64_t memory_limit_in_bytes() const;
  void set_memory_limit_in_bytes(::int64_t value);

  private:
  ::int64_t _internal_memory_limit_in_bytes() const;
  void _internal_set_memory_limit_in_bytes(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.WindowsContainerResources)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const WindowsContainerResources& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int64_t cpu_shares_;
    ::int64_t cpu_count_;
    ::int64_t cpu_maximum_;
    ::int64_t memory_limit_in_bytes_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull WindowsContainerResources_class_data_;
// -------------------------------------------------------------------

class VersionResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.VersionResponse) */ {
 public:
  inline VersionResponse() : VersionResponse(nullptr) {}
  ~VersionResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(VersionResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(VersionResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR VersionResponse(::google::protobuf::internal::ConstantInitialized);

  inline VersionResponse(const VersionResponse& from) : VersionResponse(nullptr, from) {}
  inline VersionResponse(VersionResponse&& from) noexcept
      : VersionResponse(nullptr, ::std::move(from)) {}
  inline VersionResponse& operator=(const VersionResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline VersionResponse& operator=(VersionResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const VersionResponse& default_instance() {
    return *reinterpret_cast<const VersionResponse*>(
        &_VersionResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(VersionResponse& a, VersionResponse& b) { a.Swap(&b); }
  inline void Swap(VersionResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(VersionResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  VersionResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<VersionResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const VersionResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const VersionResponse& from) { VersionResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(VersionResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.VersionResponse"; }

  explicit VersionResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  VersionResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const VersionResponse& from);
  VersionResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, VersionResponse&& from) noexcept
      : VersionResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kVersionFieldNumber = 1,
    kRuntimeNameFieldNumber = 2,
    kRuntimeVersionFieldNumber = 3,
    kRuntimeApiVersionFieldNumber = 4,
  };
  // string version = 1;
  void clear_version() ;
  const ::std::string& version() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_version(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_version();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_version();
  void set_allocated_version(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_version() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_version(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_version();

  public:
  // string runtime_name = 2;
  void clear_runtime_name() ;
  const ::std::string& runtime_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_runtime_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_runtime_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_runtime_name();
  void set_allocated_runtime_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_runtime_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_runtime_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_runtime_name();

  public:
  // string runtime_version = 3;
  void clear_runtime_version() ;
  const ::std::string& runtime_version() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_runtime_version(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_runtime_version();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_runtime_version();
  void set_allocated_runtime_version(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_runtime_version() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_runtime_version(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_runtime_version();

  public:
  // string runtime_api_version = 4;
  void clear_runtime_api_version() ;
  const ::std::string& runtime_api_version() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_runtime_api_version(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_runtime_api_version();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_runtime_api_version();
  void set_allocated_runtime_api_version(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_runtime_api_version() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_runtime_api_version(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_runtime_api_version();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.VersionResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 94,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const VersionResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr version_;
    ::google::protobuf::internal::ArenaStringPtr runtime_name_;
    ::google::protobuf::internal::ArenaStringPtr runtime_version_;
    ::google::protobuf::internal::ArenaStringPtr runtime_api_version_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull VersionResponse_class_data_;
// -------------------------------------------------------------------

class VersionRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.VersionRequest) */ {
 public:
  inline VersionRequest() : VersionRequest(nullptr) {}
  ~VersionRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(VersionRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(VersionRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR VersionRequest(::google::protobuf::internal::ConstantInitialized);

  inline VersionRequest(const VersionRequest& from) : VersionRequest(nullptr, from) {}
  inline VersionRequest(VersionRequest&& from) noexcept
      : VersionRequest(nullptr, ::std::move(from)) {}
  inline VersionRequest& operator=(const VersionRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline VersionRequest& operator=(VersionRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const VersionRequest& default_instance() {
    return *reinterpret_cast<const VersionRequest*>(
        &_VersionRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(VersionRequest& a, VersionRequest& b) { a.Swap(&b); }
  inline void Swap(VersionRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(VersionRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  VersionRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<VersionRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const VersionRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const VersionRequest& from) { VersionRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(VersionRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.VersionRequest"; }

  explicit VersionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  VersionRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const VersionRequest& from);
  VersionRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, VersionRequest&& from) noexcept
      : VersionRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kVersionFieldNumber = 1,
  };
  // string version = 1;
  void clear_version() ;
  const ::std::string& version() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_version(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_version();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_version();
  void set_allocated_version(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_version() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_version(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_version();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.VersionRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 47,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const VersionRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr version_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull VersionRequest_class_data_;
// -------------------------------------------------------------------

class UpdateRuntimeConfigResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.UpdateRuntimeConfigResponse) */ {
 public:
  inline UpdateRuntimeConfigResponse() : UpdateRuntimeConfigResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(UpdateRuntimeConfigResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(UpdateRuntimeConfigResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR UpdateRuntimeConfigResponse(::google::protobuf::internal::ConstantInitialized);

  inline UpdateRuntimeConfigResponse(const UpdateRuntimeConfigResponse& from) : UpdateRuntimeConfigResponse(nullptr, from) {}
  inline UpdateRuntimeConfigResponse(UpdateRuntimeConfigResponse&& from) noexcept
      : UpdateRuntimeConfigResponse(nullptr, ::std::move(from)) {}
  inline UpdateRuntimeConfigResponse& operator=(const UpdateRuntimeConfigResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline UpdateRuntimeConfigResponse& operator=(UpdateRuntimeConfigResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const UpdateRuntimeConfigResponse& default_instance() {
    return *reinterpret_cast<const UpdateRuntimeConfigResponse*>(
        &_UpdateRuntimeConfigResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 104;
  friend void swap(UpdateRuntimeConfigResponse& a, UpdateRuntimeConfigResponse& b) { a.Swap(&b); }
  inline void Swap(UpdateRuntimeConfigResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UpdateRuntimeConfigResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  UpdateRuntimeConfigResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<UpdateRuntimeConfigResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const UpdateRuntimeConfigResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const UpdateRuntimeConfigResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.UpdateRuntimeConfigResponse"; }

  explicit UpdateRuntimeConfigResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  UpdateRuntimeConfigResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UpdateRuntimeConfigResponse& from);
  UpdateRuntimeConfigResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, UpdateRuntimeConfigResponse&& from) noexcept
      : UpdateRuntimeConfigResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateRuntimeConfigResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull UpdateRuntimeConfigResponse_class_data_;
// -------------------------------------------------------------------

class UpdateContainerResourcesResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.UpdateContainerResourcesResponse) */ {
 public:
  inline UpdateContainerResourcesResponse() : UpdateContainerResourcesResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(UpdateContainerResourcesResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(UpdateContainerResourcesResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR UpdateContainerResourcesResponse(::google::protobuf::internal::ConstantInitialized);

  inline UpdateContainerResourcesResponse(const UpdateContainerResourcesResponse& from) : UpdateContainerResourcesResponse(nullptr, from) {}
  inline UpdateContainerResourcesResponse(UpdateContainerResourcesResponse&& from) noexcept
      : UpdateContainerResourcesResponse(nullptr, ::std::move(from)) {}
  inline UpdateContainerResourcesResponse& operator=(const UpdateContainerResourcesResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline UpdateContainerResourcesResponse& operator=(UpdateContainerResourcesResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const UpdateContainerResourcesResponse& default_instance() {
    return *reinterpret_cast<const UpdateContainerResourcesResponse*>(
        &_UpdateContainerResourcesResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 80;
  friend void swap(UpdateContainerResourcesResponse& a, UpdateContainerResourcesResponse& b) { a.Swap(&b); }
  inline void Swap(UpdateContainerResourcesResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UpdateContainerResourcesResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  UpdateContainerResourcesResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<UpdateContainerResourcesResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const UpdateContainerResourcesResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const UpdateContainerResourcesResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.UpdateContainerResourcesResponse"; }

  explicit UpdateContainerResourcesResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  UpdateContainerResourcesResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UpdateContainerResourcesResponse& from);
  UpdateContainerResourcesResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, UpdateContainerResourcesResponse&& from) noexcept
      : UpdateContainerResourcesResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateContainerResourcesResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull UpdateContainerResourcesResponse_class_data_;
// -------------------------------------------------------------------

class UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 82,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class UInt64Value final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.UInt64Value) */ {
 public:
  inline UInt64Value() : UInt64Value(nullptr) {}
  ~UInt64Value() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(UInt64Value* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(UInt64Value));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR UInt64Value(::google::protobuf::internal::ConstantInitialized);

  inline UInt64Value(const UInt64Value& from) : UInt64Value(nullptr, from) {}
  inline UInt64Value(UInt64Value&& from) noexcept
      : UInt64Value(nullptr, ::std::move(from)) {}
  inline UInt64Value& operator=(const UInt64Value& from) {
    CopyFrom(from);
    return *this;
  }
  inline UInt64Value& operator=(UInt64Value&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const UInt64Value& default_instance() {
    return *reinterpret_cast<const UInt64Value*>(
        &_UInt64Value_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 111;
  friend void swap(UInt64Value& a, UInt64Value& b) { a.Swap(&b); }
  inline void Swap(UInt64Value* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UInt64Value* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  UInt64Value* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<UInt64Value>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const UInt64Value& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const UInt64Value& from) { UInt64Value::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(UInt64Value* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.UInt64Value"; }

  explicit UInt64Value(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  UInt64Value(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UInt64Value& from);
  UInt64Value(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, UInt64Value&& from) noexcept
      : UInt64Value(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kValueFieldNumber = 1,
  };
  // uint64 value = 1;
  void clear_value() ;
  ::uint64_t value() const;
  void set_value(::uint64_t value);

  private:
  ::uint64_t _internal_value() const;
  void _internal_set_value(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.UInt64Value)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const UInt64Value& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint64_t value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull UInt64Value_class_data_;
// -------------------------------------------------------------------

class StopPodSandboxResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StopPodSandboxResponse) */ {
 public:
  inline StopPodSandboxResponse() : StopPodSandboxResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StopPodSandboxResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StopPodSandboxResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StopPodSandboxResponse(::google::protobuf::internal::ConstantInitialized);

  inline StopPodSandboxResponse(const StopPodSandboxResponse& from) : StopPodSandboxResponse(nullptr, from) {}
  inline StopPodSandboxResponse(StopPodSandboxResponse&& from) noexcept
      : StopPodSandboxResponse(nullptr, ::std::move(from)) {}
  inline StopPodSandboxResponse& operator=(const StopPodSandboxResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline StopPodSandboxResponse& operator=(StopPodSandboxResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StopPodSandboxResponse& default_instance() {
    return *reinterpret_cast<const StopPodSandboxResponse*>(
        &_StopPodSandboxResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 18;
  friend void swap(StopPodSandboxResponse& a, StopPodSandboxResponse& b) { a.Swap(&b); }
  inline void Swap(StopPodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StopPodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StopPodSandboxResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<StopPodSandboxResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const StopPodSandboxResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const StopPodSandboxResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StopPodSandboxResponse"; }

  explicit StopPodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StopPodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StopPodSandboxResponse& from);
  StopPodSandboxResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StopPodSandboxResponse&& from) noexcept
      : StopPodSandboxResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopPodSandboxResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StopPodSandboxResponse_class_data_;
// -------------------------------------------------------------------

class StopPodSandboxRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StopPodSandboxRequest) */ {
 public:
  inline StopPodSandboxRequest() : StopPodSandboxRequest(nullptr) {}
  ~StopPodSandboxRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StopPodSandboxRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StopPodSandboxRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StopPodSandboxRequest(::google::protobuf::internal::ConstantInitialized);

  inline StopPodSandboxRequest(const StopPodSandboxRequest& from) : StopPodSandboxRequest(nullptr, from) {}
  inline StopPodSandboxRequest(StopPodSandboxRequest&& from) noexcept
      : StopPodSandboxRequest(nullptr, ::std::move(from)) {}
  inline StopPodSandboxRequest& operator=(const StopPodSandboxRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline StopPodSandboxRequest& operator=(StopPodSandboxRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StopPodSandboxRequest& default_instance() {
    return *reinterpret_cast<const StopPodSandboxRequest*>(
        &_StopPodSandboxRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 17;
  friend void swap(StopPodSandboxRequest& a, StopPodSandboxRequest& b) { a.Swap(&b); }
  inline void Swap(StopPodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StopPodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StopPodSandboxRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<StopPodSandboxRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const StopPodSandboxRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const StopPodSandboxRequest& from) { StopPodSandboxRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StopPodSandboxRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StopPodSandboxRequest"; }

  explicit StopPodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StopPodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StopPodSandboxRequest& from);
  StopPodSandboxRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StopPodSandboxRequest&& from) noexcept
      : StopPodSandboxRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPodSandboxIdFieldNumber = 1,
  };
  // string pod_sandbox_id = 1;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopPodSandboxRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 61,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StopPodSandboxRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StopPodSandboxRequest_class_data_;
// -------------------------------------------------------------------

class StopContainerResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StopContainerResponse) */ {
 public:
  inline StopContainerResponse() : StopContainerResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StopContainerResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StopContainerResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StopContainerResponse(::google::protobuf::internal::ConstantInitialized);

  inline StopContainerResponse(const StopContainerResponse& from) : StopContainerResponse(nullptr, from) {}
  inline StopContainerResponse(StopContainerResponse&& from) noexcept
      : StopContainerResponse(nullptr, ::std::move(from)) {}
  inline StopContainerResponse& operator=(const StopContainerResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline StopContainerResponse& operator=(StopContainerResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StopContainerResponse& default_instance() {
    return *reinterpret_cast<const StopContainerResponse*>(
        &_StopContainerResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 61;
  friend void swap(StopContainerResponse& a, StopContainerResponse& b) { a.Swap(&b); }
  inline void Swap(StopContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StopContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StopContainerResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<StopContainerResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const StopContainerResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const StopContainerResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StopContainerResponse"; }

  explicit StopContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StopContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StopContainerResponse& from);
  StopContainerResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StopContainerResponse&& from) noexcept
      : StopContainerResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopContainerResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StopContainerResponse_class_data_;
// -------------------------------------------------------------------

class StopContainerRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StopContainerRequest) */ {
 public:
  inline StopContainerRequest() : StopContainerRequest(nullptr) {}
  ~StopContainerRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StopContainerRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StopContainerRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StopContainerRequest(::google::protobuf::internal::ConstantInitialized);

  inline StopContainerRequest(const StopContainerRequest& from) : StopContainerRequest(nullptr, from) {}
  inline StopContainerRequest(StopContainerRequest&& from) noexcept
      : StopContainerRequest(nullptr, ::std::move(from)) {}
  inline StopContainerRequest& operator=(const StopContainerRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline StopContainerRequest& operator=(StopContainerRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StopContainerRequest& default_instance() {
    return *reinterpret_cast<const StopContainerRequest*>(
        &_StopContainerRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 60;
  friend void swap(StopContainerRequest& a, StopContainerRequest& b) { a.Swap(&b); }
  inline void Swap(StopContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StopContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StopContainerRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<StopContainerRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const StopContainerRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const StopContainerRequest& from) { StopContainerRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StopContainerRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StopContainerRequest"; }

  explicit StopContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StopContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StopContainerRequest& from);
  StopContainerRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StopContainerRequest&& from) noexcept
      : StopContainerRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
    kTimeoutFieldNumber = 2,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // int64 timeout = 2;
  void clear_timeout() ;
  ::int64_t timeout() const;
  void set_timeout(::int64_t value);

  private:
  ::int64_t _internal_timeout() const;
  void _internal_set_timeout(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopContainerRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 58,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StopContainerRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    ::int64_t timeout_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StopContainerRequest_class_data_;
// -------------------------------------------------------------------

class StatusResponse_InfoEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  StatusResponse_InfoEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StatusResponse_InfoEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit StatusResponse_InfoEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_StatusResponse_InfoEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 58,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull StatusResponse_InfoEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class StatusRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StatusRequest) */ {
 public:
  inline StatusRequest() : StatusRequest(nullptr) {}
  ~StatusRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StatusRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StatusRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StatusRequest(::google::protobuf::internal::ConstantInitialized);

  inline StatusRequest(const StatusRequest& from) : StatusRequest(nullptr, from) {}
  inline StatusRequest(StatusRequest&& from) noexcept
      : StatusRequest(nullptr, ::std::move(from)) {}
  inline StatusRequest& operator=(const StatusRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline StatusRequest& operator=(StatusRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StatusRequest& default_instance() {
    return *reinterpret_cast<const StatusRequest*>(
        &_StatusRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 107;
  friend void swap(StatusRequest& a, StatusRequest& b) { a.Swap(&b); }
  inline void Swap(StatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StatusRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<StatusRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const StatusRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const StatusRequest& from) { StatusRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StatusRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StatusRequest"; }

  explicit StatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StatusRequest& from);
  StatusRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StatusRequest&& from) noexcept
      : StatusRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kVerboseFieldNumber = 1,
  };
  // bool verbose = 1;
  void clear_verbose() ;
  bool verbose() const;
  void set_verbose(bool value);

  private:
  bool _internal_verbose() const;
  void _internal_set_verbose(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StatusRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StatusRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    bool verbose_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StatusRequest_class_data_;
// -------------------------------------------------------------------

class StartContainerResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StartContainerResponse) */ {
 public:
  inline StartContainerResponse() : StartContainerResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StartContainerResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StartContainerResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StartContainerResponse(::google::protobuf::internal::ConstantInitialized);

  inline StartContainerResponse(const StartContainerResponse& from) : StartContainerResponse(nullptr, from) {}
  inline StartContainerResponse(StartContainerResponse&& from) noexcept
      : StartContainerResponse(nullptr, ::std::move(from)) {}
  inline StartContainerResponse& operator=(const StartContainerResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline StartContainerResponse& operator=(StartContainerResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StartContainerResponse& default_instance() {
    return *reinterpret_cast<const StartContainerResponse*>(
        &_StartContainerResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 59;
  friend void swap(StartContainerResponse& a, StartContainerResponse& b) { a.Swap(&b); }
  inline void Swap(StartContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StartContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StartContainerResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<StartContainerResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const StartContainerResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const StartContainerResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StartContainerResponse"; }

  explicit StartContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StartContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StartContainerResponse& from);
  StartContainerResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StartContainerResponse&& from) noexcept
      : StartContainerResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StartContainerResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StartContainerResponse_class_data_;
// -------------------------------------------------------------------

class StartContainerRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StartContainerRequest) */ {
 public:
  inline StartContainerRequest() : StartContainerRequest(nullptr) {}
  ~StartContainerRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StartContainerRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StartContainerRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StartContainerRequest(::google::protobuf::internal::ConstantInitialized);

  inline StartContainerRequest(const StartContainerRequest& from) : StartContainerRequest(nullptr, from) {}
  inline StartContainerRequest(StartContainerRequest&& from) noexcept
      : StartContainerRequest(nullptr, ::std::move(from)) {}
  inline StartContainerRequest& operator=(const StartContainerRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline StartContainerRequest& operator=(StartContainerRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StartContainerRequest& default_instance() {
    return *reinterpret_cast<const StartContainerRequest*>(
        &_StartContainerRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 58;
  friend void swap(StartContainerRequest& a, StartContainerRequest& b) { a.Swap(&b); }
  inline void Swap(StartContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StartContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StartContainerRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<StartContainerRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const StartContainerRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const StartContainerRequest& from) { StartContainerRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StartContainerRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StartContainerRequest"; }

  explicit StartContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StartContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StartContainerRequest& from);
  StartContainerRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StartContainerRequest&& from) noexcept
      : StartContainerRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StartContainerRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 59,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StartContainerRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StartContainerRequest_class_data_;
// -------------------------------------------------------------------

class SecurityProfile final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.SecurityProfile) */ {
 public:
  inline SecurityProfile() : SecurityProfile(nullptr) {}
  ~SecurityProfile() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SecurityProfile* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SecurityProfile));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SecurityProfile(::google::protobuf::internal::ConstantInitialized);

  inline SecurityProfile(const SecurityProfile& from) : SecurityProfile(nullptr, from) {}
  inline SecurityProfile(SecurityProfile&& from) noexcept
      : SecurityProfile(nullptr, ::std::move(from)) {}
  inline SecurityProfile& operator=(const SecurityProfile& from) {
    CopyFrom(from);
    return *this;
  }
  inline SecurityProfile& operator=(SecurityProfile&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SecurityProfile& default_instance() {
    return *reinterpret_cast<const SecurityProfile*>(
        &_SecurityProfile_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 8;
  friend void swap(SecurityProfile& a, SecurityProfile& b) { a.Swap(&b); }
  inline void Swap(SecurityProfile* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SecurityProfile* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SecurityProfile* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SecurityProfile>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SecurityProfile& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SecurityProfile& from) { SecurityProfile::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SecurityProfile* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.SecurityProfile"; }

  explicit SecurityProfile(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SecurityProfile(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SecurityProfile& from);
  SecurityProfile(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SecurityProfile&& from) noexcept
      : SecurityProfile(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using ProfileType = SecurityProfile_ProfileType;
  static constexpr ProfileType RuntimeDefault = SecurityProfile_ProfileType_RuntimeDefault;
  static constexpr ProfileType Unconfined = SecurityProfile_ProfileType_Unconfined;
  static constexpr ProfileType Localhost = SecurityProfile_ProfileType_Localhost;
  static inline bool ProfileType_IsValid(int value) {
    return SecurityProfile_ProfileType_IsValid(value);
  }
  static constexpr ProfileType ProfileType_MIN = SecurityProfile_ProfileType_ProfileType_MIN;
  static constexpr ProfileType ProfileType_MAX = SecurityProfile_ProfileType_ProfileType_MAX;
  static constexpr int ProfileType_ARRAYSIZE = SecurityProfile_ProfileType_ProfileType_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL ProfileType_descriptor() {
    return SecurityProfile_ProfileType_descriptor();
  }
  template <typename T>
  static inline const ::std::string& ProfileType_Name(T value) {
    return SecurityProfile_ProfileType_Name(value);
  }
  static inline bool ProfileType_Parse(
      ::absl::string_view name, ProfileType* PROTOBUF_NONNULL value) {
    return SecurityProfile_ProfileType_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kLocalhostRefFieldNumber = 2,
    kProfileTypeFieldNumber = 1,
  };
  // string localhost_ref = 2;
  void clear_localhost_ref() ;
  const ::std::string& localhost_ref() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_localhost_ref(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_localhost_ref();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_localhost_ref();
  void set_allocated_localhost_ref(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_localhost_ref() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_localhost_ref(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_localhost_ref();

  public:
  // .runtime.v1alpha2.SecurityProfile.ProfileType profile_type = 1;
  void clear_profile_type() ;
  ::runtime::v1alpha2::SecurityProfile_ProfileType profile_type() const;
  void set_profile_type(::runtime::v1alpha2::SecurityProfile_ProfileType value);

  private:
  ::runtime::v1alpha2::SecurityProfile_ProfileType _internal_profile_type() const;
  void _internal_set_profile_type(::runtime::v1alpha2::SecurityProfile_ProfileType value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.SecurityProfile)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 54,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SecurityProfile& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr localhost_ref_;
    int profile_type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull SecurityProfile_class_data_;
// -------------------------------------------------------------------

class SELinuxOption final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.SELinuxOption) */ {
 public:
  inline SELinuxOption() : SELinuxOption(nullptr) {}
  ~SELinuxOption() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SELinuxOption* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SELinuxOption));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR SELinuxOption(::google::protobuf::internal::ConstantInitialized);

  inline SELinuxOption(const SELinuxOption& from) : SELinuxOption(nullptr, from) {}
  inline SELinuxOption(SELinuxOption&& from) noexcept
      : SELinuxOption(nullptr, ::std::move(from)) {}
  inline SELinuxOption& operator=(const SELinuxOption& from) {
    CopyFrom(from);
    return *this;
  }
  inline SELinuxOption& operator=(SELinuxOption&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const SELinuxOption& default_instance() {
    return *reinterpret_cast<const SELinuxOption*>(
        &_SELinuxOption_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 44;
  friend void swap(SELinuxOption& a, SELinuxOption& b) { a.Swap(&b); }
  inline void Swap(SELinuxOption* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SELinuxOption* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  SELinuxOption* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SELinuxOption>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SELinuxOption& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SELinuxOption& from) { SELinuxOption::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SELinuxOption* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.SELinuxOption"; }

  explicit SELinuxOption(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SELinuxOption(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SELinuxOption& from);
  SELinuxOption(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SELinuxOption&& from) noexcept
      : SELinuxOption(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUserFieldNumber = 1,
    kRoleFieldNumber = 2,
    kTypeFieldNumber = 3,
    kLevelFieldNumber = 4,
  };
  // string user = 1;
  void clear_user() ;
  const ::std::string& user() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_user(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_user();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_user();
  void set_allocated_user(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_user() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_user(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_user();

  public:
  // string role = 2;
  void clear_role() ;
  const ::std::string& role() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_role(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_role();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_role();
  void set_allocated_role(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_role() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_role(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_role();

  public:
  // string type = 3;
  void clear_type() ;
  const ::std::string& type() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_type(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_type();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_type();
  void set_allocated_type(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_type() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_type(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_type();

  public:
  // string level = 4;
  void clear_level() ;
  const ::std::string& level() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_level(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_level();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_level();
  void set_allocated_level(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_level() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_level(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_level();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.SELinuxOption)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 56,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SELinuxOption& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr user_;
    ::google::protobuf::internal::ArenaStringPtr role_;
    ::google::protobuf::internal::ArenaStringPtr type_;
    ::google::protobuf::internal::ArenaStringPtr level_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull SELinuxOption_class_data_;
// -------------------------------------------------------------------

class RuntimeCondition final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RuntimeCondition) */ {
 public:
  inline RuntimeCondition() : RuntimeCondition(nullptr) {}
  ~RuntimeCondition() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RuntimeCondition* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RuntimeCondition));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RuntimeCondition(::google::protobuf::internal::ConstantInitialized);

  inline RuntimeCondition(const RuntimeCondition& from) : RuntimeCondition(nullptr, from) {}
  inline RuntimeCondition(RuntimeCondition&& from) noexcept
      : RuntimeCondition(nullptr, ::std::move(from)) {}
  inline RuntimeCondition& operator=(const RuntimeCondition& from) {
    CopyFrom(from);
    return *this;
  }
  inline RuntimeCondition& operator=(RuntimeCondition&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RuntimeCondition& default_instance() {
    return *reinterpret_cast<const RuntimeCondition*>(
        &_RuntimeCondition_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 105;
  friend void swap(RuntimeCondition& a, RuntimeCondition& b) { a.Swap(&b); }
  inline void Swap(RuntimeCondition* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RuntimeCondition* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RuntimeCondition* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RuntimeCondition>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RuntimeCondition& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RuntimeCondition& from) { RuntimeCondition::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RuntimeCondition* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RuntimeCondition"; }

  explicit RuntimeCondition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RuntimeCondition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RuntimeCondition& from);
  RuntimeCondition(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RuntimeCondition&& from) noexcept
      : RuntimeCondition(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTypeFieldNumber = 1,
    kReasonFieldNumber = 3,
    kMessageFieldNumber = 4,
    kStatusFieldNumber = 2,
  };
  // string type = 1;
  void clear_type() ;
  const ::std::string& type() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_type(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_type();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_type();
  void set_allocated_type(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_type() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_type(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_type();

  public:
  // string reason = 3;
  void clear_reason() ;
  const ::std::string& reason() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_reason(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_reason();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_reason();
  void set_allocated_reason(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_reason() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_reason(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_reason();

  public:
  // string message = 4;
  void clear_message() ;
  const ::std::string& message() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_message(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_message();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_message();
  void set_allocated_message(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_message() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_message(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_message();

  public:
  // bool status = 2;
  void clear_status() ;
  bool status() const;
  void set_status(bool value);

  private:
  bool _internal_status() const;
  void _internal_set_status(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RuntimeCondition)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 59,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RuntimeCondition& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr type_;
    ::google::protobuf::internal::ArenaStringPtr reason_;
    ::google::protobuf::internal::ArenaStringPtr message_;
    bool status_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RuntimeCondition_class_data_;
// -------------------------------------------------------------------

class RunPodSandboxResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RunPodSandboxResponse) */ {
 public:
  inline RunPodSandboxResponse() : RunPodSandboxResponse(nullptr) {}
  ~RunPodSandboxResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RunPodSandboxResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RunPodSandboxResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RunPodSandboxResponse(::google::protobuf::internal::ConstantInitialized);

  inline RunPodSandboxResponse(const RunPodSandboxResponse& from) : RunPodSandboxResponse(nullptr, from) {}
  inline RunPodSandboxResponse(RunPodSandboxResponse&& from) noexcept
      : RunPodSandboxResponse(nullptr, ::std::move(from)) {}
  inline RunPodSandboxResponse& operator=(const RunPodSandboxResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline RunPodSandboxResponse& operator=(RunPodSandboxResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RunPodSandboxResponse& default_instance() {
    return *reinterpret_cast<const RunPodSandboxResponse*>(
        &_RunPodSandboxResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 16;
  friend void swap(RunPodSandboxResponse& a, RunPodSandboxResponse& b) { a.Swap(&b); }
  inline void Swap(RunPodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RunPodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RunPodSandboxResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RunPodSandboxResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RunPodSandboxResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RunPodSandboxResponse& from) { RunPodSandboxResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RunPodSandboxResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RunPodSandboxResponse"; }

  explicit RunPodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RunPodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RunPodSandboxResponse& from);
  RunPodSandboxResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RunPodSandboxResponse&& from) noexcept
      : RunPodSandboxResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPodSandboxIdFieldNumber = 1,
  };
  // string pod_sandbox_id = 1;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RunPodSandboxResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 61,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RunPodSandboxResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RunPodSandboxResponse_class_data_;
// -------------------------------------------------------------------

class ReopenContainerLogResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ReopenContainerLogResponse) */ {
 public:
  inline ReopenContainerLogResponse() : ReopenContainerLogResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ReopenContainerLogResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ReopenContainerLogResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ReopenContainerLogResponse(::google::protobuf::internal::ConstantInitialized);

  inline ReopenContainerLogResponse(const ReopenContainerLogResponse& from) : ReopenContainerLogResponse(nullptr, from) {}
  inline ReopenContainerLogResponse(ReopenContainerLogResponse&& from) noexcept
      : ReopenContainerLogResponse(nullptr, ::std::move(from)) {}
  inline ReopenContainerLogResponse& operator=(const ReopenContainerLogResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ReopenContainerLogResponse& operator=(ReopenContainerLogResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ReopenContainerLogResponse& default_instance() {
    return *reinterpret_cast<const ReopenContainerLogResponse*>(
        &_ReopenContainerLogResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 128;
  friend void swap(ReopenContainerLogResponse& a, ReopenContainerLogResponse& b) { a.Swap(&b); }
  inline void Swap(ReopenContainerLogResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ReopenContainerLogResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ReopenContainerLogResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ReopenContainerLogResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ReopenContainerLogResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ReopenContainerLogResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ReopenContainerLogResponse"; }

  explicit ReopenContainerLogResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ReopenContainerLogResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ReopenContainerLogResponse& from);
  ReopenContainerLogResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ReopenContainerLogResponse&& from) noexcept
      : ReopenContainerLogResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ReopenContainerLogResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ReopenContainerLogResponse_class_data_;
// -------------------------------------------------------------------

class ReopenContainerLogRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ReopenContainerLogRequest) */ {
 public:
  inline ReopenContainerLogRequest() : ReopenContainerLogRequest(nullptr) {}
  ~ReopenContainerLogRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ReopenContainerLogRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ReopenContainerLogRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ReopenContainerLogRequest(::google::protobuf::internal::ConstantInitialized);

  inline ReopenContainerLogRequest(const ReopenContainerLogRequest& from) : ReopenContainerLogRequest(nullptr, from) {}
  inline ReopenContainerLogRequest(ReopenContainerLogRequest&& from) noexcept
      : ReopenContainerLogRequest(nullptr, ::std::move(from)) {}
  inline ReopenContainerLogRequest& operator=(const ReopenContainerLogRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ReopenContainerLogRequest& operator=(ReopenContainerLogRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ReopenContainerLogRequest& default_instance() {
    return *reinterpret_cast<const ReopenContainerLogRequest*>(
        &_ReopenContainerLogRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 127;
  friend void swap(ReopenContainerLogRequest& a, ReopenContainerLogRequest& b) { a.Swap(&b); }
  inline void Swap(ReopenContainerLogRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ReopenContainerLogRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ReopenContainerLogRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ReopenContainerLogRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ReopenContainerLogRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ReopenContainerLogRequest& from) { ReopenContainerLogRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ReopenContainerLogRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ReopenContainerLogRequest"; }

  explicit ReopenContainerLogRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ReopenContainerLogRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ReopenContainerLogRequest& from);
  ReopenContainerLogRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ReopenContainerLogRequest&& from) noexcept
      : ReopenContainerLogRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ReopenContainerLogRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 63,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ReopenContainerLogRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ReopenContainerLogRequest_class_data_;
// -------------------------------------------------------------------

class RemovePodSandboxResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RemovePodSandboxResponse) */ {
 public:
  inline RemovePodSandboxResponse() : RemovePodSandboxResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RemovePodSandboxResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RemovePodSandboxResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RemovePodSandboxResponse(::google::protobuf::internal::ConstantInitialized);

  inline RemovePodSandboxResponse(const RemovePodSandboxResponse& from) : RemovePodSandboxResponse(nullptr, from) {}
  inline RemovePodSandboxResponse(RemovePodSandboxResponse&& from) noexcept
      : RemovePodSandboxResponse(nullptr, ::std::move(from)) {}
  inline RemovePodSandboxResponse& operator=(const RemovePodSandboxResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline RemovePodSandboxResponse& operator=(RemovePodSandboxResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RemovePodSandboxResponse& default_instance() {
    return *reinterpret_cast<const RemovePodSandboxResponse*>(
        &_RemovePodSandboxResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 20;
  friend void swap(RemovePodSandboxResponse& a, RemovePodSandboxResponse& b) { a.Swap(&b); }
  inline void Swap(RemovePodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RemovePodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RemovePodSandboxResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<RemovePodSandboxResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const RemovePodSandboxResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const RemovePodSandboxResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RemovePodSandboxResponse"; }

  explicit RemovePodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RemovePodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RemovePodSandboxResponse& from);
  RemovePodSandboxResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RemovePodSandboxResponse&& from) noexcept
      : RemovePodSandboxResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemovePodSandboxResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RemovePodSandboxResponse_class_data_;
// -------------------------------------------------------------------

class RemovePodSandboxRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RemovePodSandboxRequest) */ {
 public:
  inline RemovePodSandboxRequest() : RemovePodSandboxRequest(nullptr) {}
  ~RemovePodSandboxRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RemovePodSandboxRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RemovePodSandboxRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RemovePodSandboxRequest(::google::protobuf::internal::ConstantInitialized);

  inline RemovePodSandboxRequest(const RemovePodSandboxRequest& from) : RemovePodSandboxRequest(nullptr, from) {}
  inline RemovePodSandboxRequest(RemovePodSandboxRequest&& from) noexcept
      : RemovePodSandboxRequest(nullptr, ::std::move(from)) {}
  inline RemovePodSandboxRequest& operator=(const RemovePodSandboxRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline RemovePodSandboxRequest& operator=(RemovePodSandboxRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RemovePodSandboxRequest& default_instance() {
    return *reinterpret_cast<const RemovePodSandboxRequest*>(
        &_RemovePodSandboxRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 19;
  friend void swap(RemovePodSandboxRequest& a, RemovePodSandboxRequest& b) { a.Swap(&b); }
  inline void Swap(RemovePodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RemovePodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RemovePodSandboxRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RemovePodSandboxRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RemovePodSandboxRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RemovePodSandboxRequest& from) { RemovePodSandboxRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RemovePodSandboxRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RemovePodSandboxRequest"; }

  explicit RemovePodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RemovePodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RemovePodSandboxRequest& from);
  RemovePodSandboxRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RemovePodSandboxRequest&& from) noexcept
      : RemovePodSandboxRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPodSandboxIdFieldNumber = 1,
  };
  // string pod_sandbox_id = 1;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemovePodSandboxRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 63,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RemovePodSandboxRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RemovePodSandboxRequest_class_data_;
// -------------------------------------------------------------------

class RemoveImageResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RemoveImageResponse) */ {
 public:
  inline RemoveImageResponse() : RemoveImageResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RemoveImageResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RemoveImageResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RemoveImageResponse(::google::protobuf::internal::ConstantInitialized);

  inline RemoveImageResponse(const RemoveImageResponse& from) : RemoveImageResponse(nullptr, from) {}
  inline RemoveImageResponse(RemoveImageResponse&& from) noexcept
      : RemoveImageResponse(nullptr, ::std::move(from)) {}
  inline RemoveImageResponse& operator=(const RemoveImageResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline RemoveImageResponse& operator=(RemoveImageResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RemoveImageResponse& default_instance() {
    return *reinterpret_cast<const RemoveImageResponse*>(
        &_RemoveImageResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 100;
  friend void swap(RemoveImageResponse& a, RemoveImageResponse& b) { a.Swap(&b); }
  inline void Swap(RemoveImageResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RemoveImageResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RemoveImageResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<RemoveImageResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const RemoveImageResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const RemoveImageResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RemoveImageResponse"; }

  explicit RemoveImageResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RemoveImageResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RemoveImageResponse& from);
  RemoveImageResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RemoveImageResponse&& from) noexcept
      : RemoveImageResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveImageResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RemoveImageResponse_class_data_;
// -------------------------------------------------------------------

class RemoveContainerResponse final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RemoveContainerResponse) */ {
 public:
  inline RemoveContainerResponse() : RemoveContainerResponse(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RemoveContainerResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RemoveContainerResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RemoveContainerResponse(::google::protobuf::internal::ConstantInitialized);

  inline RemoveContainerResponse(const RemoveContainerResponse& from) : RemoveContainerResponse(nullptr, from) {}
  inline RemoveContainerResponse(RemoveContainerResponse&& from) noexcept
      : RemoveContainerResponse(nullptr, ::std::move(from)) {}
  inline RemoveContainerResponse& operator=(const RemoveContainerResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline RemoveContainerResponse& operator=(RemoveContainerResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RemoveContainerResponse& default_instance() {
    return *reinterpret_cast<const RemoveContainerResponse*>(
        &_RemoveContainerResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 63;
  friend void swap(RemoveContainerResponse& a, RemoveContainerResponse& b) { a.Swap(&b); }
  inline void Swap(RemoveContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RemoveContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RemoveContainerResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<RemoveContainerResponse>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const RemoveContainerResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const RemoveContainerResponse& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RemoveContainerResponse"; }

  explicit RemoveContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RemoveContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RemoveContainerResponse& from);
  RemoveContainerResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RemoveContainerResponse&& from) noexcept
      : RemoveContainerResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveContainerResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RemoveContainerResponse_class_data_;
// -------------------------------------------------------------------

class RemoveContainerRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RemoveContainerRequest) */ {
 public:
  inline RemoveContainerRequest() : RemoveContainerRequest(nullptr) {}
  ~RemoveContainerRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RemoveContainerRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RemoveContainerRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RemoveContainerRequest(::google::protobuf::internal::ConstantInitialized);

  inline RemoveContainerRequest(const RemoveContainerRequest& from) : RemoveContainerRequest(nullptr, from) {}
  inline RemoveContainerRequest(RemoveContainerRequest&& from) noexcept
      : RemoveContainerRequest(nullptr, ::std::move(from)) {}
  inline RemoveContainerRequest& operator=(const RemoveContainerRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline RemoveContainerRequest& operator=(RemoveContainerRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RemoveContainerRequest& default_instance() {
    return *reinterpret_cast<const RemoveContainerRequest*>(
        &_RemoveContainerRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 62;
  friend void swap(RemoveContainerRequest& a, RemoveContainerRequest& b) { a.Swap(&b); }
  inline void Swap(RemoveContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RemoveContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RemoveContainerRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RemoveContainerRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RemoveContainerRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RemoveContainerRequest& from) { RemoveContainerRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RemoveContainerRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RemoveContainerRequest"; }

  explicit RemoveContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RemoveContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RemoveContainerRequest& from);
  RemoveContainerRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RemoveContainerRequest&& from) noexcept
      : RemoveContainerRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveContainerRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 60,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RemoveContainerRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RemoveContainerRequest_class_data_;
// -------------------------------------------------------------------

class PullImageResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PullImageResponse) */ {
 public:
  inline PullImageResponse() : PullImageResponse(nullptr) {}
  ~PullImageResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PullImageResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PullImageResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PullImageResponse(::google::protobuf::internal::ConstantInitialized);

  inline PullImageResponse(const PullImageResponse& from) : PullImageResponse(nullptr, from) {}
  inline PullImageResponse(PullImageResponse&& from) noexcept
      : PullImageResponse(nullptr, ::std::move(from)) {}
  inline PullImageResponse& operator=(const PullImageResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline PullImageResponse& operator=(PullImageResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PullImageResponse& default_instance() {
    return *reinterpret_cast<const PullImageResponse*>(
        &_PullImageResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 98;
  friend void swap(PullImageResponse& a, PullImageResponse& b) { a.Swap(&b); }
  inline void Swap(PullImageResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PullImageResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PullImageResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PullImageResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PullImageResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PullImageResponse& from) { PullImageResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PullImageResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PullImageResponse"; }

  explicit PullImageResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PullImageResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PullImageResponse& from);
  PullImageResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PullImageResponse&& from) noexcept
      : PullImageResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageRefFieldNumber = 1,
  };
  // string image_ref = 1;
  void clear_image_ref() ;
  const ::std::string& image_ref() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_image_ref(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_image_ref();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_image_ref();
  void set_allocated_image_ref(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_image_ref() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_image_ref(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_image_ref();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PullImageResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 52,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PullImageResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr image_ref_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PullImageResponse_class_data_;
// -------------------------------------------------------------------

class PortMapping final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PortMapping) */ {
 public:
  inline PortMapping() : PortMapping(nullptr) {}
  ~PortMapping() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PortMapping* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PortMapping));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PortMapping(::google::protobuf::internal::ConstantInitialized);

  inline PortMapping(const PortMapping& from) : PortMapping(nullptr, from) {}
  inline PortMapping(PortMapping&& from) noexcept
      : PortMapping(nullptr, ::std::move(from)) {}
  inline PortMapping& operator=(const PortMapping& from) {
    CopyFrom(from);
    return *this;
  }
  inline PortMapping& operator=(PortMapping&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PortMapping& default_instance() {
    return *reinterpret_cast<const PortMapping*>(
        &_PortMapping_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(PortMapping& a, PortMapping& b) { a.Swap(&b); }
  inline void Swap(PortMapping* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PortMapping* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PortMapping* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PortMapping>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PortMapping& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PortMapping& from) { PortMapping::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PortMapping* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PortMapping"; }

  explicit PortMapping(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PortMapping(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PortMapping& from);
  PortMapping(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PortMapping&& from) noexcept
      : PortMapping(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kHostIpFieldNumber = 4,
    kProtocolFieldNumber = 1,
    kContainerPortFieldNumber = 2,
    kHostPortFieldNumber = 3,
  };
  // string host_ip = 4;
  void clear_host_ip() ;
  const ::std::string& host_ip() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_host_ip(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_host_ip();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_host_ip();
  void set_allocated_host_ip(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_host_ip() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_host_ip(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_host_ip();

  public:
  // .runtime.v1alpha2.Protocol protocol = 1;
  void clear_protocol() ;
  ::runtime::v1alpha2::Protocol protocol() const;
  void set_protocol(::runtime::v1alpha2::Protocol value);

  private:
  ::runtime::v1alpha2::Protocol _internal_protocol() const;
  void _internal_set_protocol(::runtime::v1alpha2::Protocol value);

  public:
  // int32 container_port = 2;
  void clear_container_port() ;
  ::int32_t container_port() const;
  void set_container_port(::int32_t value);

  private:
  ::int32_t _internal_container_port() const;
  void _internal_set_container_port(::int32_t value);

  public:
  // int32 host_port = 3;
  void clear_host_port() ;
  ::int32_t host_port() const;
  void set_host_port(::int32_t value);

  private:
  ::int32_t _internal_host_port() const;
  void _internal_set_host_port(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PortMapping)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 44,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PortMapping& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr host_ip_;
    int protocol_;
    ::int32_t container_port_;
    ::int32_t host_port_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PortMapping_class_data_;
// -------------------------------------------------------------------

class PortForwardResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PortForwardResponse) */ {
 public:
  inline PortForwardResponse() : PortForwardResponse(nullptr) {}
  ~PortForwardResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PortForwardResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PortForwardResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PortForwardResponse(::google::protobuf::internal::ConstantInitialized);

  inline PortForwardResponse(const PortForwardResponse& from) : PortForwardResponse(nullptr, from) {}
  inline PortForwardResponse(PortForwardResponse&& from) noexcept
      : PortForwardResponse(nullptr, ::std::move(from)) {}
  inline PortForwardResponse& operator=(const PortForwardResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline PortForwardResponse& operator=(PortForwardResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PortForwardResponse& default_instance() {
    return *reinterpret_cast<const PortForwardResponse*>(
        &_PortForwardResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 88;
  friend void swap(PortForwardResponse& a, PortForwardResponse& b) { a.Swap(&b); }
  inline void Swap(PortForwardResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PortForwardResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PortForwardResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PortForwardResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PortForwardResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PortForwardResponse& from) { PortForwardResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PortForwardResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PortForwardResponse"; }

  explicit PortForwardResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PortForwardResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PortForwardResponse& from);
  PortForwardResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PortForwardResponse&& from) noexcept
      : PortForwardResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUrlFieldNumber = 1,
  };
  // string url = 1;
  void clear_url() ;
  const ::std::string& url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_url();
  void set_allocated_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_url();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PortForwardResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 48,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PortForwardResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr url_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PortForwardResponse_class_data_;
// -------------------------------------------------------------------

class PortForwardRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PortForwardRequest) */ {
 public:
  inline PortForwardRequest() : PortForwardRequest(nullptr) {}
  ~PortForwardRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PortForwardRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PortForwardRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PortForwardRequest(::google::protobuf::internal::ConstantInitialized);

  inline PortForwardRequest(const PortForwardRequest& from) : PortForwardRequest(nullptr, from) {}
  inline PortForwardRequest(PortForwardRequest&& from) noexcept
      : PortForwardRequest(nullptr, ::std::move(from)) {}
  inline PortForwardRequest& operator=(const PortForwardRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline PortForwardRequest& operator=(PortForwardRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PortForwardRequest& default_instance() {
    return *reinterpret_cast<const PortForwardRequest*>(
        &_PortForwardRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 87;
  friend void swap(PortForwardRequest& a, PortForwardRequest& b) { a.Swap(&b); }
  inline void Swap(PortForwardRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PortForwardRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PortForwardRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PortForwardRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PortForwardRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PortForwardRequest& from) { PortForwardRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PortForwardRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PortForwardRequest"; }

  explicit PortForwardRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PortForwardRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PortForwardRequest& from);
  PortForwardRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PortForwardRequest&& from) noexcept
      : PortForwardRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPortFieldNumber = 2,
    kPodSandboxIdFieldNumber = 1,
  };
  // repeated int32 port = 2;
  int port_size() const;
  private:
  int _internal_port_size() const;

  public:
  void clear_port() ;
  ::int32_t port(int index) const;
  void set_port(int index, ::int32_t value);
  void add_port(::int32_t value);
  const ::google::protobuf::RepeatedField<::int32_t>& port() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL mutable_port();

  private:
  const ::google::protobuf::RepeatedField<::int32_t>& _internal_port() const;
  ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL _internal_mutable_port();

  public:
  // string pod_sandbox_id = 1;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PortForwardRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 58,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PortForwardRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int32_t> port_;
    ::google::protobuf::internal::CachedSize _port_cached_byte_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PortForwardRequest_class_data_;
// -------------------------------------------------------------------

class PodSandboxStatusResponse_InfoEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandboxStatusResponse_InfoEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStatusResponse_InfoEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandboxStatusResponse_InfoEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandboxStatusResponse_InfoEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 68,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatusResponse_InfoEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandboxStatusRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxStatusRequest) */ {
 public:
  inline PodSandboxStatusRequest() : PodSandboxStatusRequest(nullptr) {}
  ~PodSandboxStatusRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxStatusRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxStatusRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStatusRequest(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxStatusRequest(const PodSandboxStatusRequest& from) : PodSandboxStatusRequest(nullptr, from) {}
  inline PodSandboxStatusRequest(PodSandboxStatusRequest&& from) noexcept
      : PodSandboxStatusRequest(nullptr, ::std::move(from)) {}
  inline PodSandboxStatusRequest& operator=(const PodSandboxStatusRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxStatusRequest& operator=(PodSandboxStatusRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxStatusRequest& default_instance() {
    return *reinterpret_cast<const PodSandboxStatusRequest*>(
        &_PodSandboxStatusRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 21;
  friend void swap(PodSandboxStatusRequest& a, PodSandboxStatusRequest& b) { a.Swap(&b); }
  inline void Swap(PodSandboxStatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxStatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxStatusRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxStatusRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxStatusRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxStatusRequest& from) { PodSandboxStatusRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxStatusRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxStatusRequest"; }

  explicit PodSandboxStatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxStatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxStatusRequest& from);
  PodSandboxStatusRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxStatusRequest&& from) noexcept
      : PodSandboxStatusRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPodSandboxIdFieldNumber = 1,
    kVerboseFieldNumber = 2,
  };
  // string pod_sandbox_id = 1;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // bool verbose = 2;
  void clear_verbose() ;
  bool verbose() const;
  void set_verbose(bool value);

  private:
  bool _internal_verbose() const;
  void _internal_set_verbose(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatusRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 63,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxStatusRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    bool verbose_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatusRequest_class_data_;
// -------------------------------------------------------------------

class PodSandboxStatus_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandboxStatus_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStatus_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandboxStatus_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandboxStatus_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 62,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatus_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandboxStatus_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandboxStatus_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStatus_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandboxStatus_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandboxStatus_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 67,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatus_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandboxStateValue final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxStateValue) */ {
 public:
  inline PodSandboxStateValue() : PodSandboxStateValue(nullptr) {}
  ~PodSandboxStateValue() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxStateValue* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxStateValue));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStateValue(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxStateValue(const PodSandboxStateValue& from) : PodSandboxStateValue(nullptr, from) {}
  inline PodSandboxStateValue(PodSandboxStateValue&& from) noexcept
      : PodSandboxStateValue(nullptr, ::std::move(from)) {}
  inline PodSandboxStateValue& operator=(const PodSandboxStateValue& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxStateValue& operator=(PodSandboxStateValue&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxStateValue& default_instance() {
    return *reinterpret_cast<const PodSandboxStateValue*>(
        &_PodSandboxStateValue_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 31;
  friend void swap(PodSandboxStateValue& a, PodSandboxStateValue& b) { a.Swap(&b); }
  inline void Swap(PodSandboxStateValue* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxStateValue* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxStateValue* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxStateValue>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxStateValue& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxStateValue& from) { PodSandboxStateValue::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxStateValue* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxStateValue"; }

  explicit PodSandboxStateValue(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxStateValue(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxStateValue& from);
  PodSandboxStateValue(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxStateValue&& from) noexcept
      : PodSandboxStateValue(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStateFieldNumber = 1,
  };
  // .runtime.v1alpha2.PodSandboxState state = 1;
  void clear_state() ;
  ::runtime::v1alpha2::PodSandboxState state() const;
  void set_state(::runtime::v1alpha2::PodSandboxState value);

  private:
  ::runtime::v1alpha2::PodSandboxState _internal_state() const;
  void _internal_set_state(::runtime::v1alpha2::PodSandboxState value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStateValue)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxStateValue& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int state_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxStateValue_class_data_;
// -------------------------------------------------------------------

class PodSandboxMetadata final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxMetadata) */ {
 public:
  inline PodSandboxMetadata() : PodSandboxMetadata(nullptr) {}
  ~PodSandboxMetadata() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxMetadata* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxMetadata));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxMetadata(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxMetadata(const PodSandboxMetadata& from) : PodSandboxMetadata(nullptr, from) {}
  inline PodSandboxMetadata(PodSandboxMetadata&& from) noexcept
      : PodSandboxMetadata(nullptr, ::std::move(from)) {}
  inline PodSandboxMetadata& operator=(const PodSandboxMetadata& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxMetadata& operator=(PodSandboxMetadata&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxMetadata& default_instance() {
    return *reinterpret_cast<const PodSandboxMetadata*>(
        &_PodSandboxMetadata_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 11;
  friend void swap(PodSandboxMetadata& a, PodSandboxMetadata& b) { a.Swap(&b); }
  inline void Swap(PodSandboxMetadata* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxMetadata* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxMetadata* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxMetadata>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxMetadata& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxMetadata& from) { PodSandboxMetadata::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxMetadata* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxMetadata"; }

  explicit PodSandboxMetadata(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxMetadata(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxMetadata& from);
  PodSandboxMetadata(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxMetadata&& from) noexcept
      : PodSandboxMetadata(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kUidFieldNumber = 2,
    kNamespaceFieldNumber = 3,
    kAttemptFieldNumber = 4,
  };
  // string name = 1;
  void clear_name() ;
  const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // string uid = 2;
  void clear_uid() ;
  const ::std::string& uid() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_uid(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_uid();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_uid();
  void set_allocated_uid(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_uid() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_uid(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_uid();

  public:
  // string namespace = 3;
  void clear_namespace_() ;
  const ::std::string& namespace_() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_namespace_(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_namespace_();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_namespace_();
  void set_allocated_namespace_(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_namespace_() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_namespace_(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_namespace_();

  public:
  // uint32 attempt = 4;
  void clear_attempt() ;
  ::uint32_t attempt() const;
  void set_attempt(::uint32_t value);

  private:
  ::uint32_t _internal_attempt() const;
  void _internal_set_attempt(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxMetadata)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 60,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxMetadata& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::google::protobuf::internal::ArenaStringPtr uid_;
    ::google::protobuf::internal::ArenaStringPtr namespace__;
    ::uint32_t attempt_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxMetadata_class_data_;
// -------------------------------------------------------------------

class PodSandboxFilter_LabelSelectorEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandboxFilter_LabelSelectorEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxFilter_LabelSelectorEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandboxFilter_LabelSelectorEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandboxFilter_LabelSelectorEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 69,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandboxFilter_LabelSelectorEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandboxConfig_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandboxConfig_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxConfig_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandboxConfig_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandboxConfig_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 62,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandboxConfig_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandboxConfig_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandboxConfig_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxConfig_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandboxConfig_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandboxConfig_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 67,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandboxConfig_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandbox_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandbox_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandbox_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandbox_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandbox_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 56,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandbox_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodSandbox_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  PodSandbox_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandbox_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit PodSandbox_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_PodSandbox_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 61,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull PodSandbox_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class PodIP final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodIP) */ {
 public:
  inline PodIP() : PodIP(nullptr) {}
  ~PodIP() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodIP* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodIP));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodIP(::google::protobuf::internal::ConstantInitialized);

  inline PodIP(const PodIP& from) : PodIP(nullptr, from) {}
  inline PodIP(PodIP&& from) noexcept
      : PodIP(nullptr, ::std::move(from)) {}
  inline PodIP& operator=(const PodIP& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodIP& operator=(PodIP&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodIP& default_instance() {
    return *reinterpret_cast<const PodIP*>(
        &_PodIP_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 22;
  friend void swap(PodIP& a, PodIP& b) { a.Swap(&b); }
  inline void Swap(PodIP* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodIP* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodIP* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodIP>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodIP& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodIP& from) { PodIP::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodIP* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodIP"; }

  explicit PodIP(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodIP(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodIP& from);
  PodIP(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodIP&& from) noexcept
      : PodIP(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIpFieldNumber = 1,
  };
  // string ip = 1;
  void clear_ip() ;
  const ::std::string& ip() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_ip(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_ip();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_ip();
  void set_allocated_ip(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_ip() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_ip(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_ip();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodIP)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 33,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodIP& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr ip_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodIP_class_data_;
// -------------------------------------------------------------------

class NetworkConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.NetworkConfig) */ {
 public:
  inline NetworkConfig() : NetworkConfig(nullptr) {}
  ~NetworkConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(NetworkConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(NetworkConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR NetworkConfig(::google::protobuf::internal::ConstantInitialized);

  inline NetworkConfig(const NetworkConfig& from) : NetworkConfig(nullptr, from) {}
  inline NetworkConfig(NetworkConfig&& from) noexcept
      : NetworkConfig(nullptr, ::std::move(from)) {}
  inline NetworkConfig& operator=(const NetworkConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline NetworkConfig& operator=(NetworkConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const NetworkConfig& default_instance() {
    return *reinterpret_cast<const NetworkConfig*>(
        &_NetworkConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 101;
  friend void swap(NetworkConfig& a, NetworkConfig& b) { a.Swap(&b); }
  inline void Swap(NetworkConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NetworkConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  NetworkConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<NetworkConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const NetworkConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const NetworkConfig& from) { NetworkConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(NetworkConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.NetworkConfig"; }

  explicit NetworkConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  NetworkConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const NetworkConfig& from);
  NetworkConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, NetworkConfig&& from) noexcept
      : NetworkConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPodCidrFieldNumber = 1,
  };
  // string pod_cidr = 1;
  void clear_pod_cidr() ;
  const ::std::string& pod_cidr() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_cidr(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_cidr();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_cidr();
  void set_allocated_pod_cidr(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_cidr() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_cidr(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_cidr();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.NetworkConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 47,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const NetworkConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_cidr_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull NetworkConfig_class_data_;
// -------------------------------------------------------------------

class NamespaceOption final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.NamespaceOption) */ {
 public:
  inline NamespaceOption() : NamespaceOption(nullptr) {}
  ~NamespaceOption() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(NamespaceOption* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(NamespaceOption));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR NamespaceOption(::google::protobuf::internal::ConstantInitialized);

  inline NamespaceOption(const NamespaceOption& from) : NamespaceOption(nullptr, from) {}
  inline NamespaceOption(NamespaceOption&& from) noexcept
      : NamespaceOption(nullptr, ::std::move(from)) {}
  inline NamespaceOption& operator=(const NamespaceOption& from) {
    CopyFrom(from);
    return *this;
  }
  inline NamespaceOption& operator=(NamespaceOption&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const NamespaceOption& default_instance() {
    return *reinterpret_cast<const NamespaceOption*>(
        &_NamespaceOption_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 5;
  friend void swap(NamespaceOption& a, NamespaceOption& b) { a.Swap(&b); }
  inline void Swap(NamespaceOption* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NamespaceOption* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  NamespaceOption* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<NamespaceOption>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const NamespaceOption& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const NamespaceOption& from) { NamespaceOption::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(NamespaceOption* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.NamespaceOption"; }

  explicit NamespaceOption(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  NamespaceOption(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const NamespaceOption& from);
  NamespaceOption(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, NamespaceOption&& from) noexcept
      : NamespaceOption(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kTargetIdFieldNumber = 4,
    kNetworkFieldNumber = 1,
    kPidFieldNumber = 2,
    kIpcFieldNumber = 3,
  };
  // string target_id = 4;
  void clear_target_id() ;
  const ::std::string& target_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_target_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_target_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_target_id();
  void set_allocated_target_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_target_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_target_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_target_id();

  public:
  // .runtime.v1alpha2.NamespaceMode network = 1;
  void clear_network() ;
  ::runtime::v1alpha2::NamespaceMode network() const;
  void set_network(::runtime::v1alpha2::NamespaceMode value);

  private:
  ::runtime::v1alpha2::NamespaceMode _internal_network() const;
  void _internal_set_network(::runtime::v1alpha2::NamespaceMode value);

  public:
  // .runtime.v1alpha2.NamespaceMode pid = 2;
  void clear_pid() ;
  ::runtime::v1alpha2::NamespaceMode pid() const;
  void set_pid(::runtime::v1alpha2::NamespaceMode value);

  private:
  ::runtime::v1alpha2::NamespaceMode _internal_pid() const;
  void _internal_set_pid(::runtime::v1alpha2::NamespaceMode value);

  public:
  // .runtime.v1alpha2.NamespaceMode ipc = 3;
  void clear_ipc() ;
  ::runtime::v1alpha2::NamespaceMode ipc() const;
  void set_ipc(::runtime::v1alpha2::NamespaceMode value);

  private:
  ::runtime::v1alpha2::NamespaceMode _internal_ipc() const;
  void _internal_set_ipc(::runtime::v1alpha2::NamespaceMode value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.NamespaceOption)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   0, 50,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const NamespaceOption& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr target_id_;
    int network_;
    int pid_;
    int ipc_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull NamespaceOption_class_data_;
// -------------------------------------------------------------------

class Mount final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Mount) */ {
 public:
  inline Mount() : Mount(nullptr) {}
  ~Mount() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Mount* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Mount));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Mount(::google::protobuf::internal::ConstantInitialized);

  inline Mount(const Mount& from) : Mount(nullptr, from) {}
  inline Mount(Mount&& from) noexcept
      : Mount(nullptr, ::std::move(from)) {}
  inline Mount& operator=(const Mount& from) {
    CopyFrom(from);
    return *this;
  }
  inline Mount& operator=(Mount&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Mount& default_instance() {
    return *reinterpret_cast<const Mount*>(
        &_Mount_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(Mount& a, Mount& b) { a.Swap(&b); }
  inline void Swap(Mount* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Mount* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Mount* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Mount>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Mount& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Mount& from) { Mount::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Mount* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Mount"; }

  explicit Mount(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Mount(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Mount& from);
  Mount(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Mount&& from) noexcept
      : Mount(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerPathFieldNumber = 1,
    kHostPathFieldNumber = 2,
    kReadonlyFieldNumber = 3,
    kSelinuxRelabelFieldNumber = 4,
    kPropagationFieldNumber = 5,
  };
  // string container_path = 1;
  void clear_container_path() ;
  const ::std::string& container_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_path();
  void set_allocated_container_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_path();

  public:
  // string host_path = 2;
  void clear_host_path() ;
  const ::std::string& host_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_host_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_host_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_host_path();
  void set_allocated_host_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_host_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_host_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_host_path();

  public:
  // bool readonly = 3;
  void clear_readonly() ;
  bool readonly() const;
  void set_readonly(bool value);

  private:
  bool _internal_readonly() const;
  void _internal_set_readonly(bool value);

  public:
  // bool selinux_relabel = 4;
  void clear_selinux_relabel() ;
  bool selinux_relabel() const;
  void set_selinux_relabel(bool value);

  private:
  bool _internal_selinux_relabel() const;
  void _internal_set_selinux_relabel(bool value);

  public:
  // .runtime.v1alpha2.MountPropagation propagation = 5;
  void clear_propagation() ;
  ::runtime::v1alpha2::MountPropagation propagation() const;
  void set_propagation(::runtime::v1alpha2::MountPropagation value);

  private:
  ::runtime::v1alpha2::MountPropagation _internal_propagation() const;
  void _internal_set_propagation(::runtime::v1alpha2::MountPropagation value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Mount)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 5,
                                   0, 54,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Mount& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_path_;
    ::google::protobuf::internal::ArenaStringPtr host_path_;
    bool readonly_;
    bool selinux_relabel_;
    int propagation_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Mount_class_data_;
// -------------------------------------------------------------------

class LinuxPodSandboxConfig_SysctlsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  LinuxPodSandboxConfig_SysctlsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxPodSandboxConfig_SysctlsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit LinuxPodSandboxConfig_SysctlsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_LinuxPodSandboxConfig_SysctlsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 68,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull LinuxPodSandboxConfig_SysctlsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class KeyValue final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.KeyValue) */ {
 public:
  inline KeyValue() : KeyValue(nullptr) {}
  ~KeyValue() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(KeyValue* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(KeyValue));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR KeyValue(::google::protobuf::internal::ConstantInitialized);

  inline KeyValue(const KeyValue& from) : KeyValue(nullptr, from) {}
  inline KeyValue(KeyValue&& from) noexcept
      : KeyValue(nullptr, ::std::move(from)) {}
  inline KeyValue& operator=(const KeyValue& from) {
    CopyFrom(from);
    return *this;
  }
  inline KeyValue& operator=(KeyValue&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const KeyValue& default_instance() {
    return *reinterpret_cast<const KeyValue*>(
        &_KeyValue_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 41;
  friend void swap(KeyValue& a, KeyValue& b) { a.Swap(&b); }
  inline void Swap(KeyValue* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(KeyValue* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  KeyValue* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<KeyValue>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const KeyValue& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const KeyValue& from) { KeyValue::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(KeyValue* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.KeyValue"; }

  explicit KeyValue(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  KeyValue(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const KeyValue& from);
  KeyValue(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, KeyValue&& from) noexcept
      : KeyValue(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kKeyFieldNumber = 1,
    kValueFieldNumber = 2,
  };
  // string key = 1;
  void clear_key() ;
  const ::std::string& key() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_key(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_key();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_key();
  void set_allocated_key(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_key() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_key(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_key();

  public:
  // string value = 2;
  void clear_value() ;
  const ::std::string& value() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_value(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_value();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_value();
  void set_allocated_value(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_value() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_value(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.KeyValue)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 42,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const KeyValue& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr key_;
    ::google::protobuf::internal::ArenaStringPtr value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull KeyValue_class_data_;
// -------------------------------------------------------------------

class Int64Value final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Int64Value) */ {
 public:
  inline Int64Value() : Int64Value(nullptr) {}
  ~Int64Value() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Int64Value* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Int64Value));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Int64Value(::google::protobuf::internal::ConstantInitialized);

  inline Int64Value(const Int64Value& from) : Int64Value(nullptr, from) {}
  inline Int64Value(Int64Value&& from) noexcept
      : Int64Value(nullptr, ::std::move(from)) {}
  inline Int64Value& operator=(const Int64Value& from) {
    CopyFrom(from);
    return *this;
  }
  inline Int64Value& operator=(Int64Value&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Int64Value& default_instance() {
    return *reinterpret_cast<const Int64Value*>(
        &_Int64Value_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 6;
  friend void swap(Int64Value& a, Int64Value& b) { a.Swap(&b); }
  inline void Swap(Int64Value* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Int64Value* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Int64Value* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Int64Value>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Int64Value& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Int64Value& from) { Int64Value::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Int64Value* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Int64Value"; }

  explicit Int64Value(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Int64Value(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Int64Value& from);
  Int64Value(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Int64Value&& from) noexcept
      : Int64Value(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kValueFieldNumber = 1,
  };
  // int64 value = 1;
  void clear_value() ;
  ::int64_t value() const;
  void set_value(::int64_t value);

  private:
  ::int64_t _internal_value() const;
  void _internal_set_value(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Int64Value)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Int64Value& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int64_t value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Int64Value_class_data_;
// -------------------------------------------------------------------

class ImageStatusResponse_InfoEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ImageStatusResponse_InfoEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageStatusResponse_InfoEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ImageStatusResponse_InfoEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ImageStatusResponse_InfoEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 63,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ImageStatusResponse_InfoEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ImageSpec_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ImageSpec_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageSpec_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ImageSpec_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ImageSpec_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 60,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ImageSpec_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ImageFsInfoRequest final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ImageFsInfoRequest) */ {
 public:
  inline ImageFsInfoRequest() : ImageFsInfoRequest(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ImageFsInfoRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ImageFsInfoRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageFsInfoRequest(::google::protobuf::internal::ConstantInitialized);

  inline ImageFsInfoRequest(const ImageFsInfoRequest& from) : ImageFsInfoRequest(nullptr, from) {}
  inline ImageFsInfoRequest(ImageFsInfoRequest&& from) noexcept
      : ImageFsInfoRequest(nullptr, ::std::move(from)) {}
  inline ImageFsInfoRequest& operator=(const ImageFsInfoRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ImageFsInfoRequest& operator=(ImageFsInfoRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ImageFsInfoRequest& default_instance() {
    return *reinterpret_cast<const ImageFsInfoRequest*>(
        &_ImageFsInfoRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 110;
  friend void swap(ImageFsInfoRequest& a, ImageFsInfoRequest& b) { a.Swap(&b); }
  inline void Swap(ImageFsInfoRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ImageFsInfoRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ImageFsInfoRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<ImageFsInfoRequest>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const ImageFsInfoRequest& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const ImageFsInfoRequest& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ImageFsInfoRequest"; }

  explicit ImageFsInfoRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ImageFsInfoRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImageFsInfoRequest& from);
  ImageFsInfoRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ImageFsInfoRequest&& from) noexcept
      : ImageFsInfoRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageFsInfoRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ImageFsInfoRequest_class_data_;
// -------------------------------------------------------------------

class HugepageLimit final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.HugepageLimit) */ {
 public:
  inline HugepageLimit() : HugepageLimit(nullptr) {}
  ~HugepageLimit() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(HugepageLimit* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(HugepageLimit));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR HugepageLimit(::google::protobuf::internal::ConstantInitialized);

  inline HugepageLimit(const HugepageLimit& from) : HugepageLimit(nullptr, from) {}
  inline HugepageLimit(HugepageLimit&& from) noexcept
      : HugepageLimit(nullptr, ::std::move(from)) {}
  inline HugepageLimit& operator=(const HugepageLimit& from) {
    CopyFrom(from);
    return *this;
  }
  inline HugepageLimit& operator=(HugepageLimit&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const HugepageLimit& default_instance() {
    return *reinterpret_cast<const HugepageLimit*>(
        &_HugepageLimit_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 43;
  friend void swap(HugepageLimit& a, HugepageLimit& b) { a.Swap(&b); }
  inline void Swap(HugepageLimit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(HugepageLimit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  HugepageLimit* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<HugepageLimit>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const HugepageLimit& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const HugepageLimit& from) { HugepageLimit::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(HugepageLimit* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.HugepageLimit"; }

  explicit HugepageLimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  HugepageLimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const HugepageLimit& from);
  HugepageLimit(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, HugepageLimit&& from) noexcept
      : HugepageLimit(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPageSizeFieldNumber = 1,
    kLimitFieldNumber = 2,
  };
  // string page_size = 1;
  void clear_page_size() ;
  const ::std::string& page_size() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_page_size(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_page_size();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_page_size();
  void set_allocated_page_size(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_page_size() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_page_size(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_page_size();

  public:
  // uint64 limit = 2;
  void clear_limit() ;
  ::uint64_t limit() const;
  void set_limit(::uint64_t value);

  private:
  ::uint64_t _internal_limit() const;
  void _internal_set_limit(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.HugepageLimit)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 48,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const HugepageLimit& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr page_size_;
    ::uint64_t limit_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull HugepageLimit_class_data_;
// -------------------------------------------------------------------

class FilesystemIdentifier final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.FilesystemIdentifier) */ {
 public:
  inline FilesystemIdentifier() : FilesystemIdentifier(nullptr) {}
  ~FilesystemIdentifier() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FilesystemIdentifier* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FilesystemIdentifier));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FilesystemIdentifier(::google::protobuf::internal::ConstantInitialized);

  inline FilesystemIdentifier(const FilesystemIdentifier& from) : FilesystemIdentifier(nullptr, from) {}
  inline FilesystemIdentifier(FilesystemIdentifier&& from) noexcept
      : FilesystemIdentifier(nullptr, ::std::move(from)) {}
  inline FilesystemIdentifier& operator=(const FilesystemIdentifier& from) {
    CopyFrom(from);
    return *this;
  }
  inline FilesystemIdentifier& operator=(FilesystemIdentifier&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FilesystemIdentifier& default_instance() {
    return *reinterpret_cast<const FilesystemIdentifier*>(
        &_FilesystemIdentifier_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 112;
  friend void swap(FilesystemIdentifier& a, FilesystemIdentifier& b) { a.Swap(&b); }
  inline void Swap(FilesystemIdentifier* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FilesystemIdentifier* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FilesystemIdentifier* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<FilesystemIdentifier>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const FilesystemIdentifier& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const FilesystemIdentifier& from) { FilesystemIdentifier::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FilesystemIdentifier* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.FilesystemIdentifier"; }

  explicit FilesystemIdentifier(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FilesystemIdentifier(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FilesystemIdentifier& from);
  FilesystemIdentifier(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FilesystemIdentifier&& from) noexcept
      : FilesystemIdentifier(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kMountpointFieldNumber = 1,
  };
  // string mountpoint = 1;
  void clear_mountpoint() ;
  const ::std::string& mountpoint() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_mountpoint(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_mountpoint();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_mountpoint();
  void set_allocated_mountpoint(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_mountpoint() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_mountpoint(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_mountpoint();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.FilesystemIdentifier)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 56,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FilesystemIdentifier& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr mountpoint_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull FilesystemIdentifier_class_data_;
// -------------------------------------------------------------------

class ExecSyncResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ExecSyncResponse) */ {
 public:
  inline ExecSyncResponse() : ExecSyncResponse(nullptr) {}
  ~ExecSyncResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExecSyncResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExecSyncResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExecSyncResponse(::google::protobuf::internal::ConstantInitialized);

  inline ExecSyncResponse(const ExecSyncResponse& from) : ExecSyncResponse(nullptr, from) {}
  inline ExecSyncResponse(ExecSyncResponse&& from) noexcept
      : ExecSyncResponse(nullptr, ::std::move(from)) {}
  inline ExecSyncResponse& operator=(const ExecSyncResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExecSyncResponse& operator=(ExecSyncResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExecSyncResponse& default_instance() {
    return *reinterpret_cast<const ExecSyncResponse*>(
        &_ExecSyncResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 82;
  friend void swap(ExecSyncResponse& a, ExecSyncResponse& b) { a.Swap(&b); }
  inline void Swap(ExecSyncResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExecSyncResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExecSyncResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExecSyncResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExecSyncResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExecSyncResponse& from) { ExecSyncResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExecSyncResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ExecSyncResponse"; }

  explicit ExecSyncResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExecSyncResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExecSyncResponse& from);
  ExecSyncResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExecSyncResponse&& from) noexcept
      : ExecSyncResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFStdoutFieldNumber = 1,
    kFStderrFieldNumber = 2,
    kExitCodeFieldNumber = 3,
  };
  // bytes f_stdout = 1;
  void clear_f_stdout() ;
  const ::std::string& f_stdout() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_f_stdout(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_f_stdout();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_f_stdout();
  void set_allocated_f_stdout(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_f_stdout() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_f_stdout(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_f_stdout();

  public:
  // bytes f_stderr = 2;
  void clear_f_stderr() ;
  const ::std::string& f_stderr() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_f_stderr(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_f_stderr();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_f_stderr();
  void set_allocated_f_stderr(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_f_stderr() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_f_stderr(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_f_stderr();

  public:
  // int32 exit_code = 3;
  void clear_exit_code() ;
  ::int32_t exit_code() const;
  void set_exit_code(::int32_t value);

  private:
  ::int32_t _internal_exit_code() const;
  void _internal_set_exit_code(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecSyncResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExecSyncResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr f_stdout_;
    ::google::protobuf::internal::ArenaStringPtr f_stderr_;
    ::int32_t exit_code_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ExecSyncResponse_class_data_;
// -------------------------------------------------------------------

class ExecSyncRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ExecSyncRequest) */ {
 public:
  inline ExecSyncRequest() : ExecSyncRequest(nullptr) {}
  ~ExecSyncRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExecSyncRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExecSyncRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExecSyncRequest(::google::protobuf::internal::ConstantInitialized);

  inline ExecSyncRequest(const ExecSyncRequest& from) : ExecSyncRequest(nullptr, from) {}
  inline ExecSyncRequest(ExecSyncRequest&& from) noexcept
      : ExecSyncRequest(nullptr, ::std::move(from)) {}
  inline ExecSyncRequest& operator=(const ExecSyncRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExecSyncRequest& operator=(ExecSyncRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExecSyncRequest& default_instance() {
    return *reinterpret_cast<const ExecSyncRequest*>(
        &_ExecSyncRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 81;
  friend void swap(ExecSyncRequest& a, ExecSyncRequest& b) { a.Swap(&b); }
  inline void Swap(ExecSyncRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExecSyncRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExecSyncRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExecSyncRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExecSyncRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExecSyncRequest& from) { ExecSyncRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExecSyncRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ExecSyncRequest"; }

  explicit ExecSyncRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExecSyncRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExecSyncRequest& from);
  ExecSyncRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExecSyncRequest&& from) noexcept
      : ExecSyncRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCmdFieldNumber = 2,
    kContainerIdFieldNumber = 1,
    kTimeoutFieldNumber = 3,
  };
  // repeated string cmd = 2;
  int cmd_size() const;
  private:
  int _internal_cmd_size() const;

  public:
  void clear_cmd() ;
  const ::std::string& cmd(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_cmd(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cmd(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_cmd();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_cmd(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& cmd() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_cmd();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_cmd() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_cmd();

  public:
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // int64 timeout = 3;
  void clear_timeout() ;
  ::int64_t timeout() const;
  void set_timeout(::int64_t value);

  private:
  ::int64_t _internal_timeout() const;
  void _internal_set_timeout(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecSyncRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   0, 56,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExecSyncRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> cmd_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    ::int64_t timeout_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ExecSyncRequest_class_data_;
// -------------------------------------------------------------------

class ExecResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ExecResponse) */ {
 public:
  inline ExecResponse() : ExecResponse(nullptr) {}
  ~ExecResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExecResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExecResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExecResponse(::google::protobuf::internal::ConstantInitialized);

  inline ExecResponse(const ExecResponse& from) : ExecResponse(nullptr, from) {}
  inline ExecResponse(ExecResponse&& from) noexcept
      : ExecResponse(nullptr, ::std::move(from)) {}
  inline ExecResponse& operator=(const ExecResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExecResponse& operator=(ExecResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExecResponse& default_instance() {
    return *reinterpret_cast<const ExecResponse*>(
        &_ExecResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 84;
  friend void swap(ExecResponse& a, ExecResponse& b) { a.Swap(&b); }
  inline void Swap(ExecResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExecResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExecResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExecResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExecResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExecResponse& from) { ExecResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExecResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ExecResponse"; }

  explicit ExecResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExecResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExecResponse& from);
  ExecResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExecResponse&& from) noexcept
      : ExecResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUrlFieldNumber = 1,
  };
  // string url = 1;
  void clear_url() ;
  const ::std::string& url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_url();
  void set_allocated_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_url();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 41,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExecResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr url_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ExecResponse_class_data_;
// -------------------------------------------------------------------

class ExecRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ExecRequest) */ {
 public:
  inline ExecRequest() : ExecRequest(nullptr) {}
  ~ExecRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExecRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExecRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ExecRequest(::google::protobuf::internal::ConstantInitialized);

  inline ExecRequest(const ExecRequest& from) : ExecRequest(nullptr, from) {}
  inline ExecRequest(ExecRequest&& from) noexcept
      : ExecRequest(nullptr, ::std::move(from)) {}
  inline ExecRequest& operator=(const ExecRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExecRequest& operator=(ExecRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ExecRequest& default_instance() {
    return *reinterpret_cast<const ExecRequest*>(
        &_ExecRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 83;
  friend void swap(ExecRequest& a, ExecRequest& b) { a.Swap(&b); }
  inline void Swap(ExecRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExecRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ExecRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ExecRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ExecRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ExecRequest& from) { ExecRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExecRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ExecRequest"; }

  explicit ExecRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExecRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExecRequest& from);
  ExecRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExecRequest&& from) noexcept
      : ExecRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCmdFieldNumber = 2,
    kContainerIdFieldNumber = 1,
    kTtyFieldNumber = 3,
    kFStdinFieldNumber = 4,
    kFStdoutFieldNumber = 5,
    kFStderrFieldNumber = 6,
  };
  // repeated string cmd = 2;
  int cmd_size() const;
  private:
  int _internal_cmd_size() const;

  public:
  void clear_cmd() ;
  const ::std::string& cmd(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_cmd(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cmd(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_cmd();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_cmd(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& cmd() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_cmd();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_cmd() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_cmd();

  public:
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // bool tty = 3;
  void clear_tty() ;
  bool tty() const;
  void set_tty(bool value);

  private:
  bool _internal_tty() const;
  void _internal_set_tty(bool value);

  public:
  // bool f_stdin = 4;
  void clear_f_stdin() ;
  bool f_stdin() const;
  void set_f_stdin(bool value);

  private:
  bool _internal_f_stdin() const;
  void _internal_set_f_stdin(bool value);

  public:
  // bool f_stdout = 5;
  void clear_f_stdout() ;
  bool f_stdout() const;
  void set_f_stdout(bool value);

  private:
  bool _internal_f_stdout() const;
  void _internal_set_f_stdout(bool value);

  public:
  // bool f_stderr = 6;
  void clear_f_stderr() ;
  bool f_stderr() const;
  void set_f_stderr(bool value);

  private:
  bool _internal_f_stderr() const;
  void _internal_set_f_stderr(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 6,
                                   0, 52,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExecRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> cmd_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    bool tty_;
    bool f_stdin_;
    bool f_stdout_;
    bool f_stderr_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ExecRequest_class_data_;
// -------------------------------------------------------------------

class Device final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Device) */ {
 public:
  inline Device() : Device(nullptr) {}
  ~Device() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Device* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Device));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Device(::google::protobuf::internal::ConstantInitialized);

  inline Device(const Device& from) : Device(nullptr, from) {}
  inline Device(Device&& from) noexcept
      : Device(nullptr, ::std::move(from)) {}
  inline Device& operator=(const Device& from) {
    CopyFrom(from);
    return *this;
  }
  inline Device& operator=(Device&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Device& default_instance() {
    return *reinterpret_cast<const Device*>(
        &_Device_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 52;
  friend void swap(Device& a, Device& b) { a.Swap(&b); }
  inline void Swap(Device* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Device* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Device* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Device>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Device& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Device& from) { Device::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Device* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Device"; }

  explicit Device(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Device(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Device& from);
  Device(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Device&& from) noexcept
      : Device(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerPathFieldNumber = 1,
    kHostPathFieldNumber = 2,
    kPermissionsFieldNumber = 3,
  };
  // string container_path = 1;
  void clear_container_path() ;
  const ::std::string& container_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_path();
  void set_allocated_container_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_path();

  public:
  // string host_path = 2;
  void clear_host_path() ;
  const ::std::string& host_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_host_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_host_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_host_path();
  void set_allocated_host_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_host_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_host_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_host_path();

  public:
  // string permissions = 3;
  void clear_permissions() ;
  const ::std::string& permissions() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_permissions(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_permissions();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_permissions();
  void set_allocated_permissions(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_permissions() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_permissions(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_permissions();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Device)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   0, 66,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Device& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_path_;
    ::google::protobuf::internal::ArenaStringPtr host_path_;
    ::google::protobuf::internal::ArenaStringPtr permissions_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Device_class_data_;
// -------------------------------------------------------------------

class DNSConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.DNSConfig) */ {
 public:
  inline DNSConfig() : DNSConfig(nullptr) {}
  ~DNSConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(DNSConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(DNSConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR DNSConfig(::google::protobuf::internal::ConstantInitialized);

  inline DNSConfig(const DNSConfig& from) : DNSConfig(nullptr, from) {}
  inline DNSConfig(DNSConfig&& from) noexcept
      : DNSConfig(nullptr, ::std::move(from)) {}
  inline DNSConfig& operator=(const DNSConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline DNSConfig& operator=(DNSConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const DNSConfig& default_instance() {
    return *reinterpret_cast<const DNSConfig*>(
        &_DNSConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(DNSConfig& a, DNSConfig& b) { a.Swap(&b); }
  inline void Swap(DNSConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(DNSConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  DNSConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<DNSConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const DNSConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const DNSConfig& from) { DNSConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(DNSConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.DNSConfig"; }

  explicit DNSConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  DNSConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DNSConfig& from);
  DNSConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, DNSConfig&& from) noexcept
      : DNSConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kServersFieldNumber = 1,
    kSearchesFieldNumber = 2,
    kOptionsFieldNumber = 3,
  };
  // repeated string servers = 1;
  int servers_size() const;
  private:
  int _internal_servers_size() const;

  public:
  void clear_servers() ;
  const ::std::string& servers(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_servers(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_servers(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_servers();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_servers(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& servers() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_servers();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_servers() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_servers();

  public:
  // repeated string searches = 2;
  int searches_size() const;
  private:
  int _internal_searches_size() const;

  public:
  void clear_searches() ;
  const ::std::string& searches(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_searches(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_searches(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_searches();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_searches(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& searches() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_searches();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_searches() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_searches();

  public:
  // repeated string options = 3;
  int options_size() const;
  private:
  int _internal_options_size() const;

  public:
  void clear_options() ;
  const ::std::string& options(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_options(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_options(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_options();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_options(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& options() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_options();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_options() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_options();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.DNSConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   0, 57,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const DNSConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> servers_;
    ::google::protobuf::RepeatedPtrField<::std::string> searches_;
    ::google::protobuf::RepeatedPtrField<::std::string> options_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull DNSConfig_class_data_;
// -------------------------------------------------------------------

class CreateContainerResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.CreateContainerResponse) */ {
 public:
  inline CreateContainerResponse() : CreateContainerResponse(nullptr) {}
  ~CreateContainerResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(CreateContainerResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(CreateContainerResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR CreateContainerResponse(::google::protobuf::internal::ConstantInitialized);

  inline CreateContainerResponse(const CreateContainerResponse& from) : CreateContainerResponse(nullptr, from) {}
  inline CreateContainerResponse(CreateContainerResponse&& from) noexcept
      : CreateContainerResponse(nullptr, ::std::move(from)) {}
  inline CreateContainerResponse& operator=(const CreateContainerResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline CreateContainerResponse& operator=(CreateContainerResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const CreateContainerResponse& default_instance() {
    return *reinterpret_cast<const CreateContainerResponse*>(
        &_CreateContainerResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 57;
  friend void swap(CreateContainerResponse& a, CreateContainerResponse& b) { a.Swap(&b); }
  inline void Swap(CreateContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CreateContainerResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  CreateContainerResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<CreateContainerResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const CreateContainerResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const CreateContainerResponse& from) { CreateContainerResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(CreateContainerResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.CreateContainerResponse"; }

  explicit CreateContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  CreateContainerResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const CreateContainerResponse& from);
  CreateContainerResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, CreateContainerResponse&& from) noexcept
      : CreateContainerResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.CreateContainerResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 61,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const CreateContainerResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull CreateContainerResponse_class_data_;
// -------------------------------------------------------------------

class ContainerStatusResponse_InfoEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerStatusResponse_InfoEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatusResponse_InfoEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerStatusResponse_InfoEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerStatusResponse_InfoEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 67,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerStatusResponse_InfoEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerStatusRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStatusRequest) */ {
 public:
  inline ContainerStatusRequest() : ContainerStatusRequest(nullptr) {}
  ~ContainerStatusRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStatusRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStatusRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatusRequest(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStatusRequest(const ContainerStatusRequest& from) : ContainerStatusRequest(nullptr, from) {}
  inline ContainerStatusRequest(ContainerStatusRequest&& from) noexcept
      : ContainerStatusRequest(nullptr, ::std::move(from)) {}
  inline ContainerStatusRequest& operator=(const ContainerStatusRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStatusRequest& operator=(ContainerStatusRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStatusRequest& default_instance() {
    return *reinterpret_cast<const ContainerStatusRequest*>(
        &_ContainerStatusRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 72;
  friend void swap(ContainerStatusRequest& a, ContainerStatusRequest& b) { a.Swap(&b); }
  inline void Swap(ContainerStatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStatusRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStatusRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStatusRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStatusRequest& from) { ContainerStatusRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStatusRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStatusRequest"; }

  explicit ContainerStatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStatusRequest& from);
  ContainerStatusRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStatusRequest&& from) noexcept
      : ContainerStatusRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
    kVerboseFieldNumber = 2,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // bool verbose = 2;
  void clear_verbose() ;
  bool verbose() const;
  void set_verbose(bool value);

  private:
  bool _internal_verbose() const;
  void _internal_set_verbose(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatusRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 60,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStatusRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    bool verbose_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStatusRequest_class_data_;
// -------------------------------------------------------------------

class ContainerStatus_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerStatus_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatus_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerStatus_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerStatus_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 61,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerStatus_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerStatus_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerStatus_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatus_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerStatus_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerStatus_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 66,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerStatus_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerStatsRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStatsRequest) */ {
 public:
  inline ContainerStatsRequest() : ContainerStatsRequest(nullptr) {}
  ~ContainerStatsRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStatsRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStatsRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatsRequest(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStatsRequest(const ContainerStatsRequest& from) : ContainerStatsRequest(nullptr, from) {}
  inline ContainerStatsRequest(ContainerStatsRequest&& from) noexcept
      : ContainerStatsRequest(nullptr, ::std::move(from)) {}
  inline ContainerStatsRequest& operator=(const ContainerStatsRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStatsRequest& operator=(ContainerStatsRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStatsRequest& default_instance() {
    return *reinterpret_cast<const ContainerStatsRequest*>(
        &_ContainerStatsRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 115;
  friend void swap(ContainerStatsRequest& a, ContainerStatsRequest& b) { a.Swap(&b); }
  inline void Swap(ContainerStatsRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStatsRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStatsRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStatsRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStatsRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStatsRequest& from) { ContainerStatsRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStatsRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStatsRequest"; }

  explicit ContainerStatsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStatsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStatsRequest& from);
  ContainerStatsRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStatsRequest&& from) noexcept
      : ContainerStatsRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 59,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStatsRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStatsRequest_class_data_;
// -------------------------------------------------------------------

class ContainerStatsFilter_LabelSelectorEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerStatsFilter_LabelSelectorEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatsFilter_LabelSelectorEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerStatsFilter_LabelSelectorEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerStatsFilter_LabelSelectorEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 73,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerStatsFilter_LabelSelectorEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerStateValue final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStateValue) */ {
 public:
  inline ContainerStateValue() : ContainerStateValue(nullptr) {}
  ~ContainerStateValue() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStateValue* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStateValue));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStateValue(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStateValue(const ContainerStateValue& from) : ContainerStateValue(nullptr, from) {}
  inline ContainerStateValue(ContainerStateValue&& from) noexcept
      : ContainerStateValue(nullptr, ::std::move(from)) {}
  inline ContainerStateValue& operator=(const ContainerStateValue& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStateValue& operator=(ContainerStateValue&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStateValue& default_instance() {
    return *reinterpret_cast<const ContainerStateValue*>(
        &_ContainerStateValue_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 64;
  friend void swap(ContainerStateValue& a, ContainerStateValue& b) { a.Swap(&b); }
  inline void Swap(ContainerStateValue* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStateValue* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStateValue* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStateValue>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStateValue& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStateValue& from) { ContainerStateValue::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStateValue* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStateValue"; }

  explicit ContainerStateValue(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStateValue(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStateValue& from);
  ContainerStateValue(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStateValue&& from) noexcept
      : ContainerStateValue(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStateFieldNumber = 1,
  };
  // .runtime.v1alpha2.ContainerState state = 1;
  void clear_state() ;
  ::runtime::v1alpha2::ContainerState state() const;
  void set_state(::runtime::v1alpha2::ContainerState value);

  private:
  ::runtime::v1alpha2::ContainerState _internal_state() const;
  void _internal_set_state(::runtime::v1alpha2::ContainerState value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStateValue)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStateValue& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int state_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStateValue_class_data_;
// -------------------------------------------------------------------

class ContainerMetadata final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerMetadata) */ {
 public:
  inline ContainerMetadata() : ContainerMetadata(nullptr) {}
  ~ContainerMetadata() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerMetadata* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerMetadata));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerMetadata(::google::protobuf::internal::ConstantInitialized);

  inline ContainerMetadata(const ContainerMetadata& from) : ContainerMetadata(nullptr, from) {}
  inline ContainerMetadata(ContainerMetadata&& from) noexcept
      : ContainerMetadata(nullptr, ::std::move(from)) {}
  inline ContainerMetadata& operator=(const ContainerMetadata& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerMetadata& operator=(ContainerMetadata&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerMetadata& default_instance() {
    return *reinterpret_cast<const ContainerMetadata*>(
        &_ContainerMetadata_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 51;
  friend void swap(ContainerMetadata& a, ContainerMetadata& b) { a.Swap(&b); }
  inline void Swap(ContainerMetadata* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerMetadata* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerMetadata* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerMetadata>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerMetadata& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerMetadata& from) { ContainerMetadata::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerMetadata* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerMetadata"; }

  explicit ContainerMetadata(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerMetadata(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerMetadata& from);
  ContainerMetadata(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerMetadata&& from) noexcept
      : ContainerMetadata(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNameFieldNumber = 1,
    kAttemptFieldNumber = 2,
  };
  // string name = 1;
  void clear_name() ;
  const ::std::string& name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_name();
  void set_allocated_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_name();

  public:
  // uint32 attempt = 2;
  void clear_attempt() ;
  ::uint32_t attempt() const;
  void set_attempt(::uint32_t value);

  private:
  ::uint32_t _internal_attempt() const;
  void _internal_set_attempt(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerMetadata)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 47,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerMetadata& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr name_;
    ::uint32_t attempt_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerMetadata_class_data_;
// -------------------------------------------------------------------

class ContainerFilter_LabelSelectorEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerFilter_LabelSelectorEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerFilter_LabelSelectorEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerFilter_LabelSelectorEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerFilter_LabelSelectorEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 68,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerFilter_LabelSelectorEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerConfig_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerConfig_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerConfig_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerConfig_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerConfig_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 61,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerConfig_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerConfig_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerConfig_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerConfig_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerConfig_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerConfig_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 66,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerConfig_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerAttributes_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerAttributes_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerAttributes_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerAttributes_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerAttributes_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 65,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerAttributes_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class ContainerAttributes_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  ContainerAttributes_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerAttributes_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit ContainerAttributes_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_ContainerAttributes_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 70,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull ContainerAttributes_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class Container_LabelsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  Container_LabelsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Container_LabelsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit Container_LabelsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_Container_LabelsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 55,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull Container_LabelsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class Container_AnnotationsEntry_DoNotUse final
    : public ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                             ::google::protobuf::internal::WireFormatLite::TYPE_STRING> {
 public:
  using SuperType =
      ::google::protobuf::internal::MapEntry<::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>;
  Container_AnnotationsEntry_DoNotUse();
  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Container_AnnotationsEntry_DoNotUse(::google::protobuf::internal::ConstantInitialized);
  explicit Container_AnnotationsEntry_DoNotUse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr const void* PROTOBUF_NONNULL internal_default_instance() {
    return &_Container_AnnotationsEntry_DoNotUse_default_instance_;
  }


  static constexpr auto InternalGenerateClassData_();

 private:
  friend class ::google::protobuf::MessageLite;
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;

  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 60,
                                   2>
      _table_;

  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();
};
extern const ::google::protobuf::internal::ClassDataFull Container_AnnotationsEntry_DoNotUse_class_data_;
// -------------------------------------------------------------------

class Capability final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Capability) */ {
 public:
  inline Capability() : Capability(nullptr) {}
  ~Capability() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Capability* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Capability));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Capability(::google::protobuf::internal::ConstantInitialized);

  inline Capability(const Capability& from) : Capability(nullptr, from) {}
  inline Capability(Capability&& from) noexcept
      : Capability(nullptr, ::std::move(from)) {}
  inline Capability& operator=(const Capability& from) {
    CopyFrom(from);
    return *this;
  }
  inline Capability& operator=(Capability&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Capability& default_instance() {
    return *reinterpret_cast<const Capability*>(
        &_Capability_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 45;
  friend void swap(Capability& a, Capability& b) { a.Swap(&b); }
  inline void Swap(Capability* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Capability* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Capability* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Capability>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Capability& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Capability& from) { Capability::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Capability* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Capability"; }

  explicit Capability(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Capability(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Capability& from);
  Capability(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Capability&& from) noexcept
      : Capability(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kAddCapabilitiesFieldNumber = 1,
    kDropCapabilitiesFieldNumber = 2,
  };
  // repeated string add_capabilities = 1;
  int add_capabilities_size() const;
  private:
  int _internal_add_capabilities_size() const;

  public:
  void clear_add_capabilities() ;
  const ::std::string& add_capabilities(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_add_capabilities(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_add_capabilities(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_add_capabilities();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_add_capabilities(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& add_capabilities() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_add_capabilities();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_add_capabilities() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_add_capabilities();

  public:
  // repeated string drop_capabilities = 2;
  int drop_capabilities_size() const;
  private:
  int _internal_drop_capabilities_size() const;

  public:
  void clear_drop_capabilities() ;
  const ::std::string& drop_capabilities(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_drop_capabilities(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_drop_capabilities(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_drop_capabilities();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_drop_capabilities(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& drop_capabilities() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_drop_capabilities();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_drop_capabilities() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_drop_capabilities();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Capability)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   0, 69,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Capability& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> add_capabilities_;
    ::google::protobuf::RepeatedPtrField<::std::string> drop_capabilities_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Capability_class_data_;
// -------------------------------------------------------------------

class AuthConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.AuthConfig) */ {
 public:
  inline AuthConfig() : AuthConfig(nullptr) {}
  ~AuthConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AuthConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AuthConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR AuthConfig(::google::protobuf::internal::ConstantInitialized);

  inline AuthConfig(const AuthConfig& from) : AuthConfig(nullptr, from) {}
  inline AuthConfig(AuthConfig&& from) noexcept
      : AuthConfig(nullptr, ::std::move(from)) {}
  inline AuthConfig& operator=(const AuthConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline AuthConfig& operator=(AuthConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AuthConfig& default_instance() {
    return *reinterpret_cast<const AuthConfig*>(
        &_AuthConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 96;
  friend void swap(AuthConfig& a, AuthConfig& b) { a.Swap(&b); }
  inline void Swap(AuthConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AuthConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AuthConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AuthConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AuthConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AuthConfig& from) { AuthConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AuthConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.AuthConfig"; }

  explicit AuthConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AuthConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AuthConfig& from);
  AuthConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AuthConfig&& from) noexcept
      : AuthConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUsernameFieldNumber = 1,
    kPasswordFieldNumber = 2,
    kAuthFieldNumber = 3,
    kServerAddressFieldNumber = 4,
    kIdentityTokenFieldNumber = 5,
    kRegistryTokenFieldNumber = 6,
  };
  // string username = 1;
  void clear_username() ;
  const ::std::string& username() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_username(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_username();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_username();
  void set_allocated_username(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_username() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_username(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_username();

  public:
  // string password = 2;
  void clear_password() ;
  const ::std::string& password() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_password(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_password();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_password();
  void set_allocated_password(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_password() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_password(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_password();

  public:
  // string auth = 3;
  void clear_auth() ;
  const ::std::string& auth() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_auth(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_auth();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_auth();
  void set_allocated_auth(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_auth() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_auth(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_auth();

  public:
  // string server_address = 4;
  void clear_server_address() ;
  const ::std::string& server_address() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_server_address(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_server_address();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_server_address();
  void set_allocated_server_address(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_server_address() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_server_address(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_server_address();

  public:
  // string identity_token = 5;
  void clear_identity_token() ;
  const ::std::string& identity_token() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_identity_token(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_identity_token();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_identity_token();
  void set_allocated_identity_token(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_identity_token() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_identity_token(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_identity_token();

  public:
  // string registry_token = 6;
  void clear_registry_token() ;
  const ::std::string& registry_token() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_registry_token(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_registry_token();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_registry_token();
  void set_allocated_registry_token(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_registry_token() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_registry_token(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_registry_token();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.AuthConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 6,
                                   0, 98,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AuthConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr username_;
    ::google::protobuf::internal::ArenaStringPtr password_;
    ::google::protobuf::internal::ArenaStringPtr auth_;
    ::google::protobuf::internal::ArenaStringPtr server_address_;
    ::google::protobuf::internal::ArenaStringPtr identity_token_;
    ::google::protobuf::internal::ArenaStringPtr registry_token_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull AuthConfig_class_data_;
// -------------------------------------------------------------------

class AttachResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.AttachResponse) */ {
 public:
  inline AttachResponse() : AttachResponse(nullptr) {}
  ~AttachResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AttachResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AttachResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR AttachResponse(::google::protobuf::internal::ConstantInitialized);

  inline AttachResponse(const AttachResponse& from) : AttachResponse(nullptr, from) {}
  inline AttachResponse(AttachResponse&& from) noexcept
      : AttachResponse(nullptr, ::std::move(from)) {}
  inline AttachResponse& operator=(const AttachResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline AttachResponse& operator=(AttachResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AttachResponse& default_instance() {
    return *reinterpret_cast<const AttachResponse*>(
        &_AttachResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 86;
  friend void swap(AttachResponse& a, AttachResponse& b) { a.Swap(&b); }
  inline void Swap(AttachResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AttachResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AttachResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AttachResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AttachResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AttachResponse& from) { AttachResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AttachResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.AttachResponse"; }

  explicit AttachResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AttachResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AttachResponse& from);
  AttachResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AttachResponse&& from) noexcept
      : AttachResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUrlFieldNumber = 1,
  };
  // string url = 1;
  void clear_url() ;
  const ::std::string& url() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_url(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_url();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_url();
  void set_allocated_url(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_url() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_url(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_url();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.AttachResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   0, 43,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AttachResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr url_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull AttachResponse_class_data_;
// -------------------------------------------------------------------

class AttachRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.AttachRequest) */ {
 public:
  inline AttachRequest() : AttachRequest(nullptr) {}
  ~AttachRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AttachRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AttachRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR AttachRequest(::google::protobuf::internal::ConstantInitialized);

  inline AttachRequest(const AttachRequest& from) : AttachRequest(nullptr, from) {}
  inline AttachRequest(AttachRequest&& from) noexcept
      : AttachRequest(nullptr, ::std::move(from)) {}
  inline AttachRequest& operator=(const AttachRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline AttachRequest& operator=(AttachRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const AttachRequest& default_instance() {
    return *reinterpret_cast<const AttachRequest*>(
        &_AttachRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 85;
  friend void swap(AttachRequest& a, AttachRequest& b) { a.Swap(&b); }
  inline void Swap(AttachRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AttachRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  AttachRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AttachRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AttachRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AttachRequest& from) { AttachRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AttachRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.AttachRequest"; }

  explicit AttachRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AttachRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AttachRequest& from);
  AttachRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AttachRequest&& from) noexcept
      : AttachRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
    kFStdinFieldNumber = 2,
    kTtyFieldNumber = 3,
    kFStdoutFieldNumber = 4,
    kFStderrFieldNumber = 5,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // bool f_stdin = 2;
  void clear_f_stdin() ;
  bool f_stdin() const;
  void set_f_stdin(bool value);

  private:
  bool _internal_f_stdin() const;
  void _internal_set_f_stdin(bool value);

  public:
  // bool tty = 3;
  void clear_tty() ;
  bool tty() const;
  void set_tty(bool value);

  private:
  bool _internal_tty() const;
  void _internal_set_tty(bool value);

  public:
  // bool f_stdout = 4;
  void clear_f_stdout() ;
  bool f_stdout() const;
  void set_f_stdout(bool value);

  private:
  bool _internal_f_stdout() const;
  void _internal_set_f_stdout(bool value);

  public:
  // bool f_stderr = 5;
  void clear_f_stderr() ;
  bool f_stderr() const;
  void set_f_stderr(bool value);

  private:
  bool _internal_f_stderr() const;
  void _internal_set_f_stderr(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.AttachRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 5,
                                   0, 51,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AttachRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    bool f_stdin_;
    bool tty_;
    bool f_stdout_;
    bool f_stderr_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull AttachRequest_class_data_;
// -------------------------------------------------------------------

class WindowsContainerConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.WindowsContainerConfig) */ {
 public:
  inline WindowsContainerConfig() : WindowsContainerConfig(nullptr) {}
  ~WindowsContainerConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(WindowsContainerConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(WindowsContainerConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR WindowsContainerConfig(::google::protobuf::internal::ConstantInitialized);

  inline WindowsContainerConfig(const WindowsContainerConfig& from) : WindowsContainerConfig(nullptr, from) {}
  inline WindowsContainerConfig(WindowsContainerConfig&& from) noexcept
      : WindowsContainerConfig(nullptr, ::std::move(from)) {}
  inline WindowsContainerConfig& operator=(const WindowsContainerConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline WindowsContainerConfig& operator=(WindowsContainerConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const WindowsContainerConfig& default_instance() {
    return *reinterpret_cast<const WindowsContainerConfig*>(
        &_WindowsContainerConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 49;
  friend void swap(WindowsContainerConfig& a, WindowsContainerConfig& b) { a.Swap(&b); }
  inline void Swap(WindowsContainerConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(WindowsContainerConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  WindowsContainerConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<WindowsContainerConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const WindowsContainerConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const WindowsContainerConfig& from) { WindowsContainerConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(WindowsContainerConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.WindowsContainerConfig"; }

  explicit WindowsContainerConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  WindowsContainerConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const WindowsContainerConfig& from);
  WindowsContainerConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, WindowsContainerConfig&& from) noexcept
      : WindowsContainerConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResourcesFieldNumber = 1,
    kSecurityContextFieldNumber = 2,
  };
  // .runtime.v1alpha2.WindowsContainerResources resources = 1;
  bool has_resources() const;
  void clear_resources() ;
  const ::runtime::v1alpha2::WindowsContainerResources& resources() const;
  [[nodiscard]] ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE release_resources();
  ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL mutable_resources();
  void set_allocated_resources(::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_resources(::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE unsafe_arena_release_resources();

  private:
  const ::runtime::v1alpha2::WindowsContainerResources& _internal_resources() const;
  ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL _internal_mutable_resources();

  public:
  // .runtime.v1alpha2.WindowsContainerSecurityContext security_context = 2;
  bool has_security_context() const;
  void clear_security_context() ;
  const ::runtime::v1alpha2::WindowsContainerSecurityContext& security_context() const;
  [[nodiscard]] ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE release_security_context();
  ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NONNULL mutable_security_context();
  void set_allocated_security_context(::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_security_context(::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE unsafe_arena_release_security_context();

  private:
  const ::runtime::v1alpha2::WindowsContainerSecurityContext& _internal_security_context() const;
  ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NONNULL _internal_mutable_security_context();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.WindowsContainerConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const WindowsContainerConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE resources_;
    ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE security_context_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull WindowsContainerConfig_class_data_;
// -------------------------------------------------------------------

class RuntimeStatus final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RuntimeStatus) */ {
 public:
  inline RuntimeStatus() : RuntimeStatus(nullptr) {}
  ~RuntimeStatus() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RuntimeStatus* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RuntimeStatus));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RuntimeStatus(::google::protobuf::internal::ConstantInitialized);

  inline RuntimeStatus(const RuntimeStatus& from) : RuntimeStatus(nullptr, from) {}
  inline RuntimeStatus(RuntimeStatus&& from) noexcept
      : RuntimeStatus(nullptr, ::std::move(from)) {}
  inline RuntimeStatus& operator=(const RuntimeStatus& from) {
    CopyFrom(from);
    return *this;
  }
  inline RuntimeStatus& operator=(RuntimeStatus&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RuntimeStatus& default_instance() {
    return *reinterpret_cast<const RuntimeStatus*>(
        &_RuntimeStatus_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 106;
  friend void swap(RuntimeStatus& a, RuntimeStatus& b) { a.Swap(&b); }
  inline void Swap(RuntimeStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RuntimeStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RuntimeStatus* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RuntimeStatus>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RuntimeStatus& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RuntimeStatus& from) { RuntimeStatus::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RuntimeStatus* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RuntimeStatus"; }

  explicit RuntimeStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RuntimeStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RuntimeStatus& from);
  RuntimeStatus(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RuntimeStatus&& from) noexcept
      : RuntimeStatus(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kConditionsFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.RuntimeCondition conditions = 1;
  int conditions_size() const;
  private:
  int _internal_conditions_size() const;

  public:
  void clear_conditions() ;
  ::runtime::v1alpha2::RuntimeCondition* PROTOBUF_NONNULL mutable_conditions(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>* PROTOBUF_NONNULL mutable_conditions();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>& _internal_conditions() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>* PROTOBUF_NONNULL _internal_mutable_conditions();
  public:
  const ::runtime::v1alpha2::RuntimeCondition& conditions(int index) const;
  ::runtime::v1alpha2::RuntimeCondition* PROTOBUF_NONNULL add_conditions();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>& conditions() const;
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RuntimeStatus)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RuntimeStatus& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::RuntimeCondition > conditions_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RuntimeStatus_class_data_;
// -------------------------------------------------------------------

class RuntimeConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RuntimeConfig) */ {
 public:
  inline RuntimeConfig() : RuntimeConfig(nullptr) {}
  ~RuntimeConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RuntimeConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RuntimeConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RuntimeConfig(::google::protobuf::internal::ConstantInitialized);

  inline RuntimeConfig(const RuntimeConfig& from) : RuntimeConfig(nullptr, from) {}
  inline RuntimeConfig(RuntimeConfig&& from) noexcept
      : RuntimeConfig(nullptr, ::std::move(from)) {}
  inline RuntimeConfig& operator=(const RuntimeConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline RuntimeConfig& operator=(RuntimeConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RuntimeConfig& default_instance() {
    return *reinterpret_cast<const RuntimeConfig*>(
        &_RuntimeConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 102;
  friend void swap(RuntimeConfig& a, RuntimeConfig& b) { a.Swap(&b); }
  inline void Swap(RuntimeConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RuntimeConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RuntimeConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RuntimeConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RuntimeConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RuntimeConfig& from) { RuntimeConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RuntimeConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RuntimeConfig"; }

  explicit RuntimeConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RuntimeConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RuntimeConfig& from);
  RuntimeConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RuntimeConfig&& from) noexcept
      : RuntimeConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNetworkConfigFieldNumber = 1,
  };
  // .runtime.v1alpha2.NetworkConfig network_config = 1;
  bool has_network_config() const;
  void clear_network_config() ;
  const ::runtime::v1alpha2::NetworkConfig& network_config() const;
  [[nodiscard]] ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE release_network_config();
  ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NONNULL mutable_network_config();
  void set_allocated_network_config(::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_network_config(::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE unsafe_arena_release_network_config();

  private:
  const ::runtime::v1alpha2::NetworkConfig& _internal_network_config() const;
  ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NONNULL _internal_mutable_network_config();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RuntimeConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RuntimeConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE network_config_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RuntimeConfig_class_data_;
// -------------------------------------------------------------------

class PodSandboxNetworkStatus final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxNetworkStatus) */ {
 public:
  inline PodSandboxNetworkStatus() : PodSandboxNetworkStatus(nullptr) {}
  ~PodSandboxNetworkStatus() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxNetworkStatus* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxNetworkStatus));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxNetworkStatus(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxNetworkStatus(const PodSandboxNetworkStatus& from) : PodSandboxNetworkStatus(nullptr, from) {}
  inline PodSandboxNetworkStatus(PodSandboxNetworkStatus&& from) noexcept
      : PodSandboxNetworkStatus(nullptr, ::std::move(from)) {}
  inline PodSandboxNetworkStatus& operator=(const PodSandboxNetworkStatus& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxNetworkStatus& operator=(PodSandboxNetworkStatus&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxNetworkStatus& default_instance() {
    return *reinterpret_cast<const PodSandboxNetworkStatus*>(
        &_PodSandboxNetworkStatus_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 23;
  friend void swap(PodSandboxNetworkStatus& a, PodSandboxNetworkStatus& b) { a.Swap(&b); }
  inline void Swap(PodSandboxNetworkStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxNetworkStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxNetworkStatus* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxNetworkStatus>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxNetworkStatus& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxNetworkStatus& from) { PodSandboxNetworkStatus::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxNetworkStatus* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxNetworkStatus"; }

  explicit PodSandboxNetworkStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxNetworkStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxNetworkStatus& from);
  PodSandboxNetworkStatus(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxNetworkStatus&& from) noexcept
      : PodSandboxNetworkStatus(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kAdditionalIpsFieldNumber = 2,
    kIpFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.PodIP additional_ips = 2;
  int additional_ips_size() const;
  private:
  int _internal_additional_ips_size() const;

  public:
  void clear_additional_ips() ;
  ::runtime::v1alpha2::PodIP* PROTOBUF_NONNULL mutable_additional_ips(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>* PROTOBUF_NONNULL mutable_additional_ips();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>& _internal_additional_ips() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>* PROTOBUF_NONNULL _internal_mutable_additional_ips();
  public:
  const ::runtime::v1alpha2::PodIP& additional_ips(int index) const;
  ::runtime::v1alpha2::PodIP* PROTOBUF_NONNULL add_additional_ips();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>& additional_ips() const;
  // string ip = 1;
  void clear_ip() ;
  const ::std::string& ip() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_ip(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_ip();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_ip();
  void set_allocated_ip(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_ip() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_ip(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_ip();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxNetworkStatus)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 51,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxNetworkStatus& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::PodIP > additional_ips_;
    ::google::protobuf::internal::ArenaStringPtr ip_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxNetworkStatus_class_data_;
// -------------------------------------------------------------------

class PodSandboxFilter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxFilter) */ {
 public:
  inline PodSandboxFilter() : PodSandboxFilter(nullptr) {}
  ~PodSandboxFilter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxFilter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxFilter));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxFilter(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxFilter(const PodSandboxFilter& from) : PodSandboxFilter(nullptr, from) {}
  inline PodSandboxFilter(PodSandboxFilter&& from) noexcept
      : PodSandboxFilter(nullptr, ::std::move(from)) {}
  inline PodSandboxFilter& operator=(const PodSandboxFilter& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxFilter& operator=(PodSandboxFilter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxFilter& default_instance() {
    return *reinterpret_cast<const PodSandboxFilter*>(
        &_PodSandboxFilter_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 33;
  friend void swap(PodSandboxFilter& a, PodSandboxFilter& b) { a.Swap(&b); }
  inline void Swap(PodSandboxFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxFilter* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxFilter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxFilter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxFilter& from) { PodSandboxFilter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxFilter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxFilter"; }

  explicit PodSandboxFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxFilter& from);
  PodSandboxFilter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxFilter&& from) noexcept
      : PodSandboxFilter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kStateFieldNumber = 2,
    kLabelSelectorFieldNumber = 3,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // .runtime.v1alpha2.PodSandboxStateValue state = 2;
  bool has_state() const;
  void clear_state() ;
  const ::runtime::v1alpha2::PodSandboxStateValue& state() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE release_state();
  ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NONNULL mutable_state();
  void set_allocated_state(::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_state(::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE unsafe_arena_release_state();

  private:
  const ::runtime::v1alpha2::PodSandboxStateValue& _internal_state() const;
  ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NONNULL _internal_mutable_state();

  public:
  // map<string, string> label_selector = 3;
  int label_selector_size() const;
  private:
  int _internal_label_selector_size() const;

  public:
  void clear_label_selector() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& label_selector() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_label_selector();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_label_selector() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_label_selector();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxFilter)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 3,
                                   2, 58,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxFilter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE state_;
    ::google::protobuf::internal::MapField<PodSandboxFilter_LabelSelectorEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        label_selector_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxFilter_class_data_;
// -------------------------------------------------------------------

class PodSandbox final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandbox) */ {
 public:
  inline PodSandbox() : PodSandbox(nullptr) {}
  ~PodSandbox() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandbox* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandbox));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandbox(::google::protobuf::internal::ConstantInitialized);

  inline PodSandbox(const PodSandbox& from) : PodSandbox(nullptr, from) {}
  inline PodSandbox(PodSandbox&& from) noexcept
      : PodSandbox(nullptr, ::std::move(from)) {}
  inline PodSandbox& operator=(const PodSandbox& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandbox& operator=(PodSandbox&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandbox& default_instance() {
    return *reinterpret_cast<const PodSandbox*>(
        &_PodSandbox_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 37;
  friend void swap(PodSandbox& a, PodSandbox& b) { a.Swap(&b); }
  inline void Swap(PodSandbox* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandbox* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandbox* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandbox>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandbox& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandbox& from) { PodSandbox::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandbox* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandbox"; }

  explicit PodSandbox(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandbox(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandbox& from);
  PodSandbox(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandbox&& from) noexcept
      : PodSandbox(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kRuntimeHandlerFieldNumber = 7,
    kMetadataFieldNumber = 2,
    kCreatedAtFieldNumber = 4,
    kStateFieldNumber = 3,
    kLabelsFieldNumber = 5,
    kAnnotationsFieldNumber = 6,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string runtime_handler = 7;
  void clear_runtime_handler() ;
  const ::std::string& runtime_handler() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_runtime_handler(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_runtime_handler();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_runtime_handler();
  void set_allocated_runtime_handler(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_runtime_handler() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_runtime_handler(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_runtime_handler();

  public:
  // .runtime.v1alpha2.PodSandboxMetadata metadata = 2;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::PodSandboxMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::PodSandboxMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // int64 created_at = 4;
  void clear_created_at() ;
  ::int64_t created_at() const;
  void set_created_at(::int64_t value);

  private:
  ::int64_t _internal_created_at() const;
  void _internal_set_created_at(::int64_t value);

  public:
  // .runtime.v1alpha2.PodSandboxState state = 3;
  void clear_state() ;
  ::runtime::v1alpha2::PodSandboxState state() const;
  void set_state(::runtime::v1alpha2::PodSandboxState value);

  private:
  ::runtime::v1alpha2::PodSandboxState _internal_state() const;
  void _internal_set_state(::runtime::v1alpha2::PodSandboxState value);

  public:
  // map<string, string> labels = 5;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 6;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandbox)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 7,
                                   3, 70,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandbox& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr runtime_handler_;
    ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE metadata_;
    ::int64_t created_at_;
    int state_;
    ::google::protobuf::internal::MapField<PodSandbox_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<PodSandbox_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandbox_class_data_;
// -------------------------------------------------------------------

class Namespace final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Namespace) */ {
 public:
  inline Namespace() : Namespace(nullptr) {}
  ~Namespace() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Namespace* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Namespace));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Namespace(::google::protobuf::internal::ConstantInitialized);

  inline Namespace(const Namespace& from) : Namespace(nullptr, from) {}
  inline Namespace(Namespace&& from) noexcept
      : Namespace(nullptr, ::std::move(from)) {}
  inline Namespace& operator=(const Namespace& from) {
    CopyFrom(from);
    return *this;
  }
  inline Namespace& operator=(Namespace&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Namespace& default_instance() {
    return *reinterpret_cast<const Namespace*>(
        &_Namespace_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 24;
  friend void swap(Namespace& a, Namespace& b) { a.Swap(&b); }
  inline void Swap(Namespace* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Namespace* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Namespace* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Namespace>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Namespace& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Namespace& from) { Namespace::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Namespace* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Namespace"; }

  explicit Namespace(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Namespace(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Namespace& from);
  Namespace(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Namespace&& from) noexcept
      : Namespace(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kOptionsFieldNumber = 2,
  };
  // .runtime.v1alpha2.NamespaceOption options = 2;
  bool has_options() const;
  void clear_options() ;
  const ::runtime::v1alpha2::NamespaceOption& options() const;
  [[nodiscard]] ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE release_options();
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL mutable_options();
  void set_allocated_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE unsafe_arena_release_options();

  private:
  const ::runtime::v1alpha2::NamespaceOption& _internal_options() const;
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL _internal_mutable_options();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Namespace)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Namespace& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE options_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Namespace_class_data_;
// -------------------------------------------------------------------

class MemoryUsage final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.MemoryUsage) */ {
 public:
  inline MemoryUsage() : MemoryUsage(nullptr) {}
  ~MemoryUsage() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(MemoryUsage* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(MemoryUsage));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR MemoryUsage(::google::protobuf::internal::ConstantInitialized);

  inline MemoryUsage(const MemoryUsage& from) : MemoryUsage(nullptr, from) {}
  inline MemoryUsage(MemoryUsage&& from) noexcept
      : MemoryUsage(nullptr, ::std::move(from)) {}
  inline MemoryUsage& operator=(const MemoryUsage& from) {
    CopyFrom(from);
    return *this;
  }
  inline MemoryUsage& operator=(MemoryUsage&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const MemoryUsage& default_instance() {
    return *reinterpret_cast<const MemoryUsage*>(
        &_MemoryUsage_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 126;
  friend void swap(MemoryUsage& a, MemoryUsage& b) { a.Swap(&b); }
  inline void Swap(MemoryUsage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(MemoryUsage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  MemoryUsage* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<MemoryUsage>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const MemoryUsage& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const MemoryUsage& from) { MemoryUsage::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(MemoryUsage* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.MemoryUsage"; }

  explicit MemoryUsage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  MemoryUsage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const MemoryUsage& from);
  MemoryUsage(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, MemoryUsage&& from) noexcept
      : MemoryUsage(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kWorkingSetBytesFieldNumber = 2,
    kTimestampFieldNumber = 1,
  };
  // .runtime.v1alpha2.UInt64Value working_set_bytes = 2;
  bool has_working_set_bytes() const;
  void clear_working_set_bytes() ;
  const ::runtime::v1alpha2::UInt64Value& working_set_bytes() const;
  [[nodiscard]] ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE release_working_set_bytes();
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL mutable_working_set_bytes();
  void set_allocated_working_set_bytes(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_working_set_bytes(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE unsafe_arena_release_working_set_bytes();

  private:
  const ::runtime::v1alpha2::UInt64Value& _internal_working_set_bytes() const;
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL _internal_mutable_working_set_bytes();

  public:
  // int64 timestamp = 1;
  void clear_timestamp() ;
  ::int64_t timestamp() const;
  void set_timestamp(::int64_t value);

  private:
  ::int64_t _internal_timestamp() const;
  void _internal_set_timestamp(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.MemoryUsage)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const MemoryUsage& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE working_set_bytes_;
    ::int64_t timestamp_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull MemoryUsage_class_data_;
// -------------------------------------------------------------------

class LinuxSandboxSecurityContext final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.LinuxSandboxSecurityContext) */ {
 public:
  inline LinuxSandboxSecurityContext() : LinuxSandboxSecurityContext(nullptr) {}
  ~LinuxSandboxSecurityContext() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinuxSandboxSecurityContext* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinuxSandboxSecurityContext));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxSandboxSecurityContext(::google::protobuf::internal::ConstantInitialized);

  inline LinuxSandboxSecurityContext(const LinuxSandboxSecurityContext& from) : LinuxSandboxSecurityContext(nullptr, from) {}
  inline LinuxSandboxSecurityContext(LinuxSandboxSecurityContext&& from) noexcept
      : LinuxSandboxSecurityContext(nullptr, ::std::move(from)) {}
  inline LinuxSandboxSecurityContext& operator=(const LinuxSandboxSecurityContext& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinuxSandboxSecurityContext& operator=(LinuxSandboxSecurityContext&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LinuxSandboxSecurityContext& default_instance() {
    return *reinterpret_cast<const LinuxSandboxSecurityContext*>(
        &_LinuxSandboxSecurityContext_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 7;
  friend void swap(LinuxSandboxSecurityContext& a, LinuxSandboxSecurityContext& b) { a.Swap(&b); }
  inline void Swap(LinuxSandboxSecurityContext* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinuxSandboxSecurityContext* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LinuxSandboxSecurityContext* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinuxSandboxSecurityContext>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinuxSandboxSecurityContext& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinuxSandboxSecurityContext& from) { LinuxSandboxSecurityContext::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinuxSandboxSecurityContext* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.LinuxSandboxSecurityContext"; }

  explicit LinuxSandboxSecurityContext(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinuxSandboxSecurityContext(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinuxSandboxSecurityContext& from);
  LinuxSandboxSecurityContext(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinuxSandboxSecurityContext&& from) noexcept
      : LinuxSandboxSecurityContext(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSupplementalGroupsFieldNumber = 5,
    kSeccompProfilePathFieldNumber = 7,
    kNamespaceOptionsFieldNumber = 1,
    kSelinuxOptionsFieldNumber = 2,
    kRunAsUserFieldNumber = 3,
    kRunAsGroupFieldNumber = 8,
    kSeccompFieldNumber = 9,
    kApparmorFieldNumber = 10,
    kReadonlyRootfsFieldNumber = 4,
    kPrivilegedFieldNumber = 6,
  };
  // repeated int64 supplemental_groups = 5;
  int supplemental_groups_size() const;
  private:
  int _internal_supplemental_groups_size() const;

  public:
  void clear_supplemental_groups() ;
  ::int64_t supplemental_groups(int index) const;
  void set_supplemental_groups(int index, ::int64_t value);
  void add_supplemental_groups(::int64_t value);
  const ::google::protobuf::RepeatedField<::int64_t>& supplemental_groups() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_supplemental_groups();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_supplemental_groups() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_supplemental_groups();

  public:
  // string seccomp_profile_path = 7;
  void clear_seccomp_profile_path() ;
  const ::std::string& seccomp_profile_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_seccomp_profile_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_seccomp_profile_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_seccomp_profile_path();
  void set_allocated_seccomp_profile_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_seccomp_profile_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_seccomp_profile_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_seccomp_profile_path();

  public:
  // .runtime.v1alpha2.NamespaceOption namespace_options = 1;
  bool has_namespace_options() const;
  void clear_namespace_options() ;
  const ::runtime::v1alpha2::NamespaceOption& namespace_options() const;
  [[nodiscard]] ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE release_namespace_options();
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL mutable_namespace_options();
  void set_allocated_namespace_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_namespace_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE unsafe_arena_release_namespace_options();

  private:
  const ::runtime::v1alpha2::NamespaceOption& _internal_namespace_options() const;
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL _internal_mutable_namespace_options();

  public:
  // .runtime.v1alpha2.SELinuxOption selinux_options = 2;
  bool has_selinux_options() const;
  void clear_selinux_options() ;
  const ::runtime::v1alpha2::SELinuxOption& selinux_options() const;
  [[nodiscard]] ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE release_selinux_options();
  ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL mutable_selinux_options();
  void set_allocated_selinux_options(::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_selinux_options(::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE unsafe_arena_release_selinux_options();

  private:
  const ::runtime::v1alpha2::SELinuxOption& _internal_selinux_options() const;
  ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL _internal_mutable_selinux_options();

  public:
  // .runtime.v1alpha2.Int64Value run_as_user = 3;
  bool has_run_as_user() const;
  void clear_run_as_user() ;
  const ::runtime::v1alpha2::Int64Value& run_as_user() const;
  [[nodiscard]] ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE release_run_as_user();
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL mutable_run_as_user();
  void set_allocated_run_as_user(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_run_as_user(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE unsafe_arena_release_run_as_user();

  private:
  const ::runtime::v1alpha2::Int64Value& _internal_run_as_user() const;
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL _internal_mutable_run_as_user();

  public:
  // .runtime.v1alpha2.Int64Value run_as_group = 8;
  bool has_run_as_group() const;
  void clear_run_as_group() ;
  const ::runtime::v1alpha2::Int64Value& run_as_group() const;
  [[nodiscard]] ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE release_run_as_group();
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL mutable_run_as_group();
  void set_allocated_run_as_group(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_run_as_group(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE unsafe_arena_release_run_as_group();

  private:
  const ::runtime::v1alpha2::Int64Value& _internal_run_as_group() const;
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL _internal_mutable_run_as_group();

  public:
  // .runtime.v1alpha2.SecurityProfile seccomp = 9;
  bool has_seccomp() const;
  void clear_seccomp() ;
  const ::runtime::v1alpha2::SecurityProfile& seccomp() const;
  [[nodiscard]] ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE release_seccomp();
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL mutable_seccomp();
  void set_allocated_seccomp(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_seccomp(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE unsafe_arena_release_seccomp();

  private:
  const ::runtime::v1alpha2::SecurityProfile& _internal_seccomp() const;
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL _internal_mutable_seccomp();

  public:
  // .runtime.v1alpha2.SecurityProfile apparmor = 10;
  bool has_apparmor() const;
  void clear_apparmor() ;
  const ::runtime::v1alpha2::SecurityProfile& apparmor() const;
  [[nodiscard]] ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE release_apparmor();
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL mutable_apparmor();
  void set_allocated_apparmor(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_apparmor(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE unsafe_arena_release_apparmor();

  private:
  const ::runtime::v1alpha2::SecurityProfile& _internal_apparmor() const;
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL _internal_mutable_apparmor();

  public:
  // bool readonly_rootfs = 4;
  void clear_readonly_rootfs() ;
  bool readonly_rootfs() const;
  void set_readonly_rootfs(bool value);

  private:
  bool _internal_readonly_rootfs() const;
  void _internal_set_readonly_rootfs(bool value);

  public:
  // bool privileged = 6;
  void clear_privileged() ;
  bool privileged() const;
  void set_privileged(bool value);

  private:
  bool _internal_privileged() const;
  void _internal_set_privileged(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxSandboxSecurityContext)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 10,
                                   6, 81,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinuxSandboxSecurityContext& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int64_t> supplemental_groups_;
    ::google::protobuf::internal::CachedSize _supplemental_groups_cached_byte_size_;
    ::google::protobuf::internal::ArenaStringPtr seccomp_profile_path_;
    ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE namespace_options_;
    ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE selinux_options_;
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE run_as_user_;
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE run_as_group_;
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE seccomp_;
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE apparmor_;
    bool readonly_rootfs_;
    bool privileged_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LinuxSandboxSecurityContext_class_data_;
// -------------------------------------------------------------------

class LinuxContainerSecurityContext final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.LinuxContainerSecurityContext) */ {
 public:
  inline LinuxContainerSecurityContext() : LinuxContainerSecurityContext(nullptr) {}
  ~LinuxContainerSecurityContext() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinuxContainerSecurityContext* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinuxContainerSecurityContext));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxContainerSecurityContext(::google::protobuf::internal::ConstantInitialized);

  inline LinuxContainerSecurityContext(const LinuxContainerSecurityContext& from) : LinuxContainerSecurityContext(nullptr, from) {}
  inline LinuxContainerSecurityContext(LinuxContainerSecurityContext&& from) noexcept
      : LinuxContainerSecurityContext(nullptr, ::std::move(from)) {}
  inline LinuxContainerSecurityContext& operator=(const LinuxContainerSecurityContext& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinuxContainerSecurityContext& operator=(LinuxContainerSecurityContext&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LinuxContainerSecurityContext& default_instance() {
    return *reinterpret_cast<const LinuxContainerSecurityContext*>(
        &_LinuxContainerSecurityContext_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 46;
  friend void swap(LinuxContainerSecurityContext& a, LinuxContainerSecurityContext& b) { a.Swap(&b); }
  inline void Swap(LinuxContainerSecurityContext* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinuxContainerSecurityContext* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LinuxContainerSecurityContext* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinuxContainerSecurityContext>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinuxContainerSecurityContext& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinuxContainerSecurityContext& from) { LinuxContainerSecurityContext::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinuxContainerSecurityContext* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.LinuxContainerSecurityContext"; }

  explicit LinuxContainerSecurityContext(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinuxContainerSecurityContext(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinuxContainerSecurityContext& from);
  LinuxContainerSecurityContext(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinuxContainerSecurityContext&& from) noexcept
      : LinuxContainerSecurityContext(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSupplementalGroupsFieldNumber = 8,
    kMaskedPathsFieldNumber = 13,
    kReadonlyPathsFieldNumber = 14,
    kRunAsUsernameFieldNumber = 6,
    kApparmorProfileFieldNumber = 9,
    kSeccompProfilePathFieldNumber = 10,
    kCapabilitiesFieldNumber = 1,
    kNamespaceOptionsFieldNumber = 3,
    kSelinuxOptionsFieldNumber = 4,
    kRunAsUserFieldNumber = 5,
    kRunAsGroupFieldNumber = 12,
    kSeccompFieldNumber = 15,
    kApparmorFieldNumber = 16,
    kPrivilegedFieldNumber = 2,
    kReadonlyRootfsFieldNumber = 7,
    kNoNewPrivsFieldNumber = 11,
  };
  // repeated int64 supplemental_groups = 8;
  int supplemental_groups_size() const;
  private:
  int _internal_supplemental_groups_size() const;

  public:
  void clear_supplemental_groups() ;
  ::int64_t supplemental_groups(int index) const;
  void set_supplemental_groups(int index, ::int64_t value);
  void add_supplemental_groups(::int64_t value);
  const ::google::protobuf::RepeatedField<::int64_t>& supplemental_groups() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL mutable_supplemental_groups();

  private:
  const ::google::protobuf::RepeatedField<::int64_t>& _internal_supplemental_groups() const;
  ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL _internal_mutable_supplemental_groups();

  public:
  // repeated string masked_paths = 13;
  int masked_paths_size() const;
  private:
  int _internal_masked_paths_size() const;

  public:
  void clear_masked_paths() ;
  const ::std::string& masked_paths(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_masked_paths(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_masked_paths(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_masked_paths();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_masked_paths(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& masked_paths() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_masked_paths();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_masked_paths() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_masked_paths();

  public:
  // repeated string readonly_paths = 14;
  int readonly_paths_size() const;
  private:
  int _internal_readonly_paths_size() const;

  public:
  void clear_readonly_paths() ;
  const ::std::string& readonly_paths(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_readonly_paths(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_readonly_paths(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_readonly_paths();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_readonly_paths(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& readonly_paths() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_readonly_paths();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_readonly_paths() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_readonly_paths();

  public:
  // string run_as_username = 6;
  void clear_run_as_username() ;
  const ::std::string& run_as_username() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_run_as_username(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_run_as_username();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_run_as_username();
  void set_allocated_run_as_username(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_run_as_username() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_run_as_username(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_run_as_username();

  public:
  // string apparmor_profile = 9;
  void clear_apparmor_profile() ;
  const ::std::string& apparmor_profile() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_apparmor_profile(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_apparmor_profile();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_apparmor_profile();
  void set_allocated_apparmor_profile(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_apparmor_profile() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_apparmor_profile(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_apparmor_profile();

  public:
  // string seccomp_profile_path = 10;
  void clear_seccomp_profile_path() ;
  const ::std::string& seccomp_profile_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_seccomp_profile_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_seccomp_profile_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_seccomp_profile_path();
  void set_allocated_seccomp_profile_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_seccomp_profile_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_seccomp_profile_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_seccomp_profile_path();

  public:
  // .runtime.v1alpha2.Capability capabilities = 1;
  bool has_capabilities() const;
  void clear_capabilities() ;
  const ::runtime::v1alpha2::Capability& capabilities() const;
  [[nodiscard]] ::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE release_capabilities();
  ::runtime::v1alpha2::Capability* PROTOBUF_NONNULL mutable_capabilities();
  void set_allocated_capabilities(::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_capabilities(::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE unsafe_arena_release_capabilities();

  private:
  const ::runtime::v1alpha2::Capability& _internal_capabilities() const;
  ::runtime::v1alpha2::Capability* PROTOBUF_NONNULL _internal_mutable_capabilities();

  public:
  // .runtime.v1alpha2.NamespaceOption namespace_options = 3;
  bool has_namespace_options() const;
  void clear_namespace_options() ;
  const ::runtime::v1alpha2::NamespaceOption& namespace_options() const;
  [[nodiscard]] ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE release_namespace_options();
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL mutable_namespace_options();
  void set_allocated_namespace_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_namespace_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE unsafe_arena_release_namespace_options();

  private:
  const ::runtime::v1alpha2::NamespaceOption& _internal_namespace_options() const;
  ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL _internal_mutable_namespace_options();

  public:
  // .runtime.v1alpha2.SELinuxOption selinux_options = 4;
  bool has_selinux_options() const;
  void clear_selinux_options() ;
  const ::runtime::v1alpha2::SELinuxOption& selinux_options() const;
  [[nodiscard]] ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE release_selinux_options();
  ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL mutable_selinux_options();
  void set_allocated_selinux_options(::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_selinux_options(::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE unsafe_arena_release_selinux_options();

  private:
  const ::runtime::v1alpha2::SELinuxOption& _internal_selinux_options() const;
  ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL _internal_mutable_selinux_options();

  public:
  // .runtime.v1alpha2.Int64Value run_as_user = 5;
  bool has_run_as_user() const;
  void clear_run_as_user() ;
  const ::runtime::v1alpha2::Int64Value& run_as_user() const;
  [[nodiscard]] ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE release_run_as_user();
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL mutable_run_as_user();
  void set_allocated_run_as_user(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_run_as_user(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE unsafe_arena_release_run_as_user();

  private:
  const ::runtime::v1alpha2::Int64Value& _internal_run_as_user() const;
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL _internal_mutable_run_as_user();

  public:
  // .runtime.v1alpha2.Int64Value run_as_group = 12;
  bool has_run_as_group() const;
  void clear_run_as_group() ;
  const ::runtime::v1alpha2::Int64Value& run_as_group() const;
  [[nodiscard]] ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE release_run_as_group();
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL mutable_run_as_group();
  void set_allocated_run_as_group(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_run_as_group(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE unsafe_arena_release_run_as_group();

  private:
  const ::runtime::v1alpha2::Int64Value& _internal_run_as_group() const;
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL _internal_mutable_run_as_group();

  public:
  // .runtime.v1alpha2.SecurityProfile seccomp = 15;
  bool has_seccomp() const;
  void clear_seccomp() ;
  const ::runtime::v1alpha2::SecurityProfile& seccomp() const;
  [[nodiscard]] ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE release_seccomp();
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL mutable_seccomp();
  void set_allocated_seccomp(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_seccomp(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE unsafe_arena_release_seccomp();

  private:
  const ::runtime::v1alpha2::SecurityProfile& _internal_seccomp() const;
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL _internal_mutable_seccomp();

  public:
  // .runtime.v1alpha2.SecurityProfile apparmor = 16;
  bool has_apparmor() const;
  void clear_apparmor() ;
  const ::runtime::v1alpha2::SecurityProfile& apparmor() const;
  [[nodiscard]] ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE release_apparmor();
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL mutable_apparmor();
  void set_allocated_apparmor(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_apparmor(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE unsafe_arena_release_apparmor();

  private:
  const ::runtime::v1alpha2::SecurityProfile& _internal_apparmor() const;
  ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL _internal_mutable_apparmor();

  public:
  // bool privileged = 2;
  void clear_privileged() ;
  bool privileged() const;
  void set_privileged(bool value);

  private:
  bool _internal_privileged() const;
  void _internal_set_privileged(bool value);

  public:
  // bool readonly_rootfs = 7;
  void clear_readonly_rootfs() ;
  bool readonly_rootfs() const;
  void set_readonly_rootfs(bool value);

  private:
  bool _internal_readonly_rootfs() const;
  void _internal_set_readonly_rootfs(bool value);

  public:
  // bool no_new_privs = 11;
  void clear_no_new_privs() ;
  bool no_new_privs() const;
  void set_no_new_privs(bool value);

  private:
  bool _internal_no_new_privs() const;
  void _internal_set_no_new_privs(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxContainerSecurityContext)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 16,
                                   7, 148,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinuxContainerSecurityContext& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<::int64_t> supplemental_groups_;
    ::google::protobuf::internal::CachedSize _supplemental_groups_cached_byte_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> masked_paths_;
    ::google::protobuf::RepeatedPtrField<::std::string> readonly_paths_;
    ::google::protobuf::internal::ArenaStringPtr run_as_username_;
    ::google::protobuf::internal::ArenaStringPtr apparmor_profile_;
    ::google::protobuf::internal::ArenaStringPtr seccomp_profile_path_;
    ::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE capabilities_;
    ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE namespace_options_;
    ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE selinux_options_;
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE run_as_user_;
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE run_as_group_;
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE seccomp_;
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE apparmor_;
    bool privileged_;
    bool readonly_rootfs_;
    bool no_new_privs_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LinuxContainerSecurityContext_class_data_;
// -------------------------------------------------------------------

class LinuxContainerResources final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.LinuxContainerResources) */ {
 public:
  inline LinuxContainerResources() : LinuxContainerResources(nullptr) {}
  ~LinuxContainerResources() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinuxContainerResources* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinuxContainerResources));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxContainerResources(::google::protobuf::internal::ConstantInitialized);

  inline LinuxContainerResources(const LinuxContainerResources& from) : LinuxContainerResources(nullptr, from) {}
  inline LinuxContainerResources(LinuxContainerResources&& from) noexcept
      : LinuxContainerResources(nullptr, ::std::move(from)) {}
  inline LinuxContainerResources& operator=(const LinuxContainerResources& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinuxContainerResources& operator=(LinuxContainerResources&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LinuxContainerResources& default_instance() {
    return *reinterpret_cast<const LinuxContainerResources*>(
        &_LinuxContainerResources_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 42;
  friend void swap(LinuxContainerResources& a, LinuxContainerResources& b) { a.Swap(&b); }
  inline void Swap(LinuxContainerResources* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinuxContainerResources* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LinuxContainerResources* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinuxContainerResources>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinuxContainerResources& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinuxContainerResources& from) { LinuxContainerResources::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinuxContainerResources* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.LinuxContainerResources"; }

  explicit LinuxContainerResources(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinuxContainerResources(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinuxContainerResources& from);
  LinuxContainerResources(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinuxContainerResources&& from) noexcept
      : LinuxContainerResources(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kHugepageLimitsFieldNumber = 8,
    kCpusetCpusFieldNumber = 6,
    kCpusetMemsFieldNumber = 7,
    kCpuPeriodFieldNumber = 1,
    kCpuQuotaFieldNumber = 2,
    kCpuSharesFieldNumber = 3,
    kMemoryLimitInBytesFieldNumber = 4,
    kOomScoreAdjFieldNumber = 5,
  };
  // repeated .runtime.v1alpha2.HugepageLimit hugepage_limits = 8;
  int hugepage_limits_size() const;
  private:
  int _internal_hugepage_limits_size() const;

  public:
  void clear_hugepage_limits() ;
  ::runtime::v1alpha2::HugepageLimit* PROTOBUF_NONNULL mutable_hugepage_limits(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>* PROTOBUF_NONNULL mutable_hugepage_limits();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>& _internal_hugepage_limits() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>* PROTOBUF_NONNULL _internal_mutable_hugepage_limits();
  public:
  const ::runtime::v1alpha2::HugepageLimit& hugepage_limits(int index) const;
  ::runtime::v1alpha2::HugepageLimit* PROTOBUF_NONNULL add_hugepage_limits();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>& hugepage_limits() const;
  // string cpuset_cpus = 6;
  void clear_cpuset_cpus() ;
  const ::std::string& cpuset_cpus() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cpuset_cpus(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_cpuset_cpus();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_cpuset_cpus();
  void set_allocated_cpuset_cpus(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_cpuset_cpus() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_cpuset_cpus(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_cpuset_cpus();

  public:
  // string cpuset_mems = 7;
  void clear_cpuset_mems() ;
  const ::std::string& cpuset_mems() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cpuset_mems(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_cpuset_mems();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_cpuset_mems();
  void set_allocated_cpuset_mems(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_cpuset_mems() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_cpuset_mems(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_cpuset_mems();

  public:
  // int64 cpu_period = 1;
  void clear_cpu_period() ;
  ::int64_t cpu_period() const;
  void set_cpu_period(::int64_t value);

  private:
  ::int64_t _internal_cpu_period() const;
  void _internal_set_cpu_period(::int64_t value);

  public:
  // int64 cpu_quota = 2;
  void clear_cpu_quota() ;
  ::int64_t cpu_quota() const;
  void set_cpu_quota(::int64_t value);

  private:
  ::int64_t _internal_cpu_quota() const;
  void _internal_set_cpu_quota(::int64_t value);

  public:
  // int64 cpu_shares = 3;
  void clear_cpu_shares() ;
  ::int64_t cpu_shares() const;
  void set_cpu_shares(::int64_t value);

  private:
  ::int64_t _internal_cpu_shares() const;
  void _internal_set_cpu_shares(::int64_t value);

  public:
  // int64 memory_limit_in_bytes = 4;
  void clear_memory_limit_in_bytes() ;
  ::int64_t memory_limit_in_bytes() const;
  void set_memory_limit_in_bytes(::int64_t value);

  private:
  ::int64_t _internal_memory_limit_in_bytes() const;
  void _internal_set_memory_limit_in_bytes(::int64_t value);

  public:
  // int64 oom_score_adj = 5;
  void clear_oom_score_adj() ;
  ::int64_t oom_score_adj() const;
  void set_oom_score_adj(::int64_t value);

  private:
  ::int64_t _internal_oom_score_adj() const;
  void _internal_set_oom_score_adj(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxContainerResources)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 8,
                                   1, 79,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinuxContainerResources& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::HugepageLimit > hugepage_limits_;
    ::google::protobuf::internal::ArenaStringPtr cpuset_cpus_;
    ::google::protobuf::internal::ArenaStringPtr cpuset_mems_;
    ::int64_t cpu_period_;
    ::int64_t cpu_quota_;
    ::int64_t cpu_shares_;
    ::int64_t memory_limit_in_bytes_;
    ::int64_t oom_score_adj_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LinuxContainerResources_class_data_;
// -------------------------------------------------------------------

class ImageSpec final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ImageSpec) */ {
 public:
  inline ImageSpec() : ImageSpec(nullptr) {}
  ~ImageSpec() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ImageSpec* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ImageSpec));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageSpec(::google::protobuf::internal::ConstantInitialized);

  inline ImageSpec(const ImageSpec& from) : ImageSpec(nullptr, from) {}
  inline ImageSpec(ImageSpec&& from) noexcept
      : ImageSpec(nullptr, ::std::move(from)) {}
  inline ImageSpec& operator=(const ImageSpec& from) {
    CopyFrom(from);
    return *this;
  }
  inline ImageSpec& operator=(ImageSpec&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ImageSpec& default_instance() {
    return *reinterpret_cast<const ImageSpec*>(
        &_ImageSpec_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 40;
  friend void swap(ImageSpec& a, ImageSpec& b) { a.Swap(&b); }
  inline void Swap(ImageSpec* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ImageSpec* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ImageSpec* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ImageSpec>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ImageSpec& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ImageSpec& from) { ImageSpec::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ImageSpec* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ImageSpec"; }

  explicit ImageSpec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ImageSpec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImageSpec& from);
  ImageSpec(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ImageSpec&& from) noexcept
      : ImageSpec(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFieldNumber = 1,
    kAnnotationsFieldNumber = 2,
  };
  // string image = 1;
  void clear_image() ;
  const ::std::string& image() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_image(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_image();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_image();
  void set_allocated_image(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_image() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_image(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // map<string, string> annotations = 2;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageSpec)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 2,
                                   1, 51,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ImageSpec& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr image_;
    ::google::protobuf::internal::MapField<ImageSpec_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ImageSpec_class_data_;
// -------------------------------------------------------------------

class FilesystemUsage final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.FilesystemUsage) */ {
 public:
  inline FilesystemUsage() : FilesystemUsage(nullptr) {}
  ~FilesystemUsage() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(FilesystemUsage* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(FilesystemUsage));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR FilesystemUsage(::google::protobuf::internal::ConstantInitialized);

  inline FilesystemUsage(const FilesystemUsage& from) : FilesystemUsage(nullptr, from) {}
  inline FilesystemUsage(FilesystemUsage&& from) noexcept
      : FilesystemUsage(nullptr, ::std::move(from)) {}
  inline FilesystemUsage& operator=(const FilesystemUsage& from) {
    CopyFrom(from);
    return *this;
  }
  inline FilesystemUsage& operator=(FilesystemUsage&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const FilesystemUsage& default_instance() {
    return *reinterpret_cast<const FilesystemUsage*>(
        &_FilesystemUsage_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 113;
  friend void swap(FilesystemUsage& a, FilesystemUsage& b) { a.Swap(&b); }
  inline void Swap(FilesystemUsage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(FilesystemUsage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  FilesystemUsage* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<FilesystemUsage>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const FilesystemUsage& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const FilesystemUsage& from) { FilesystemUsage::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(FilesystemUsage* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.FilesystemUsage"; }

  explicit FilesystemUsage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  FilesystemUsage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FilesystemUsage& from);
  FilesystemUsage(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, FilesystemUsage&& from) noexcept
      : FilesystemUsage(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFsIdFieldNumber = 2,
    kUsedBytesFieldNumber = 3,
    kInodesUsedFieldNumber = 4,
    kTimestampFieldNumber = 1,
  };
  // .runtime.v1alpha2.FilesystemIdentifier fs_id = 2;
  bool has_fs_id() const;
  void clear_fs_id() ;
  const ::runtime::v1alpha2::FilesystemIdentifier& fs_id() const;
  [[nodiscard]] ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE release_fs_id();
  ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NONNULL mutable_fs_id();
  void set_allocated_fs_id(::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_fs_id(::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE unsafe_arena_release_fs_id();

  private:
  const ::runtime::v1alpha2::FilesystemIdentifier& _internal_fs_id() const;
  ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NONNULL _internal_mutable_fs_id();

  public:
  // .runtime.v1alpha2.UInt64Value used_bytes = 3;
  bool has_used_bytes() const;
  void clear_used_bytes() ;
  const ::runtime::v1alpha2::UInt64Value& used_bytes() const;
  [[nodiscard]] ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE release_used_bytes();
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL mutable_used_bytes();
  void set_allocated_used_bytes(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_used_bytes(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE unsafe_arena_release_used_bytes();

  private:
  const ::runtime::v1alpha2::UInt64Value& _internal_used_bytes() const;
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL _internal_mutable_used_bytes();

  public:
  // .runtime.v1alpha2.UInt64Value inodes_used = 4;
  bool has_inodes_used() const;
  void clear_inodes_used() ;
  const ::runtime::v1alpha2::UInt64Value& inodes_used() const;
  [[nodiscard]] ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE release_inodes_used();
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL mutable_inodes_used();
  void set_allocated_inodes_used(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_inodes_used(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE unsafe_arena_release_inodes_used();

  private:
  const ::runtime::v1alpha2::UInt64Value& _internal_inodes_used() const;
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL _internal_mutable_inodes_used();

  public:
  // int64 timestamp = 1;
  void clear_timestamp() ;
  ::int64_t timestamp() const;
  void set_timestamp(::int64_t value);

  private:
  ::int64_t _internal_timestamp() const;
  void _internal_set_timestamp(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.FilesystemUsage)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   3, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const FilesystemUsage& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE fs_id_;
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE used_bytes_;
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE inodes_used_;
    ::int64_t timestamp_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull FilesystemUsage_class_data_;
// -------------------------------------------------------------------

class CpuUsage final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.CpuUsage) */ {
 public:
  inline CpuUsage() : CpuUsage(nullptr) {}
  ~CpuUsage() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(CpuUsage* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(CpuUsage));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR CpuUsage(::google::protobuf::internal::ConstantInitialized);

  inline CpuUsage(const CpuUsage& from) : CpuUsage(nullptr, from) {}
  inline CpuUsage(CpuUsage&& from) noexcept
      : CpuUsage(nullptr, ::std::move(from)) {}
  inline CpuUsage& operator=(const CpuUsage& from) {
    CopyFrom(from);
    return *this;
  }
  inline CpuUsage& operator=(CpuUsage&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const CpuUsage& default_instance() {
    return *reinterpret_cast<const CpuUsage*>(
        &_CpuUsage_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 125;
  friend void swap(CpuUsage& a, CpuUsage& b) { a.Swap(&b); }
  inline void Swap(CpuUsage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CpuUsage* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  CpuUsage* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<CpuUsage>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const CpuUsage& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const CpuUsage& from) { CpuUsage::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(CpuUsage* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.CpuUsage"; }

  explicit CpuUsage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  CpuUsage(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const CpuUsage& from);
  CpuUsage(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, CpuUsage&& from) noexcept
      : CpuUsage(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kUsageCoreNanoSecondsFieldNumber = 2,
    kTimestampFieldNumber = 1,
  };
  // .runtime.v1alpha2.UInt64Value usage_core_nano_seconds = 2;
  bool has_usage_core_nano_seconds() const;
  void clear_usage_core_nano_seconds() ;
  const ::runtime::v1alpha2::UInt64Value& usage_core_nano_seconds() const;
  [[nodiscard]] ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE release_usage_core_nano_seconds();
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL mutable_usage_core_nano_seconds();
  void set_allocated_usage_core_nano_seconds(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_usage_core_nano_seconds(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE unsafe_arena_release_usage_core_nano_seconds();

  private:
  const ::runtime::v1alpha2::UInt64Value& _internal_usage_core_nano_seconds() const;
  ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL _internal_mutable_usage_core_nano_seconds();

  public:
  // int64 timestamp = 1;
  void clear_timestamp() ;
  ::int64_t timestamp() const;
  void set_timestamp(::int64_t value);

  private:
  ::int64_t _internal_timestamp() const;
  void _internal_set_timestamp(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.CpuUsage)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const CpuUsage& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE usage_core_nano_seconds_;
    ::int64_t timestamp_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull CpuUsage_class_data_;
// -------------------------------------------------------------------

class ContainerStatsFilter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStatsFilter) */ {
 public:
  inline ContainerStatsFilter() : ContainerStatsFilter(nullptr) {}
  ~ContainerStatsFilter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStatsFilter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStatsFilter));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatsFilter(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStatsFilter(const ContainerStatsFilter& from) : ContainerStatsFilter(nullptr, from) {}
  inline ContainerStatsFilter(ContainerStatsFilter&& from) noexcept
      : ContainerStatsFilter(nullptr, ::std::move(from)) {}
  inline ContainerStatsFilter& operator=(const ContainerStatsFilter& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStatsFilter& operator=(ContainerStatsFilter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStatsFilter& default_instance() {
    return *reinterpret_cast<const ContainerStatsFilter*>(
        &_ContainerStatsFilter_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 119;
  friend void swap(ContainerStatsFilter& a, ContainerStatsFilter& b) { a.Swap(&b); }
  inline void Swap(ContainerStatsFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStatsFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStatsFilter* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStatsFilter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStatsFilter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStatsFilter& from) { ContainerStatsFilter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStatsFilter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStatsFilter"; }

  explicit ContainerStatsFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStatsFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStatsFilter& from);
  ContainerStatsFilter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStatsFilter&& from) noexcept
      : ContainerStatsFilter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kPodSandboxIdFieldNumber = 2,
    kLabelSelectorFieldNumber = 3,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string pod_sandbox_id = 2;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // map<string, string> label_selector = 3;
  int label_selector_size() const;
  private:
  int _internal_label_selector_size() const;

  public:
  void clear_label_selector() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& label_selector() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_label_selector();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_label_selector() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_label_selector();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsFilter)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 3,
                                   1, 76,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStatsFilter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    ::google::protobuf::internal::MapField<ContainerStatsFilter_LabelSelectorEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        label_selector_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStatsFilter_class_data_;
// -------------------------------------------------------------------

class ContainerFilter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerFilter) */ {
 public:
  inline ContainerFilter() : ContainerFilter(nullptr) {}
  ~ContainerFilter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerFilter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerFilter));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerFilter(::google::protobuf::internal::ConstantInitialized);

  inline ContainerFilter(const ContainerFilter& from) : ContainerFilter(nullptr, from) {}
  inline ContainerFilter(ContainerFilter&& from) noexcept
      : ContainerFilter(nullptr, ::std::move(from)) {}
  inline ContainerFilter& operator=(const ContainerFilter& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerFilter& operator=(ContainerFilter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerFilter& default_instance() {
    return *reinterpret_cast<const ContainerFilter*>(
        &_ContainerFilter_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 66;
  friend void swap(ContainerFilter& a, ContainerFilter& b) { a.Swap(&b); }
  inline void Swap(ContainerFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerFilter* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerFilter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerFilter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerFilter& from) { ContainerFilter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerFilter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerFilter"; }

  explicit ContainerFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerFilter& from);
  ContainerFilter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerFilter&& from) noexcept
      : ContainerFilter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kPodSandboxIdFieldNumber = 3,
    kStateFieldNumber = 2,
    kLabelSelectorFieldNumber = 4,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string pod_sandbox_id = 3;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // .runtime.v1alpha2.ContainerStateValue state = 2;
  bool has_state() const;
  void clear_state() ;
  const ::runtime::v1alpha2::ContainerStateValue& state() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE release_state();
  ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NONNULL mutable_state();
  void set_allocated_state(::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_state(::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE unsafe_arena_release_state();

  private:
  const ::runtime::v1alpha2::ContainerStateValue& _internal_state() const;
  ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NONNULL _internal_mutable_state();

  public:
  // map<string, string> label_selector = 4;
  int label_selector_size() const;
  private:
  int _internal_label_selector_size() const;

  public:
  void clear_label_selector() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& label_selector() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_label_selector();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_label_selector() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_label_selector();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerFilter)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   2, 71,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerFilter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE state_;
    ::google::protobuf::internal::MapField<ContainerFilter_LabelSelectorEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        label_selector_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerFilter_class_data_;
// -------------------------------------------------------------------

class ContainerAttributes final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerAttributes) */ {
 public:
  inline ContainerAttributes() : ContainerAttributes(nullptr) {}
  ~ContainerAttributes() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerAttributes* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerAttributes));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerAttributes(::google::protobuf::internal::ConstantInitialized);

  inline ContainerAttributes(const ContainerAttributes& from) : ContainerAttributes(nullptr, from) {}
  inline ContainerAttributes(ContainerAttributes&& from) noexcept
      : ContainerAttributes(nullptr, ::std::move(from)) {}
  inline ContainerAttributes& operator=(const ContainerAttributes& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerAttributes& operator=(ContainerAttributes&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerAttributes& default_instance() {
    return *reinterpret_cast<const ContainerAttributes*>(
        &_ContainerAttributes_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 123;
  friend void swap(ContainerAttributes& a, ContainerAttributes& b) { a.Swap(&b); }
  inline void Swap(ContainerAttributes* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerAttributes* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerAttributes* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerAttributes>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerAttributes& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerAttributes& from) { ContainerAttributes::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerAttributes* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerAttributes"; }

  explicit ContainerAttributes(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerAttributes(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerAttributes& from);
  ContainerAttributes(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerAttributes&& from) noexcept
      : ContainerAttributes(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kMetadataFieldNumber = 2,
    kLabelsFieldNumber = 3,
    kAnnotationsFieldNumber = 4,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // .runtime.v1alpha2.ContainerMetadata metadata = 2;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::ContainerMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::ContainerMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // map<string, string> labels = 3;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 4;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerAttributes)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 4,
                                   3, 64,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerAttributes& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE metadata_;
    ::google::protobuf::internal::MapField<ContainerAttributes_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<ContainerAttributes_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerAttributes_class_data_;
// -------------------------------------------------------------------

class UpdateRuntimeConfigRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.UpdateRuntimeConfigRequest) */ {
 public:
  inline UpdateRuntimeConfigRequest() : UpdateRuntimeConfigRequest(nullptr) {}
  ~UpdateRuntimeConfigRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(UpdateRuntimeConfigRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(UpdateRuntimeConfigRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR UpdateRuntimeConfigRequest(::google::protobuf::internal::ConstantInitialized);

  inline UpdateRuntimeConfigRequest(const UpdateRuntimeConfigRequest& from) : UpdateRuntimeConfigRequest(nullptr, from) {}
  inline UpdateRuntimeConfigRequest(UpdateRuntimeConfigRequest&& from) noexcept
      : UpdateRuntimeConfigRequest(nullptr, ::std::move(from)) {}
  inline UpdateRuntimeConfigRequest& operator=(const UpdateRuntimeConfigRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline UpdateRuntimeConfigRequest& operator=(UpdateRuntimeConfigRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const UpdateRuntimeConfigRequest& default_instance() {
    return *reinterpret_cast<const UpdateRuntimeConfigRequest*>(
        &_UpdateRuntimeConfigRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 103;
  friend void swap(UpdateRuntimeConfigRequest& a, UpdateRuntimeConfigRequest& b) { a.Swap(&b); }
  inline void Swap(UpdateRuntimeConfigRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UpdateRuntimeConfigRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  UpdateRuntimeConfigRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<UpdateRuntimeConfigRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const UpdateRuntimeConfigRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const UpdateRuntimeConfigRequest& from) { UpdateRuntimeConfigRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(UpdateRuntimeConfigRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.UpdateRuntimeConfigRequest"; }

  explicit UpdateRuntimeConfigRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  UpdateRuntimeConfigRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UpdateRuntimeConfigRequest& from);
  UpdateRuntimeConfigRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, UpdateRuntimeConfigRequest&& from) noexcept
      : UpdateRuntimeConfigRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRuntimeConfigFieldNumber = 1,
  };
  // .runtime.v1alpha2.RuntimeConfig runtime_config = 1;
  bool has_runtime_config() const;
  void clear_runtime_config() ;
  const ::runtime::v1alpha2::RuntimeConfig& runtime_config() const;
  [[nodiscard]] ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE release_runtime_config();
  ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NONNULL mutable_runtime_config();
  void set_allocated_runtime_config(::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_runtime_config(::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE unsafe_arena_release_runtime_config();

  private:
  const ::runtime::v1alpha2::RuntimeConfig& _internal_runtime_config() const;
  ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NONNULL _internal_mutable_runtime_config();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateRuntimeConfigRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const UpdateRuntimeConfigRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE runtime_config_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull UpdateRuntimeConfigRequest_class_data_;
// -------------------------------------------------------------------

class UpdateContainerResourcesRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.UpdateContainerResourcesRequest) */ {
 public:
  inline UpdateContainerResourcesRequest() : UpdateContainerResourcesRequest(nullptr) {}
  ~UpdateContainerResourcesRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(UpdateContainerResourcesRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(UpdateContainerResourcesRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR UpdateContainerResourcesRequest(::google::protobuf::internal::ConstantInitialized);

  inline UpdateContainerResourcesRequest(const UpdateContainerResourcesRequest& from) : UpdateContainerResourcesRequest(nullptr, from) {}
  inline UpdateContainerResourcesRequest(UpdateContainerResourcesRequest&& from) noexcept
      : UpdateContainerResourcesRequest(nullptr, ::std::move(from)) {}
  inline UpdateContainerResourcesRequest& operator=(const UpdateContainerResourcesRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline UpdateContainerResourcesRequest& operator=(UpdateContainerResourcesRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const UpdateContainerResourcesRequest& default_instance() {
    return *reinterpret_cast<const UpdateContainerResourcesRequest*>(
        &_UpdateContainerResourcesRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 79;
  friend void swap(UpdateContainerResourcesRequest& a, UpdateContainerResourcesRequest& b) { a.Swap(&b); }
  inline void Swap(UpdateContainerResourcesRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(UpdateContainerResourcesRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  UpdateContainerResourcesRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<UpdateContainerResourcesRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const UpdateContainerResourcesRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const UpdateContainerResourcesRequest& from) { UpdateContainerResourcesRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(UpdateContainerResourcesRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.UpdateContainerResourcesRequest"; }

  explicit UpdateContainerResourcesRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  UpdateContainerResourcesRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const UpdateContainerResourcesRequest& from);
  UpdateContainerResourcesRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, UpdateContainerResourcesRequest&& from) noexcept
      : UpdateContainerResourcesRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 1,
    kLinuxFieldNumber = 2,
    kWindowsFieldNumber = 3,
    kAnnotationsFieldNumber = 4,
  };
  // string container_id = 1;
  void clear_container_id() ;
  const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // .runtime.v1alpha2.LinuxContainerResources linux = 2;
  bool has_linux() const;
  void clear_linux() ;
  const ::runtime::v1alpha2::LinuxContainerResources& linux() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE release_linux();
  ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL mutable_linux();
  void set_allocated_linux(::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_linux(::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE unsafe_arena_release_linux();

  private:
  const ::runtime::v1alpha2::LinuxContainerResources& _internal_linux() const;
  ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL _internal_mutable_linux();

  public:
  // .runtime.v1alpha2.WindowsContainerResources windows = 3;
  bool has_windows() const;
  void clear_windows() ;
  const ::runtime::v1alpha2::WindowsContainerResources& windows() const;
  [[nodiscard]] ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE release_windows();
  ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL mutable_windows();
  void set_allocated_windows(::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_windows(::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE unsafe_arena_release_windows();

  private:
  const ::runtime::v1alpha2::WindowsContainerResources& _internal_windows() const;
  ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL _internal_mutable_windows();

  public:
  // map<string, string> annotations = 4;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateContainerResourcesRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   3, 80,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const UpdateContainerResourcesRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE linux_;
    ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE windows_;
    ::google::protobuf::internal::MapField<UpdateContainerResourcesRequest_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull UpdateContainerResourcesRequest_class_data_;
// -------------------------------------------------------------------

class StatusResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.StatusResponse) */ {
 public:
  inline StatusResponse() : StatusResponse(nullptr) {}
  ~StatusResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StatusResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StatusResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR StatusResponse(::google::protobuf::internal::ConstantInitialized);

  inline StatusResponse(const StatusResponse& from) : StatusResponse(nullptr, from) {}
  inline StatusResponse(StatusResponse&& from) noexcept
      : StatusResponse(nullptr, ::std::move(from)) {}
  inline StatusResponse& operator=(const StatusResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline StatusResponse& operator=(StatusResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const StatusResponse& default_instance() {
    return *reinterpret_cast<const StatusResponse*>(
        &_StatusResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 109;
  friend void swap(StatusResponse& a, StatusResponse& b) { a.Swap(&b); }
  inline void Swap(StatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  StatusResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<StatusResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const StatusResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const StatusResponse& from) { StatusResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StatusResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.StatusResponse"; }

  explicit StatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StatusResponse& from);
  StatusResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StatusResponse&& from) noexcept
      : StatusResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStatusFieldNumber = 1,
    kInfoFieldNumber = 2,
  };
  // .runtime.v1alpha2.RuntimeStatus status = 1;
  bool has_status() const;
  void clear_status() ;
  const ::runtime::v1alpha2::RuntimeStatus& status() const;
  [[nodiscard]] ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE release_status();
  ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NONNULL mutable_status();
  void set_allocated_status(::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_status(::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE unsafe_arena_release_status();

  private:
  const ::runtime::v1alpha2::RuntimeStatus& _internal_status() const;
  ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NONNULL _internal_mutable_status();

  public:
  // map<string, string> info = 2;
  int info_size() const;
  private:
  int _internal_info_size() const;

  public:
  void clear_info() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_info();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_info();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.StatusResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 2,
                                   2, 44,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StatusResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE status_;
    ::google::protobuf::internal::MapField<StatusResponse_InfoEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        info_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull StatusResponse_class_data_;
// -------------------------------------------------------------------

class RemoveImageRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RemoveImageRequest) */ {
 public:
  inline RemoveImageRequest() : RemoveImageRequest(nullptr) {}
  ~RemoveImageRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RemoveImageRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RemoveImageRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RemoveImageRequest(::google::protobuf::internal::ConstantInitialized);

  inline RemoveImageRequest(const RemoveImageRequest& from) : RemoveImageRequest(nullptr, from) {}
  inline RemoveImageRequest(RemoveImageRequest&& from) noexcept
      : RemoveImageRequest(nullptr, ::std::move(from)) {}
  inline RemoveImageRequest& operator=(const RemoveImageRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline RemoveImageRequest& operator=(RemoveImageRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RemoveImageRequest& default_instance() {
    return *reinterpret_cast<const RemoveImageRequest*>(
        &_RemoveImageRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 99;
  friend void swap(RemoveImageRequest& a, RemoveImageRequest& b) { a.Swap(&b); }
  inline void Swap(RemoveImageRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RemoveImageRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RemoveImageRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RemoveImageRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RemoveImageRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RemoveImageRequest& from) { RemoveImageRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RemoveImageRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RemoveImageRequest"; }

  explicit RemoveImageRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RemoveImageRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RemoveImageRequest& from);
  RemoveImageRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RemoveImageRequest&& from) noexcept
      : RemoveImageRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFieldNumber = 1,
  };
  // .runtime.v1alpha2.ImageSpec image = 1;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveImageRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RemoveImageRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RemoveImageRequest_class_data_;
// -------------------------------------------------------------------

class ListPodSandboxResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListPodSandboxResponse) */ {
 public:
  inline ListPodSandboxResponse() : ListPodSandboxResponse(nullptr) {}
  ~ListPodSandboxResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListPodSandboxResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListPodSandboxResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListPodSandboxResponse(::google::protobuf::internal::ConstantInitialized);

  inline ListPodSandboxResponse(const ListPodSandboxResponse& from) : ListPodSandboxResponse(nullptr, from) {}
  inline ListPodSandboxResponse(ListPodSandboxResponse&& from) noexcept
      : ListPodSandboxResponse(nullptr, ::std::move(from)) {}
  inline ListPodSandboxResponse& operator=(const ListPodSandboxResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListPodSandboxResponse& operator=(ListPodSandboxResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListPodSandboxResponse& default_instance() {
    return *reinterpret_cast<const ListPodSandboxResponse*>(
        &_ListPodSandboxResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 38;
  friend void swap(ListPodSandboxResponse& a, ListPodSandboxResponse& b) { a.Swap(&b); }
  inline void Swap(ListPodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListPodSandboxResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListPodSandboxResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListPodSandboxResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListPodSandboxResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListPodSandboxResponse& from) { ListPodSandboxResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListPodSandboxResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListPodSandboxResponse"; }

  explicit ListPodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListPodSandboxResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListPodSandboxResponse& from);
  ListPodSandboxResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListPodSandboxResponse&& from) noexcept
      : ListPodSandboxResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kItemsFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.PodSandbox items = 1;
  int items_size() const;
  private:
  int _internal_items_size() const;

  public:
  void clear_items() ;
  ::runtime::v1alpha2::PodSandbox* PROTOBUF_NONNULL mutable_items(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>* PROTOBUF_NONNULL mutable_items();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>& _internal_items() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>* PROTOBUF_NONNULL _internal_mutable_items();
  public:
  const ::runtime::v1alpha2::PodSandbox& items(int index) const;
  ::runtime::v1alpha2::PodSandbox* PROTOBUF_NONNULL add_items();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>& items() const;
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListPodSandboxResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListPodSandboxResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::PodSandbox > items_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListPodSandboxResponse_class_data_;
// -------------------------------------------------------------------

class ListPodSandboxRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListPodSandboxRequest) */ {
 public:
  inline ListPodSandboxRequest() : ListPodSandboxRequest(nullptr) {}
  ~ListPodSandboxRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListPodSandboxRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListPodSandboxRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListPodSandboxRequest(::google::protobuf::internal::ConstantInitialized);

  inline ListPodSandboxRequest(const ListPodSandboxRequest& from) : ListPodSandboxRequest(nullptr, from) {}
  inline ListPodSandboxRequest(ListPodSandboxRequest&& from) noexcept
      : ListPodSandboxRequest(nullptr, ::std::move(from)) {}
  inline ListPodSandboxRequest& operator=(const ListPodSandboxRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListPodSandboxRequest& operator=(ListPodSandboxRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListPodSandboxRequest& default_instance() {
    return *reinterpret_cast<const ListPodSandboxRequest*>(
        &_ListPodSandboxRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 34;
  friend void swap(ListPodSandboxRequest& a, ListPodSandboxRequest& b) { a.Swap(&b); }
  inline void Swap(ListPodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListPodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListPodSandboxRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListPodSandboxRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListPodSandboxRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListPodSandboxRequest& from) { ListPodSandboxRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListPodSandboxRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListPodSandboxRequest"; }

  explicit ListPodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListPodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListPodSandboxRequest& from);
  ListPodSandboxRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListPodSandboxRequest&& from) noexcept
      : ListPodSandboxRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFilterFieldNumber = 1,
  };
  // .runtime.v1alpha2.PodSandboxFilter filter = 1;
  bool has_filter() const;
  void clear_filter() ;
  const ::runtime::v1alpha2::PodSandboxFilter& filter() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE release_filter();
  ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NONNULL mutable_filter();
  void set_allocated_filter(::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_filter(::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE unsafe_arena_release_filter();

  private:
  const ::runtime::v1alpha2::PodSandboxFilter& _internal_filter() const;
  ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NONNULL _internal_mutable_filter();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListPodSandboxRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListPodSandboxRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE filter_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListPodSandboxRequest_class_data_;
// -------------------------------------------------------------------

class ListContainersRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListContainersRequest) */ {
 public:
  inline ListContainersRequest() : ListContainersRequest(nullptr) {}
  ~ListContainersRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListContainersRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListContainersRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListContainersRequest(::google::protobuf::internal::ConstantInitialized);

  inline ListContainersRequest(const ListContainersRequest& from) : ListContainersRequest(nullptr, from) {}
  inline ListContainersRequest(ListContainersRequest&& from) noexcept
      : ListContainersRequest(nullptr, ::std::move(from)) {}
  inline ListContainersRequest& operator=(const ListContainersRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListContainersRequest& operator=(ListContainersRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListContainersRequest& default_instance() {
    return *reinterpret_cast<const ListContainersRequest*>(
        &_ListContainersRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 67;
  friend void swap(ListContainersRequest& a, ListContainersRequest& b) { a.Swap(&b); }
  inline void Swap(ListContainersRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListContainersRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListContainersRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListContainersRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListContainersRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListContainersRequest& from) { ListContainersRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListContainersRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListContainersRequest"; }

  explicit ListContainersRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListContainersRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListContainersRequest& from);
  ListContainersRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListContainersRequest&& from) noexcept
      : ListContainersRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFilterFieldNumber = 1,
  };
  // .runtime.v1alpha2.ContainerFilter filter = 1;
  bool has_filter() const;
  void clear_filter() ;
  const ::runtime::v1alpha2::ContainerFilter& filter() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE release_filter();
  ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NONNULL mutable_filter();
  void set_allocated_filter(::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_filter(::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE unsafe_arena_release_filter();

  private:
  const ::runtime::v1alpha2::ContainerFilter& _internal_filter() const;
  ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NONNULL _internal_mutable_filter();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainersRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListContainersRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE filter_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListContainersRequest_class_data_;
// -------------------------------------------------------------------

class ListContainerStatsRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListContainerStatsRequest) */ {
 public:
  inline ListContainerStatsRequest() : ListContainerStatsRequest(nullptr) {}
  ~ListContainerStatsRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListContainerStatsRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListContainerStatsRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListContainerStatsRequest(::google::protobuf::internal::ConstantInitialized);

  inline ListContainerStatsRequest(const ListContainerStatsRequest& from) : ListContainerStatsRequest(nullptr, from) {}
  inline ListContainerStatsRequest(ListContainerStatsRequest&& from) noexcept
      : ListContainerStatsRequest(nullptr, ::std::move(from)) {}
  inline ListContainerStatsRequest& operator=(const ListContainerStatsRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListContainerStatsRequest& operator=(ListContainerStatsRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListContainerStatsRequest& default_instance() {
    return *reinterpret_cast<const ListContainerStatsRequest*>(
        &_ListContainerStatsRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 117;
  friend void swap(ListContainerStatsRequest& a, ListContainerStatsRequest& b) { a.Swap(&b); }
  inline void Swap(ListContainerStatsRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListContainerStatsRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListContainerStatsRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListContainerStatsRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListContainerStatsRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListContainerStatsRequest& from) { ListContainerStatsRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListContainerStatsRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListContainerStatsRequest"; }

  explicit ListContainerStatsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListContainerStatsRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListContainerStatsRequest& from);
  ListContainerStatsRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListContainerStatsRequest&& from) noexcept
      : ListContainerStatsRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFilterFieldNumber = 1,
  };
  // .runtime.v1alpha2.ContainerStatsFilter filter = 1;
  bool has_filter() const;
  void clear_filter() ;
  const ::runtime::v1alpha2::ContainerStatsFilter& filter() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE release_filter();
  ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NONNULL mutable_filter();
  void set_allocated_filter(::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_filter(::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE unsafe_arena_release_filter();

  private:
  const ::runtime::v1alpha2::ContainerStatsFilter& _internal_filter() const;
  ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NONNULL _internal_mutable_filter();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainerStatsRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListContainerStatsRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE filter_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListContainerStatsRequest_class_data_;
// -------------------------------------------------------------------

class LinuxPodSandboxStatus final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.LinuxPodSandboxStatus) */ {
 public:
  inline LinuxPodSandboxStatus() : LinuxPodSandboxStatus(nullptr) {}
  ~LinuxPodSandboxStatus() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinuxPodSandboxStatus* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinuxPodSandboxStatus));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxPodSandboxStatus(::google::protobuf::internal::ConstantInitialized);

  inline LinuxPodSandboxStatus(const LinuxPodSandboxStatus& from) : LinuxPodSandboxStatus(nullptr, from) {}
  inline LinuxPodSandboxStatus(LinuxPodSandboxStatus&& from) noexcept
      : LinuxPodSandboxStatus(nullptr, ::std::move(from)) {}
  inline LinuxPodSandboxStatus& operator=(const LinuxPodSandboxStatus& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinuxPodSandboxStatus& operator=(LinuxPodSandboxStatus&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LinuxPodSandboxStatus& default_instance() {
    return *reinterpret_cast<const LinuxPodSandboxStatus*>(
        &_LinuxPodSandboxStatus_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 25;
  friend void swap(LinuxPodSandboxStatus& a, LinuxPodSandboxStatus& b) { a.Swap(&b); }
  inline void Swap(LinuxPodSandboxStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinuxPodSandboxStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LinuxPodSandboxStatus* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinuxPodSandboxStatus>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinuxPodSandboxStatus& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinuxPodSandboxStatus& from) { LinuxPodSandboxStatus::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinuxPodSandboxStatus* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.LinuxPodSandboxStatus"; }

  explicit LinuxPodSandboxStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinuxPodSandboxStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinuxPodSandboxStatus& from);
  LinuxPodSandboxStatus(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinuxPodSandboxStatus&& from) noexcept
      : LinuxPodSandboxStatus(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kNamespacesFieldNumber = 1,
  };
  // .runtime.v1alpha2.Namespace namespaces = 1;
  bool has_namespaces() const;
  void clear_namespaces() ;
  const ::runtime::v1alpha2::Namespace& namespaces() const;
  [[nodiscard]] ::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE release_namespaces();
  ::runtime::v1alpha2::Namespace* PROTOBUF_NONNULL mutable_namespaces();
  void set_allocated_namespaces(::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_namespaces(::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE unsafe_arena_release_namespaces();

  private:
  const ::runtime::v1alpha2::Namespace& _internal_namespaces() const;
  ::runtime::v1alpha2::Namespace* PROTOBUF_NONNULL _internal_mutable_namespaces();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxPodSandboxStatus)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinuxPodSandboxStatus& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE namespaces_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LinuxPodSandboxStatus_class_data_;
// -------------------------------------------------------------------

class LinuxPodSandboxConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.LinuxPodSandboxConfig) */ {
 public:
  inline LinuxPodSandboxConfig() : LinuxPodSandboxConfig(nullptr) {}
  ~LinuxPodSandboxConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinuxPodSandboxConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinuxPodSandboxConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxPodSandboxConfig(::google::protobuf::internal::ConstantInitialized);

  inline LinuxPodSandboxConfig(const LinuxPodSandboxConfig& from) : LinuxPodSandboxConfig(nullptr, from) {}
  inline LinuxPodSandboxConfig(LinuxPodSandboxConfig&& from) noexcept
      : LinuxPodSandboxConfig(nullptr, ::std::move(from)) {}
  inline LinuxPodSandboxConfig& operator=(const LinuxPodSandboxConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinuxPodSandboxConfig& operator=(LinuxPodSandboxConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LinuxPodSandboxConfig& default_instance() {
    return *reinterpret_cast<const LinuxPodSandboxConfig*>(
        &_LinuxPodSandboxConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 10;
  friend void swap(LinuxPodSandboxConfig& a, LinuxPodSandboxConfig& b) { a.Swap(&b); }
  inline void Swap(LinuxPodSandboxConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinuxPodSandboxConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LinuxPodSandboxConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinuxPodSandboxConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinuxPodSandboxConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinuxPodSandboxConfig& from) { LinuxPodSandboxConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinuxPodSandboxConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.LinuxPodSandboxConfig"; }

  explicit LinuxPodSandboxConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinuxPodSandboxConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinuxPodSandboxConfig& from);
  LinuxPodSandboxConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinuxPodSandboxConfig&& from) noexcept
      : LinuxPodSandboxConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCgroupParentFieldNumber = 1,
    kSecurityContextFieldNumber = 2,
    kSysctlsFieldNumber = 3,
  };
  // string cgroup_parent = 1;
  void clear_cgroup_parent() ;
  const ::std::string& cgroup_parent() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cgroup_parent(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_cgroup_parent();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_cgroup_parent();
  void set_allocated_cgroup_parent(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_cgroup_parent() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_cgroup_parent(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_cgroup_parent();

  public:
  // .runtime.v1alpha2.LinuxSandboxSecurityContext security_context = 2;
  bool has_security_context() const;
  void clear_security_context() ;
  const ::runtime::v1alpha2::LinuxSandboxSecurityContext& security_context() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE release_security_context();
  ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NONNULL mutable_security_context();
  void set_allocated_security_context(::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_security_context(::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE unsafe_arena_release_security_context();

  private:
  const ::runtime::v1alpha2::LinuxSandboxSecurityContext& _internal_security_context() const;
  ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NONNULL _internal_mutable_security_context();

  public:
  // map<string, string> sysctls = 3;
  int sysctls_size() const;
  private:
  int _internal_sysctls_size() const;

  public:
  void clear_sysctls() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& sysctls() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_sysctls();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_sysctls() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_sysctls();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxPodSandboxConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 3,
                                   2, 67,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinuxPodSandboxConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr cgroup_parent_;
    ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE security_context_;
    ::google::protobuf::internal::MapField<LinuxPodSandboxConfig_SysctlsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        sysctls_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LinuxPodSandboxConfig_class_data_;
// -------------------------------------------------------------------

class LinuxContainerConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.LinuxContainerConfig) */ {
 public:
  inline LinuxContainerConfig() : LinuxContainerConfig(nullptr) {}
  ~LinuxContainerConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(LinuxContainerConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(LinuxContainerConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR LinuxContainerConfig(::google::protobuf::internal::ConstantInitialized);

  inline LinuxContainerConfig(const LinuxContainerConfig& from) : LinuxContainerConfig(nullptr, from) {}
  inline LinuxContainerConfig(LinuxContainerConfig&& from) noexcept
      : LinuxContainerConfig(nullptr, ::std::move(from)) {}
  inline LinuxContainerConfig& operator=(const LinuxContainerConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline LinuxContainerConfig& operator=(LinuxContainerConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const LinuxContainerConfig& default_instance() {
    return *reinterpret_cast<const LinuxContainerConfig*>(
        &_LinuxContainerConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 47;
  friend void swap(LinuxContainerConfig& a, LinuxContainerConfig& b) { a.Swap(&b); }
  inline void Swap(LinuxContainerConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(LinuxContainerConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  LinuxContainerConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<LinuxContainerConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const LinuxContainerConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const LinuxContainerConfig& from) { LinuxContainerConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(LinuxContainerConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.LinuxContainerConfig"; }

  explicit LinuxContainerConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  LinuxContainerConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const LinuxContainerConfig& from);
  LinuxContainerConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, LinuxContainerConfig&& from) noexcept
      : LinuxContainerConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResourcesFieldNumber = 1,
    kSecurityContextFieldNumber = 2,
  };
  // .runtime.v1alpha2.LinuxContainerResources resources = 1;
  bool has_resources() const;
  void clear_resources() ;
  const ::runtime::v1alpha2::LinuxContainerResources& resources() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE release_resources();
  ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL mutable_resources();
  void set_allocated_resources(::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_resources(::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE unsafe_arena_release_resources();

  private:
  const ::runtime::v1alpha2::LinuxContainerResources& _internal_resources() const;
  ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL _internal_mutable_resources();

  public:
  // .runtime.v1alpha2.LinuxContainerSecurityContext security_context = 2;
  bool has_security_context() const;
  void clear_security_context() ;
  const ::runtime::v1alpha2::LinuxContainerSecurityContext& security_context() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE release_security_context();
  ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NONNULL mutable_security_context();
  void set_allocated_security_context(::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_security_context(::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE unsafe_arena_release_security_context();

  private:
  const ::runtime::v1alpha2::LinuxContainerSecurityContext& _internal_security_context() const;
  ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NONNULL _internal_mutable_security_context();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxContainerConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const LinuxContainerConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE resources_;
    ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE security_context_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull LinuxContainerConfig_class_data_;
// -------------------------------------------------------------------

class ImageStatusRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ImageStatusRequest) */ {
 public:
  inline ImageStatusRequest() : ImageStatusRequest(nullptr) {}
  ~ImageStatusRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ImageStatusRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ImageStatusRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageStatusRequest(::google::protobuf::internal::ConstantInitialized);

  inline ImageStatusRequest(const ImageStatusRequest& from) : ImageStatusRequest(nullptr, from) {}
  inline ImageStatusRequest(ImageStatusRequest&& from) noexcept
      : ImageStatusRequest(nullptr, ::std::move(from)) {}
  inline ImageStatusRequest& operator=(const ImageStatusRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ImageStatusRequest& operator=(ImageStatusRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ImageStatusRequest& default_instance() {
    return *reinterpret_cast<const ImageStatusRequest*>(
        &_ImageStatusRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 93;
  friend void swap(ImageStatusRequest& a, ImageStatusRequest& b) { a.Swap(&b); }
  inline void Swap(ImageStatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ImageStatusRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ImageStatusRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ImageStatusRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ImageStatusRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ImageStatusRequest& from) { ImageStatusRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ImageStatusRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ImageStatusRequest"; }

  explicit ImageStatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ImageStatusRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImageStatusRequest& from);
  ImageStatusRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ImageStatusRequest&& from) noexcept
      : ImageStatusRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFieldNumber = 1,
    kVerboseFieldNumber = 2,
  };
  // .runtime.v1alpha2.ImageSpec image = 1;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // bool verbose = 2;
  void clear_verbose() ;
  bool verbose() const;
  void set_verbose(bool value);

  private:
  bool _internal_verbose() const;
  void _internal_set_verbose(bool value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageStatusRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ImageStatusRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    bool verbose_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ImageStatusRequest_class_data_;
// -------------------------------------------------------------------

class ImageFsInfoResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ImageFsInfoResponse) */ {
 public:
  inline ImageFsInfoResponse() : ImageFsInfoResponse(nullptr) {}
  ~ImageFsInfoResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ImageFsInfoResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ImageFsInfoResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageFsInfoResponse(::google::protobuf::internal::ConstantInitialized);

  inline ImageFsInfoResponse(const ImageFsInfoResponse& from) : ImageFsInfoResponse(nullptr, from) {}
  inline ImageFsInfoResponse(ImageFsInfoResponse&& from) noexcept
      : ImageFsInfoResponse(nullptr, ::std::move(from)) {}
  inline ImageFsInfoResponse& operator=(const ImageFsInfoResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ImageFsInfoResponse& operator=(ImageFsInfoResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ImageFsInfoResponse& default_instance() {
    return *reinterpret_cast<const ImageFsInfoResponse*>(
        &_ImageFsInfoResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 114;
  friend void swap(ImageFsInfoResponse& a, ImageFsInfoResponse& b) { a.Swap(&b); }
  inline void Swap(ImageFsInfoResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ImageFsInfoResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ImageFsInfoResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ImageFsInfoResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ImageFsInfoResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ImageFsInfoResponse& from) { ImageFsInfoResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ImageFsInfoResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ImageFsInfoResponse"; }

  explicit ImageFsInfoResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ImageFsInfoResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImageFsInfoResponse& from);
  ImageFsInfoResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ImageFsInfoResponse&& from) noexcept
      : ImageFsInfoResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFilesystemsFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.FilesystemUsage image_filesystems = 1;
  int image_filesystems_size() const;
  private:
  int _internal_image_filesystems_size() const;

  public:
  void clear_image_filesystems() ;
  ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL mutable_image_filesystems(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>* PROTOBUF_NONNULL mutable_image_filesystems();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>& _internal_image_filesystems() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>* PROTOBUF_NONNULL _internal_mutable_image_filesystems();
  public:
  const ::runtime::v1alpha2::FilesystemUsage& image_filesystems(int index) const;
  ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL add_image_filesystems();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>& image_filesystems() const;
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageFsInfoResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ImageFsInfoResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::FilesystemUsage > image_filesystems_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ImageFsInfoResponse_class_data_;
// -------------------------------------------------------------------

class ImageFilter final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ImageFilter) */ {
 public:
  inline ImageFilter() : ImageFilter(nullptr) {}
  ~ImageFilter() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ImageFilter* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ImageFilter));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageFilter(::google::protobuf::internal::ConstantInitialized);

  inline ImageFilter(const ImageFilter& from) : ImageFilter(nullptr, from) {}
  inline ImageFilter(ImageFilter&& from) noexcept
      : ImageFilter(nullptr, ::std::move(from)) {}
  inline ImageFilter& operator=(const ImageFilter& from) {
    CopyFrom(from);
    return *this;
  }
  inline ImageFilter& operator=(ImageFilter&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ImageFilter& default_instance() {
    return *reinterpret_cast<const ImageFilter*>(
        &_ImageFilter_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 89;
  friend void swap(ImageFilter& a, ImageFilter& b) { a.Swap(&b); }
  inline void Swap(ImageFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ImageFilter* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ImageFilter* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ImageFilter>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ImageFilter& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ImageFilter& from) { ImageFilter::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ImageFilter* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ImageFilter"; }

  explicit ImageFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ImageFilter(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImageFilter& from);
  ImageFilter(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ImageFilter&& from) noexcept
      : ImageFilter(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFieldNumber = 1,
  };
  // .runtime.v1alpha2.ImageSpec image = 1;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageFilter)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ImageFilter& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ImageFilter_class_data_;
// -------------------------------------------------------------------

class Image final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Image) */ {
 public:
  inline Image() : Image(nullptr) {}
  ~Image() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Image* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Image));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Image(::google::protobuf::internal::ConstantInitialized);

  inline Image(const Image& from) : Image(nullptr, from) {}
  inline Image(Image&& from) noexcept
      : Image(nullptr, ::std::move(from)) {}
  inline Image& operator=(const Image& from) {
    CopyFrom(from);
    return *this;
  }
  inline Image& operator=(Image&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Image& default_instance() {
    return *reinterpret_cast<const Image*>(
        &_Image_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 91;
  friend void swap(Image& a, Image& b) { a.Swap(&b); }
  inline void Swap(Image* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Image* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Image* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Image>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Image& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Image& from) { Image::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Image* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Image"; }

  explicit Image(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Image(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Image& from);
  Image(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Image&& from) noexcept
      : Image(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRepoTagsFieldNumber = 2,
    kRepoDigestsFieldNumber = 3,
    kIdFieldNumber = 1,
    kUsernameFieldNumber = 6,
    kUidFieldNumber = 5,
    kSpecFieldNumber = 7,
    kSizeFieldNumber = 4,
  };
  // repeated string repo_tags = 2;
  int repo_tags_size() const;
  private:
  int _internal_repo_tags_size() const;

  public:
  void clear_repo_tags() ;
  const ::std::string& repo_tags(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_repo_tags(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_repo_tags(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_repo_tags();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_repo_tags(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& repo_tags() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_repo_tags();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_repo_tags() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_repo_tags();

  public:
  // repeated string repo_digests = 3;
  int repo_digests_size() const;
  private:
  int _internal_repo_digests_size() const;

  public:
  void clear_repo_digests() ;
  const ::std::string& repo_digests(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_repo_digests(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_repo_digests(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_repo_digests();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_repo_digests(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& repo_digests() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_repo_digests();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_repo_digests() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_repo_digests();

  public:
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string username = 6;
  void clear_username() ;
  const ::std::string& username() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_username(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_username();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_username();
  void set_allocated_username(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_username() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_username(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_username();

  public:
  // .runtime.v1alpha2.Int64Value uid = 5;
  bool has_uid() const;
  void clear_uid() ;
  const ::runtime::v1alpha2::Int64Value& uid() const;
  [[nodiscard]] ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE release_uid();
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL mutable_uid();
  void set_allocated_uid(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_uid(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE unsafe_arena_release_uid();

  private:
  const ::runtime::v1alpha2::Int64Value& _internal_uid() const;
  ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL _internal_mutable_uid();

  public:
  // .runtime.v1alpha2.ImageSpec spec = 7;
  bool has_spec() const;
  void clear_spec() ;
  const ::runtime::v1alpha2::ImageSpec& spec() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_spec();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_spec();
  void set_allocated_spec(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_spec(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_spec();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_spec() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_spec();

  public:
  // uint64 size = 4;
  void clear_size() ;
  ::uint64_t size() const;
  void set_size(::uint64_t value);

  private:
  ::uint64_t _internal_size() const;
  void _internal_set_size(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Image)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 7,
                                   2, 62,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Image& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> repo_tags_;
    ::google::protobuf::RepeatedPtrField<::std::string> repo_digests_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr username_;
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE uid_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE spec_;
    ::uint64_t size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Image_class_data_;
// -------------------------------------------------------------------

class ContainerStatus final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStatus) */ {
 public:
  inline ContainerStatus() : ContainerStatus(nullptr) {}
  ~ContainerStatus() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStatus* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStatus));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatus(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStatus(const ContainerStatus& from) : ContainerStatus(nullptr, from) {}
  inline ContainerStatus(ContainerStatus&& from) noexcept
      : ContainerStatus(nullptr, ::std::move(from)) {}
  inline ContainerStatus& operator=(const ContainerStatus& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStatus& operator=(ContainerStatus&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStatus& default_instance() {
    return *reinterpret_cast<const ContainerStatus*>(
        &_ContainerStatus_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 75;
  friend void swap(ContainerStatus& a, ContainerStatus& b) { a.Swap(&b); }
  inline void Swap(ContainerStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStatus* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStatus>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStatus& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStatus& from) { ContainerStatus::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStatus* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStatus"; }

  explicit ContainerStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStatus& from);
  ContainerStatus(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStatus&& from) noexcept
      : ContainerStatus(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kMountsFieldNumber = 14,
    kIdFieldNumber = 1,
    kImageRefFieldNumber = 9,
    kReasonFieldNumber = 10,
    kMessageFieldNumber = 11,
    kLogPathFieldNumber = 15,
    kMetadataFieldNumber = 2,
    kImageFieldNumber = 8,
    kCreatedAtFieldNumber = 4,
    kStartedAtFieldNumber = 5,
    kStateFieldNumber = 3,
    kExitCodeFieldNumber = 7,
    kFinishedAtFieldNumber = 6,
    kLabelsFieldNumber = 12,
    kAnnotationsFieldNumber = 13,
  };
  // repeated .runtime.v1alpha2.Mount mounts = 14;
  int mounts_size() const;
  private:
  int _internal_mounts_size() const;

  public:
  void clear_mounts() ;
  ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL mutable_mounts(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL mutable_mounts();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>& _internal_mounts() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL _internal_mutable_mounts();
  public:
  const ::runtime::v1alpha2::Mount& mounts(int index) const;
  ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL add_mounts();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>& mounts() const;
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string image_ref = 9;
  void clear_image_ref() ;
  const ::std::string& image_ref() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_image_ref(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_image_ref();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_image_ref();
  void set_allocated_image_ref(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_image_ref() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_image_ref(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_image_ref();

  public:
  // string reason = 10;
  void clear_reason() ;
  const ::std::string& reason() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_reason(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_reason();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_reason();
  void set_allocated_reason(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_reason() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_reason(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_reason();

  public:
  // string message = 11;
  void clear_message() ;
  const ::std::string& message() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_message(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_message();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_message();
  void set_allocated_message(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_message() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_message(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_message();

  public:
  // string log_path = 15;
  void clear_log_path() ;
  const ::std::string& log_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_log_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_log_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_log_path();
  void set_allocated_log_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_log_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_log_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_log_path();

  public:
  // .runtime.v1alpha2.ContainerMetadata metadata = 2;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::ContainerMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::ContainerMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // .runtime.v1alpha2.ImageSpec image = 8;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // int64 created_at = 4;
  void clear_created_at() ;
  ::int64_t created_at() const;
  void set_created_at(::int64_t value);

  private:
  ::int64_t _internal_created_at() const;
  void _internal_set_created_at(::int64_t value);

  public:
  // int64 started_at = 5;
  void clear_started_at() ;
  ::int64_t started_at() const;
  void set_started_at(::int64_t value);

  private:
  ::int64_t _internal_started_at() const;
  void _internal_set_started_at(::int64_t value);

  public:
  // .runtime.v1alpha2.ContainerState state = 3;
  void clear_state() ;
  ::runtime::v1alpha2::ContainerState state() const;
  void set_state(::runtime::v1alpha2::ContainerState value);

  private:
  ::runtime::v1alpha2::ContainerState _internal_state() const;
  void _internal_set_state(::runtime::v1alpha2::ContainerState value);

  public:
  // int32 exit_code = 7;
  void clear_exit_code() ;
  ::int32_t exit_code() const;
  void set_exit_code(::int32_t value);

  private:
  ::int32_t _internal_exit_code() const;
  void _internal_set_exit_code(::int32_t value);

  public:
  // int64 finished_at = 6;
  void clear_finished_at() ;
  ::int64_t finished_at() const;
  void set_finished_at(::int64_t value);

  private:
  ::int64_t _internal_finished_at() const;
  void _internal_set_finished_at(::int64_t value);

  public:
  // map<string, string> labels = 12;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 13;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatus)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 15,
                                   5, 98,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStatus& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::Mount > mounts_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr image_ref_;
    ::google::protobuf::internal::ArenaStringPtr reason_;
    ::google::protobuf::internal::ArenaStringPtr message_;
    ::google::protobuf::internal::ArenaStringPtr log_path_;
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE metadata_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    ::int64_t created_at_;
    ::int64_t started_at_;
    int state_;
    ::int32_t exit_code_;
    ::int64_t finished_at_;
    ::google::protobuf::internal::MapField<ContainerStatus_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<ContainerStatus_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStatus_class_data_;
// -------------------------------------------------------------------

class ContainerStats final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStats) */ {
 public:
  inline ContainerStats() : ContainerStats(nullptr) {}
  ~ContainerStats() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStats* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStats));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStats(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStats(const ContainerStats& from) : ContainerStats(nullptr, from) {}
  inline ContainerStats(ContainerStats&& from) noexcept
      : ContainerStats(nullptr, ::std::move(from)) {}
  inline ContainerStats& operator=(const ContainerStats& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStats& operator=(ContainerStats&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStats& default_instance() {
    return *reinterpret_cast<const ContainerStats*>(
        &_ContainerStats_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 124;
  friend void swap(ContainerStats& a, ContainerStats& b) { a.Swap(&b); }
  inline void Swap(ContainerStats* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStats* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStats* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStats>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStats& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStats& from) { ContainerStats::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStats* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStats"; }

  explicit ContainerStats(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStats(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStats& from);
  ContainerStats(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStats&& from) noexcept
      : ContainerStats(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kAttributesFieldNumber = 1,
    kCpuFieldNumber = 2,
    kMemoryFieldNumber = 3,
    kWritableLayerFieldNumber = 4,
  };
  // .runtime.v1alpha2.ContainerAttributes attributes = 1;
  bool has_attributes() const;
  void clear_attributes() ;
  const ::runtime::v1alpha2::ContainerAttributes& attributes() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE release_attributes();
  ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NONNULL mutable_attributes();
  void set_allocated_attributes(::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_attributes(::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE unsafe_arena_release_attributes();

  private:
  const ::runtime::v1alpha2::ContainerAttributes& _internal_attributes() const;
  ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NONNULL _internal_mutable_attributes();

  public:
  // .runtime.v1alpha2.CpuUsage cpu = 2;
  bool has_cpu() const;
  void clear_cpu() ;
  const ::runtime::v1alpha2::CpuUsage& cpu() const;
  [[nodiscard]] ::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE release_cpu();
  ::runtime::v1alpha2::CpuUsage* PROTOBUF_NONNULL mutable_cpu();
  void set_allocated_cpu(::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_cpu(::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE unsafe_arena_release_cpu();

  private:
  const ::runtime::v1alpha2::CpuUsage& _internal_cpu() const;
  ::runtime::v1alpha2::CpuUsage* PROTOBUF_NONNULL _internal_mutable_cpu();

  public:
  // .runtime.v1alpha2.MemoryUsage memory = 3;
  bool has_memory() const;
  void clear_memory() ;
  const ::runtime::v1alpha2::MemoryUsage& memory() const;
  [[nodiscard]] ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE release_memory();
  ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NONNULL mutable_memory();
  void set_allocated_memory(::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_memory(::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE unsafe_arena_release_memory();

  private:
  const ::runtime::v1alpha2::MemoryUsage& _internal_memory() const;
  ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NONNULL _internal_mutable_memory();

  public:
  // .runtime.v1alpha2.FilesystemUsage writable_layer = 4;
  bool has_writable_layer() const;
  void clear_writable_layer() ;
  const ::runtime::v1alpha2::FilesystemUsage& writable_layer() const;
  [[nodiscard]] ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE release_writable_layer();
  ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL mutable_writable_layer();
  void set_allocated_writable_layer(::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_writable_layer(::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE unsafe_arena_release_writable_layer();

  private:
  const ::runtime::v1alpha2::FilesystemUsage& _internal_writable_layer() const;
  ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL _internal_mutable_writable_layer();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStats)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 4,
                                   4, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStats& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE attributes_;
    ::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE cpu_;
    ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE memory_;
    ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE writable_layer_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStats_class_data_;
// -------------------------------------------------------------------

class Container final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.Container) */ {
 public:
  inline Container() : Container(nullptr) {}
  ~Container() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Container* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Container));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR Container(::google::protobuf::internal::ConstantInitialized);

  inline Container(const Container& from) : Container(nullptr, from) {}
  inline Container(Container&& from) noexcept
      : Container(nullptr, ::std::move(from)) {}
  inline Container& operator=(const Container& from) {
    CopyFrom(from);
    return *this;
  }
  inline Container& operator=(Container&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Container& default_instance() {
    return *reinterpret_cast<const Container*>(
        &_Container_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 70;
  friend void swap(Container& a, Container& b) { a.Swap(&b); }
  inline void Swap(Container* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Container* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Container* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Container>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Container& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Container& from) { Container::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Container* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.Container"; }

  explicit Container(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Container(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Container& from);
  Container(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Container&& from) noexcept
      : Container(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kPodSandboxIdFieldNumber = 2,
    kImageRefFieldNumber = 5,
    kMetadataFieldNumber = 3,
    kImageFieldNumber = 4,
    kCreatedAtFieldNumber = 7,
    kStateFieldNumber = 6,
    kLabelsFieldNumber = 8,
    kAnnotationsFieldNumber = 9,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string pod_sandbox_id = 2;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // string image_ref = 5;
  void clear_image_ref() ;
  const ::std::string& image_ref() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_image_ref(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_image_ref();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_image_ref();
  void set_allocated_image_ref(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_image_ref() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_image_ref(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_image_ref();

  public:
  // .runtime.v1alpha2.ContainerMetadata metadata = 3;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::ContainerMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::ContainerMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // .runtime.v1alpha2.ImageSpec image = 4;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // int64 created_at = 7;
  void clear_created_at() ;
  ::int64_t created_at() const;
  void set_created_at(::int64_t value);

  private:
  ::int64_t _internal_created_at() const;
  void _internal_set_created_at(::int64_t value);

  public:
  // .runtime.v1alpha2.ContainerState state = 6;
  void clear_state() ;
  ::runtime::v1alpha2::ContainerState state() const;
  void set_state(::runtime::v1alpha2::ContainerState value);

  private:
  ::runtime::v1alpha2::ContainerState _internal_state() const;
  void _internal_set_state(::runtime::v1alpha2::ContainerState value);

  public:
  // map<string, string> labels = 8;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 9;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.Container)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 9,
                                   4, 85,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Container& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    ::google::protobuf::internal::ArenaStringPtr image_ref_;
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE metadata_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    ::int64_t created_at_;
    int state_;
    ::google::protobuf::internal::MapField<Container_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<Container_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Container_class_data_;
// -------------------------------------------------------------------

class PodSandboxStatus final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxStatus) */ {
 public:
  inline PodSandboxStatus() : PodSandboxStatus(nullptr) {}
  ~PodSandboxStatus() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxStatus* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxStatus));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStatus(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxStatus(const PodSandboxStatus& from) : PodSandboxStatus(nullptr, from) {}
  inline PodSandboxStatus(PodSandboxStatus&& from) noexcept
      : PodSandboxStatus(nullptr, ::std::move(from)) {}
  inline PodSandboxStatus& operator=(const PodSandboxStatus& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxStatus& operator=(PodSandboxStatus&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxStatus& default_instance() {
    return *reinterpret_cast<const PodSandboxStatus*>(
        &_PodSandboxStatus_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 28;
  friend void swap(PodSandboxStatus& a, PodSandboxStatus& b) { a.Swap(&b); }
  inline void Swap(PodSandboxStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxStatus* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxStatus* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxStatus>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxStatus& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxStatus& from) { PodSandboxStatus::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxStatus* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxStatus"; }

  explicit PodSandboxStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxStatus(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxStatus& from);
  PodSandboxStatus(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxStatus&& from) noexcept
      : PodSandboxStatus(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIdFieldNumber = 1,
    kRuntimeHandlerFieldNumber = 9,
    kMetadataFieldNumber = 2,
    kNetworkFieldNumber = 5,
    kLinuxFieldNumber = 6,
    kCreatedAtFieldNumber = 4,
    kStateFieldNumber = 3,
    kLabelsFieldNumber = 7,
    kAnnotationsFieldNumber = 8,
  };
  // string id = 1;
  void clear_id() ;
  const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string runtime_handler = 9;
  void clear_runtime_handler() ;
  const ::std::string& runtime_handler() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_runtime_handler(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_runtime_handler();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_runtime_handler();
  void set_allocated_runtime_handler(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_runtime_handler() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_runtime_handler(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_runtime_handler();

  public:
  // .runtime.v1alpha2.PodSandboxMetadata metadata = 2;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::PodSandboxMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::PodSandboxMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // .runtime.v1alpha2.PodSandboxNetworkStatus network = 5;
  bool has_network() const;
  void clear_network() ;
  const ::runtime::v1alpha2::PodSandboxNetworkStatus& network() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE release_network();
  ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NONNULL mutable_network();
  void set_allocated_network(::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_network(::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE unsafe_arena_release_network();

  private:
  const ::runtime::v1alpha2::PodSandboxNetworkStatus& _internal_network() const;
  ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NONNULL _internal_mutable_network();

  public:
  // .runtime.v1alpha2.LinuxPodSandboxStatus linux = 6;
  bool has_linux() const;
  void clear_linux() ;
  const ::runtime::v1alpha2::LinuxPodSandboxStatus& linux() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE release_linux();
  ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NONNULL mutable_linux();
  void set_allocated_linux(::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_linux(::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE unsafe_arena_release_linux();

  private:
  const ::runtime::v1alpha2::LinuxPodSandboxStatus& _internal_linux() const;
  ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NONNULL _internal_mutable_linux();

  public:
  // int64 created_at = 4;
  void clear_created_at() ;
  ::int64_t created_at() const;
  void set_created_at(::int64_t value);

  private:
  ::int64_t _internal_created_at() const;
  void _internal_set_created_at(::int64_t value);

  public:
  // .runtime.v1alpha2.PodSandboxState state = 3;
  void clear_state() ;
  ::runtime::v1alpha2::PodSandboxState state() const;
  void set_state(::runtime::v1alpha2::PodSandboxState value);

  private:
  ::runtime::v1alpha2::PodSandboxState _internal_state() const;
  void _internal_set_state(::runtime::v1alpha2::PodSandboxState value);

  public:
  // map<string, string> labels = 7;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 8;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatus)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 9,
                                   5, 84,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxStatus& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr runtime_handler_;
    ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE metadata_;
    ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE network_;
    ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE linux_;
    ::int64_t created_at_;
    int state_;
    ::google::protobuf::internal::MapField<PodSandboxStatus_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<PodSandboxStatus_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatus_class_data_;
// -------------------------------------------------------------------

class PodSandboxConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxConfig) */ {
 public:
  inline PodSandboxConfig() : PodSandboxConfig(nullptr) {}
  ~PodSandboxConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxConfig(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxConfig(const PodSandboxConfig& from) : PodSandboxConfig(nullptr, from) {}
  inline PodSandboxConfig(PodSandboxConfig&& from) noexcept
      : PodSandboxConfig(nullptr, ::std::move(from)) {}
  inline PodSandboxConfig& operator=(const PodSandboxConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxConfig& operator=(PodSandboxConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxConfig& default_instance() {
    return *reinterpret_cast<const PodSandboxConfig*>(
        &_PodSandboxConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 14;
  friend void swap(PodSandboxConfig& a, PodSandboxConfig& b) { a.Swap(&b); }
  inline void Swap(PodSandboxConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxConfig& from) { PodSandboxConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxConfig"; }

  explicit PodSandboxConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxConfig& from);
  PodSandboxConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxConfig&& from) noexcept
      : PodSandboxConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPortMappingsFieldNumber = 5,
    kHostnameFieldNumber = 2,
    kLogDirectoryFieldNumber = 3,
    kMetadataFieldNumber = 1,
    kDnsConfigFieldNumber = 4,
    kLinuxFieldNumber = 8,
    kLabelsFieldNumber = 6,
    kAnnotationsFieldNumber = 7,
  };
  // repeated .runtime.v1alpha2.PortMapping port_mappings = 5;
  int port_mappings_size() const;
  private:
  int _internal_port_mappings_size() const;

  public:
  void clear_port_mappings() ;
  ::runtime::v1alpha2::PortMapping* PROTOBUF_NONNULL mutable_port_mappings(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>* PROTOBUF_NONNULL mutable_port_mappings();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>& _internal_port_mappings() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>* PROTOBUF_NONNULL _internal_mutable_port_mappings();
  public:
  const ::runtime::v1alpha2::PortMapping& port_mappings(int index) const;
  ::runtime::v1alpha2::PortMapping* PROTOBUF_NONNULL add_port_mappings();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>& port_mappings() const;
  // string hostname = 2;
  void clear_hostname() ;
  const ::std::string& hostname() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_hostname(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_hostname();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_hostname();
  void set_allocated_hostname(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_hostname() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_hostname(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_hostname();

  public:
  // string log_directory = 3;
  void clear_log_directory() ;
  const ::std::string& log_directory() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_log_directory(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_log_directory();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_log_directory();
  void set_allocated_log_directory(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_log_directory() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_log_directory(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_log_directory();

  public:
  // .runtime.v1alpha2.PodSandboxMetadata metadata = 1;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::PodSandboxMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::PodSandboxMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // .runtime.v1alpha2.DNSConfig dns_config = 4;
  bool has_dns_config() const;
  void clear_dns_config() ;
  const ::runtime::v1alpha2::DNSConfig& dns_config() const;
  [[nodiscard]] ::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE release_dns_config();
  ::runtime::v1alpha2::DNSConfig* PROTOBUF_NONNULL mutable_dns_config();
  void set_allocated_dns_config(::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_dns_config(::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE unsafe_arena_release_dns_config();

  private:
  const ::runtime::v1alpha2::DNSConfig& _internal_dns_config() const;
  ::runtime::v1alpha2::DNSConfig* PROTOBUF_NONNULL _internal_mutable_dns_config();

  public:
  // .runtime.v1alpha2.LinuxPodSandboxConfig linux = 8;
  bool has_linux() const;
  void clear_linux() ;
  const ::runtime::v1alpha2::LinuxPodSandboxConfig& linux() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE release_linux();
  ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NONNULL mutable_linux();
  void set_allocated_linux(::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_linux(::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE unsafe_arena_release_linux();

  private:
  const ::runtime::v1alpha2::LinuxPodSandboxConfig& _internal_linux() const;
  ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NONNULL _internal_mutable_linux();

  public:
  // map<string, string> labels = 6;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 7;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 8,
                                   6, 88,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::PortMapping > port_mappings_;
    ::google::protobuf::internal::ArenaStringPtr hostname_;
    ::google::protobuf::internal::ArenaStringPtr log_directory_;
    ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE metadata_;
    ::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE dns_config_;
    ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE linux_;
    ::google::protobuf::internal::MapField<PodSandboxConfig_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<PodSandboxConfig_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxConfig_class_data_;
// -------------------------------------------------------------------

class ListImagesResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListImagesResponse) */ {
 public:
  inline ListImagesResponse() : ListImagesResponse(nullptr) {}
  ~ListImagesResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListImagesResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListImagesResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListImagesResponse(::google::protobuf::internal::ConstantInitialized);

  inline ListImagesResponse(const ListImagesResponse& from) : ListImagesResponse(nullptr, from) {}
  inline ListImagesResponse(ListImagesResponse&& from) noexcept
      : ListImagesResponse(nullptr, ::std::move(from)) {}
  inline ListImagesResponse& operator=(const ListImagesResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListImagesResponse& operator=(ListImagesResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListImagesResponse& default_instance() {
    return *reinterpret_cast<const ListImagesResponse*>(
        &_ListImagesResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 92;
  friend void swap(ListImagesResponse& a, ListImagesResponse& b) { a.Swap(&b); }
  inline void Swap(ListImagesResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListImagesResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListImagesResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListImagesResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListImagesResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListImagesResponse& from) { ListImagesResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListImagesResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListImagesResponse"; }

  explicit ListImagesResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListImagesResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListImagesResponse& from);
  ListImagesResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListImagesResponse&& from) noexcept
      : ListImagesResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImagesFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.Image images = 1;
  int images_size() const;
  private:
  int _internal_images_size() const;

  public:
  void clear_images() ;
  ::runtime::v1alpha2::Image* PROTOBUF_NONNULL mutable_images(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>* PROTOBUF_NONNULL mutable_images();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>& _internal_images() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>* PROTOBUF_NONNULL _internal_mutable_images();
  public:
  const ::runtime::v1alpha2::Image& images(int index) const;
  ::runtime::v1alpha2::Image* PROTOBUF_NONNULL add_images();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>& images() const;
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListImagesResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListImagesResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::Image > images_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListImagesResponse_class_data_;
// -------------------------------------------------------------------

class ListImagesRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListImagesRequest) */ {
 public:
  inline ListImagesRequest() : ListImagesRequest(nullptr) {}
  ~ListImagesRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListImagesRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListImagesRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListImagesRequest(::google::protobuf::internal::ConstantInitialized);

  inline ListImagesRequest(const ListImagesRequest& from) : ListImagesRequest(nullptr, from) {}
  inline ListImagesRequest(ListImagesRequest&& from) noexcept
      : ListImagesRequest(nullptr, ::std::move(from)) {}
  inline ListImagesRequest& operator=(const ListImagesRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListImagesRequest& operator=(ListImagesRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListImagesRequest& default_instance() {
    return *reinterpret_cast<const ListImagesRequest*>(
        &_ListImagesRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 90;
  friend void swap(ListImagesRequest& a, ListImagesRequest& b) { a.Swap(&b); }
  inline void Swap(ListImagesRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListImagesRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListImagesRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListImagesRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListImagesRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListImagesRequest& from) { ListImagesRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListImagesRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListImagesRequest"; }

  explicit ListImagesRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListImagesRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListImagesRequest& from);
  ListImagesRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListImagesRequest&& from) noexcept
      : ListImagesRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFilterFieldNumber = 1,
  };
  // .runtime.v1alpha2.ImageFilter filter = 1;
  bool has_filter() const;
  void clear_filter() ;
  const ::runtime::v1alpha2::ImageFilter& filter() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE release_filter();
  ::runtime::v1alpha2::ImageFilter* PROTOBUF_NONNULL mutable_filter();
  void set_allocated_filter(::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_filter(::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE unsafe_arena_release_filter();

  private:
  const ::runtime::v1alpha2::ImageFilter& _internal_filter() const;
  ::runtime::v1alpha2::ImageFilter* PROTOBUF_NONNULL _internal_mutable_filter();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListImagesRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListImagesRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE filter_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListImagesRequest_class_data_;
// -------------------------------------------------------------------

class ListContainersResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListContainersResponse) */ {
 public:
  inline ListContainersResponse() : ListContainersResponse(nullptr) {}
  ~ListContainersResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListContainersResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListContainersResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListContainersResponse(::google::protobuf::internal::ConstantInitialized);

  inline ListContainersResponse(const ListContainersResponse& from) : ListContainersResponse(nullptr, from) {}
  inline ListContainersResponse(ListContainersResponse&& from) noexcept
      : ListContainersResponse(nullptr, ::std::move(from)) {}
  inline ListContainersResponse& operator=(const ListContainersResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListContainersResponse& operator=(ListContainersResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListContainersResponse& default_instance() {
    return *reinterpret_cast<const ListContainersResponse*>(
        &_ListContainersResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 71;
  friend void swap(ListContainersResponse& a, ListContainersResponse& b) { a.Swap(&b); }
  inline void Swap(ListContainersResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListContainersResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListContainersResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListContainersResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListContainersResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListContainersResponse& from) { ListContainersResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListContainersResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListContainersResponse"; }

  explicit ListContainersResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListContainersResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListContainersResponse& from);
  ListContainersResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListContainersResponse&& from) noexcept
      : ListContainersResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainersFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.Container containers = 1;
  int containers_size() const;
  private:
  int _internal_containers_size() const;

  public:
  void clear_containers() ;
  ::runtime::v1alpha2::Container* PROTOBUF_NONNULL mutable_containers(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>* PROTOBUF_NONNULL mutable_containers();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>& _internal_containers() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>* PROTOBUF_NONNULL _internal_mutable_containers();
  public:
  const ::runtime::v1alpha2::Container& containers(int index) const;
  ::runtime::v1alpha2::Container* PROTOBUF_NONNULL add_containers();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>& containers() const;
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainersResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListContainersResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::Container > containers_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListContainersResponse_class_data_;
// -------------------------------------------------------------------

class ListContainerStatsResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ListContainerStatsResponse) */ {
 public:
  inline ListContainerStatsResponse() : ListContainerStatsResponse(nullptr) {}
  ~ListContainerStatsResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ListContainerStatsResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ListContainerStatsResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ListContainerStatsResponse(::google::protobuf::internal::ConstantInitialized);

  inline ListContainerStatsResponse(const ListContainerStatsResponse& from) : ListContainerStatsResponse(nullptr, from) {}
  inline ListContainerStatsResponse(ListContainerStatsResponse&& from) noexcept
      : ListContainerStatsResponse(nullptr, ::std::move(from)) {}
  inline ListContainerStatsResponse& operator=(const ListContainerStatsResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ListContainerStatsResponse& operator=(ListContainerStatsResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ListContainerStatsResponse& default_instance() {
    return *reinterpret_cast<const ListContainerStatsResponse*>(
        &_ListContainerStatsResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 120;
  friend void swap(ListContainerStatsResponse& a, ListContainerStatsResponse& b) { a.Swap(&b); }
  inline void Swap(ListContainerStatsResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ListContainerStatsResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ListContainerStatsResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ListContainerStatsResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ListContainerStatsResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ListContainerStatsResponse& from) { ListContainerStatsResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ListContainerStatsResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ListContainerStatsResponse"; }

  explicit ListContainerStatsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ListContainerStatsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ListContainerStatsResponse& from);
  ListContainerStatsResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ListContainerStatsResponse&& from) noexcept
      : ListContainerStatsResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStatsFieldNumber = 1,
  };
  // repeated .runtime.v1alpha2.ContainerStats stats = 1;
  int stats_size() const;
  private:
  int _internal_stats_size() const;

  public:
  void clear_stats() ;
  ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL mutable_stats(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>* PROTOBUF_NONNULL mutable_stats();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>& _internal_stats() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>* PROTOBUF_NONNULL _internal_mutable_stats();
  public:
  const ::runtime::v1alpha2::ContainerStats& stats(int index) const;
  ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL add_stats();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>& stats() const;
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainerStatsResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ListContainerStatsResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::ContainerStats > stats_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ListContainerStatsResponse_class_data_;
// -------------------------------------------------------------------

class ImageStatusResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ImageStatusResponse) */ {
 public:
  inline ImageStatusResponse() : ImageStatusResponse(nullptr) {}
  ~ImageStatusResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ImageStatusResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ImageStatusResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ImageStatusResponse(::google::protobuf::internal::ConstantInitialized);

  inline ImageStatusResponse(const ImageStatusResponse& from) : ImageStatusResponse(nullptr, from) {}
  inline ImageStatusResponse(ImageStatusResponse&& from) noexcept
      : ImageStatusResponse(nullptr, ::std::move(from)) {}
  inline ImageStatusResponse& operator=(const ImageStatusResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ImageStatusResponse& operator=(ImageStatusResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ImageStatusResponse& default_instance() {
    return *reinterpret_cast<const ImageStatusResponse*>(
        &_ImageStatusResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 95;
  friend void swap(ImageStatusResponse& a, ImageStatusResponse& b) { a.Swap(&b); }
  inline void Swap(ImageStatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ImageStatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ImageStatusResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ImageStatusResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ImageStatusResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ImageStatusResponse& from) { ImageStatusResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ImageStatusResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ImageStatusResponse"; }

  explicit ImageStatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ImageStatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ImageStatusResponse& from);
  ImageStatusResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ImageStatusResponse&& from) noexcept
      : ImageStatusResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFieldNumber = 1,
    kInfoFieldNumber = 2,
  };
  // .runtime.v1alpha2.Image image = 1;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::Image& image() const;
  [[nodiscard]] ::runtime::v1alpha2::Image* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::Image* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::Image* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::Image* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::Image* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::Image& _internal_image() const;
  ::runtime::v1alpha2::Image* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // map<string, string> info = 2;
  int info_size() const;
  private:
  int _internal_info_size() const;

  public:
  void clear_info() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_info();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_info();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageStatusResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 2,
                                   2, 49,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ImageStatusResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::Image* PROTOBUF_NULLABLE image_;
    ::google::protobuf::internal::MapField<ImageStatusResponse_InfoEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        info_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ImageStatusResponse_class_data_;
// -------------------------------------------------------------------

class ContainerStatusResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStatusResponse) */ {
 public:
  inline ContainerStatusResponse() : ContainerStatusResponse(nullptr) {}
  ~ContainerStatusResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStatusResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStatusResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatusResponse(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStatusResponse(const ContainerStatusResponse& from) : ContainerStatusResponse(nullptr, from) {}
  inline ContainerStatusResponse(ContainerStatusResponse&& from) noexcept
      : ContainerStatusResponse(nullptr, ::std::move(from)) {}
  inline ContainerStatusResponse& operator=(const ContainerStatusResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStatusResponse& operator=(ContainerStatusResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStatusResponse& default_instance() {
    return *reinterpret_cast<const ContainerStatusResponse*>(
        &_ContainerStatusResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 77;
  friend void swap(ContainerStatusResponse& a, ContainerStatusResponse& b) { a.Swap(&b); }
  inline void Swap(ContainerStatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStatusResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStatusResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStatusResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStatusResponse& from) { ContainerStatusResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStatusResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStatusResponse"; }

  explicit ContainerStatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStatusResponse& from);
  ContainerStatusResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStatusResponse&& from) noexcept
      : ContainerStatusResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStatusFieldNumber = 1,
    kInfoFieldNumber = 2,
  };
  // .runtime.v1alpha2.ContainerStatus status = 1;
  bool has_status() const;
  void clear_status() ;
  const ::runtime::v1alpha2::ContainerStatus& status() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE release_status();
  ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NONNULL mutable_status();
  void set_allocated_status(::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_status(::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE unsafe_arena_release_status();

  private:
  const ::runtime::v1alpha2::ContainerStatus& _internal_status() const;
  ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NONNULL _internal_mutable_status();

  public:
  // map<string, string> info = 2;
  int info_size() const;
  private:
  int _internal_info_size() const;

  public:
  void clear_info() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_info();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_info();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatusResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 2,
                                   2, 53,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStatusResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE status_;
    ::google::protobuf::internal::MapField<ContainerStatusResponse_InfoEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        info_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStatusResponse_class_data_;
// -------------------------------------------------------------------

class ContainerStatsResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerStatsResponse) */ {
 public:
  inline ContainerStatsResponse() : ContainerStatsResponse(nullptr) {}
  ~ContainerStatsResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerStatsResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerStatsResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerStatsResponse(::google::protobuf::internal::ConstantInitialized);

  inline ContainerStatsResponse(const ContainerStatsResponse& from) : ContainerStatsResponse(nullptr, from) {}
  inline ContainerStatsResponse(ContainerStatsResponse&& from) noexcept
      : ContainerStatsResponse(nullptr, ::std::move(from)) {}
  inline ContainerStatsResponse& operator=(const ContainerStatsResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerStatsResponse& operator=(ContainerStatsResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerStatsResponse& default_instance() {
    return *reinterpret_cast<const ContainerStatsResponse*>(
        &_ContainerStatsResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 116;
  friend void swap(ContainerStatsResponse& a, ContainerStatsResponse& b) { a.Swap(&b); }
  inline void Swap(ContainerStatsResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerStatsResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerStatsResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerStatsResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerStatsResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerStatsResponse& from) { ContainerStatsResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerStatsResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerStatsResponse"; }

  explicit ContainerStatsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerStatsResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerStatsResponse& from);
  ContainerStatsResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerStatsResponse&& from) noexcept
      : ContainerStatsResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStatsFieldNumber = 1,
  };
  // .runtime.v1alpha2.ContainerStats stats = 1;
  bool has_stats() const;
  void clear_stats() ;
  const ::runtime::v1alpha2::ContainerStats& stats() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE release_stats();
  ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL mutable_stats();
  void set_allocated_stats(::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_stats(::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE unsafe_arena_release_stats();

  private:
  const ::runtime::v1alpha2::ContainerStats& _internal_stats() const;
  ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL _internal_mutable_stats();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerStatsResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE stats_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerStatsResponse_class_data_;
// -------------------------------------------------------------------

class ContainerConfig final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.ContainerConfig) */ {
 public:
  inline ContainerConfig() : ContainerConfig(nullptr) {}
  ~ContainerConfig() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContainerConfig* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContainerConfig));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR ContainerConfig(::google::protobuf::internal::ConstantInitialized);

  inline ContainerConfig(const ContainerConfig& from) : ContainerConfig(nullptr, from) {}
  inline ContainerConfig(ContainerConfig&& from) noexcept
      : ContainerConfig(nullptr, ::std::move(from)) {}
  inline ContainerConfig& operator=(const ContainerConfig& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContainerConfig& operator=(ContainerConfig&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const ContainerConfig& default_instance() {
    return *reinterpret_cast<const ContainerConfig*>(
        &_ContainerConfig_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 55;
  friend void swap(ContainerConfig& a, ContainerConfig& b) { a.Swap(&b); }
  inline void Swap(ContainerConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContainerConfig* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  ContainerConfig* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<ContainerConfig>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const ContainerConfig& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const ContainerConfig& from) { ContainerConfig::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContainerConfig* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.ContainerConfig"; }

  explicit ContainerConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContainerConfig(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContainerConfig& from);
  ContainerConfig(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContainerConfig&& from) noexcept
      : ContainerConfig(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCommandFieldNumber = 3,
    kArgsFieldNumber = 4,
    kEnvsFieldNumber = 6,
    kMountsFieldNumber = 7,
    kDevicesFieldNumber = 8,
    kWorkingDirFieldNumber = 5,
    kLogPathFieldNumber = 11,
    kMetadataFieldNumber = 1,
    kImageFieldNumber = 2,
    kLinuxFieldNumber = 15,
    kWindowsFieldNumber = 16,
    kFStdinFieldNumber = 12,
    kStdinOnceFieldNumber = 13,
    kTtyFieldNumber = 14,
    kLabelsFieldNumber = 9,
    kAnnotationsFieldNumber = 10,
  };
  // repeated string command = 3;
  int command_size() const;
  private:
  int _internal_command_size() const;

  public:
  void clear_command() ;
  const ::std::string& command(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_command(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_command(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_command();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_command(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& command() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_command();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_command() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_command();

  public:
  // repeated string args = 4;
  int args_size() const;
  private:
  int _internal_args_size() const;

  public:
  void clear_args() ;
  const ::std::string& args(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_args(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_args(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_args();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_args(Arg_&& value, Args_... args);
  const ::google::protobuf::RepeatedPtrField<::std::string>& args() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL mutable_args();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_args() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_args();

  public:
  // repeated .runtime.v1alpha2.KeyValue envs = 6;
  int envs_size() const;
  private:
  int _internal_envs_size() const;

  public:
  void clear_envs() ;
  ::runtime::v1alpha2::KeyValue* PROTOBUF_NONNULL mutable_envs(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>* PROTOBUF_NONNULL mutable_envs();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>& _internal_envs() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>* PROTOBUF_NONNULL _internal_mutable_envs();
  public:
  const ::runtime::v1alpha2::KeyValue& envs(int index) const;
  ::runtime::v1alpha2::KeyValue* PROTOBUF_NONNULL add_envs();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>& envs() const;
  // repeated .runtime.v1alpha2.Mount mounts = 7;
  int mounts_size() const;
  private:
  int _internal_mounts_size() const;

  public:
  void clear_mounts() ;
  ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL mutable_mounts(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL mutable_mounts();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>& _internal_mounts() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL _internal_mutable_mounts();
  public:
  const ::runtime::v1alpha2::Mount& mounts(int index) const;
  ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL add_mounts();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>& mounts() const;
  // repeated .runtime.v1alpha2.Device devices = 8;
  int devices_size() const;
  private:
  int _internal_devices_size() const;

  public:
  void clear_devices() ;
  ::runtime::v1alpha2::Device* PROTOBUF_NONNULL mutable_devices(int index);
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>* PROTOBUF_NONNULL mutable_devices();

  private:
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>& _internal_devices() const;
  ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>* PROTOBUF_NONNULL _internal_mutable_devices();
  public:
  const ::runtime::v1alpha2::Device& devices(int index) const;
  ::runtime::v1alpha2::Device* PROTOBUF_NONNULL add_devices();
  const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>& devices() const;
  // string working_dir = 5;
  void clear_working_dir() ;
  const ::std::string& working_dir() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_working_dir(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_working_dir();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_working_dir();
  void set_allocated_working_dir(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_working_dir() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_working_dir(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_working_dir();

  public:
  // string log_path = 11;
  void clear_log_path() ;
  const ::std::string& log_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_log_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_log_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_log_path();
  void set_allocated_log_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_log_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_log_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_log_path();

  public:
  // .runtime.v1alpha2.ContainerMetadata metadata = 1;
  bool has_metadata() const;
  void clear_metadata() ;
  const ::runtime::v1alpha2::ContainerMetadata& metadata() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE release_metadata();
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL mutable_metadata();
  void set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE unsafe_arena_release_metadata();

  private:
  const ::runtime::v1alpha2::ContainerMetadata& _internal_metadata() const;
  ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL _internal_mutable_metadata();

  public:
  // .runtime.v1alpha2.ImageSpec image = 2;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // .runtime.v1alpha2.LinuxContainerConfig linux = 15;
  bool has_linux() const;
  void clear_linux() ;
  const ::runtime::v1alpha2::LinuxContainerConfig& linux() const;
  [[nodiscard]] ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE release_linux();
  ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NONNULL mutable_linux();
  void set_allocated_linux(::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_linux(::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE unsafe_arena_release_linux();

  private:
  const ::runtime::v1alpha2::LinuxContainerConfig& _internal_linux() const;
  ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NONNULL _internal_mutable_linux();

  public:
  // .runtime.v1alpha2.WindowsContainerConfig windows = 16;
  bool has_windows() const;
  void clear_windows() ;
  const ::runtime::v1alpha2::WindowsContainerConfig& windows() const;
  [[nodiscard]] ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE release_windows();
  ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NONNULL mutable_windows();
  void set_allocated_windows(::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_windows(::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE unsafe_arena_release_windows();

  private:
  const ::runtime::v1alpha2::WindowsContainerConfig& _internal_windows() const;
  ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NONNULL _internal_mutable_windows();

  public:
  // bool f_stdin = 12;
  void clear_f_stdin() ;
  bool f_stdin() const;
  void set_f_stdin(bool value);

  private:
  bool _internal_f_stdin() const;
  void _internal_set_f_stdin(bool value);

  public:
  // bool stdin_once = 13;
  void clear_stdin_once() ;
  bool stdin_once() const;
  void set_stdin_once(bool value);

  private:
  bool _internal_stdin_once() const;
  void _internal_set_stdin_once(bool value);

  public:
  // bool tty = 14;
  void clear_tty() ;
  bool tty() const;
  void set_tty(bool value);

  private:
  bool _internal_tty() const;
  void _internal_set_tty(bool value);

  public:
  // map<string, string> labels = 9;
  int labels_size() const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_labels() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // map<string, string> annotations = 10;
  int annotations_size() const;
  private:
  int _internal_annotations_size() const;

  public:
  void clear_annotations() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_annotations();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_annotations() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_annotations();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerConfig)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<4, 16,
                                   9, 104,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContainerConfig& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> command_;
    ::google::protobuf::RepeatedPtrField<::std::string> args_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::KeyValue > envs_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::Mount > mounts_;
    ::google::protobuf::RepeatedPtrField< ::runtime::v1alpha2::Device > devices_;
    ::google::protobuf::internal::ArenaStringPtr working_dir_;
    ::google::protobuf::internal::ArenaStringPtr log_path_;
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE metadata_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE linux_;
    ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE windows_;
    bool f_stdin_;
    bool stdin_once_;
    bool tty_;
    ::google::protobuf::internal::MapField<ContainerConfig_LabelsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        labels_;
    ::google::protobuf::internal::MapField<ContainerConfig_AnnotationsEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        annotations_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull ContainerConfig_class_data_;
// -------------------------------------------------------------------

class RunPodSandboxRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.RunPodSandboxRequest) */ {
 public:
  inline RunPodSandboxRequest() : RunPodSandboxRequest(nullptr) {}
  ~RunPodSandboxRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(RunPodSandboxRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(RunPodSandboxRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR RunPodSandboxRequest(::google::protobuf::internal::ConstantInitialized);

  inline RunPodSandboxRequest(const RunPodSandboxRequest& from) : RunPodSandboxRequest(nullptr, from) {}
  inline RunPodSandboxRequest(RunPodSandboxRequest&& from) noexcept
      : RunPodSandboxRequest(nullptr, ::std::move(from)) {}
  inline RunPodSandboxRequest& operator=(const RunPodSandboxRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline RunPodSandboxRequest& operator=(RunPodSandboxRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const RunPodSandboxRequest& default_instance() {
    return *reinterpret_cast<const RunPodSandboxRequest*>(
        &_RunPodSandboxRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 15;
  friend void swap(RunPodSandboxRequest& a, RunPodSandboxRequest& b) { a.Swap(&b); }
  inline void Swap(RunPodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(RunPodSandboxRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  RunPodSandboxRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<RunPodSandboxRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const RunPodSandboxRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const RunPodSandboxRequest& from) { RunPodSandboxRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(RunPodSandboxRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.RunPodSandboxRequest"; }

  explicit RunPodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  RunPodSandboxRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const RunPodSandboxRequest& from);
  RunPodSandboxRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, RunPodSandboxRequest&& from) noexcept
      : RunPodSandboxRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRuntimeHandlerFieldNumber = 2,
    kConfigFieldNumber = 1,
  };
  // string runtime_handler = 2;
  void clear_runtime_handler() ;
  const ::std::string& runtime_handler() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_runtime_handler(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_runtime_handler();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_runtime_handler();
  void set_allocated_runtime_handler(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_runtime_handler() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_runtime_handler(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_runtime_handler();

  public:
  // .runtime.v1alpha2.PodSandboxConfig config = 1;
  bool has_config() const;
  void clear_config() ;
  const ::runtime::v1alpha2::PodSandboxConfig& config() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE release_config();
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL mutable_config();
  void set_allocated_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE unsafe_arena_release_config();

  private:
  const ::runtime::v1alpha2::PodSandboxConfig& _internal_config() const;
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL _internal_mutable_config();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.RunPodSandboxRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 61,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const RunPodSandboxRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr runtime_handler_;
    ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE config_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull RunPodSandboxRequest_class_data_;
// -------------------------------------------------------------------

class PullImageRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PullImageRequest) */ {
 public:
  inline PullImageRequest() : PullImageRequest(nullptr) {}
  ~PullImageRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PullImageRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PullImageRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PullImageRequest(::google::protobuf::internal::ConstantInitialized);

  inline PullImageRequest(const PullImageRequest& from) : PullImageRequest(nullptr, from) {}
  inline PullImageRequest(PullImageRequest&& from) noexcept
      : PullImageRequest(nullptr, ::std::move(from)) {}
  inline PullImageRequest& operator=(const PullImageRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline PullImageRequest& operator=(PullImageRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PullImageRequest& default_instance() {
    return *reinterpret_cast<const PullImageRequest*>(
        &_PullImageRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 97;
  friend void swap(PullImageRequest& a, PullImageRequest& b) { a.Swap(&b); }
  inline void Swap(PullImageRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PullImageRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PullImageRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PullImageRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PullImageRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PullImageRequest& from) { PullImageRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PullImageRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PullImageRequest"; }

  explicit PullImageRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PullImageRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PullImageRequest& from);
  PullImageRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PullImageRequest&& from) noexcept
      : PullImageRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kImageFieldNumber = 1,
    kAuthFieldNumber = 2,
    kSandboxConfigFieldNumber = 3,
  };
  // .runtime.v1alpha2.ImageSpec image = 1;
  bool has_image() const;
  void clear_image() ;
  const ::runtime::v1alpha2::ImageSpec& image() const;
  [[nodiscard]] ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE release_image();
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL mutable_image();
  void set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE unsafe_arena_release_image();

  private:
  const ::runtime::v1alpha2::ImageSpec& _internal_image() const;
  ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL _internal_mutable_image();

  public:
  // .runtime.v1alpha2.AuthConfig auth = 2;
  bool has_auth() const;
  void clear_auth() ;
  const ::runtime::v1alpha2::AuthConfig& auth() const;
  [[nodiscard]] ::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE release_auth();
  ::runtime::v1alpha2::AuthConfig* PROTOBUF_NONNULL mutable_auth();
  void set_allocated_auth(::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_auth(::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE unsafe_arena_release_auth();

  private:
  const ::runtime::v1alpha2::AuthConfig& _internal_auth() const;
  ::runtime::v1alpha2::AuthConfig* PROTOBUF_NONNULL _internal_mutable_auth();

  public:
  // .runtime.v1alpha2.PodSandboxConfig sandbox_config = 3;
  bool has_sandbox_config() const;
  void clear_sandbox_config() ;
  const ::runtime::v1alpha2::PodSandboxConfig& sandbox_config() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE release_sandbox_config();
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL mutable_sandbox_config();
  void set_allocated_sandbox_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_sandbox_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE unsafe_arena_release_sandbox_config();

  private:
  const ::runtime::v1alpha2::PodSandboxConfig& _internal_sandbox_config() const;
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL _internal_mutable_sandbox_config();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PullImageRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   3, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PullImageRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE image_;
    ::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE auth_;
    ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE sandbox_config_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PullImageRequest_class_data_;
// -------------------------------------------------------------------

class PodSandboxStatusResponse final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.PodSandboxStatusResponse) */ {
 public:
  inline PodSandboxStatusResponse() : PodSandboxStatusResponse(nullptr) {}
  ~PodSandboxStatusResponse() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(PodSandboxStatusResponse* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(PodSandboxStatusResponse));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR PodSandboxStatusResponse(::google::protobuf::internal::ConstantInitialized);

  inline PodSandboxStatusResponse(const PodSandboxStatusResponse& from) : PodSandboxStatusResponse(nullptr, from) {}
  inline PodSandboxStatusResponse(PodSandboxStatusResponse&& from) noexcept
      : PodSandboxStatusResponse(nullptr, ::std::move(from)) {}
  inline PodSandboxStatusResponse& operator=(const PodSandboxStatusResponse& from) {
    CopyFrom(from);
    return *this;
  }
  inline PodSandboxStatusResponse& operator=(PodSandboxStatusResponse&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const PodSandboxStatusResponse& default_instance() {
    return *reinterpret_cast<const PodSandboxStatusResponse*>(
        &_PodSandboxStatusResponse_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 30;
  friend void swap(PodSandboxStatusResponse& a, PodSandboxStatusResponse& b) { a.Swap(&b); }
  inline void Swap(PodSandboxStatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(PodSandboxStatusResponse* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  PodSandboxStatusResponse* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<PodSandboxStatusResponse>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const PodSandboxStatusResponse& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const PodSandboxStatusResponse& from) { PodSandboxStatusResponse::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(PodSandboxStatusResponse* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.PodSandboxStatusResponse"; }

  explicit PodSandboxStatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  PodSandboxStatusResponse(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const PodSandboxStatusResponse& from);
  PodSandboxStatusResponse(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, PodSandboxStatusResponse&& from) noexcept
      : PodSandboxStatusResponse(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kStatusFieldNumber = 1,
    kInfoFieldNumber = 2,
  };
  // .runtime.v1alpha2.PodSandboxStatus status = 1;
  bool has_status() const;
  void clear_status() ;
  const ::runtime::v1alpha2::PodSandboxStatus& status() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE release_status();
  ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NONNULL mutable_status();
  void set_allocated_status(::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_status(::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE unsafe_arena_release_status();

  private:
  const ::runtime::v1alpha2::PodSandboxStatus& _internal_status() const;
  ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NONNULL _internal_mutable_status();

  public:
  // map<string, string> info = 2;
  int info_size() const;
  private:
  int _internal_info_size() const;

  public:
  void clear_info() ;
  const ::google::protobuf::Map<::std::string, ::std::string>& info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL mutable_info();

  private:
  const ::google::protobuf::Map<::std::string, ::std::string>& _internal_info() const;
  ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL _internal_mutable_info();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatusResponse)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 2,
                                   2, 54,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const PodSandboxStatusResponse& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE status_;
    ::google::protobuf::internal::MapField<PodSandboxStatusResponse_InfoEntry_DoNotUse, ::std::string, ::std::string,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING,
                      ::google::protobuf::internal::WireFormatLite::TYPE_STRING>
        info_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull PodSandboxStatusResponse_class_data_;
// -------------------------------------------------------------------

class CreateContainerRequest final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:runtime.v1alpha2.CreateContainerRequest) */ {
 public:
  inline CreateContainerRequest() : CreateContainerRequest(nullptr) {}
  ~CreateContainerRequest() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(CreateContainerRequest* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(CreateContainerRequest));
  }
#endif

  template <typename = void>
  explicit PROTOBUF_CONSTEXPR CreateContainerRequest(::google::protobuf::internal::ConstantInitialized);

  inline CreateContainerRequest(const CreateContainerRequest& from) : CreateContainerRequest(nullptr, from) {}
  inline CreateContainerRequest(CreateContainerRequest&& from) noexcept
      : CreateContainerRequest(nullptr, ::std::move(from)) {}
  inline CreateContainerRequest& operator=(const CreateContainerRequest& from) {
    CopyFrom(from);
    return *this;
  }
  inline CreateContainerRequest& operator=(CreateContainerRequest&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL mutable_unknown_fields()
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const CreateContainerRequest& default_instance() {
    return *reinterpret_cast<const CreateContainerRequest*>(
        &_CreateContainerRequest_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 56;
  friend void swap(CreateContainerRequest& a, CreateContainerRequest& b) { a.Swap(&b); }
  inline void Swap(CreateContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CreateContainerRequest* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  CreateContainerRequest* PROTOBUF_NONNULL New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<CreateContainerRequest>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const CreateContainerRequest& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const CreateContainerRequest& from) { CreateContainerRequest::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  ::size_t ByteSizeLong() const final;
  ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  int GetCachedSize() const { return _impl_._cached_size_.Get(); }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(CreateContainerRequest* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "runtime.v1alpha2.CreateContainerRequest"; }

  explicit CreateContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  CreateContainerRequest(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const CreateContainerRequest& from);
  CreateContainerRequest(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, CreateContainerRequest&& from) noexcept
      : CreateContainerRequest(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPodSandboxIdFieldNumber = 1,
    kConfigFieldNumber = 2,
    kSandboxConfigFieldNumber = 3,
  };
  // string pod_sandbox_id = 1;
  void clear_pod_sandbox_id() ;
  const ::std::string& pod_sandbox_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pod_sandbox_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pod_sandbox_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pod_sandbox_id();
  void set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pod_sandbox_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pod_sandbox_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pod_sandbox_id();

  public:
  // .runtime.v1alpha2.ContainerConfig config = 2;
  bool has_config() const;
  void clear_config() ;
  const ::runtime::v1alpha2::ContainerConfig& config() const;
  [[nodiscard]] ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE release_config();
  ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NONNULL mutable_config();
  void set_allocated_config(::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_config(::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE unsafe_arena_release_config();

  private:
  const ::runtime::v1alpha2::ContainerConfig& _internal_config() const;
  ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NONNULL _internal_mutable_config();

  public:
  // .runtime.v1alpha2.PodSandboxConfig sandbox_config = 3;
  bool has_sandbox_config() const;
  void clear_sandbox_config() ;
  const ::runtime::v1alpha2::PodSandboxConfig& sandbox_config() const;
  [[nodiscard]] ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE release_sandbox_config();
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL mutable_sandbox_config();
  void set_allocated_sandbox_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_sandbox_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value);
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE unsafe_arena_release_sandbox_config();

  private:
  const ::runtime::v1alpha2::PodSandboxConfig& _internal_sandbox_config() const;
  ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL _internal_mutable_sandbox_config();

  public:
  // @@protoc_insertion_point(class_scope:runtime.v1alpha2.CreateContainerRequest)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   2, 62,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const CreateContainerRequest& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pod_sandbox_id_;
    ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE config_;
    ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE sandbox_config_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_cri_2dv1alpha2_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull CreateContainerRequest_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// VersionRequest

// string version = 1;
inline void VersionRequest::clear_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& VersionRequest::version() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.VersionRequest.version)
  return _internal_version();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void VersionRequest::set_version(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.version_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.VersionRequest.version)
}
inline ::std::string* PROTOBUF_NONNULL VersionRequest::mutable_version()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_version();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.VersionRequest.version)
  return _s;
}
inline const ::std::string& VersionRequest::_internal_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_.Get();
}
inline void VersionRequest::_internal_set_version(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL VersionRequest::_internal_mutable_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.version_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE VersionRequest::release_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.VersionRequest.version)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.version_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.version_.Set("", GetArena());
  }
  return released;
}
inline void VersionRequest::set_allocated_version(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.version_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.version_.IsDefault()) {
    _impl_.version_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.VersionRequest.version)
}

// -------------------------------------------------------------------

// VersionResponse

// string version = 1;
inline void VersionResponse::clear_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& VersionResponse::version() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.VersionResponse.version)
  return _internal_version();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void VersionResponse::set_version(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.version_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.VersionResponse.version)
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::mutable_version()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_version();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.VersionResponse.version)
  return _s;
}
inline const ::std::string& VersionResponse::_internal_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_.Get();
}
inline void VersionResponse::_internal_set_version(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::_internal_mutable_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.version_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE VersionResponse::release_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.VersionResponse.version)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.version_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.version_.Set("", GetArena());
  }
  return released;
}
inline void VersionResponse::set_allocated_version(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.version_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.version_.IsDefault()) {
    _impl_.version_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.VersionResponse.version)
}

// string runtime_name = 2;
inline void VersionResponse::clear_runtime_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& VersionResponse::runtime_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.VersionResponse.runtime_name)
  return _internal_runtime_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void VersionResponse::set_runtime_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.runtime_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.VersionResponse.runtime_name)
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::mutable_runtime_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_runtime_name();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.VersionResponse.runtime_name)
  return _s;
}
inline const ::std::string& VersionResponse::_internal_runtime_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.runtime_name_.Get();
}
inline void VersionResponse::_internal_set_runtime_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::_internal_mutable_runtime_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.runtime_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE VersionResponse::release_runtime_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.VersionResponse.runtime_name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.runtime_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.runtime_name_.Set("", GetArena());
  }
  return released;
}
inline void VersionResponse::set_allocated_runtime_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.runtime_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.runtime_name_.IsDefault()) {
    _impl_.runtime_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.VersionResponse.runtime_name)
}

// string runtime_version = 3;
inline void VersionResponse::clear_runtime_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_version_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& VersionResponse::runtime_version() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.VersionResponse.runtime_version)
  return _internal_runtime_version();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void VersionResponse::set_runtime_version(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.runtime_version_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.VersionResponse.runtime_version)
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::mutable_runtime_version()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_runtime_version();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.VersionResponse.runtime_version)
  return _s;
}
inline const ::std::string& VersionResponse::_internal_runtime_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.runtime_version_.Get();
}
inline void VersionResponse::_internal_set_runtime_version(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_version_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::_internal_mutable_runtime_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.runtime_version_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE VersionResponse::release_runtime_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.VersionResponse.runtime_version)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.runtime_version_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.runtime_version_.Set("", GetArena());
  }
  return released;
}
inline void VersionResponse::set_allocated_runtime_version(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.runtime_version_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.runtime_version_.IsDefault()) {
    _impl_.runtime_version_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.VersionResponse.runtime_version)
}

// string runtime_api_version = 4;
inline void VersionResponse::clear_runtime_api_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_api_version_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::std::string& VersionResponse::runtime_api_version() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.VersionResponse.runtime_api_version)
  return _internal_runtime_api_version();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void VersionResponse::set_runtime_api_version(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.runtime_api_version_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.VersionResponse.runtime_api_version)
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::mutable_runtime_api_version()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_runtime_api_version();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.VersionResponse.runtime_api_version)
  return _s;
}
inline const ::std::string& VersionResponse::_internal_runtime_api_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.runtime_api_version_.Get();
}
inline void VersionResponse::_internal_set_runtime_api_version(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_api_version_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL VersionResponse::_internal_mutable_runtime_api_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.runtime_api_version_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE VersionResponse::release_runtime_api_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.VersionResponse.runtime_api_version)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.runtime_api_version_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.runtime_api_version_.Set("", GetArena());
  }
  return released;
}
inline void VersionResponse::set_allocated_runtime_api_version(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.runtime_api_version_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.runtime_api_version_.IsDefault()) {
    _impl_.runtime_api_version_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.VersionResponse.runtime_api_version)
}

// -------------------------------------------------------------------

// DNSConfig

// repeated string servers = 1;
inline int DNSConfig::_internal_servers_size() const {
  return _internal_servers().size();
}
inline int DNSConfig::servers_size() const {
  return _internal_servers_size();
}
inline void DNSConfig::clear_servers() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.servers_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL DNSConfig::add_servers()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_servers()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.DNSConfig.servers)
  return _s;
}
inline const ::std::string& DNSConfig::servers(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.DNSConfig.servers)
  return _internal_servers().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL DNSConfig::mutable_servers(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.DNSConfig.servers)
  return _internal_mutable_servers()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void DNSConfig::set_servers(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_servers()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.DNSConfig.servers)
}
template <typename Arg_, typename... Args_>
inline void DNSConfig::add_servers(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_servers(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.DNSConfig.servers)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& DNSConfig::servers()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.DNSConfig.servers)
  return _internal_servers();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
DNSConfig::mutable_servers() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.DNSConfig.servers)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_servers();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
DNSConfig::_internal_servers() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.servers_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
DNSConfig::_internal_mutable_servers() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.servers_;
}

// repeated string searches = 2;
inline int DNSConfig::_internal_searches_size() const {
  return _internal_searches().size();
}
inline int DNSConfig::searches_size() const {
  return _internal_searches_size();
}
inline void DNSConfig::clear_searches() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.searches_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL DNSConfig::add_searches()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_searches()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.DNSConfig.searches)
  return _s;
}
inline const ::std::string& DNSConfig::searches(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.DNSConfig.searches)
  return _internal_searches().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL DNSConfig::mutable_searches(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.DNSConfig.searches)
  return _internal_mutable_searches()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void DNSConfig::set_searches(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_searches()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.DNSConfig.searches)
}
template <typename Arg_, typename... Args_>
inline void DNSConfig::add_searches(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_searches(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.DNSConfig.searches)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& DNSConfig::searches()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.DNSConfig.searches)
  return _internal_searches();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
DNSConfig::mutable_searches() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.DNSConfig.searches)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_searches();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
DNSConfig::_internal_searches() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.searches_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
DNSConfig::_internal_mutable_searches() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.searches_;
}

// repeated string options = 3;
inline int DNSConfig::_internal_options_size() const {
  return _internal_options().size();
}
inline int DNSConfig::options_size() const {
  return _internal_options_size();
}
inline void DNSConfig::clear_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.options_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::std::string* PROTOBUF_NONNULL DNSConfig::add_options()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_options()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.DNSConfig.options)
  return _s;
}
inline const ::std::string& DNSConfig::options(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.DNSConfig.options)
  return _internal_options().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL DNSConfig::mutable_options(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.DNSConfig.options)
  return _internal_mutable_options()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void DNSConfig::set_options(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_options()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.DNSConfig.options)
}
template <typename Arg_, typename... Args_>
inline void DNSConfig::add_options(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_options(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.DNSConfig.options)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& DNSConfig::options()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.DNSConfig.options)
  return _internal_options();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
DNSConfig::mutable_options() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.DNSConfig.options)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_options();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
DNSConfig::_internal_options() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.options_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
DNSConfig::_internal_mutable_options() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.options_;
}

// -------------------------------------------------------------------

// PortMapping

// .runtime.v1alpha2.Protocol protocol = 1;
inline void PortMapping::clear_protocol() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.protocol_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::runtime::v1alpha2::Protocol PortMapping::protocol() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortMapping.protocol)
  return _internal_protocol();
}
inline void PortMapping::set_protocol(::runtime::v1alpha2::Protocol value) {
  _internal_set_protocol(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortMapping.protocol)
}
inline ::runtime::v1alpha2::Protocol PortMapping::_internal_protocol() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::Protocol>(_impl_.protocol_);
}
inline void PortMapping::_internal_set_protocol(::runtime::v1alpha2::Protocol value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.protocol_ = value;
}

// int32 container_port = 2;
inline void PortMapping::clear_container_port() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_port_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::int32_t PortMapping::container_port() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortMapping.container_port)
  return _internal_container_port();
}
inline void PortMapping::set_container_port(::int32_t value) {
  _internal_set_container_port(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortMapping.container_port)
}
inline ::int32_t PortMapping::_internal_container_port() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_port_;
}
inline void PortMapping::_internal_set_container_port(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_port_ = value;
}

// int32 host_port = 3;
inline void PortMapping::clear_host_port() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_port_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::int32_t PortMapping::host_port() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortMapping.host_port)
  return _internal_host_port();
}
inline void PortMapping::set_host_port(::int32_t value) {
  _internal_set_host_port(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortMapping.host_port)
}
inline ::int32_t PortMapping::_internal_host_port() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.host_port_;
}
inline void PortMapping::_internal_set_host_port(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_port_ = value;
}

// string host_ip = 4;
inline void PortMapping::clear_host_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_ip_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PortMapping::host_ip() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortMapping.host_ip)
  return _internal_host_ip();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PortMapping::set_host_ip(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.host_ip_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortMapping.host_ip)
}
inline ::std::string* PROTOBUF_NONNULL PortMapping::mutable_host_ip()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_host_ip();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PortMapping.host_ip)
  return _s;
}
inline const ::std::string& PortMapping::_internal_host_ip() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.host_ip_.Get();
}
inline void PortMapping::_internal_set_host_ip(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_ip_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PortMapping::_internal_mutable_host_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.host_ip_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PortMapping::release_host_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PortMapping.host_ip)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.host_ip_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.host_ip_.Set("", GetArena());
  }
  return released;
}
inline void PortMapping::set_allocated_host_ip(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.host_ip_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.host_ip_.IsDefault()) {
    _impl_.host_ip_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PortMapping.host_ip)
}

// -------------------------------------------------------------------

// Mount

// string container_path = 1;
inline void Mount::clear_container_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& Mount::container_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Mount.container_path)
  return _internal_container_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Mount::set_container_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Mount.container_path)
}
inline ::std::string* PROTOBUF_NONNULL Mount::mutable_container_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Mount.container_path)
  return _s;
}
inline const ::std::string& Mount::_internal_container_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_path_.Get();
}
inline void Mount::_internal_set_container_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Mount::_internal_mutable_container_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Mount::release_container_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Mount.container_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_path_.Set("", GetArena());
  }
  return released;
}
inline void Mount::set_allocated_container_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_path_.IsDefault()) {
    _impl_.container_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Mount.container_path)
}

// string host_path = 2;
inline void Mount::clear_host_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& Mount::host_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Mount.host_path)
  return _internal_host_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Mount::set_host_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.host_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Mount.host_path)
}
inline ::std::string* PROTOBUF_NONNULL Mount::mutable_host_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_host_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Mount.host_path)
  return _s;
}
inline const ::std::string& Mount::_internal_host_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.host_path_.Get();
}
inline void Mount::_internal_set_host_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Mount::_internal_mutable_host_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.host_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Mount::release_host_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Mount.host_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.host_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.host_path_.Set("", GetArena());
  }
  return released;
}
inline void Mount::set_allocated_host_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.host_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.host_path_.IsDefault()) {
    _impl_.host_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Mount.host_path)
}

// bool readonly = 3;
inline void Mount::clear_readonly() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline bool Mount::readonly() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Mount.readonly)
  return _internal_readonly();
}
inline void Mount::set_readonly(bool value) {
  _internal_set_readonly(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Mount.readonly)
}
inline bool Mount::_internal_readonly() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.readonly_;
}
inline void Mount::_internal_set_readonly(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_ = value;
}

// bool selinux_relabel = 4;
inline void Mount::clear_selinux_relabel() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.selinux_relabel_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline bool Mount::selinux_relabel() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Mount.selinux_relabel)
  return _internal_selinux_relabel();
}
inline void Mount::set_selinux_relabel(bool value) {
  _internal_set_selinux_relabel(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Mount.selinux_relabel)
}
inline bool Mount::_internal_selinux_relabel() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.selinux_relabel_;
}
inline void Mount::_internal_set_selinux_relabel(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.selinux_relabel_ = value;
}

// .runtime.v1alpha2.MountPropagation propagation = 5;
inline void Mount::clear_propagation() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.propagation_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::runtime::v1alpha2::MountPropagation Mount::propagation() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Mount.propagation)
  return _internal_propagation();
}
inline void Mount::set_propagation(::runtime::v1alpha2::MountPropagation value) {
  _internal_set_propagation(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Mount.propagation)
}
inline ::runtime::v1alpha2::MountPropagation Mount::_internal_propagation() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::MountPropagation>(_impl_.propagation_);
}
inline void Mount::_internal_set_propagation(::runtime::v1alpha2::MountPropagation value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.propagation_ = value;
}

// -------------------------------------------------------------------

// NamespaceOption

// .runtime.v1alpha2.NamespaceMode network = 1;
inline void NamespaceOption::clear_network() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.network_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::runtime::v1alpha2::NamespaceMode NamespaceOption::network() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.NamespaceOption.network)
  return _internal_network();
}
inline void NamespaceOption::set_network(::runtime::v1alpha2::NamespaceMode value) {
  _internal_set_network(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.NamespaceOption.network)
}
inline ::runtime::v1alpha2::NamespaceMode NamespaceOption::_internal_network() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::NamespaceMode>(_impl_.network_);
}
inline void NamespaceOption::_internal_set_network(::runtime::v1alpha2::NamespaceMode value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.network_ = value;
}

// .runtime.v1alpha2.NamespaceMode pid = 2;
inline void NamespaceOption::clear_pid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pid_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::runtime::v1alpha2::NamespaceMode NamespaceOption::pid() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.NamespaceOption.pid)
  return _internal_pid();
}
inline void NamespaceOption::set_pid(::runtime::v1alpha2::NamespaceMode value) {
  _internal_set_pid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.NamespaceOption.pid)
}
inline ::runtime::v1alpha2::NamespaceMode NamespaceOption::_internal_pid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::NamespaceMode>(_impl_.pid_);
}
inline void NamespaceOption::_internal_set_pid(::runtime::v1alpha2::NamespaceMode value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pid_ = value;
}

// .runtime.v1alpha2.NamespaceMode ipc = 3;
inline void NamespaceOption::clear_ipc() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ipc_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::runtime::v1alpha2::NamespaceMode NamespaceOption::ipc() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.NamespaceOption.ipc)
  return _internal_ipc();
}
inline void NamespaceOption::set_ipc(::runtime::v1alpha2::NamespaceMode value) {
  _internal_set_ipc(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.NamespaceOption.ipc)
}
inline ::runtime::v1alpha2::NamespaceMode NamespaceOption::_internal_ipc() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::NamespaceMode>(_impl_.ipc_);
}
inline void NamespaceOption::_internal_set_ipc(::runtime::v1alpha2::NamespaceMode value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ipc_ = value;
}

// string target_id = 4;
inline void NamespaceOption::clear_target_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.target_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& NamespaceOption::target_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.NamespaceOption.target_id)
  return _internal_target_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void NamespaceOption::set_target_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.target_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.NamespaceOption.target_id)
}
inline ::std::string* PROTOBUF_NONNULL NamespaceOption::mutable_target_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_target_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.NamespaceOption.target_id)
  return _s;
}
inline const ::std::string& NamespaceOption::_internal_target_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.target_id_.Get();
}
inline void NamespaceOption::_internal_set_target_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.target_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL NamespaceOption::_internal_mutable_target_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.target_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE NamespaceOption::release_target_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.NamespaceOption.target_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.target_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.target_id_.Set("", GetArena());
  }
  return released;
}
inline void NamespaceOption::set_allocated_target_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.target_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.target_id_.IsDefault()) {
    _impl_.target_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.NamespaceOption.target_id)
}

// -------------------------------------------------------------------

// Int64Value

// int64 value = 1;
inline void Int64Value::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::int64_t Int64Value::value() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Int64Value.value)
  return _internal_value();
}
inline void Int64Value::set_value(::int64_t value) {
  _internal_set_value(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Int64Value.value)
}
inline ::int64_t Int64Value::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_;
}
inline void Int64Value::_internal_set_value(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_ = value;
}

// -------------------------------------------------------------------

// LinuxSandboxSecurityContext

// .runtime.v1alpha2.NamespaceOption namespace_options = 1;
inline bool LinuxSandboxSecurityContext::has_namespace_options() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.namespace_options_ != nullptr);
  return value;
}
inline void LinuxSandboxSecurityContext::clear_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.namespace_options_ != nullptr) _impl_.namespace_options_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::NamespaceOption& LinuxSandboxSecurityContext::_internal_namespace_options() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::NamespaceOption* p = _impl_.namespace_options_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::NamespaceOption&>(::runtime::v1alpha2::_NamespaceOption_default_instance_);
}
inline const ::runtime::v1alpha2::NamespaceOption& LinuxSandboxSecurityContext::namespace_options() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.namespace_options)
  return _internal_namespace_options();
}
inline void LinuxSandboxSecurityContext::unsafe_arena_set_allocated_namespace_options(
    ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.namespace_options_);
  }
  _impl_.namespace_options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.namespace_options)
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::NamespaceOption* released = _impl_.namespace_options_;
  _impl_.namespace_options_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::unsafe_arena_release_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.namespace_options)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::NamespaceOption* temp = _impl_.namespace_options_;
  _impl_.namespace_options_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.namespace_options_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::NamespaceOption>(GetArena());
    _impl_.namespace_options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(p);
  }
  return _impl_.namespace_options_;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_namespace_options()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::NamespaceOption* _msg = _internal_mutable_namespace_options();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.namespace_options)
  return _msg;
}
inline void LinuxSandboxSecurityContext::set_allocated_namespace_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.namespace_options_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.namespace_options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.namespace_options)
}

// .runtime.v1alpha2.SELinuxOption selinux_options = 2;
inline bool LinuxSandboxSecurityContext::has_selinux_options() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.selinux_options_ != nullptr);
  return value;
}
inline void LinuxSandboxSecurityContext::clear_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.selinux_options_ != nullptr) _impl_.selinux_options_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::runtime::v1alpha2::SELinuxOption& LinuxSandboxSecurityContext::_internal_selinux_options() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::SELinuxOption* p = _impl_.selinux_options_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::SELinuxOption&>(::runtime::v1alpha2::_SELinuxOption_default_instance_);
}
inline const ::runtime::v1alpha2::SELinuxOption& LinuxSandboxSecurityContext::selinux_options() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.selinux_options)
  return _internal_selinux_options();
}
inline void LinuxSandboxSecurityContext::unsafe_arena_set_allocated_selinux_options(
    ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.selinux_options_);
  }
  _impl_.selinux_options_ = reinterpret_cast<::runtime::v1alpha2::SELinuxOption*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.selinux_options)
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::SELinuxOption* released = _impl_.selinux_options_;
  _impl_.selinux_options_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::unsafe_arena_release_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.selinux_options)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::SELinuxOption* temp = _impl_.selinux_options_;
  _impl_.selinux_options_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.selinux_options_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::SELinuxOption>(GetArena());
    _impl_.selinux_options_ = reinterpret_cast<::runtime::v1alpha2::SELinuxOption*>(p);
  }
  return _impl_.selinux_options_;
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_selinux_options()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::SELinuxOption* _msg = _internal_mutable_selinux_options();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.selinux_options)
  return _msg;
}
inline void LinuxSandboxSecurityContext::set_allocated_selinux_options(::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.selinux_options_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.selinux_options_ = reinterpret_cast<::runtime::v1alpha2::SELinuxOption*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.selinux_options)
}

// .runtime.v1alpha2.Int64Value run_as_user = 3;
inline bool LinuxSandboxSecurityContext::has_run_as_user() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.run_as_user_ != nullptr);
  return value;
}
inline void LinuxSandboxSecurityContext::clear_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_user_ != nullptr) _impl_.run_as_user_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxSandboxSecurityContext::_internal_run_as_user() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Int64Value* p = _impl_.run_as_user_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Int64Value&>(::runtime::v1alpha2::_Int64Value_default_instance_);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxSandboxSecurityContext::run_as_user() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_user)
  return _internal_run_as_user();
}
inline void LinuxSandboxSecurityContext::unsafe_arena_set_allocated_run_as_user(
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_user_);
  }
  _impl_.run_as_user_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_user)
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::Int64Value* released = _impl_.run_as_user_;
  _impl_.run_as_user_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::unsafe_arena_release_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_user)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::Int64Value* temp = _impl_.run_as_user_;
  _impl_.run_as_user_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_user_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Int64Value>(GetArena());
    _impl_.run_as_user_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(p);
  }
  return _impl_.run_as_user_;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_run_as_user()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::Int64Value* _msg = _internal_mutable_run_as_user();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_user)
  return _msg;
}
inline void LinuxSandboxSecurityContext::set_allocated_run_as_user(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_user_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.run_as_user_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_user)
}

// .runtime.v1alpha2.Int64Value run_as_group = 8;
inline bool LinuxSandboxSecurityContext::has_run_as_group() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  PROTOBUF_ASSUME(!value || _impl_.run_as_group_ != nullptr);
  return value;
}
inline void LinuxSandboxSecurityContext::clear_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_group_ != nullptr) _impl_.run_as_group_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxSandboxSecurityContext::_internal_run_as_group() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Int64Value* p = _impl_.run_as_group_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Int64Value&>(::runtime::v1alpha2::_Int64Value_default_instance_);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxSandboxSecurityContext::run_as_group() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_group)
  return _internal_run_as_group();
}
inline void LinuxSandboxSecurityContext::unsafe_arena_set_allocated_run_as_group(
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_group_);
  }
  _impl_.run_as_group_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_group)
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::Int64Value* released = _impl_.run_as_group_;
  _impl_.run_as_group_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::unsafe_arena_release_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_group)

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::Int64Value* temp = _impl_.run_as_group_;
  _impl_.run_as_group_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_group_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Int64Value>(GetArena());
    _impl_.run_as_group_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(p);
  }
  return _impl_.run_as_group_;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_run_as_group()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::Int64Value* _msg = _internal_mutable_run_as_group();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_group)
  return _msg;
}
inline void LinuxSandboxSecurityContext::set_allocated_run_as_group(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_group_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }

  _impl_.run_as_group_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_group)
}

// bool readonly_rootfs = 4;
inline void LinuxSandboxSecurityContext::clear_readonly_rootfs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_rootfs_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000100U);
}
inline bool LinuxSandboxSecurityContext::readonly_rootfs() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.readonly_rootfs)
  return _internal_readonly_rootfs();
}
inline void LinuxSandboxSecurityContext::set_readonly_rootfs(bool value) {
  _internal_set_readonly_rootfs(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxSandboxSecurityContext.readonly_rootfs)
}
inline bool LinuxSandboxSecurityContext::_internal_readonly_rootfs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.readonly_rootfs_;
}
inline void LinuxSandboxSecurityContext::_internal_set_readonly_rootfs(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_rootfs_ = value;
}

// repeated int64 supplemental_groups = 5;
inline int LinuxSandboxSecurityContext::_internal_supplemental_groups_size() const {
  return _internal_supplemental_groups().size();
}
inline int LinuxSandboxSecurityContext::supplemental_groups_size() const {
  return _internal_supplemental_groups_size();
}
inline void LinuxSandboxSecurityContext::clear_supplemental_groups() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.supplemental_groups_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::int64_t LinuxSandboxSecurityContext::supplemental_groups(int index) const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.supplemental_groups)
  return _internal_supplemental_groups().Get(index);
}
inline void LinuxSandboxSecurityContext::set_supplemental_groups(int index, ::int64_t value) {
  _internal_mutable_supplemental_groups()->Set(index, value);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxSandboxSecurityContext.supplemental_groups)
}
inline void LinuxSandboxSecurityContext::add_supplemental_groups(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_supplemental_groups()->Add(value);
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.LinuxSandboxSecurityContext.supplemental_groups)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& LinuxSandboxSecurityContext::supplemental_groups() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.LinuxSandboxSecurityContext.supplemental_groups)
  return _internal_supplemental_groups();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_supplemental_groups()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.LinuxSandboxSecurityContext.supplemental_groups)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_supplemental_groups();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
LinuxSandboxSecurityContext::_internal_supplemental_groups() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.supplemental_groups_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
LinuxSandboxSecurityContext::_internal_mutable_supplemental_groups() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.supplemental_groups_;
}

// bool privileged = 6;
inline void LinuxSandboxSecurityContext::clear_privileged() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.privileged_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000200U);
}
inline bool LinuxSandboxSecurityContext::privileged() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.privileged)
  return _internal_privileged();
}
inline void LinuxSandboxSecurityContext::set_privileged(bool value) {
  _internal_set_privileged(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxSandboxSecurityContext.privileged)
}
inline bool LinuxSandboxSecurityContext::_internal_privileged() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.privileged_;
}
inline void LinuxSandboxSecurityContext::_internal_set_privileged(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.privileged_ = value;
}

// .runtime.v1alpha2.SecurityProfile seccomp = 9;
inline bool LinuxSandboxSecurityContext::has_seccomp() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  PROTOBUF_ASSUME(!value || _impl_.seccomp_ != nullptr);
  return value;
}
inline void LinuxSandboxSecurityContext::clear_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.seccomp_ != nullptr) _impl_.seccomp_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxSandboxSecurityContext::_internal_seccomp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::SecurityProfile* p = _impl_.seccomp_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::SecurityProfile&>(::runtime::v1alpha2::_SecurityProfile_default_instance_);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxSandboxSecurityContext::seccomp() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp)
  return _internal_seccomp();
}
inline void LinuxSandboxSecurityContext::unsafe_arena_set_allocated_seccomp(
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.seccomp_);
  }
  _impl_.seccomp_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp)
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::SecurityProfile* released = _impl_.seccomp_;
  _impl_.seccomp_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::unsafe_arena_release_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp)

  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::SecurityProfile* temp = _impl_.seccomp_;
  _impl_.seccomp_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.seccomp_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::SecurityProfile>(GetArena());
    _impl_.seccomp_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(p);
  }
  return _impl_.seccomp_;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_seccomp()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::SecurityProfile* _msg = _internal_mutable_seccomp();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp)
  return _msg;
}
inline void LinuxSandboxSecurityContext::set_allocated_seccomp(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.seccomp_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }

  _impl_.seccomp_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp)
}

// .runtime.v1alpha2.SecurityProfile apparmor = 10;
inline bool LinuxSandboxSecurityContext::has_apparmor() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  PROTOBUF_ASSUME(!value || _impl_.apparmor_ != nullptr);
  return value;
}
inline void LinuxSandboxSecurityContext::clear_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.apparmor_ != nullptr) _impl_.apparmor_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxSandboxSecurityContext::_internal_apparmor() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::SecurityProfile* p = _impl_.apparmor_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::SecurityProfile&>(::runtime::v1alpha2::_SecurityProfile_default_instance_);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxSandboxSecurityContext::apparmor() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.apparmor)
  return _internal_apparmor();
}
inline void LinuxSandboxSecurityContext::unsafe_arena_set_allocated_apparmor(
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.apparmor_);
  }
  _impl_.apparmor_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.apparmor)
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::SecurityProfile* released = _impl_.apparmor_;
  _impl_.apparmor_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::unsafe_arena_release_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.apparmor)

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::SecurityProfile* temp = _impl_.apparmor_;
  _impl_.apparmor_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.apparmor_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::SecurityProfile>(GetArena());
    _impl_.apparmor_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(p);
  }
  return _impl_.apparmor_;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_apparmor()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::SecurityProfile* _msg = _internal_mutable_apparmor();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.apparmor)
  return _msg;
}
inline void LinuxSandboxSecurityContext::set_allocated_apparmor(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.apparmor_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }

  _impl_.apparmor_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.apparmor)
}

// string seccomp_profile_path = 7;
inline void LinuxSandboxSecurityContext::clear_seccomp_profile_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.seccomp_profile_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& LinuxSandboxSecurityContext::seccomp_profile_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp_profile_path)
  return _internal_seccomp_profile_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxSandboxSecurityContext::set_seccomp_profile_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.seccomp_profile_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp_profile_path)
}
inline ::std::string* PROTOBUF_NONNULL LinuxSandboxSecurityContext::mutable_seccomp_profile_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_seccomp_profile_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp_profile_path)
  return _s;
}
inline const ::std::string& LinuxSandboxSecurityContext::_internal_seccomp_profile_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.seccomp_profile_path_.Get();
}
inline void LinuxSandboxSecurityContext::_internal_set_seccomp_profile_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.seccomp_profile_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxSandboxSecurityContext::_internal_mutable_seccomp_profile_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.seccomp_profile_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxSandboxSecurityContext::release_seccomp_profile_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp_profile_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.seccomp_profile_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.seccomp_profile_path_.Set("", GetArena());
  }
  return released;
}
inline void LinuxSandboxSecurityContext::set_allocated_seccomp_profile_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.seccomp_profile_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.seccomp_profile_path_.IsDefault()) {
    _impl_.seccomp_profile_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp_profile_path)
}

// -------------------------------------------------------------------

// SecurityProfile

// .runtime.v1alpha2.SecurityProfile.ProfileType profile_type = 1;
inline void SecurityProfile::clear_profile_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.profile_type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::runtime::v1alpha2::SecurityProfile_ProfileType SecurityProfile::profile_type() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.SecurityProfile.profile_type)
  return _internal_profile_type();
}
inline void SecurityProfile::set_profile_type(::runtime::v1alpha2::SecurityProfile_ProfileType value) {
  _internal_set_profile_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.SecurityProfile.profile_type)
}
inline ::runtime::v1alpha2::SecurityProfile_ProfileType SecurityProfile::_internal_profile_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::SecurityProfile_ProfileType>(_impl_.profile_type_);
}
inline void SecurityProfile::_internal_set_profile_type(::runtime::v1alpha2::SecurityProfile_ProfileType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.profile_type_ = value;
}

// string localhost_ref = 2;
inline void SecurityProfile::clear_localhost_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.localhost_ref_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& SecurityProfile::localhost_ref() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.SecurityProfile.localhost_ref)
  return _internal_localhost_ref();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SecurityProfile::set_localhost_ref(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.localhost_ref_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.SecurityProfile.localhost_ref)
}
inline ::std::string* PROTOBUF_NONNULL SecurityProfile::mutable_localhost_ref()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_localhost_ref();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.SecurityProfile.localhost_ref)
  return _s;
}
inline const ::std::string& SecurityProfile::_internal_localhost_ref() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.localhost_ref_.Get();
}
inline void SecurityProfile::_internal_set_localhost_ref(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.localhost_ref_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SecurityProfile::_internal_mutable_localhost_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.localhost_ref_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SecurityProfile::release_localhost_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.SecurityProfile.localhost_ref)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.localhost_ref_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.localhost_ref_.Set("", GetArena());
  }
  return released;
}
inline void SecurityProfile::set_allocated_localhost_ref(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.localhost_ref_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.localhost_ref_.IsDefault()) {
    _impl_.localhost_ref_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.SecurityProfile.localhost_ref)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// LinuxPodSandboxConfig

// string cgroup_parent = 1;
inline void LinuxPodSandboxConfig::clear_cgroup_parent() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cgroup_parent_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& LinuxPodSandboxConfig::cgroup_parent() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxPodSandboxConfig.cgroup_parent)
  return _internal_cgroup_parent();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxPodSandboxConfig::set_cgroup_parent(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.cgroup_parent_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxPodSandboxConfig.cgroup_parent)
}
inline ::std::string* PROTOBUF_NONNULL LinuxPodSandboxConfig::mutable_cgroup_parent()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_cgroup_parent();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxPodSandboxConfig.cgroup_parent)
  return _s;
}
inline const ::std::string& LinuxPodSandboxConfig::_internal_cgroup_parent() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cgroup_parent_.Get();
}
inline void LinuxPodSandboxConfig::_internal_set_cgroup_parent(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cgroup_parent_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxPodSandboxConfig::_internal_mutable_cgroup_parent() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.cgroup_parent_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxPodSandboxConfig::release_cgroup_parent() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxPodSandboxConfig.cgroup_parent)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.cgroup_parent_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.cgroup_parent_.Set("", GetArena());
  }
  return released;
}
inline void LinuxPodSandboxConfig::set_allocated_cgroup_parent(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.cgroup_parent_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.cgroup_parent_.IsDefault()) {
    _impl_.cgroup_parent_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxPodSandboxConfig.cgroup_parent)
}

// .runtime.v1alpha2.LinuxSandboxSecurityContext security_context = 2;
inline bool LinuxPodSandboxConfig::has_security_context() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.security_context_ != nullptr);
  return value;
}
inline void LinuxPodSandboxConfig::clear_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.security_context_ != nullptr) _impl_.security_context_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::LinuxSandboxSecurityContext& LinuxPodSandboxConfig::_internal_security_context() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxSandboxSecurityContext* p = _impl_.security_context_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxSandboxSecurityContext&>(::runtime::v1alpha2::_LinuxSandboxSecurityContext_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxSandboxSecurityContext& LinuxPodSandboxConfig::security_context() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxPodSandboxConfig.security_context)
  return _internal_security_context();
}
inline void LinuxPodSandboxConfig::unsafe_arena_set_allocated_security_context(
    ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.security_context_);
  }
  _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::LinuxSandboxSecurityContext*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxPodSandboxConfig.security_context)
}
inline ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE LinuxPodSandboxConfig::release_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxSandboxSecurityContext* released = _impl_.security_context_;
  _impl_.security_context_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE LinuxPodSandboxConfig::unsafe_arena_release_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxPodSandboxConfig.security_context)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxSandboxSecurityContext* temp = _impl_.security_context_;
  _impl_.security_context_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NONNULL LinuxPodSandboxConfig::_internal_mutable_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.security_context_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxSandboxSecurityContext>(GetArena());
    _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::LinuxSandboxSecurityContext*>(p);
  }
  return _impl_.security_context_;
}
inline ::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NONNULL LinuxPodSandboxConfig::mutable_security_context()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxSandboxSecurityContext* _msg = _internal_mutable_security_context();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxPodSandboxConfig.security_context)
  return _msg;
}
inline void LinuxPodSandboxConfig::set_allocated_security_context(::runtime::v1alpha2::LinuxSandboxSecurityContext* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.security_context_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::LinuxSandboxSecurityContext*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxPodSandboxConfig.security_context)
}

// map<string, string> sysctls = 3;
inline int LinuxPodSandboxConfig::_internal_sysctls_size() const {
  return _internal_sysctls().size();
}
inline int LinuxPodSandboxConfig::sysctls_size() const {
  return _internal_sysctls_size();
}
inline void LinuxPodSandboxConfig::clear_sysctls() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysctls_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& LinuxPodSandboxConfig::_internal_sysctls() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysctls_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& LinuxPodSandboxConfig::sysctls() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.LinuxPodSandboxConfig.sysctls)
  return _internal_sysctls();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL LinuxPodSandboxConfig::_internal_mutable_sysctls() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.sysctls_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL LinuxPodSandboxConfig::mutable_sysctls()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.LinuxPodSandboxConfig.sysctls)
  return _internal_mutable_sysctls();
}

// -------------------------------------------------------------------

// PodSandboxMetadata

// string name = 1;
inline void PodSandboxMetadata::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PodSandboxMetadata::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxMetadata.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxMetadata::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxMetadata.name)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxMetadata::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxMetadata.name)
  return _s;
}
inline const ::std::string& PodSandboxMetadata::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void PodSandboxMetadata::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxMetadata::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxMetadata::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxMetadata.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxMetadata::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxMetadata.name)
}

// string uid = 2;
inline void PodSandboxMetadata::clear_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.uid_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& PodSandboxMetadata::uid() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxMetadata.uid)
  return _internal_uid();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxMetadata::set_uid(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.uid_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxMetadata.uid)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxMetadata::mutable_uid()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_uid();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxMetadata.uid)
  return _s;
}
inline const ::std::string& PodSandboxMetadata::_internal_uid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.uid_.Get();
}
inline void PodSandboxMetadata::_internal_set_uid(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.uid_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxMetadata::_internal_mutable_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.uid_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxMetadata::release_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxMetadata.uid)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.uid_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.uid_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxMetadata::set_allocated_uid(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.uid_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.uid_.IsDefault()) {
    _impl_.uid_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxMetadata.uid)
}

// string namespace = 3;
inline void PodSandboxMetadata::clear_namespace_() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.namespace__.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& PodSandboxMetadata::namespace_() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxMetadata.namespace)
  return _internal_namespace_();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxMetadata::set_namespace_(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.namespace__.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxMetadata.namespace)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxMetadata::mutable_namespace_()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_namespace_();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxMetadata.namespace)
  return _s;
}
inline const ::std::string& PodSandboxMetadata::_internal_namespace_() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.namespace__.Get();
}
inline void PodSandboxMetadata::_internal_set_namespace_(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.namespace__.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxMetadata::_internal_mutable_namespace_() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.namespace__.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxMetadata::release_namespace_() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxMetadata.namespace)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.namespace__.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.namespace__.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxMetadata::set_allocated_namespace_(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.namespace__.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.namespace__.IsDefault()) {
    _impl_.namespace__.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxMetadata.namespace)
}

// uint32 attempt = 4;
inline void PodSandboxMetadata::clear_attempt() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.attempt_ = 0u;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::uint32_t PodSandboxMetadata::attempt() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxMetadata.attempt)
  return _internal_attempt();
}
inline void PodSandboxMetadata::set_attempt(::uint32_t value) {
  _internal_set_attempt(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxMetadata.attempt)
}
inline ::uint32_t PodSandboxMetadata::_internal_attempt() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attempt_;
}
inline void PodSandboxMetadata::_internal_set_attempt(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.attempt_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// PodSandboxConfig

// .runtime.v1alpha2.PodSandboxMetadata metadata = 1;
inline bool PodSandboxConfig::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void PodSandboxConfig::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::runtime::v1alpha2::PodSandboxMetadata& PodSandboxConfig::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxMetadata&>(::runtime::v1alpha2::_PodSandboxMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxMetadata& PodSandboxConfig::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxConfig.metadata)
  return _internal_metadata();
}
inline void PodSandboxConfig::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxConfig.metadata)
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE PodSandboxConfig::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::PodSandboxMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE PodSandboxConfig::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxConfig.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::PodSandboxMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL PodSandboxConfig::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::PodSandboxMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxConfig.metadata)
  return _msg;
}
inline void PodSandboxConfig::set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxConfig.metadata)
}

// string hostname = 2;
inline void PodSandboxConfig::clear_hostname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hostname_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& PodSandboxConfig::hostname() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxConfig.hostname)
  return _internal_hostname();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxConfig::set_hostname(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.hostname_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxConfig.hostname)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxConfig::mutable_hostname()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_hostname();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxConfig.hostname)
  return _s;
}
inline const ::std::string& PodSandboxConfig::_internal_hostname() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.hostname_.Get();
}
inline void PodSandboxConfig::_internal_set_hostname(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hostname_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_hostname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.hostname_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxConfig::release_hostname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxConfig.hostname)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.hostname_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.hostname_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxConfig::set_allocated_hostname(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.hostname_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.hostname_.IsDefault()) {
    _impl_.hostname_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxConfig.hostname)
}

// string log_directory = 3;
inline void PodSandboxConfig::clear_log_directory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_directory_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& PodSandboxConfig::log_directory() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxConfig.log_directory)
  return _internal_log_directory();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxConfig::set_log_directory(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.log_directory_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxConfig.log_directory)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxConfig::mutable_log_directory()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_log_directory();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxConfig.log_directory)
  return _s;
}
inline const ::std::string& PodSandboxConfig::_internal_log_directory() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.log_directory_.Get();
}
inline void PodSandboxConfig::_internal_set_log_directory(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_directory_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_log_directory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.log_directory_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxConfig::release_log_directory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxConfig.log_directory)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.log_directory_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.log_directory_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxConfig::set_allocated_log_directory(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.log_directory_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.log_directory_.IsDefault()) {
    _impl_.log_directory_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxConfig.log_directory)
}

// .runtime.v1alpha2.DNSConfig dns_config = 4;
inline bool PodSandboxConfig::has_dns_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.dns_config_ != nullptr);
  return value;
}
inline void PodSandboxConfig::clear_dns_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.dns_config_ != nullptr) _impl_.dns_config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::runtime::v1alpha2::DNSConfig& PodSandboxConfig::_internal_dns_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::DNSConfig* p = _impl_.dns_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::DNSConfig&>(::runtime::v1alpha2::_DNSConfig_default_instance_);
}
inline const ::runtime::v1alpha2::DNSConfig& PodSandboxConfig::dns_config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxConfig.dns_config)
  return _internal_dns_config();
}
inline void PodSandboxConfig::unsafe_arena_set_allocated_dns_config(
    ::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.dns_config_);
  }
  _impl_.dns_config_ = reinterpret_cast<::runtime::v1alpha2::DNSConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxConfig.dns_config)
}
inline ::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE PodSandboxConfig::release_dns_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::DNSConfig* released = _impl_.dns_config_;
  _impl_.dns_config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE PodSandboxConfig::unsafe_arena_release_dns_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxConfig.dns_config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::DNSConfig* temp = _impl_.dns_config_;
  _impl_.dns_config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::DNSConfig* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_dns_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.dns_config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::DNSConfig>(GetArena());
    _impl_.dns_config_ = reinterpret_cast<::runtime::v1alpha2::DNSConfig*>(p);
  }
  return _impl_.dns_config_;
}
inline ::runtime::v1alpha2::DNSConfig* PROTOBUF_NONNULL PodSandboxConfig::mutable_dns_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::DNSConfig* _msg = _internal_mutable_dns_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxConfig.dns_config)
  return _msg;
}
inline void PodSandboxConfig::set_allocated_dns_config(::runtime::v1alpha2::DNSConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.dns_config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.dns_config_ = reinterpret_cast<::runtime::v1alpha2::DNSConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxConfig.dns_config)
}

// repeated .runtime.v1alpha2.PortMapping port_mappings = 5;
inline int PodSandboxConfig::_internal_port_mappings_size() const {
  return _internal_port_mappings().size();
}
inline int PodSandboxConfig::port_mappings_size() const {
  return _internal_port_mappings_size();
}
inline void PodSandboxConfig::clear_port_mappings() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.port_mappings_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::PortMapping* PROTOBUF_NONNULL PodSandboxConfig::mutable_port_mappings(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxConfig.port_mappings)
  return _internal_mutable_port_mappings()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>* PROTOBUF_NONNULL PodSandboxConfig::mutable_port_mappings()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.PodSandboxConfig.port_mappings)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_port_mappings();
}
inline const ::runtime::v1alpha2::PortMapping& PodSandboxConfig::port_mappings(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxConfig.port_mappings)
  return _internal_port_mappings().Get(index);
}
inline ::runtime::v1alpha2::PortMapping* PROTOBUF_NONNULL PodSandboxConfig::add_port_mappings()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::PortMapping* _add =
      _internal_mutable_port_mappings()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.PodSandboxConfig.port_mappings)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>& PodSandboxConfig::port_mappings() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.PodSandboxConfig.port_mappings)
  return _internal_port_mappings();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>&
PodSandboxConfig::_internal_port_mappings() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.port_mappings_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PortMapping>* PROTOBUF_NONNULL
PodSandboxConfig::_internal_mutable_port_mappings() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.port_mappings_;
}

// map<string, string> labels = 6;
inline int PodSandboxConfig::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int PodSandboxConfig::labels_size() const {
  return _internal_labels_size();
}
inline void PodSandboxConfig::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000040U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxConfig::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxConfig::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandboxConfig.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxConfig::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandboxConfig.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 7;
inline int PodSandboxConfig::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int PodSandboxConfig::annotations_size() const {
  return _internal_annotations_size();
}
inline void PodSandboxConfig::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxConfig::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxConfig::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandboxConfig.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxConfig::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandboxConfig.annotations)
  return _internal_mutable_annotations();
}

// .runtime.v1alpha2.LinuxPodSandboxConfig linux = 8;
inline bool PodSandboxConfig::has_linux() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  PROTOBUF_ASSUME(!value || _impl_.linux_ != nullptr);
  return value;
}
inline void PodSandboxConfig::clear_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ != nullptr) _impl_.linux_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::runtime::v1alpha2::LinuxPodSandboxConfig& PodSandboxConfig::_internal_linux() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxPodSandboxConfig* p = _impl_.linux_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxPodSandboxConfig&>(::runtime::v1alpha2::_LinuxPodSandboxConfig_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxPodSandboxConfig& PodSandboxConfig::linux() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxConfig.linux)
  return _internal_linux();
}
inline void PodSandboxConfig::unsafe_arena_set_allocated_linux(
    ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }
  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxPodSandboxConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxConfig.linux)
}
inline ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE PodSandboxConfig::release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::LinuxPodSandboxConfig* released = _impl_.linux_;
  _impl_.linux_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE PodSandboxConfig::unsafe_arena_release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxConfig.linux)

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::LinuxPodSandboxConfig* temp = _impl_.linux_;
  _impl_.linux_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NONNULL PodSandboxConfig::_internal_mutable_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxPodSandboxConfig>(GetArena());
    _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxPodSandboxConfig*>(p);
  }
  return _impl_.linux_;
}
inline ::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NONNULL PodSandboxConfig::mutable_linux()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::LinuxPodSandboxConfig* _msg = _internal_mutable_linux();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxConfig.linux)
  return _msg;
}
inline void PodSandboxConfig::set_allocated_linux(::runtime::v1alpha2::LinuxPodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }

  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxPodSandboxConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxConfig.linux)
}

// -------------------------------------------------------------------

// RunPodSandboxRequest

// .runtime.v1alpha2.PodSandboxConfig config = 1;
inline bool RunPodSandboxRequest::has_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.config_ != nullptr);
  return value;
}
inline void RunPodSandboxRequest::clear_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.config_ != nullptr) _impl_.config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::PodSandboxConfig& RunPodSandboxRequest::_internal_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxConfig* p = _impl_.config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxConfig&>(::runtime::v1alpha2::_PodSandboxConfig_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxConfig& RunPodSandboxRequest::config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RunPodSandboxRequest.config)
  return _internal_config();
}
inline void RunPodSandboxRequest::unsafe_arena_set_allocated_config(
    ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.config_);
  }
  _impl_.config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.RunPodSandboxRequest.config)
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE RunPodSandboxRequest::release_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::PodSandboxConfig* released = _impl_.config_;
  _impl_.config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE RunPodSandboxRequest::unsafe_arena_release_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RunPodSandboxRequest.config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::PodSandboxConfig* temp = _impl_.config_;
  _impl_.config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL RunPodSandboxRequest::_internal_mutable_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxConfig>(GetArena());
    _impl_.config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(p);
  }
  return _impl_.config_;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL RunPodSandboxRequest::mutable_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::PodSandboxConfig* _msg = _internal_mutable_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RunPodSandboxRequest.config)
  return _msg;
}
inline void RunPodSandboxRequest::set_allocated_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RunPodSandboxRequest.config)
}

// string runtime_handler = 2;
inline void RunPodSandboxRequest::clear_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_handler_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& RunPodSandboxRequest::runtime_handler() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RunPodSandboxRequest.runtime_handler)
  return _internal_runtime_handler();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RunPodSandboxRequest::set_runtime_handler(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.runtime_handler_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RunPodSandboxRequest.runtime_handler)
}
inline ::std::string* PROTOBUF_NONNULL RunPodSandboxRequest::mutable_runtime_handler()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_runtime_handler();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RunPodSandboxRequest.runtime_handler)
  return _s;
}
inline const ::std::string& RunPodSandboxRequest::_internal_runtime_handler() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.runtime_handler_.Get();
}
inline void RunPodSandboxRequest::_internal_set_runtime_handler(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_handler_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RunPodSandboxRequest::_internal_mutable_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.runtime_handler_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RunPodSandboxRequest::release_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RunPodSandboxRequest.runtime_handler)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.runtime_handler_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.runtime_handler_.Set("", GetArena());
  }
  return released;
}
inline void RunPodSandboxRequest::set_allocated_runtime_handler(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.runtime_handler_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.runtime_handler_.IsDefault()) {
    _impl_.runtime_handler_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RunPodSandboxRequest.runtime_handler)
}

// -------------------------------------------------------------------

// RunPodSandboxResponse

// string pod_sandbox_id = 1;
inline void RunPodSandboxResponse::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& RunPodSandboxResponse::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RunPodSandboxResponse.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RunPodSandboxResponse::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RunPodSandboxResponse.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL RunPodSandboxResponse::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RunPodSandboxResponse.pod_sandbox_id)
  return _s;
}
inline const ::std::string& RunPodSandboxResponse::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void RunPodSandboxResponse::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RunPodSandboxResponse::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RunPodSandboxResponse::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RunPodSandboxResponse.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void RunPodSandboxResponse::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RunPodSandboxResponse.pod_sandbox_id)
}

// -------------------------------------------------------------------

// StopPodSandboxRequest

// string pod_sandbox_id = 1;
inline void StopPodSandboxRequest::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& StopPodSandboxRequest::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.StopPodSandboxRequest.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void StopPodSandboxRequest::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.StopPodSandboxRequest.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL StopPodSandboxRequest::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.StopPodSandboxRequest.pod_sandbox_id)
  return _s;
}
inline const ::std::string& StopPodSandboxRequest::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void StopPodSandboxRequest::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL StopPodSandboxRequest::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE StopPodSandboxRequest::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.StopPodSandboxRequest.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void StopPodSandboxRequest::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.StopPodSandboxRequest.pod_sandbox_id)
}

// -------------------------------------------------------------------

// StopPodSandboxResponse

// -------------------------------------------------------------------

// RemovePodSandboxRequest

// string pod_sandbox_id = 1;
inline void RemovePodSandboxRequest::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& RemovePodSandboxRequest::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RemovePodSandboxRequest.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RemovePodSandboxRequest::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RemovePodSandboxRequest.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL RemovePodSandboxRequest::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RemovePodSandboxRequest.pod_sandbox_id)
  return _s;
}
inline const ::std::string& RemovePodSandboxRequest::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void RemovePodSandboxRequest::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RemovePodSandboxRequest::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RemovePodSandboxRequest::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RemovePodSandboxRequest.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void RemovePodSandboxRequest::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RemovePodSandboxRequest.pod_sandbox_id)
}

// -------------------------------------------------------------------

// RemovePodSandboxResponse

// -------------------------------------------------------------------

// PodSandboxStatusRequest

// string pod_sandbox_id = 1;
inline void PodSandboxStatusRequest::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PodSandboxStatusRequest::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatusRequest.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxStatusRequest::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStatusRequest.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxStatusRequest::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatusRequest.pod_sandbox_id)
  return _s;
}
inline const ::std::string& PodSandboxStatusRequest::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void PodSandboxStatusRequest::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxStatusRequest::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxStatusRequest::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatusRequest.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxStatusRequest::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatusRequest.pod_sandbox_id)
}

// bool verbose = 2;
inline void PodSandboxStatusRequest::clear_verbose() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline bool PodSandboxStatusRequest::verbose() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatusRequest.verbose)
  return _internal_verbose();
}
inline void PodSandboxStatusRequest::set_verbose(bool value) {
  _internal_set_verbose(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStatusRequest.verbose)
}
inline bool PodSandboxStatusRequest::_internal_verbose() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.verbose_;
}
inline void PodSandboxStatusRequest::_internal_set_verbose(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = value;
}

// -------------------------------------------------------------------

// PodIP

// string ip = 1;
inline void PodIP::clear_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ip_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PodIP::ip() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodIP.ip)
  return _internal_ip();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodIP::set_ip(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.ip_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodIP.ip)
}
inline ::std::string* PROTOBUF_NONNULL PodIP::mutable_ip()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_ip();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodIP.ip)
  return _s;
}
inline const ::std::string& PodIP::_internal_ip() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.ip_.Get();
}
inline void PodIP::_internal_set_ip(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ip_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodIP::_internal_mutable_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.ip_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodIP::release_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodIP.ip)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.ip_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.ip_.Set("", GetArena());
  }
  return released;
}
inline void PodIP::set_allocated_ip(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.ip_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.ip_.IsDefault()) {
    _impl_.ip_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodIP.ip)
}

// -------------------------------------------------------------------

// PodSandboxNetworkStatus

// string ip = 1;
inline void PodSandboxNetworkStatus::clear_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ip_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& PodSandboxNetworkStatus::ip() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxNetworkStatus.ip)
  return _internal_ip();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxNetworkStatus::set_ip(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.ip_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxNetworkStatus.ip)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxNetworkStatus::mutable_ip()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_ip();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxNetworkStatus.ip)
  return _s;
}
inline const ::std::string& PodSandboxNetworkStatus::_internal_ip() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.ip_.Get();
}
inline void PodSandboxNetworkStatus::_internal_set_ip(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ip_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxNetworkStatus::_internal_mutable_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.ip_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxNetworkStatus::release_ip() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxNetworkStatus.ip)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.ip_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.ip_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxNetworkStatus::set_allocated_ip(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.ip_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.ip_.IsDefault()) {
    _impl_.ip_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxNetworkStatus.ip)
}

// repeated .runtime.v1alpha2.PodIP additional_ips = 2;
inline int PodSandboxNetworkStatus::_internal_additional_ips_size() const {
  return _internal_additional_ips().size();
}
inline int PodSandboxNetworkStatus::additional_ips_size() const {
  return _internal_additional_ips_size();
}
inline void PodSandboxNetworkStatus::clear_additional_ips() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.additional_ips_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::PodIP* PROTOBUF_NONNULL PodSandboxNetworkStatus::mutable_additional_ips(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxNetworkStatus.additional_ips)
  return _internal_mutable_additional_ips()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>* PROTOBUF_NONNULL PodSandboxNetworkStatus::mutable_additional_ips()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.PodSandboxNetworkStatus.additional_ips)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_additional_ips();
}
inline const ::runtime::v1alpha2::PodIP& PodSandboxNetworkStatus::additional_ips(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxNetworkStatus.additional_ips)
  return _internal_additional_ips().Get(index);
}
inline ::runtime::v1alpha2::PodIP* PROTOBUF_NONNULL PodSandboxNetworkStatus::add_additional_ips()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::PodIP* _add =
      _internal_mutable_additional_ips()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.PodSandboxNetworkStatus.additional_ips)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>& PodSandboxNetworkStatus::additional_ips() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.PodSandboxNetworkStatus.additional_ips)
  return _internal_additional_ips();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>&
PodSandboxNetworkStatus::_internal_additional_ips() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.additional_ips_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodIP>* PROTOBUF_NONNULL
PodSandboxNetworkStatus::_internal_mutable_additional_ips() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.additional_ips_;
}

// -------------------------------------------------------------------

// Namespace

// .runtime.v1alpha2.NamespaceOption options = 2;
inline bool Namespace::has_options() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.options_ != nullptr);
  return value;
}
inline void Namespace::clear_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.options_ != nullptr) _impl_.options_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::NamespaceOption& Namespace::_internal_options() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::NamespaceOption* p = _impl_.options_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::NamespaceOption&>(::runtime::v1alpha2::_NamespaceOption_default_instance_);
}
inline const ::runtime::v1alpha2::NamespaceOption& Namespace::options() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Namespace.options)
  return _internal_options();
}
inline void Namespace::unsafe_arena_set_allocated_options(
    ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.options_);
  }
  _impl_.options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.Namespace.options)
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE Namespace::release_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::NamespaceOption* released = _impl_.options_;
  _impl_.options_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE Namespace::unsafe_arena_release_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Namespace.options)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::NamespaceOption* temp = _impl_.options_;
  _impl_.options_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL Namespace::_internal_mutable_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.options_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::NamespaceOption>(GetArena());
    _impl_.options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(p);
  }
  return _impl_.options_;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL Namespace::mutable_options()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::NamespaceOption* _msg = _internal_mutable_options();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Namespace.options)
  return _msg;
}
inline void Namespace::set_allocated_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.options_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Namespace.options)
}

// -------------------------------------------------------------------

// LinuxPodSandboxStatus

// .runtime.v1alpha2.Namespace namespaces = 1;
inline bool LinuxPodSandboxStatus::has_namespaces() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.namespaces_ != nullptr);
  return value;
}
inline void LinuxPodSandboxStatus::clear_namespaces() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.namespaces_ != nullptr) _impl_.namespaces_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::Namespace& LinuxPodSandboxStatus::_internal_namespaces() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Namespace* p = _impl_.namespaces_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Namespace&>(::runtime::v1alpha2::_Namespace_default_instance_);
}
inline const ::runtime::v1alpha2::Namespace& LinuxPodSandboxStatus::namespaces() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxPodSandboxStatus.namespaces)
  return _internal_namespaces();
}
inline void LinuxPodSandboxStatus::unsafe_arena_set_allocated_namespaces(
    ::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.namespaces_);
  }
  _impl_.namespaces_ = reinterpret_cast<::runtime::v1alpha2::Namespace*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxPodSandboxStatus.namespaces)
}
inline ::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE LinuxPodSandboxStatus::release_namespaces() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::Namespace* released = _impl_.namespaces_;
  _impl_.namespaces_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE LinuxPodSandboxStatus::unsafe_arena_release_namespaces() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxPodSandboxStatus.namespaces)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::Namespace* temp = _impl_.namespaces_;
  _impl_.namespaces_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Namespace* PROTOBUF_NONNULL LinuxPodSandboxStatus::_internal_mutable_namespaces() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.namespaces_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Namespace>(GetArena());
    _impl_.namespaces_ = reinterpret_cast<::runtime::v1alpha2::Namespace*>(p);
  }
  return _impl_.namespaces_;
}
inline ::runtime::v1alpha2::Namespace* PROTOBUF_NONNULL LinuxPodSandboxStatus::mutable_namespaces()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::Namespace* _msg = _internal_mutable_namespaces();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxPodSandboxStatus.namespaces)
  return _msg;
}
inline void LinuxPodSandboxStatus::set_allocated_namespaces(::runtime::v1alpha2::Namespace* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.namespaces_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.namespaces_ = reinterpret_cast<::runtime::v1alpha2::Namespace*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxPodSandboxStatus.namespaces)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// PodSandboxStatus

// string id = 1;
inline void PodSandboxStatus::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PodSandboxStatus::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxStatus::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStatus.id)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxStatus::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatus.id)
  return _s;
}
inline const ::std::string& PodSandboxStatus::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void PodSandboxStatus::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxStatus::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatus.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxStatus::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatus.id)
}

// .runtime.v1alpha2.PodSandboxMetadata metadata = 2;
inline bool PodSandboxStatus::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void PodSandboxStatus::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::PodSandboxMetadata& PodSandboxStatus::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxMetadata&>(::runtime::v1alpha2::_PodSandboxMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxMetadata& PodSandboxStatus::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.metadata)
  return _internal_metadata();
}
inline void PodSandboxStatus::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxStatus.metadata)
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE PodSandboxStatus::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE PodSandboxStatus::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatus.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL PodSandboxStatus::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatus.metadata)
  return _msg;
}
inline void PodSandboxStatus::set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatus.metadata)
}

// .runtime.v1alpha2.PodSandboxState state = 3;
inline void PodSandboxStatus::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline ::runtime::v1alpha2::PodSandboxState PodSandboxStatus::state() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.state)
  return _internal_state();
}
inline void PodSandboxStatus::set_state(::runtime::v1alpha2::PodSandboxState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStatus.state)
}
inline ::runtime::v1alpha2::PodSandboxState PodSandboxStatus::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::PodSandboxState>(_impl_.state_);
}
inline void PodSandboxStatus::_internal_set_state(::runtime::v1alpha2::PodSandboxState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// int64 created_at = 4;
inline void PodSandboxStatus::clear_created_at() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline ::int64_t PodSandboxStatus::created_at() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.created_at)
  return _internal_created_at();
}
inline void PodSandboxStatus::set_created_at(::int64_t value) {
  _internal_set_created_at(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStatus.created_at)
}
inline ::int64_t PodSandboxStatus::_internal_created_at() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_at_;
}
inline void PodSandboxStatus::_internal_set_created_at(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = value;
}

// .runtime.v1alpha2.PodSandboxNetworkStatus network = 5;
inline bool PodSandboxStatus::has_network() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.network_ != nullptr);
  return value;
}
inline void PodSandboxStatus::clear_network() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.network_ != nullptr) _impl_.network_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::runtime::v1alpha2::PodSandboxNetworkStatus& PodSandboxStatus::_internal_network() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxNetworkStatus* p = _impl_.network_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxNetworkStatus&>(::runtime::v1alpha2::_PodSandboxNetworkStatus_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxNetworkStatus& PodSandboxStatus::network() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.network)
  return _internal_network();
}
inline void PodSandboxStatus::unsafe_arena_set_allocated_network(
    ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.network_);
  }
  _impl_.network_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxNetworkStatus*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxStatus.network)
}
inline ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE PodSandboxStatus::release_network() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::PodSandboxNetworkStatus* released = _impl_.network_;
  _impl_.network_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE PodSandboxStatus::unsafe_arena_release_network() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatus.network)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::PodSandboxNetworkStatus* temp = _impl_.network_;
  _impl_.network_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_network() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.network_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxNetworkStatus>(GetArena());
    _impl_.network_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxNetworkStatus*>(p);
  }
  return _impl_.network_;
}
inline ::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NONNULL PodSandboxStatus::mutable_network()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::PodSandboxNetworkStatus* _msg = _internal_mutable_network();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatus.network)
  return _msg;
}
inline void PodSandboxStatus::set_allocated_network(::runtime::v1alpha2::PodSandboxNetworkStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.network_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.network_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxNetworkStatus*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatus.network)
}

// .runtime.v1alpha2.LinuxPodSandboxStatus linux = 6;
inline bool PodSandboxStatus::has_linux() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.linux_ != nullptr);
  return value;
}
inline void PodSandboxStatus::clear_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ != nullptr) _impl_.linux_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::runtime::v1alpha2::LinuxPodSandboxStatus& PodSandboxStatus::_internal_linux() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxPodSandboxStatus* p = _impl_.linux_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxPodSandboxStatus&>(::runtime::v1alpha2::_LinuxPodSandboxStatus_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxPodSandboxStatus& PodSandboxStatus::linux() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.linux)
  return _internal_linux();
}
inline void PodSandboxStatus::unsafe_arena_set_allocated_linux(
    ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }
  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxPodSandboxStatus*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxStatus.linux)
}
inline ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE PodSandboxStatus::release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::LinuxPodSandboxStatus* released = _impl_.linux_;
  _impl_.linux_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE PodSandboxStatus::unsafe_arena_release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatus.linux)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::LinuxPodSandboxStatus* temp = _impl_.linux_;
  _impl_.linux_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxPodSandboxStatus>(GetArena());
    _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxPodSandboxStatus*>(p);
  }
  return _impl_.linux_;
}
inline ::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NONNULL PodSandboxStatus::mutable_linux()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::LinuxPodSandboxStatus* _msg = _internal_mutable_linux();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatus.linux)
  return _msg;
}
inline void PodSandboxStatus::set_allocated_linux(::runtime::v1alpha2::LinuxPodSandboxStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxPodSandboxStatus*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatus.linux)
}

// map<string, string> labels = 7;
inline int PodSandboxStatus::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int PodSandboxStatus::labels_size() const {
  return _internal_labels_size();
}
inline void PodSandboxStatus::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxStatus::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxStatus::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandboxStatus.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxStatus::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandboxStatus.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 8;
inline int PodSandboxStatus::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int PodSandboxStatus::annotations_size() const {
  return _internal_annotations_size();
}
inline void PodSandboxStatus::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000100U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxStatus::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxStatus::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandboxStatus.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxStatus::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandboxStatus.annotations)
  return _internal_mutable_annotations();
}

// string runtime_handler = 9;
inline void PodSandboxStatus::clear_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_handler_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& PodSandboxStatus::runtime_handler() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatus.runtime_handler)
  return _internal_runtime_handler();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxStatus::set_runtime_handler(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.runtime_handler_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStatus.runtime_handler)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxStatus::mutable_runtime_handler()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_runtime_handler();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatus.runtime_handler)
  return _s;
}
inline const ::std::string& PodSandboxStatus::_internal_runtime_handler() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.runtime_handler_.Get();
}
inline void PodSandboxStatus::_internal_set_runtime_handler(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_handler_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxStatus::_internal_mutable_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.runtime_handler_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxStatus::release_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatus.runtime_handler)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.runtime_handler_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.runtime_handler_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxStatus::set_allocated_runtime_handler(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.runtime_handler_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.runtime_handler_.IsDefault()) {
    _impl_.runtime_handler_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatus.runtime_handler)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// PodSandboxStatusResponse

// .runtime.v1alpha2.PodSandboxStatus status = 1;
inline bool PodSandboxStatusResponse::has_status() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.status_ != nullptr);
  return value;
}
inline void PodSandboxStatusResponse::clear_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.status_ != nullptr) _impl_.status_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::PodSandboxStatus& PodSandboxStatusResponse::_internal_status() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxStatus* p = _impl_.status_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxStatus&>(::runtime::v1alpha2::_PodSandboxStatus_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxStatus& PodSandboxStatusResponse::status() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStatusResponse.status)
  return _internal_status();
}
inline void PodSandboxStatusResponse::unsafe_arena_set_allocated_status(
    ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.status_);
  }
  _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxStatus*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxStatusResponse.status)
}
inline ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE PodSandboxStatusResponse::release_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::PodSandboxStatus* released = _impl_.status_;
  _impl_.status_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE PodSandboxStatusResponse::unsafe_arena_release_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxStatusResponse.status)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::PodSandboxStatus* temp = _impl_.status_;
  _impl_.status_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NONNULL PodSandboxStatusResponse::_internal_mutable_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.status_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxStatus>(GetArena());
    _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxStatus*>(p);
  }
  return _impl_.status_;
}
inline ::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NONNULL PodSandboxStatusResponse::mutable_status()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::PodSandboxStatus* _msg = _internal_mutable_status();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxStatusResponse.status)
  return _msg;
}
inline void PodSandboxStatusResponse::set_allocated_status(::runtime::v1alpha2::PodSandboxStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.status_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxStatus*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxStatusResponse.status)
}

// map<string, string> info = 2;
inline int PodSandboxStatusResponse::_internal_info_size() const {
  return _internal_info().size();
}
inline int PodSandboxStatusResponse::info_size() const {
  return _internal_info_size();
}
inline void PodSandboxStatusResponse::clear_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.info_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxStatusResponse::_internal_info() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.info_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxStatusResponse::info() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandboxStatusResponse.info)
  return _internal_info();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxStatusResponse::_internal_mutable_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.info_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxStatusResponse::mutable_info()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandboxStatusResponse.info)
  return _internal_mutable_info();
}

// -------------------------------------------------------------------

// PodSandboxStateValue

// .runtime.v1alpha2.PodSandboxState state = 1;
inline void PodSandboxStateValue::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::PodSandboxState PodSandboxStateValue::state() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxStateValue.state)
  return _internal_state();
}
inline void PodSandboxStateValue::set_state(::runtime::v1alpha2::PodSandboxState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxStateValue.state)
}
inline ::runtime::v1alpha2::PodSandboxState PodSandboxStateValue::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::PodSandboxState>(_impl_.state_);
}
inline void PodSandboxStateValue::_internal_set_state(::runtime::v1alpha2::PodSandboxState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// PodSandboxFilter

// string id = 1;
inline void PodSandboxFilter::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PodSandboxFilter::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxFilter.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandboxFilter::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandboxFilter.id)
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxFilter::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxFilter.id)
  return _s;
}
inline const ::std::string& PodSandboxFilter::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void PodSandboxFilter::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandboxFilter::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandboxFilter::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxFilter.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void PodSandboxFilter::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxFilter.id)
}

// .runtime.v1alpha2.PodSandboxStateValue state = 2;
inline bool PodSandboxFilter::has_state() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.state_ != nullptr);
  return value;
}
inline void PodSandboxFilter::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.state_ != nullptr) _impl_.state_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::PodSandboxStateValue& PodSandboxFilter::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxStateValue* p = _impl_.state_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxStateValue&>(::runtime::v1alpha2::_PodSandboxStateValue_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxStateValue& PodSandboxFilter::state() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandboxFilter.state)
  return _internal_state();
}
inline void PodSandboxFilter::unsafe_arena_set_allocated_state(
    ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.state_);
  }
  _impl_.state_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxStateValue*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandboxFilter.state)
}
inline ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE PodSandboxFilter::release_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::PodSandboxStateValue* released = _impl_.state_;
  _impl_.state_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE PodSandboxFilter::unsafe_arena_release_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandboxFilter.state)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::PodSandboxStateValue* temp = _impl_.state_;
  _impl_.state_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NONNULL PodSandboxFilter::_internal_mutable_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.state_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxStateValue>(GetArena());
    _impl_.state_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxStateValue*>(p);
  }
  return _impl_.state_;
}
inline ::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NONNULL PodSandboxFilter::mutable_state()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::PodSandboxStateValue* _msg = _internal_mutable_state();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandboxFilter.state)
  return _msg;
}
inline void PodSandboxFilter::set_allocated_state(::runtime::v1alpha2::PodSandboxStateValue* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.state_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.state_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxStateValue*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandboxFilter.state)
}

// map<string, string> label_selector = 3;
inline int PodSandboxFilter::_internal_label_selector_size() const {
  return _internal_label_selector().size();
}
inline int PodSandboxFilter::label_selector_size() const {
  return _internal_label_selector_size();
}
inline void PodSandboxFilter::clear_label_selector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_selector_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxFilter::_internal_label_selector() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.label_selector_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandboxFilter::label_selector() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandboxFilter.label_selector)
  return _internal_label_selector();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxFilter::_internal_mutable_label_selector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.label_selector_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandboxFilter::mutable_label_selector()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandboxFilter.label_selector)
  return _internal_mutable_label_selector();
}

// -------------------------------------------------------------------

// ListPodSandboxRequest

// .runtime.v1alpha2.PodSandboxFilter filter = 1;
inline bool ListPodSandboxRequest::has_filter() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.filter_ != nullptr);
  return value;
}
inline void ListPodSandboxRequest::clear_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ != nullptr) _impl_.filter_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::PodSandboxFilter& ListPodSandboxRequest::_internal_filter() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxFilter* p = _impl_.filter_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxFilter&>(::runtime::v1alpha2::_PodSandboxFilter_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxFilter& ListPodSandboxRequest::filter() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListPodSandboxRequest.filter)
  return _internal_filter();
}
inline void ListPodSandboxRequest::unsafe_arena_set_allocated_filter(
    ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }
  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxFilter*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ListPodSandboxRequest.filter)
}
inline ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE ListPodSandboxRequest::release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::PodSandboxFilter* released = _impl_.filter_;
  _impl_.filter_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE ListPodSandboxRequest::unsafe_arena_release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ListPodSandboxRequest.filter)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::PodSandboxFilter* temp = _impl_.filter_;
  _impl_.filter_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NONNULL ListPodSandboxRequest::_internal_mutable_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxFilter>(GetArena());
    _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxFilter*>(p);
  }
  return _impl_.filter_;
}
inline ::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NONNULL ListPodSandboxRequest::mutable_filter()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::PodSandboxFilter* _msg = _internal_mutable_filter();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListPodSandboxRequest.filter)
  return _msg;
}
inline void ListPodSandboxRequest::set_allocated_filter(::runtime::v1alpha2::PodSandboxFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxFilter*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ListPodSandboxRequest.filter)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// PodSandbox

// string id = 1;
inline void PodSandbox::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PodSandbox::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandbox.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandbox::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandbox.id)
}
inline ::std::string* PROTOBUF_NONNULL PodSandbox::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandbox.id)
  return _s;
}
inline const ::std::string& PodSandbox::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void PodSandbox::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandbox::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandbox::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandbox.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void PodSandbox::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandbox.id)
}

// .runtime.v1alpha2.PodSandboxMetadata metadata = 2;
inline bool PodSandbox::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void PodSandbox::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::PodSandboxMetadata& PodSandbox::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxMetadata&>(::runtime::v1alpha2::_PodSandboxMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxMetadata& PodSandbox::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandbox.metadata)
  return _internal_metadata();
}
inline void PodSandbox::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PodSandbox.metadata)
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE PodSandbox::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE PodSandbox::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandbox.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL PodSandbox::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NONNULL PodSandbox::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandbox.metadata)
  return _msg;
}
inline void PodSandbox::set_allocated_metadata(::runtime::v1alpha2::PodSandboxMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandbox.metadata)
}

// .runtime.v1alpha2.PodSandboxState state = 3;
inline void PodSandbox::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::runtime::v1alpha2::PodSandboxState PodSandbox::state() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandbox.state)
  return _internal_state();
}
inline void PodSandbox::set_state(::runtime::v1alpha2::PodSandboxState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandbox.state)
}
inline ::runtime::v1alpha2::PodSandboxState PodSandbox::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::PodSandboxState>(_impl_.state_);
}
inline void PodSandbox::_internal_set_state(::runtime::v1alpha2::PodSandboxState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// int64 created_at = 4;
inline void PodSandbox::clear_created_at() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::int64_t PodSandbox::created_at() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandbox.created_at)
  return _internal_created_at();
}
inline void PodSandbox::set_created_at(::int64_t value) {
  _internal_set_created_at(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandbox.created_at)
}
inline ::int64_t PodSandbox::_internal_created_at() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_at_;
}
inline void PodSandbox::_internal_set_created_at(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = value;
}

// map<string, string> labels = 5;
inline int PodSandbox::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int PodSandbox::labels_size() const {
  return _internal_labels_size();
}
inline void PodSandbox::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandbox::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandbox::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandbox.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandbox::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandbox::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandbox.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 6;
inline int PodSandbox::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int PodSandbox::annotations_size() const {
  return _internal_annotations_size();
}
inline void PodSandbox::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000040U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandbox::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& PodSandbox::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.PodSandbox.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandbox::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL PodSandbox::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.PodSandbox.annotations)
  return _internal_mutable_annotations();
}

// string runtime_handler = 7;
inline void PodSandbox::clear_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_handler_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& PodSandbox::runtime_handler() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PodSandbox.runtime_handler)
  return _internal_runtime_handler();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PodSandbox::set_runtime_handler(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.runtime_handler_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PodSandbox.runtime_handler)
}
inline ::std::string* PROTOBUF_NONNULL PodSandbox::mutable_runtime_handler()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_runtime_handler();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PodSandbox.runtime_handler)
  return _s;
}
inline const ::std::string& PodSandbox::_internal_runtime_handler() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.runtime_handler_.Get();
}
inline void PodSandbox::_internal_set_runtime_handler(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.runtime_handler_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PodSandbox::_internal_mutable_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.runtime_handler_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PodSandbox::release_runtime_handler() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PodSandbox.runtime_handler)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.runtime_handler_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.runtime_handler_.Set("", GetArena());
  }
  return released;
}
inline void PodSandbox::set_allocated_runtime_handler(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.runtime_handler_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.runtime_handler_.IsDefault()) {
    _impl_.runtime_handler_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PodSandbox.runtime_handler)
}

// -------------------------------------------------------------------

// ListPodSandboxResponse

// repeated .runtime.v1alpha2.PodSandbox items = 1;
inline int ListPodSandboxResponse::_internal_items_size() const {
  return _internal_items().size();
}
inline int ListPodSandboxResponse::items_size() const {
  return _internal_items_size();
}
inline void ListPodSandboxResponse::clear_items() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.items_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::PodSandbox* PROTOBUF_NONNULL ListPodSandboxResponse::mutable_items(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListPodSandboxResponse.items)
  return _internal_mutable_items()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>* PROTOBUF_NONNULL ListPodSandboxResponse::mutable_items()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ListPodSandboxResponse.items)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_items();
}
inline const ::runtime::v1alpha2::PodSandbox& ListPodSandboxResponse::items(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListPodSandboxResponse.items)
  return _internal_items().Get(index);
}
inline ::runtime::v1alpha2::PodSandbox* PROTOBUF_NONNULL ListPodSandboxResponse::add_items()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::PodSandbox* _add =
      _internal_mutable_items()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ListPodSandboxResponse.items)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>& ListPodSandboxResponse::items() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ListPodSandboxResponse.items)
  return _internal_items();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>&
ListPodSandboxResponse::_internal_items() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.items_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::PodSandbox>* PROTOBUF_NONNULL
ListPodSandboxResponse::_internal_mutable_items() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.items_;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ImageSpec

// string image = 1;
inline void ImageSpec::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ImageSpec::image() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ImageSpec.image)
  return _internal_image();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ImageSpec::set_image(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.image_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ImageSpec.image)
}
inline ::std::string* PROTOBUF_NONNULL ImageSpec::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ImageSpec.image)
  return _s;
}
inline const ::std::string& ImageSpec::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.image_.Get();
}
inline void ImageSpec::_internal_set_image(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ImageSpec::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.image_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ImageSpec::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ImageSpec.image)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.image_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.image_.Set("", GetArena());
  }
  return released;
}
inline void ImageSpec::set_allocated_image(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.image_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.image_.IsDefault()) {
    _impl_.image_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ImageSpec.image)
}

// map<string, string> annotations = 2;
inline int ImageSpec::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int ImageSpec::annotations_size() const {
  return _internal_annotations_size();
}
inline void ImageSpec::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ImageSpec::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ImageSpec::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ImageSpec.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ImageSpec::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ImageSpec::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ImageSpec.annotations)
  return _internal_mutable_annotations();
}

// -------------------------------------------------------------------

// KeyValue

// string key = 1;
inline void KeyValue::clear_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& KeyValue::key() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.KeyValue.key)
  return _internal_key();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void KeyValue::set_key(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.key_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.KeyValue.key)
}
inline ::std::string* PROTOBUF_NONNULL KeyValue::mutable_key()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_key();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.KeyValue.key)
  return _s;
}
inline const ::std::string& KeyValue::_internal_key() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.key_.Get();
}
inline void KeyValue::_internal_set_key(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.key_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL KeyValue::_internal_mutable_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.key_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE KeyValue::release_key() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.KeyValue.key)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.key_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.key_.Set("", GetArena());
  }
  return released;
}
inline void KeyValue::set_allocated_key(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.key_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.key_.IsDefault()) {
    _impl_.key_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.KeyValue.key)
}

// string value = 2;
inline void KeyValue::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& KeyValue::value() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.KeyValue.value)
  return _internal_value();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void KeyValue::set_value(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.value_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.KeyValue.value)
}
inline ::std::string* PROTOBUF_NONNULL KeyValue::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_value();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.KeyValue.value)
  return _s;
}
inline const ::std::string& KeyValue::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_.Get();
}
inline void KeyValue::_internal_set_value(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL KeyValue::_internal_mutable_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.value_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE KeyValue::release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.KeyValue.value)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.value_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.value_.Set("", GetArena());
  }
  return released;
}
inline void KeyValue::set_allocated_value(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.value_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.value_.IsDefault()) {
    _impl_.value_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.KeyValue.value)
}

// -------------------------------------------------------------------

// LinuxContainerResources

// int64 cpu_period = 1;
inline void LinuxContainerResources::clear_cpu_period() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_period_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::int64_t LinuxContainerResources::cpu_period() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.cpu_period)
  return _internal_cpu_period();
}
inline void LinuxContainerResources::set_cpu_period(::int64_t value) {
  _internal_set_cpu_period(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.cpu_period)
}
inline ::int64_t LinuxContainerResources::_internal_cpu_period() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_period_;
}
inline void LinuxContainerResources::_internal_set_cpu_period(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_period_ = value;
}

// int64 cpu_quota = 2;
inline void LinuxContainerResources::clear_cpu_quota() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_quota_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::int64_t LinuxContainerResources::cpu_quota() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.cpu_quota)
  return _internal_cpu_quota();
}
inline void LinuxContainerResources::set_cpu_quota(::int64_t value) {
  _internal_set_cpu_quota(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.cpu_quota)
}
inline ::int64_t LinuxContainerResources::_internal_cpu_quota() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_quota_;
}
inline void LinuxContainerResources::_internal_set_cpu_quota(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_quota_ = value;
}

// int64 cpu_shares = 3;
inline void LinuxContainerResources::clear_cpu_shares() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_shares_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline ::int64_t LinuxContainerResources::cpu_shares() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.cpu_shares)
  return _internal_cpu_shares();
}
inline void LinuxContainerResources::set_cpu_shares(::int64_t value) {
  _internal_set_cpu_shares(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.cpu_shares)
}
inline ::int64_t LinuxContainerResources::_internal_cpu_shares() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_shares_;
}
inline void LinuxContainerResources::_internal_set_cpu_shares(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_shares_ = value;
}

// int64 memory_limit_in_bytes = 4;
inline void LinuxContainerResources::clear_memory_limit_in_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.memory_limit_in_bytes_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline ::int64_t LinuxContainerResources::memory_limit_in_bytes() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.memory_limit_in_bytes)
  return _internal_memory_limit_in_bytes();
}
inline void LinuxContainerResources::set_memory_limit_in_bytes(::int64_t value) {
  _internal_set_memory_limit_in_bytes(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.memory_limit_in_bytes)
}
inline ::int64_t LinuxContainerResources::_internal_memory_limit_in_bytes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.memory_limit_in_bytes_;
}
inline void LinuxContainerResources::_internal_set_memory_limit_in_bytes(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.memory_limit_in_bytes_ = value;
}

// int64 oom_score_adj = 5;
inline void LinuxContainerResources::clear_oom_score_adj() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.oom_score_adj_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline ::int64_t LinuxContainerResources::oom_score_adj() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.oom_score_adj)
  return _internal_oom_score_adj();
}
inline void LinuxContainerResources::set_oom_score_adj(::int64_t value) {
  _internal_set_oom_score_adj(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.oom_score_adj)
}
inline ::int64_t LinuxContainerResources::_internal_oom_score_adj() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.oom_score_adj_;
}
inline void LinuxContainerResources::_internal_set_oom_score_adj(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.oom_score_adj_ = value;
}

// string cpuset_cpus = 6;
inline void LinuxContainerResources::clear_cpuset_cpus() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpuset_cpus_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& LinuxContainerResources::cpuset_cpus() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.cpuset_cpus)
  return _internal_cpuset_cpus();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxContainerResources::set_cpuset_cpus(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.cpuset_cpus_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.cpuset_cpus)
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerResources::mutable_cpuset_cpus()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_cpuset_cpus();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerResources.cpuset_cpus)
  return _s;
}
inline const ::std::string& LinuxContainerResources::_internal_cpuset_cpus() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpuset_cpus_.Get();
}
inline void LinuxContainerResources::_internal_set_cpuset_cpus(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpuset_cpus_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerResources::_internal_mutable_cpuset_cpus() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.cpuset_cpus_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxContainerResources::release_cpuset_cpus() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerResources.cpuset_cpus)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.cpuset_cpus_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.cpuset_cpus_.Set("", GetArena());
  }
  return released;
}
inline void LinuxContainerResources::set_allocated_cpuset_cpus(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.cpuset_cpus_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.cpuset_cpus_.IsDefault()) {
    _impl_.cpuset_cpus_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerResources.cpuset_cpus)
}

// string cpuset_mems = 7;
inline void LinuxContainerResources::clear_cpuset_mems() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpuset_mems_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& LinuxContainerResources::cpuset_mems() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.cpuset_mems)
  return _internal_cpuset_mems();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxContainerResources::set_cpuset_mems(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.cpuset_mems_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerResources.cpuset_mems)
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerResources::mutable_cpuset_mems()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_cpuset_mems();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerResources.cpuset_mems)
  return _s;
}
inline const ::std::string& LinuxContainerResources::_internal_cpuset_mems() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpuset_mems_.Get();
}
inline void LinuxContainerResources::_internal_set_cpuset_mems(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpuset_mems_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerResources::_internal_mutable_cpuset_mems() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.cpuset_mems_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxContainerResources::release_cpuset_mems() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerResources.cpuset_mems)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.cpuset_mems_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.cpuset_mems_.Set("", GetArena());
  }
  return released;
}
inline void LinuxContainerResources::set_allocated_cpuset_mems(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.cpuset_mems_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.cpuset_mems_.IsDefault()) {
    _impl_.cpuset_mems_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerResources.cpuset_mems)
}

// repeated .runtime.v1alpha2.HugepageLimit hugepage_limits = 8;
inline int LinuxContainerResources::_internal_hugepage_limits_size() const {
  return _internal_hugepage_limits().size();
}
inline int LinuxContainerResources::hugepage_limits_size() const {
  return _internal_hugepage_limits_size();
}
inline void LinuxContainerResources::clear_hugepage_limits() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.hugepage_limits_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::HugepageLimit* PROTOBUF_NONNULL LinuxContainerResources::mutable_hugepage_limits(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerResources.hugepage_limits)
  return _internal_mutable_hugepage_limits()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>* PROTOBUF_NONNULL LinuxContainerResources::mutable_hugepage_limits()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.LinuxContainerResources.hugepage_limits)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_hugepage_limits();
}
inline const ::runtime::v1alpha2::HugepageLimit& LinuxContainerResources::hugepage_limits(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerResources.hugepage_limits)
  return _internal_hugepage_limits().Get(index);
}
inline ::runtime::v1alpha2::HugepageLimit* PROTOBUF_NONNULL LinuxContainerResources::add_hugepage_limits()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::HugepageLimit* _add =
      _internal_mutable_hugepage_limits()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.LinuxContainerResources.hugepage_limits)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>& LinuxContainerResources::hugepage_limits() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.LinuxContainerResources.hugepage_limits)
  return _internal_hugepage_limits();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>&
LinuxContainerResources::_internal_hugepage_limits() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.hugepage_limits_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::HugepageLimit>* PROTOBUF_NONNULL
LinuxContainerResources::_internal_mutable_hugepage_limits() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.hugepage_limits_;
}

// -------------------------------------------------------------------

// HugepageLimit

// string page_size = 1;
inline void HugepageLimit::clear_page_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.page_size_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& HugepageLimit::page_size() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.HugepageLimit.page_size)
  return _internal_page_size();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void HugepageLimit::set_page_size(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.page_size_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.HugepageLimit.page_size)
}
inline ::std::string* PROTOBUF_NONNULL HugepageLimit::mutable_page_size()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_page_size();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.HugepageLimit.page_size)
  return _s;
}
inline const ::std::string& HugepageLimit::_internal_page_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.page_size_.Get();
}
inline void HugepageLimit::_internal_set_page_size(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.page_size_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL HugepageLimit::_internal_mutable_page_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.page_size_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE HugepageLimit::release_page_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.HugepageLimit.page_size)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.page_size_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.page_size_.Set("", GetArena());
  }
  return released;
}
inline void HugepageLimit::set_allocated_page_size(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.page_size_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.page_size_.IsDefault()) {
    _impl_.page_size_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.HugepageLimit.page_size)
}

// uint64 limit = 2;
inline void HugepageLimit::clear_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.limit_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::uint64_t HugepageLimit::limit() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.HugepageLimit.limit)
  return _internal_limit();
}
inline void HugepageLimit::set_limit(::uint64_t value) {
  _internal_set_limit(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.HugepageLimit.limit)
}
inline ::uint64_t HugepageLimit::_internal_limit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.limit_;
}
inline void HugepageLimit::_internal_set_limit(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.limit_ = value;
}

// -------------------------------------------------------------------

// SELinuxOption

// string user = 1;
inline void SELinuxOption::clear_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.user_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& SELinuxOption::user() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.SELinuxOption.user)
  return _internal_user();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SELinuxOption::set_user(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.user_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.SELinuxOption.user)
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::mutable_user()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_user();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.SELinuxOption.user)
  return _s;
}
inline const ::std::string& SELinuxOption::_internal_user() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.user_.Get();
}
inline void SELinuxOption::_internal_set_user(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.user_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::_internal_mutable_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.user_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SELinuxOption::release_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.SELinuxOption.user)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.user_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.user_.Set("", GetArena());
  }
  return released;
}
inline void SELinuxOption::set_allocated_user(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.user_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.user_.IsDefault()) {
    _impl_.user_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.SELinuxOption.user)
}

// string role = 2;
inline void SELinuxOption::clear_role() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.role_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& SELinuxOption::role() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.SELinuxOption.role)
  return _internal_role();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SELinuxOption::set_role(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.role_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.SELinuxOption.role)
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::mutable_role()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_role();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.SELinuxOption.role)
  return _s;
}
inline const ::std::string& SELinuxOption::_internal_role() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.role_.Get();
}
inline void SELinuxOption::_internal_set_role(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.role_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::_internal_mutable_role() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.role_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SELinuxOption::release_role() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.SELinuxOption.role)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.role_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.role_.Set("", GetArena());
  }
  return released;
}
inline void SELinuxOption::set_allocated_role(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.role_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.role_.IsDefault()) {
    _impl_.role_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.SELinuxOption.role)
}

// string type = 3;
inline void SELinuxOption::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& SELinuxOption::type() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.SELinuxOption.type)
  return _internal_type();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SELinuxOption::set_type(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.type_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.SELinuxOption.type)
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::mutable_type()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_type();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.SELinuxOption.type)
  return _s;
}
inline const ::std::string& SELinuxOption::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.type_.Get();
}
inline void SELinuxOption::_internal_set_type(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::_internal_mutable_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.type_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SELinuxOption::release_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.SELinuxOption.type)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.type_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.type_.Set("", GetArena());
  }
  return released;
}
inline void SELinuxOption::set_allocated_type(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.type_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.type_.IsDefault()) {
    _impl_.type_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.SELinuxOption.type)
}

// string level = 4;
inline void SELinuxOption::clear_level() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.level_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::std::string& SELinuxOption::level() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.SELinuxOption.level)
  return _internal_level();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void SELinuxOption::set_level(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.level_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.SELinuxOption.level)
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::mutable_level()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_level();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.SELinuxOption.level)
  return _s;
}
inline const ::std::string& SELinuxOption::_internal_level() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.level_.Get();
}
inline void SELinuxOption::_internal_set_level(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.level_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL SELinuxOption::_internal_mutable_level() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.level_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE SELinuxOption::release_level() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.SELinuxOption.level)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.level_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.level_.Set("", GetArena());
  }
  return released;
}
inline void SELinuxOption::set_allocated_level(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.level_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.level_.IsDefault()) {
    _impl_.level_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.SELinuxOption.level)
}

// -------------------------------------------------------------------

// Capability

// repeated string add_capabilities = 1;
inline int Capability::_internal_add_capabilities_size() const {
  return _internal_add_capabilities().size();
}
inline int Capability::add_capabilities_size() const {
  return _internal_add_capabilities_size();
}
inline void Capability::clear_add_capabilities() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.add_capabilities_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL Capability::add_add_capabilities()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_add_capabilities()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.Capability.add_capabilities)
  return _s;
}
inline const ::std::string& Capability::add_capabilities(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Capability.add_capabilities)
  return _internal_add_capabilities().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Capability::mutable_add_capabilities(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Capability.add_capabilities)
  return _internal_mutable_add_capabilities()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Capability::set_add_capabilities(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_add_capabilities()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Capability.add_capabilities)
}
template <typename Arg_, typename... Args_>
inline void Capability::add_add_capabilities(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_add_capabilities(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.Capability.add_capabilities)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Capability::add_capabilities()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.Capability.add_capabilities)
  return _internal_add_capabilities();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Capability::mutable_add_capabilities() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.Capability.add_capabilities)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_add_capabilities();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Capability::_internal_add_capabilities() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.add_capabilities_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Capability::_internal_mutable_add_capabilities() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.add_capabilities_;
}

// repeated string drop_capabilities = 2;
inline int Capability::_internal_drop_capabilities_size() const {
  return _internal_drop_capabilities().size();
}
inline int Capability::drop_capabilities_size() const {
  return _internal_drop_capabilities_size();
}
inline void Capability::clear_drop_capabilities() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.drop_capabilities_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL Capability::add_drop_capabilities()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_drop_capabilities()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.Capability.drop_capabilities)
  return _s;
}
inline const ::std::string& Capability::drop_capabilities(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Capability.drop_capabilities)
  return _internal_drop_capabilities().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Capability::mutable_drop_capabilities(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Capability.drop_capabilities)
  return _internal_mutable_drop_capabilities()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Capability::set_drop_capabilities(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_drop_capabilities()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Capability.drop_capabilities)
}
template <typename Arg_, typename... Args_>
inline void Capability::add_drop_capabilities(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_drop_capabilities(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.Capability.drop_capabilities)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Capability::drop_capabilities()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.Capability.drop_capabilities)
  return _internal_drop_capabilities();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Capability::mutable_drop_capabilities() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.Capability.drop_capabilities)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_drop_capabilities();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Capability::_internal_drop_capabilities() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.drop_capabilities_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Capability::_internal_mutable_drop_capabilities() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.drop_capabilities_;
}

// -------------------------------------------------------------------

// LinuxContainerSecurityContext

// .runtime.v1alpha2.Capability capabilities = 1;
inline bool LinuxContainerSecurityContext::has_capabilities() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  PROTOBUF_ASSUME(!value || _impl_.capabilities_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_capabilities() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.capabilities_ != nullptr) _impl_.capabilities_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline const ::runtime::v1alpha2::Capability& LinuxContainerSecurityContext::_internal_capabilities() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Capability* p = _impl_.capabilities_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Capability&>(::runtime::v1alpha2::_Capability_default_instance_);
}
inline const ::runtime::v1alpha2::Capability& LinuxContainerSecurityContext::capabilities() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.capabilities)
  return _internal_capabilities();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_capabilities(
    ::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.capabilities_);
  }
  _impl_.capabilities_ = reinterpret_cast<::runtime::v1alpha2::Capability*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.capabilities)
}
inline ::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_capabilities() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::Capability* released = _impl_.capabilities_;
  _impl_.capabilities_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_capabilities() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.capabilities)

  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::Capability* temp = _impl_.capabilities_;
  _impl_.capabilities_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Capability* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_capabilities() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.capabilities_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Capability>(GetArena());
    _impl_.capabilities_ = reinterpret_cast<::runtime::v1alpha2::Capability*>(p);
  }
  return _impl_.capabilities_;
}
inline ::runtime::v1alpha2::Capability* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_capabilities()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::Capability* _msg = _internal_mutable_capabilities();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.capabilities)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_capabilities(::runtime::v1alpha2::Capability* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.capabilities_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }

  _impl_.capabilities_ = reinterpret_cast<::runtime::v1alpha2::Capability*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.capabilities)
}

// bool privileged = 2;
inline void LinuxContainerSecurityContext::clear_privileged() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.privileged_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00002000U);
}
inline bool LinuxContainerSecurityContext::privileged() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.privileged)
  return _internal_privileged();
}
inline void LinuxContainerSecurityContext::set_privileged(bool value) {
  _internal_set_privileged(value);
  SetHasBit(_impl_._has_bits_[0], 0x00002000U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.privileged)
}
inline bool LinuxContainerSecurityContext::_internal_privileged() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.privileged_;
}
inline void LinuxContainerSecurityContext::_internal_set_privileged(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.privileged_ = value;
}

// .runtime.v1alpha2.NamespaceOption namespace_options = 3;
inline bool LinuxContainerSecurityContext::has_namespace_options() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  PROTOBUF_ASSUME(!value || _impl_.namespace_options_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.namespace_options_ != nullptr) _impl_.namespace_options_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::runtime::v1alpha2::NamespaceOption& LinuxContainerSecurityContext::_internal_namespace_options() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::NamespaceOption* p = _impl_.namespace_options_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::NamespaceOption&>(::runtime::v1alpha2::_NamespaceOption_default_instance_);
}
inline const ::runtime::v1alpha2::NamespaceOption& LinuxContainerSecurityContext::namespace_options() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.namespace_options)
  return _internal_namespace_options();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_namespace_options(
    ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.namespace_options_);
  }
  _impl_.namespace_options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.namespace_options)
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::NamespaceOption* released = _impl_.namespace_options_;
  _impl_.namespace_options_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.namespace_options)

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::NamespaceOption* temp = _impl_.namespace_options_;
  _impl_.namespace_options_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_namespace_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.namespace_options_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::NamespaceOption>(GetArena());
    _impl_.namespace_options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(p);
  }
  return _impl_.namespace_options_;
}
inline ::runtime::v1alpha2::NamespaceOption* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_namespace_options()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::NamespaceOption* _msg = _internal_mutable_namespace_options();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.namespace_options)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_namespace_options(::runtime::v1alpha2::NamespaceOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.namespace_options_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }

  _impl_.namespace_options_ = reinterpret_cast<::runtime::v1alpha2::NamespaceOption*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.namespace_options)
}

// .runtime.v1alpha2.SELinuxOption selinux_options = 4;
inline bool LinuxContainerSecurityContext::has_selinux_options() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000100U);
  PROTOBUF_ASSUME(!value || _impl_.selinux_options_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.selinux_options_ != nullptr) _impl_.selinux_options_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000100U);
}
inline const ::runtime::v1alpha2::SELinuxOption& LinuxContainerSecurityContext::_internal_selinux_options() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::SELinuxOption* p = _impl_.selinux_options_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::SELinuxOption&>(::runtime::v1alpha2::_SELinuxOption_default_instance_);
}
inline const ::runtime::v1alpha2::SELinuxOption& LinuxContainerSecurityContext::selinux_options() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.selinux_options)
  return _internal_selinux_options();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_selinux_options(
    ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.selinux_options_);
  }
  _impl_.selinux_options_ = reinterpret_cast<::runtime::v1alpha2::SELinuxOption*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.selinux_options)
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  ::runtime::v1alpha2::SELinuxOption* released = _impl_.selinux_options_;
  _impl_.selinux_options_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.selinux_options)

  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  ::runtime::v1alpha2::SELinuxOption* temp = _impl_.selinux_options_;
  _impl_.selinux_options_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_selinux_options() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.selinux_options_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::SELinuxOption>(GetArena());
    _impl_.selinux_options_ = reinterpret_cast<::runtime::v1alpha2::SELinuxOption*>(p);
  }
  return _impl_.selinux_options_;
}
inline ::runtime::v1alpha2::SELinuxOption* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_selinux_options()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  ::runtime::v1alpha2::SELinuxOption* _msg = _internal_mutable_selinux_options();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.selinux_options)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_selinux_options(::runtime::v1alpha2::SELinuxOption* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.selinux_options_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  }

  _impl_.selinux_options_ = reinterpret_cast<::runtime::v1alpha2::SELinuxOption*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.selinux_options)
}

// .runtime.v1alpha2.Int64Value run_as_user = 5;
inline bool LinuxContainerSecurityContext::has_run_as_user() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000200U);
  PROTOBUF_ASSUME(!value || _impl_.run_as_user_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_user_ != nullptr) _impl_.run_as_user_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000200U);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxContainerSecurityContext::_internal_run_as_user() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Int64Value* p = _impl_.run_as_user_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Int64Value&>(::runtime::v1alpha2::_Int64Value_default_instance_);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxContainerSecurityContext::run_as_user() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_user)
  return _internal_run_as_user();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_run_as_user(
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_user_);
  }
  _impl_.run_as_user_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_user)
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  ::runtime::v1alpha2::Int64Value* released = _impl_.run_as_user_;
  _impl_.run_as_user_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_user)

  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  ::runtime::v1alpha2::Int64Value* temp = _impl_.run_as_user_;
  _impl_.run_as_user_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_run_as_user() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_user_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Int64Value>(GetArena());
    _impl_.run_as_user_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(p);
  }
  return _impl_.run_as_user_;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_run_as_user()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  ::runtime::v1alpha2::Int64Value* _msg = _internal_mutable_run_as_user();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_user)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_run_as_user(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_user_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  }

  _impl_.run_as_user_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_user)
}

// .runtime.v1alpha2.Int64Value run_as_group = 12;
inline bool LinuxContainerSecurityContext::has_run_as_group() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000400U);
  PROTOBUF_ASSUME(!value || _impl_.run_as_group_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_group_ != nullptr) _impl_.run_as_group_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000400U);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxContainerSecurityContext::_internal_run_as_group() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Int64Value* p = _impl_.run_as_group_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Int64Value&>(::runtime::v1alpha2::_Int64Value_default_instance_);
}
inline const ::runtime::v1alpha2::Int64Value& LinuxContainerSecurityContext::run_as_group() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_group)
  return _internal_run_as_group();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_run_as_group(
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_group_);
  }
  _impl_.run_as_group_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_group)
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  ::runtime::v1alpha2::Int64Value* released = _impl_.run_as_group_;
  _impl_.run_as_group_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_group)

  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  ::runtime::v1alpha2::Int64Value* temp = _impl_.run_as_group_;
  _impl_.run_as_group_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_run_as_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.run_as_group_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Int64Value>(GetArena());
    _impl_.run_as_group_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(p);
  }
  return _impl_.run_as_group_;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_run_as_group()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  ::runtime::v1alpha2::Int64Value* _msg = _internal_mutable_run_as_group();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_group)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_run_as_group(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.run_as_group_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  }

  _impl_.run_as_group_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_group)
}

// string run_as_username = 6;
inline void LinuxContainerSecurityContext::clear_run_as_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.run_as_username_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::std::string& LinuxContainerSecurityContext::run_as_username() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_username)
  return _internal_run_as_username();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxContainerSecurityContext::set_run_as_username(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.run_as_username_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_username)
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_run_as_username()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_run_as_username();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_username)
  return _s;
}
inline const ::std::string& LinuxContainerSecurityContext::_internal_run_as_username() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.run_as_username_.Get();
}
inline void LinuxContainerSecurityContext::_internal_set_run_as_username(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.run_as_username_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_run_as_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.run_as_username_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_run_as_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_username)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.run_as_username_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.run_as_username_.Set("", GetArena());
  }
  return released;
}
inline void LinuxContainerSecurityContext::set_allocated_run_as_username(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.run_as_username_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.run_as_username_.IsDefault()) {
    _impl_.run_as_username_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.run_as_username)
}

// bool readonly_rootfs = 7;
inline void LinuxContainerSecurityContext::clear_readonly_rootfs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_rootfs_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00004000U);
}
inline bool LinuxContainerSecurityContext::readonly_rootfs() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_rootfs)
  return _internal_readonly_rootfs();
}
inline void LinuxContainerSecurityContext::set_readonly_rootfs(bool value) {
  _internal_set_readonly_rootfs(value);
  SetHasBit(_impl_._has_bits_[0], 0x00004000U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_rootfs)
}
inline bool LinuxContainerSecurityContext::_internal_readonly_rootfs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.readonly_rootfs_;
}
inline void LinuxContainerSecurityContext::_internal_set_readonly_rootfs(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_rootfs_ = value;
}

// repeated int64 supplemental_groups = 8;
inline int LinuxContainerSecurityContext::_internal_supplemental_groups_size() const {
  return _internal_supplemental_groups().size();
}
inline int LinuxContainerSecurityContext::supplemental_groups_size() const {
  return _internal_supplemental_groups_size();
}
inline void LinuxContainerSecurityContext::clear_supplemental_groups() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.supplemental_groups_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::int64_t LinuxContainerSecurityContext::supplemental_groups(int index) const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.supplemental_groups)
  return _internal_supplemental_groups().Get(index);
}
inline void LinuxContainerSecurityContext::set_supplemental_groups(int index, ::int64_t value) {
  _internal_mutable_supplemental_groups()->Set(index, value);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.supplemental_groups)
}
inline void LinuxContainerSecurityContext::add_supplemental_groups(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_supplemental_groups()->Add(value);
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.LinuxContainerSecurityContext.supplemental_groups)
}
inline const ::google::protobuf::RepeatedField<::int64_t>& LinuxContainerSecurityContext::supplemental_groups() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.LinuxContainerSecurityContext.supplemental_groups)
  return _internal_supplemental_groups();
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_supplemental_groups()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.LinuxContainerSecurityContext.supplemental_groups)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_supplemental_groups();
}
inline const ::google::protobuf::RepeatedField<::int64_t>&
LinuxContainerSecurityContext::_internal_supplemental_groups() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.supplemental_groups_;
}
inline ::google::protobuf::RepeatedField<::int64_t>* PROTOBUF_NONNULL
LinuxContainerSecurityContext::_internal_mutable_supplemental_groups() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.supplemental_groups_;
}

// bool no_new_privs = 11;
inline void LinuxContainerSecurityContext::clear_no_new_privs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.no_new_privs_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00008000U);
}
inline bool LinuxContainerSecurityContext::no_new_privs() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.no_new_privs)
  return _internal_no_new_privs();
}
inline void LinuxContainerSecurityContext::set_no_new_privs(bool value) {
  _internal_set_no_new_privs(value);
  SetHasBit(_impl_._has_bits_[0], 0x00008000U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.no_new_privs)
}
inline bool LinuxContainerSecurityContext::_internal_no_new_privs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.no_new_privs_;
}
inline void LinuxContainerSecurityContext::_internal_set_no_new_privs(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.no_new_privs_ = value;
}

// repeated string masked_paths = 13;
inline int LinuxContainerSecurityContext::_internal_masked_paths_size() const {
  return _internal_masked_paths().size();
}
inline int LinuxContainerSecurityContext::masked_paths_size() const {
  return _internal_masked_paths_size();
}
inline void LinuxContainerSecurityContext::clear_masked_paths() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.masked_paths_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::add_masked_paths()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_masked_paths()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
  return _s;
}
inline const ::std::string& LinuxContainerSecurityContext::masked_paths(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
  return _internal_masked_paths().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_masked_paths(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
  return _internal_mutable_masked_paths()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void LinuxContainerSecurityContext::set_masked_paths(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_masked_paths()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
}
template <typename Arg_, typename... Args_>
inline void LinuxContainerSecurityContext::add_masked_paths(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_masked_paths(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& LinuxContainerSecurityContext::masked_paths()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
  return _internal_masked_paths();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
LinuxContainerSecurityContext::mutable_masked_paths() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_masked_paths();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
LinuxContainerSecurityContext::_internal_masked_paths() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.masked_paths_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
LinuxContainerSecurityContext::_internal_mutable_masked_paths() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.masked_paths_;
}

// repeated string readonly_paths = 14;
inline int LinuxContainerSecurityContext::_internal_readonly_paths_size() const {
  return _internal_readonly_paths().size();
}
inline int LinuxContainerSecurityContext::readonly_paths_size() const {
  return _internal_readonly_paths_size();
}
inline void LinuxContainerSecurityContext::clear_readonly_paths() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.readonly_paths_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::add_readonly_paths()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_readonly_paths()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
  return _s;
}
inline const ::std::string& LinuxContainerSecurityContext::readonly_paths(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
  return _internal_readonly_paths().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_readonly_paths(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
  return _internal_mutable_readonly_paths()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void LinuxContainerSecurityContext::set_readonly_paths(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_readonly_paths()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
}
template <typename Arg_, typename... Args_>
inline void LinuxContainerSecurityContext::add_readonly_paths(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_readonly_paths(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& LinuxContainerSecurityContext::readonly_paths()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
  return _internal_readonly_paths();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
LinuxContainerSecurityContext::mutable_readonly_paths() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_readonly_paths();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
LinuxContainerSecurityContext::_internal_readonly_paths() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.readonly_paths_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
LinuxContainerSecurityContext::_internal_mutable_readonly_paths() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.readonly_paths_;
}

// .runtime.v1alpha2.SecurityProfile seccomp = 15;
inline bool LinuxContainerSecurityContext::has_seccomp() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000800U);
  PROTOBUF_ASSUME(!value || _impl_.seccomp_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.seccomp_ != nullptr) _impl_.seccomp_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000800U);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxContainerSecurityContext::_internal_seccomp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::SecurityProfile* p = _impl_.seccomp_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::SecurityProfile&>(::runtime::v1alpha2::_SecurityProfile_default_instance_);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxContainerSecurityContext::seccomp() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp)
  return _internal_seccomp();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_seccomp(
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.seccomp_);
  }
  _impl_.seccomp_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000800U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp)
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000800U);
  ::runtime::v1alpha2::SecurityProfile* released = _impl_.seccomp_;
  _impl_.seccomp_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp)

  ClearHasBit(_impl_._has_bits_[0], 0x00000800U);
  ::runtime::v1alpha2::SecurityProfile* temp = _impl_.seccomp_;
  _impl_.seccomp_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_seccomp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.seccomp_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::SecurityProfile>(GetArena());
    _impl_.seccomp_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(p);
  }
  return _impl_.seccomp_;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_seccomp()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  ::runtime::v1alpha2::SecurityProfile* _msg = _internal_mutable_seccomp();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_seccomp(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.seccomp_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000800U);
  }

  _impl_.seccomp_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp)
}

// .runtime.v1alpha2.SecurityProfile apparmor = 16;
inline bool LinuxContainerSecurityContext::has_apparmor() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00001000U);
  PROTOBUF_ASSUME(!value || _impl_.apparmor_ != nullptr);
  return value;
}
inline void LinuxContainerSecurityContext::clear_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.apparmor_ != nullptr) _impl_.apparmor_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00001000U);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxContainerSecurityContext::_internal_apparmor() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::SecurityProfile* p = _impl_.apparmor_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::SecurityProfile&>(::runtime::v1alpha2::_SecurityProfile_default_instance_);
}
inline const ::runtime::v1alpha2::SecurityProfile& LinuxContainerSecurityContext::apparmor() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor)
  return _internal_apparmor();
}
inline void LinuxContainerSecurityContext::unsafe_arena_set_allocated_apparmor(
    ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.apparmor_);
  }
  _impl_.apparmor_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00001000U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor)
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00001000U);
  ::runtime::v1alpha2::SecurityProfile* released = _impl_.apparmor_;
  _impl_.apparmor_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE LinuxContainerSecurityContext::unsafe_arena_release_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor)

  ClearHasBit(_impl_._has_bits_[0], 0x00001000U);
  ::runtime::v1alpha2::SecurityProfile* temp = _impl_.apparmor_;
  _impl_.apparmor_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_apparmor() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.apparmor_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::SecurityProfile>(GetArena());
    _impl_.apparmor_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(p);
  }
  return _impl_.apparmor_;
}
inline ::runtime::v1alpha2::SecurityProfile* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_apparmor()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  ::runtime::v1alpha2::SecurityProfile* _msg = _internal_mutable_apparmor();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor)
  return _msg;
}
inline void LinuxContainerSecurityContext::set_allocated_apparmor(::runtime::v1alpha2::SecurityProfile* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.apparmor_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00001000U);
  }

  _impl_.apparmor_ = reinterpret_cast<::runtime::v1alpha2::SecurityProfile*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor)
}

// string apparmor_profile = 9;
inline void LinuxContainerSecurityContext::clear_apparmor_profile() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.apparmor_profile_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::std::string& LinuxContainerSecurityContext::apparmor_profile() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor_profile)
  return _internal_apparmor_profile();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxContainerSecurityContext::set_apparmor_profile(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.apparmor_profile_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor_profile)
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_apparmor_profile()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_apparmor_profile();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor_profile)
  return _s;
}
inline const ::std::string& LinuxContainerSecurityContext::_internal_apparmor_profile() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.apparmor_profile_.Get();
}
inline void LinuxContainerSecurityContext::_internal_set_apparmor_profile(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.apparmor_profile_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_apparmor_profile() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.apparmor_profile_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_apparmor_profile() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor_profile)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.apparmor_profile_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.apparmor_profile_.Set("", GetArena());
  }
  return released;
}
inline void LinuxContainerSecurityContext::set_allocated_apparmor_profile(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.apparmor_profile_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.apparmor_profile_.IsDefault()) {
    _impl_.apparmor_profile_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.apparmor_profile)
}

// string seccomp_profile_path = 10;
inline void LinuxContainerSecurityContext::clear_seccomp_profile_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.seccomp_profile_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::std::string& LinuxContainerSecurityContext::seccomp_profile_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp_profile_path)
  return _internal_seccomp_profile_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void LinuxContainerSecurityContext::set_seccomp_profile_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  _impl_.seccomp_profile_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp_profile_path)
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::mutable_seccomp_profile_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::std::string* _s = _internal_mutable_seccomp_profile_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp_profile_path)
  return _s;
}
inline const ::std::string& LinuxContainerSecurityContext::_internal_seccomp_profile_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.seccomp_profile_path_.Get();
}
inline void LinuxContainerSecurityContext::_internal_set_seccomp_profile_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.seccomp_profile_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL LinuxContainerSecurityContext::_internal_mutable_seccomp_profile_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.seccomp_profile_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE LinuxContainerSecurityContext::release_seccomp_profile_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp_profile_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000020U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  auto* released = _impl_.seccomp_profile_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.seccomp_profile_path_.Set("", GetArena());
  }
  return released;
}
inline void LinuxContainerSecurityContext::set_allocated_seccomp_profile_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  _impl_.seccomp_profile_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.seccomp_profile_path_.IsDefault()) {
    _impl_.seccomp_profile_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerSecurityContext.seccomp_profile_path)
}

// -------------------------------------------------------------------

// LinuxContainerConfig

// .runtime.v1alpha2.LinuxContainerResources resources = 1;
inline bool LinuxContainerConfig::has_resources() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.resources_ != nullptr);
  return value;
}
inline void LinuxContainerConfig::clear_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.resources_ != nullptr) _impl_.resources_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::LinuxContainerResources& LinuxContainerConfig::_internal_resources() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxContainerResources* p = _impl_.resources_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxContainerResources&>(::runtime::v1alpha2::_LinuxContainerResources_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxContainerResources& LinuxContainerConfig::resources() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerConfig.resources)
  return _internal_resources();
}
inline void LinuxContainerConfig::unsafe_arena_set_allocated_resources(
    ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resources_);
  }
  _impl_.resources_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerResources*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerConfig.resources)
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE LinuxContainerConfig::release_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::LinuxContainerResources* released = _impl_.resources_;
  _impl_.resources_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE LinuxContainerConfig::unsafe_arena_release_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerConfig.resources)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::LinuxContainerResources* temp = _impl_.resources_;
  _impl_.resources_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL LinuxContainerConfig::_internal_mutable_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.resources_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxContainerResources>(GetArena());
    _impl_.resources_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerResources*>(p);
  }
  return _impl_.resources_;
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL LinuxContainerConfig::mutable_resources()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::LinuxContainerResources* _msg = _internal_mutable_resources();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerConfig.resources)
  return _msg;
}
inline void LinuxContainerConfig::set_allocated_resources(::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resources_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.resources_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerResources*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerConfig.resources)
}

// .runtime.v1alpha2.LinuxContainerSecurityContext security_context = 2;
inline bool LinuxContainerConfig::has_security_context() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.security_context_ != nullptr);
  return value;
}
inline void LinuxContainerConfig::clear_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.security_context_ != nullptr) _impl_.security_context_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::LinuxContainerSecurityContext& LinuxContainerConfig::_internal_security_context() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxContainerSecurityContext* p = _impl_.security_context_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxContainerSecurityContext&>(::runtime::v1alpha2::_LinuxContainerSecurityContext_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxContainerSecurityContext& LinuxContainerConfig::security_context() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.LinuxContainerConfig.security_context)
  return _internal_security_context();
}
inline void LinuxContainerConfig::unsafe_arena_set_allocated_security_context(
    ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.security_context_);
  }
  _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerSecurityContext*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.LinuxContainerConfig.security_context)
}
inline ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE LinuxContainerConfig::release_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxContainerSecurityContext* released = _impl_.security_context_;
  _impl_.security_context_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE LinuxContainerConfig::unsafe_arena_release_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.LinuxContainerConfig.security_context)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxContainerSecurityContext* temp = _impl_.security_context_;
  _impl_.security_context_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NONNULL LinuxContainerConfig::_internal_mutable_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.security_context_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxContainerSecurityContext>(GetArena());
    _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerSecurityContext*>(p);
  }
  return _impl_.security_context_;
}
inline ::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NONNULL LinuxContainerConfig::mutable_security_context()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxContainerSecurityContext* _msg = _internal_mutable_security_context();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.LinuxContainerConfig.security_context)
  return _msg;
}
inline void LinuxContainerConfig::set_allocated_security_context(::runtime::v1alpha2::LinuxContainerSecurityContext* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.security_context_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerSecurityContext*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.LinuxContainerConfig.security_context)
}

// -------------------------------------------------------------------

// WindowsContainerSecurityContext

// string run_as_username = 1;
inline void WindowsContainerSecurityContext::clear_run_as_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.run_as_username_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& WindowsContainerSecurityContext::run_as_username() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerSecurityContext.run_as_username)
  return _internal_run_as_username();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void WindowsContainerSecurityContext::set_run_as_username(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.run_as_username_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.WindowsContainerSecurityContext.run_as_username)
}
inline ::std::string* PROTOBUF_NONNULL WindowsContainerSecurityContext::mutable_run_as_username()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_run_as_username();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.WindowsContainerSecurityContext.run_as_username)
  return _s;
}
inline const ::std::string& WindowsContainerSecurityContext::_internal_run_as_username() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.run_as_username_.Get();
}
inline void WindowsContainerSecurityContext::_internal_set_run_as_username(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.run_as_username_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL WindowsContainerSecurityContext::_internal_mutable_run_as_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.run_as_username_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE WindowsContainerSecurityContext::release_run_as_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.WindowsContainerSecurityContext.run_as_username)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.run_as_username_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.run_as_username_.Set("", GetArena());
  }
  return released;
}
inline void WindowsContainerSecurityContext::set_allocated_run_as_username(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.run_as_username_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.run_as_username_.IsDefault()) {
    _impl_.run_as_username_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.WindowsContainerSecurityContext.run_as_username)
}

// string credential_spec = 2;
inline void WindowsContainerSecurityContext::clear_credential_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.credential_spec_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& WindowsContainerSecurityContext::credential_spec() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerSecurityContext.credential_spec)
  return _internal_credential_spec();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void WindowsContainerSecurityContext::set_credential_spec(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.credential_spec_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.WindowsContainerSecurityContext.credential_spec)
}
inline ::std::string* PROTOBUF_NONNULL WindowsContainerSecurityContext::mutable_credential_spec()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_credential_spec();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.WindowsContainerSecurityContext.credential_spec)
  return _s;
}
inline const ::std::string& WindowsContainerSecurityContext::_internal_credential_spec() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.credential_spec_.Get();
}
inline void WindowsContainerSecurityContext::_internal_set_credential_spec(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.credential_spec_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL WindowsContainerSecurityContext::_internal_mutable_credential_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.credential_spec_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE WindowsContainerSecurityContext::release_credential_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.WindowsContainerSecurityContext.credential_spec)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.credential_spec_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.credential_spec_.Set("", GetArena());
  }
  return released;
}
inline void WindowsContainerSecurityContext::set_allocated_credential_spec(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.credential_spec_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.credential_spec_.IsDefault()) {
    _impl_.credential_spec_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.WindowsContainerSecurityContext.credential_spec)
}

// -------------------------------------------------------------------

// WindowsContainerConfig

// .runtime.v1alpha2.WindowsContainerResources resources = 1;
inline bool WindowsContainerConfig::has_resources() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.resources_ != nullptr);
  return value;
}
inline void WindowsContainerConfig::clear_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.resources_ != nullptr) _impl_.resources_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::WindowsContainerResources& WindowsContainerConfig::_internal_resources() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::WindowsContainerResources* p = _impl_.resources_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::WindowsContainerResources&>(::runtime::v1alpha2::_WindowsContainerResources_default_instance_);
}
inline const ::runtime::v1alpha2::WindowsContainerResources& WindowsContainerConfig::resources() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerConfig.resources)
  return _internal_resources();
}
inline void WindowsContainerConfig::unsafe_arena_set_allocated_resources(
    ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resources_);
  }
  _impl_.resources_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerResources*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.WindowsContainerConfig.resources)
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE WindowsContainerConfig::release_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::WindowsContainerResources* released = _impl_.resources_;
  _impl_.resources_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE WindowsContainerConfig::unsafe_arena_release_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.WindowsContainerConfig.resources)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::WindowsContainerResources* temp = _impl_.resources_;
  _impl_.resources_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL WindowsContainerConfig::_internal_mutable_resources() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.resources_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::WindowsContainerResources>(GetArena());
    _impl_.resources_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerResources*>(p);
  }
  return _impl_.resources_;
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL WindowsContainerConfig::mutable_resources()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::WindowsContainerResources* _msg = _internal_mutable_resources();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.WindowsContainerConfig.resources)
  return _msg;
}
inline void WindowsContainerConfig::set_allocated_resources(::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.resources_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.resources_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerResources*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.WindowsContainerConfig.resources)
}

// .runtime.v1alpha2.WindowsContainerSecurityContext security_context = 2;
inline bool WindowsContainerConfig::has_security_context() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.security_context_ != nullptr);
  return value;
}
inline void WindowsContainerConfig::clear_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.security_context_ != nullptr) _impl_.security_context_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::WindowsContainerSecurityContext& WindowsContainerConfig::_internal_security_context() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::WindowsContainerSecurityContext* p = _impl_.security_context_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::WindowsContainerSecurityContext&>(::runtime::v1alpha2::_WindowsContainerSecurityContext_default_instance_);
}
inline const ::runtime::v1alpha2::WindowsContainerSecurityContext& WindowsContainerConfig::security_context() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerConfig.security_context)
  return _internal_security_context();
}
inline void WindowsContainerConfig::unsafe_arena_set_allocated_security_context(
    ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.security_context_);
  }
  _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerSecurityContext*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.WindowsContainerConfig.security_context)
}
inline ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE WindowsContainerConfig::release_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::WindowsContainerSecurityContext* released = _impl_.security_context_;
  _impl_.security_context_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE WindowsContainerConfig::unsafe_arena_release_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.WindowsContainerConfig.security_context)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::WindowsContainerSecurityContext* temp = _impl_.security_context_;
  _impl_.security_context_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NONNULL WindowsContainerConfig::_internal_mutable_security_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.security_context_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::WindowsContainerSecurityContext>(GetArena());
    _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerSecurityContext*>(p);
  }
  return _impl_.security_context_;
}
inline ::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NONNULL WindowsContainerConfig::mutable_security_context()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::WindowsContainerSecurityContext* _msg = _internal_mutable_security_context();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.WindowsContainerConfig.security_context)
  return _msg;
}
inline void WindowsContainerConfig::set_allocated_security_context(::runtime::v1alpha2::WindowsContainerSecurityContext* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.security_context_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.security_context_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerSecurityContext*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.WindowsContainerConfig.security_context)
}

// -------------------------------------------------------------------

// WindowsContainerResources

// int64 cpu_shares = 1;
inline void WindowsContainerResources::clear_cpu_shares() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_shares_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::int64_t WindowsContainerResources::cpu_shares() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerResources.cpu_shares)
  return _internal_cpu_shares();
}
inline void WindowsContainerResources::set_cpu_shares(::int64_t value) {
  _internal_set_cpu_shares(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.WindowsContainerResources.cpu_shares)
}
inline ::int64_t WindowsContainerResources::_internal_cpu_shares() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_shares_;
}
inline void WindowsContainerResources::_internal_set_cpu_shares(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_shares_ = value;
}

// int64 cpu_count = 2;
inline void WindowsContainerResources::clear_cpu_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_count_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int64_t WindowsContainerResources::cpu_count() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerResources.cpu_count)
  return _internal_cpu_count();
}
inline void WindowsContainerResources::set_cpu_count(::int64_t value) {
  _internal_set_cpu_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.WindowsContainerResources.cpu_count)
}
inline ::int64_t WindowsContainerResources::_internal_cpu_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_count_;
}
inline void WindowsContainerResources::_internal_set_cpu_count(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_count_ = value;
}

// int64 cpu_maximum = 3;
inline void WindowsContainerResources::clear_cpu_maximum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_maximum_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::int64_t WindowsContainerResources::cpu_maximum() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerResources.cpu_maximum)
  return _internal_cpu_maximum();
}
inline void WindowsContainerResources::set_cpu_maximum(::int64_t value) {
  _internal_set_cpu_maximum(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.WindowsContainerResources.cpu_maximum)
}
inline ::int64_t WindowsContainerResources::_internal_cpu_maximum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cpu_maximum_;
}
inline void WindowsContainerResources::_internal_set_cpu_maximum(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cpu_maximum_ = value;
}

// int64 memory_limit_in_bytes = 4;
inline void WindowsContainerResources::clear_memory_limit_in_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.memory_limit_in_bytes_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::int64_t WindowsContainerResources::memory_limit_in_bytes() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.WindowsContainerResources.memory_limit_in_bytes)
  return _internal_memory_limit_in_bytes();
}
inline void WindowsContainerResources::set_memory_limit_in_bytes(::int64_t value) {
  _internal_set_memory_limit_in_bytes(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.WindowsContainerResources.memory_limit_in_bytes)
}
inline ::int64_t WindowsContainerResources::_internal_memory_limit_in_bytes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.memory_limit_in_bytes_;
}
inline void WindowsContainerResources::_internal_set_memory_limit_in_bytes(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.memory_limit_in_bytes_ = value;
}

// -------------------------------------------------------------------

// ContainerMetadata

// string name = 1;
inline void ContainerMetadata::clear_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ContainerMetadata::name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerMetadata.name)
  return _internal_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerMetadata::set_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerMetadata.name)
}
inline ::std::string* PROTOBUF_NONNULL ContainerMetadata::mutable_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerMetadata.name)
  return _s;
}
inline const ::std::string& ContainerMetadata::_internal_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.name_.Get();
}
inline void ContainerMetadata::_internal_set_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerMetadata::_internal_mutable_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerMetadata::release_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerMetadata.name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.name_.Set("", GetArena());
  }
  return released;
}
inline void ContainerMetadata::set_allocated_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.name_.IsDefault()) {
    _impl_.name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerMetadata.name)
}

// uint32 attempt = 2;
inline void ContainerMetadata::clear_attempt() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.attempt_ = 0u;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::uint32_t ContainerMetadata::attempt() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerMetadata.attempt)
  return _internal_attempt();
}
inline void ContainerMetadata::set_attempt(::uint32_t value) {
  _internal_set_attempt(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerMetadata.attempt)
}
inline ::uint32_t ContainerMetadata::_internal_attempt() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.attempt_;
}
inline void ContainerMetadata::_internal_set_attempt(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.attempt_ = value;
}

// -------------------------------------------------------------------

// Device

// string container_path = 1;
inline void Device::clear_container_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& Device::container_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Device.container_path)
  return _internal_container_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_container_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Device.container_path)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_container_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Device.container_path)
  return _s;
}
inline const ::std::string& Device::_internal_container_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_path_.Get();
}
inline void Device::_internal_set_container_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_container_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_container_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Device.container_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_path_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_container_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_path_.IsDefault()) {
    _impl_.container_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Device.container_path)
}

// string host_path = 2;
inline void Device::clear_host_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& Device::host_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Device.host_path)
  return _internal_host_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_host_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.host_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Device.host_path)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_host_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_host_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Device.host_path)
  return _s;
}
inline const ::std::string& Device::_internal_host_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.host_path_.Get();
}
inline void Device::_internal_set_host_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.host_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_host_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.host_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_host_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Device.host_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.host_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.host_path_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_host_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.host_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.host_path_.IsDefault()) {
    _impl_.host_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Device.host_path)
}

// string permissions = 3;
inline void Device::clear_permissions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.permissions_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& Device::permissions() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Device.permissions)
  return _internal_permissions();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Device::set_permissions(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.permissions_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Device.permissions)
}
inline ::std::string* PROTOBUF_NONNULL Device::mutable_permissions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_permissions();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Device.permissions)
  return _s;
}
inline const ::std::string& Device::_internal_permissions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.permissions_.Get();
}
inline void Device::_internal_set_permissions(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.permissions_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Device::_internal_mutable_permissions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.permissions_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Device::release_permissions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Device.permissions)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.permissions_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.permissions_.Set("", GetArena());
  }
  return released;
}
inline void Device::set_allocated_permissions(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.permissions_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.permissions_.IsDefault()) {
    _impl_.permissions_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Device.permissions)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ContainerConfig

// .runtime.v1alpha2.ContainerMetadata metadata = 1;
inline bool ContainerConfig::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void ContainerConfig::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::runtime::v1alpha2::ContainerMetadata& ContainerConfig::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerMetadata&>(::runtime::v1alpha2::_ContainerMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerMetadata& ContainerConfig::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.metadata)
  return _internal_metadata();
}
inline void ContainerConfig::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerConfig.metadata)
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE ContainerConfig::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::ContainerMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE ContainerConfig::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerConfig.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::ContainerMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL ContainerConfig::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::ContainerMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.metadata)
  return _msg;
}
inline void ContainerConfig::set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerConfig.metadata)
}

// .runtime.v1alpha2.ImageSpec image = 2;
inline bool ContainerConfig::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000100U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void ContainerConfig::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000100U);
}
inline const ::runtime::v1alpha2::ImageSpec& ContainerConfig::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& ContainerConfig::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.image)
  return _internal_image();
}
inline void ContainerConfig::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerConfig.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ContainerConfig::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ContainerConfig::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerConfig.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ContainerConfig::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.image)
  return _msg;
}
inline void ContainerConfig::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerConfig.image)
}

// repeated string command = 3;
inline int ContainerConfig::_internal_command_size() const {
  return _internal_command().size();
}
inline int ContainerConfig::command_size() const {
  return _internal_command_size();
}
inline void ContainerConfig::clear_command() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.command_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::add_command()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_command()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.ContainerConfig.command)
  return _s;
}
inline const ::std::string& ContainerConfig::command(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.command)
  return _internal_command().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::mutable_command(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.command)
  return _internal_mutable_command()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ContainerConfig::set_command(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_command()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.command)
}
template <typename Arg_, typename... Args_>
inline void ContainerConfig::add_command(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_command(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ContainerConfig.command)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ContainerConfig::command()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ContainerConfig.command)
  return _internal_command();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ContainerConfig::mutable_command() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ContainerConfig.command)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_command();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ContainerConfig::_internal_command() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.command_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ContainerConfig::_internal_mutable_command() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.command_;
}

// repeated string args = 4;
inline int ContainerConfig::_internal_args_size() const {
  return _internal_args().size();
}
inline int ContainerConfig::args_size() const {
  return _internal_args_size();
}
inline void ContainerConfig::clear_args() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.args_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::add_args()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_args()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.ContainerConfig.args)
  return _s;
}
inline const ::std::string& ContainerConfig::args(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.args)
  return _internal_args().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::mutable_args(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.args)
  return _internal_mutable_args()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ContainerConfig::set_args(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_args()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.args)
}
template <typename Arg_, typename... Args_>
inline void ContainerConfig::add_args(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_args(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ContainerConfig.args)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ContainerConfig::args()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ContainerConfig.args)
  return _internal_args();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ContainerConfig::mutable_args() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ContainerConfig.args)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_args();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ContainerConfig::_internal_args() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.args_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ContainerConfig::_internal_mutable_args() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.args_;
}

// string working_dir = 5;
inline void ContainerConfig::clear_working_dir() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.working_dir_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::std::string& ContainerConfig::working_dir() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.working_dir)
  return _internal_working_dir();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerConfig::set_working_dir(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  _impl_.working_dir_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.working_dir)
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::mutable_working_dir()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::std::string* _s = _internal_mutable_working_dir();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.working_dir)
  return _s;
}
inline const ::std::string& ContainerConfig::_internal_working_dir() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.working_dir_.Get();
}
inline void ContainerConfig::_internal_set_working_dir(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.working_dir_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_working_dir() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.working_dir_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerConfig::release_working_dir() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerConfig.working_dir)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000020U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  auto* released = _impl_.working_dir_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.working_dir_.Set("", GetArena());
  }
  return released;
}
inline void ContainerConfig::set_allocated_working_dir(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  _impl_.working_dir_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.working_dir_.IsDefault()) {
    _impl_.working_dir_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerConfig.working_dir)
}

// repeated .runtime.v1alpha2.KeyValue envs = 6;
inline int ContainerConfig::_internal_envs_size() const {
  return _internal_envs().size();
}
inline int ContainerConfig::envs_size() const {
  return _internal_envs_size();
}
inline void ContainerConfig::clear_envs() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.envs_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::runtime::v1alpha2::KeyValue* PROTOBUF_NONNULL ContainerConfig::mutable_envs(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.envs)
  return _internal_mutable_envs()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>* PROTOBUF_NONNULL ContainerConfig::mutable_envs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ContainerConfig.envs)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_envs();
}
inline const ::runtime::v1alpha2::KeyValue& ContainerConfig::envs(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.envs)
  return _internal_envs().Get(index);
}
inline ::runtime::v1alpha2::KeyValue* PROTOBUF_NONNULL ContainerConfig::add_envs()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::KeyValue* _add =
      _internal_mutable_envs()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ContainerConfig.envs)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>& ContainerConfig::envs() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ContainerConfig.envs)
  return _internal_envs();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>&
ContainerConfig::_internal_envs() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.envs_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::KeyValue>* PROTOBUF_NONNULL
ContainerConfig::_internal_mutable_envs() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.envs_;
}

// repeated .runtime.v1alpha2.Mount mounts = 7;
inline int ContainerConfig::_internal_mounts_size() const {
  return _internal_mounts().size();
}
inline int ContainerConfig::mounts_size() const {
  return _internal_mounts_size();
}
inline void ContainerConfig::clear_mounts() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mounts_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL ContainerConfig::mutable_mounts(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.mounts)
  return _internal_mutable_mounts()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL ContainerConfig::mutable_mounts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ContainerConfig.mounts)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_mounts();
}
inline const ::runtime::v1alpha2::Mount& ContainerConfig::mounts(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.mounts)
  return _internal_mounts().Get(index);
}
inline ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL ContainerConfig::add_mounts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::Mount* _add =
      _internal_mutable_mounts()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ContainerConfig.mounts)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>& ContainerConfig::mounts() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ContainerConfig.mounts)
  return _internal_mounts();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>&
ContainerConfig::_internal_mounts() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mounts_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL
ContainerConfig::_internal_mutable_mounts() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.mounts_;
}

// repeated .runtime.v1alpha2.Device devices = 8;
inline int ContainerConfig::_internal_devices_size() const {
  return _internal_devices().size();
}
inline int ContainerConfig::devices_size() const {
  return _internal_devices_size();
}
inline void ContainerConfig::clear_devices() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.devices_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::runtime::v1alpha2::Device* PROTOBUF_NONNULL ContainerConfig::mutable_devices(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.devices)
  return _internal_mutable_devices()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>* PROTOBUF_NONNULL ContainerConfig::mutable_devices()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ContainerConfig.devices)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_devices();
}
inline const ::runtime::v1alpha2::Device& ContainerConfig::devices(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.devices)
  return _internal_devices().Get(index);
}
inline ::runtime::v1alpha2::Device* PROTOBUF_NONNULL ContainerConfig::add_devices()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::Device* _add =
      _internal_mutable_devices()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ContainerConfig.devices)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>& ContainerConfig::devices() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ContainerConfig.devices)
  return _internal_devices();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>&
ContainerConfig::_internal_devices() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.devices_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Device>* PROTOBUF_NONNULL
ContainerConfig::_internal_mutable_devices() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.devices_;
}

// map<string, string> labels = 9;
inline int ContainerConfig::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int ContainerConfig::labels_size() const {
  return _internal_labels_size();
}
inline void ContainerConfig::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00004000U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerConfig::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerConfig::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerConfig.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerConfig::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00004000U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerConfig.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 10;
inline int ContainerConfig::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int ContainerConfig::annotations_size() const {
  return _internal_annotations_size();
}
inline void ContainerConfig::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00008000U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerConfig::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerConfig::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerConfig.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerConfig::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00008000U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerConfig.annotations)
  return _internal_mutable_annotations();
}

// string log_path = 11;
inline void ContainerConfig::clear_log_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline const ::std::string& ContainerConfig::log_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.log_path)
  return _internal_log_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerConfig::set_log_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  _impl_.log_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.log_path)
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::mutable_log_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::std::string* _s = _internal_mutable_log_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.log_path)
  return _s;
}
inline const ::std::string& ContainerConfig::_internal_log_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.log_path_.Get();
}
inline void ContainerConfig::_internal_set_log_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_log_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.log_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerConfig::release_log_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerConfig.log_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000040U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  auto* released = _impl_.log_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.log_path_.Set("", GetArena());
  }
  return released;
}
inline void ContainerConfig::set_allocated_log_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }
  _impl_.log_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.log_path_.IsDefault()) {
    _impl_.log_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerConfig.log_path)
}

// bool f_stdin = 12;
inline void ContainerConfig::clear_f_stdin() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdin_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000800U);
}
inline bool ContainerConfig::f_stdin() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.f_stdin)
  return _internal_f_stdin();
}
inline void ContainerConfig::set_f_stdin(bool value) {
  _internal_set_f_stdin(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.f_stdin)
}
inline bool ContainerConfig::_internal_f_stdin() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stdin_;
}
inline void ContainerConfig::_internal_set_f_stdin(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdin_ = value;
}

// bool stdin_once = 13;
inline void ContainerConfig::clear_stdin_once() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stdin_once_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00001000U);
}
inline bool ContainerConfig::stdin_once() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.stdin_once)
  return _internal_stdin_once();
}
inline void ContainerConfig::set_stdin_once(bool value) {
  _internal_set_stdin_once(value);
  SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.stdin_once)
}
inline bool ContainerConfig::_internal_stdin_once() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.stdin_once_;
}
inline void ContainerConfig::_internal_set_stdin_once(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stdin_once_ = value;
}

// bool tty = 14;
inline void ContainerConfig::clear_tty() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tty_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00002000U);
}
inline bool ContainerConfig::tty() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.tty)
  return _internal_tty();
}
inline void ContainerConfig::set_tty(bool value) {
  _internal_set_tty(value);
  SetHasBit(_impl_._has_bits_[0], 0x00002000U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerConfig.tty)
}
inline bool ContainerConfig::_internal_tty() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tty_;
}
inline void ContainerConfig::_internal_set_tty(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tty_ = value;
}

// .runtime.v1alpha2.LinuxContainerConfig linux = 15;
inline bool ContainerConfig::has_linux() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000200U);
  PROTOBUF_ASSUME(!value || _impl_.linux_ != nullptr);
  return value;
}
inline void ContainerConfig::clear_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ != nullptr) _impl_.linux_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000200U);
}
inline const ::runtime::v1alpha2::LinuxContainerConfig& ContainerConfig::_internal_linux() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxContainerConfig* p = _impl_.linux_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxContainerConfig&>(::runtime::v1alpha2::_LinuxContainerConfig_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxContainerConfig& ContainerConfig::linux() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.linux)
  return _internal_linux();
}
inline void ContainerConfig::unsafe_arena_set_allocated_linux(
    ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }
  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerConfig.linux)
}
inline ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE ContainerConfig::release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  ::runtime::v1alpha2::LinuxContainerConfig* released = _impl_.linux_;
  _impl_.linux_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE ContainerConfig::unsafe_arena_release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerConfig.linux)

  ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  ::runtime::v1alpha2::LinuxContainerConfig* temp = _impl_.linux_;
  _impl_.linux_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxContainerConfig>(GetArena());
    _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerConfig*>(p);
  }
  return _impl_.linux_;
}
inline ::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NONNULL ContainerConfig::mutable_linux()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  ::runtime::v1alpha2::LinuxContainerConfig* _msg = _internal_mutable_linux();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.linux)
  return _msg;
}
inline void ContainerConfig::set_allocated_linux(::runtime::v1alpha2::LinuxContainerConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000200U);
  }

  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerConfig.linux)
}

// .runtime.v1alpha2.WindowsContainerConfig windows = 16;
inline bool ContainerConfig::has_windows() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000400U);
  PROTOBUF_ASSUME(!value || _impl_.windows_ != nullptr);
  return value;
}
inline void ContainerConfig::clear_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.windows_ != nullptr) _impl_.windows_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000400U);
}
inline const ::runtime::v1alpha2::WindowsContainerConfig& ContainerConfig::_internal_windows() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::WindowsContainerConfig* p = _impl_.windows_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::WindowsContainerConfig&>(::runtime::v1alpha2::_WindowsContainerConfig_default_instance_);
}
inline const ::runtime::v1alpha2::WindowsContainerConfig& ContainerConfig::windows() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerConfig.windows)
  return _internal_windows();
}
inline void ContainerConfig::unsafe_arena_set_allocated_windows(
    ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.windows_);
  }
  _impl_.windows_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerConfig.windows)
}
inline ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE ContainerConfig::release_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  ::runtime::v1alpha2::WindowsContainerConfig* released = _impl_.windows_;
  _impl_.windows_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE ContainerConfig::unsafe_arena_release_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerConfig.windows)

  ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  ::runtime::v1alpha2::WindowsContainerConfig* temp = _impl_.windows_;
  _impl_.windows_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NONNULL ContainerConfig::_internal_mutable_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.windows_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::WindowsContainerConfig>(GetArena());
    _impl_.windows_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerConfig*>(p);
  }
  return _impl_.windows_;
}
inline ::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NONNULL ContainerConfig::mutable_windows()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  ::runtime::v1alpha2::WindowsContainerConfig* _msg = _internal_mutable_windows();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerConfig.windows)
  return _msg;
}
inline void ContainerConfig::set_allocated_windows(::runtime::v1alpha2::WindowsContainerConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.windows_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000400U);
  }

  _impl_.windows_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerConfig.windows)
}

// -------------------------------------------------------------------

// CreateContainerRequest

// string pod_sandbox_id = 1;
inline void CreateContainerRequest::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& CreateContainerRequest::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.CreateContainerRequest.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void CreateContainerRequest::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.CreateContainerRequest.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL CreateContainerRequest::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.CreateContainerRequest.pod_sandbox_id)
  return _s;
}
inline const ::std::string& CreateContainerRequest::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void CreateContainerRequest::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL CreateContainerRequest::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE CreateContainerRequest::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.CreateContainerRequest.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void CreateContainerRequest::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.CreateContainerRequest.pod_sandbox_id)
}

// .runtime.v1alpha2.ContainerConfig config = 2;
inline bool CreateContainerRequest::has_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.config_ != nullptr);
  return value;
}
inline void CreateContainerRequest::clear_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.config_ != nullptr) _impl_.config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::ContainerConfig& CreateContainerRequest::_internal_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerConfig* p = _impl_.config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerConfig&>(::runtime::v1alpha2::_ContainerConfig_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerConfig& CreateContainerRequest::config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.CreateContainerRequest.config)
  return _internal_config();
}
inline void CreateContainerRequest::unsafe_arena_set_allocated_config(
    ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.config_);
  }
  _impl_.config_ = reinterpret_cast<::runtime::v1alpha2::ContainerConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.CreateContainerRequest.config)
}
inline ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE CreateContainerRequest::release_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::ContainerConfig* released = _impl_.config_;
  _impl_.config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE CreateContainerRequest::unsafe_arena_release_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.CreateContainerRequest.config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::ContainerConfig* temp = _impl_.config_;
  _impl_.config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NONNULL CreateContainerRequest::_internal_mutable_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerConfig>(GetArena());
    _impl_.config_ = reinterpret_cast<::runtime::v1alpha2::ContainerConfig*>(p);
  }
  return _impl_.config_;
}
inline ::runtime::v1alpha2::ContainerConfig* PROTOBUF_NONNULL CreateContainerRequest::mutable_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::ContainerConfig* _msg = _internal_mutable_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.CreateContainerRequest.config)
  return _msg;
}
inline void CreateContainerRequest::set_allocated_config(::runtime::v1alpha2::ContainerConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.config_ = reinterpret_cast<::runtime::v1alpha2::ContainerConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.CreateContainerRequest.config)
}

// .runtime.v1alpha2.PodSandboxConfig sandbox_config = 3;
inline bool CreateContainerRequest::has_sandbox_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.sandbox_config_ != nullptr);
  return value;
}
inline void CreateContainerRequest::clear_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.sandbox_config_ != nullptr) _impl_.sandbox_config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::PodSandboxConfig& CreateContainerRequest::_internal_sandbox_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxConfig* p = _impl_.sandbox_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxConfig&>(::runtime::v1alpha2::_PodSandboxConfig_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxConfig& CreateContainerRequest::sandbox_config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.CreateContainerRequest.sandbox_config)
  return _internal_sandbox_config();
}
inline void CreateContainerRequest::unsafe_arena_set_allocated_sandbox_config(
    ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.sandbox_config_);
  }
  _impl_.sandbox_config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.CreateContainerRequest.sandbox_config)
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE CreateContainerRequest::release_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxConfig* released = _impl_.sandbox_config_;
  _impl_.sandbox_config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE CreateContainerRequest::unsafe_arena_release_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.CreateContainerRequest.sandbox_config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxConfig* temp = _impl_.sandbox_config_;
  _impl_.sandbox_config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL CreateContainerRequest::_internal_mutable_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.sandbox_config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxConfig>(GetArena());
    _impl_.sandbox_config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(p);
  }
  return _impl_.sandbox_config_;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL CreateContainerRequest::mutable_sandbox_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxConfig* _msg = _internal_mutable_sandbox_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.CreateContainerRequest.sandbox_config)
  return _msg;
}
inline void CreateContainerRequest::set_allocated_sandbox_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.sandbox_config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.sandbox_config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.CreateContainerRequest.sandbox_config)
}

// -------------------------------------------------------------------

// CreateContainerResponse

// string container_id = 1;
inline void CreateContainerResponse::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& CreateContainerResponse::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.CreateContainerResponse.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void CreateContainerResponse::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.CreateContainerResponse.container_id)
}
inline ::std::string* PROTOBUF_NONNULL CreateContainerResponse::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.CreateContainerResponse.container_id)
  return _s;
}
inline const ::std::string& CreateContainerResponse::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void CreateContainerResponse::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL CreateContainerResponse::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE CreateContainerResponse::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.CreateContainerResponse.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void CreateContainerResponse::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.CreateContainerResponse.container_id)
}

// -------------------------------------------------------------------

// StartContainerRequest

// string container_id = 1;
inline void StartContainerRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& StartContainerRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.StartContainerRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void StartContainerRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.StartContainerRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL StartContainerRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.StartContainerRequest.container_id)
  return _s;
}
inline const ::std::string& StartContainerRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void StartContainerRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL StartContainerRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE StartContainerRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.StartContainerRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void StartContainerRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.StartContainerRequest.container_id)
}

// -------------------------------------------------------------------

// StartContainerResponse

// -------------------------------------------------------------------

// StopContainerRequest

// string container_id = 1;
inline void StopContainerRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& StopContainerRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.StopContainerRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void StopContainerRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.StopContainerRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL StopContainerRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.StopContainerRequest.container_id)
  return _s;
}
inline const ::std::string& StopContainerRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void StopContainerRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL StopContainerRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE StopContainerRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.StopContainerRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void StopContainerRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.StopContainerRequest.container_id)
}

// int64 timeout = 2;
inline void StopContainerRequest::clear_timeout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timeout_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int64_t StopContainerRequest::timeout() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.StopContainerRequest.timeout)
  return _internal_timeout();
}
inline void StopContainerRequest::set_timeout(::int64_t value) {
  _internal_set_timeout(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.StopContainerRequest.timeout)
}
inline ::int64_t StopContainerRequest::_internal_timeout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.timeout_;
}
inline void StopContainerRequest::_internal_set_timeout(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timeout_ = value;
}

// -------------------------------------------------------------------

// StopContainerResponse

// -------------------------------------------------------------------

// RemoveContainerRequest

// string container_id = 1;
inline void RemoveContainerRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& RemoveContainerRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RemoveContainerRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RemoveContainerRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RemoveContainerRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL RemoveContainerRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RemoveContainerRequest.container_id)
  return _s;
}
inline const ::std::string& RemoveContainerRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void RemoveContainerRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RemoveContainerRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RemoveContainerRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RemoveContainerRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void RemoveContainerRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RemoveContainerRequest.container_id)
}

// -------------------------------------------------------------------

// RemoveContainerResponse

// -------------------------------------------------------------------

// ContainerStateValue

// .runtime.v1alpha2.ContainerState state = 1;
inline void ContainerStateValue::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::ContainerState ContainerStateValue::state() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStateValue.state)
  return _internal_state();
}
inline void ContainerStateValue::set_state(::runtime::v1alpha2::ContainerState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStateValue.state)
}
inline ::runtime::v1alpha2::ContainerState ContainerStateValue::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::ContainerState>(_impl_.state_);
}
inline void ContainerStateValue::_internal_set_state(::runtime::v1alpha2::ContainerState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ContainerFilter

// string id = 1;
inline void ContainerFilter::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ContainerFilter::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerFilter.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerFilter::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerFilter.id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerFilter::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerFilter.id)
  return _s;
}
inline const ::std::string& ContainerFilter::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void ContainerFilter::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerFilter::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerFilter::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerFilter.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerFilter::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerFilter.id)
}

// .runtime.v1alpha2.ContainerStateValue state = 2;
inline bool ContainerFilter::has_state() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.state_ != nullptr);
  return value;
}
inline void ContainerFilter::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.state_ != nullptr) _impl_.state_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::ContainerStateValue& ContainerFilter::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerStateValue* p = _impl_.state_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerStateValue&>(::runtime::v1alpha2::_ContainerStateValue_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerStateValue& ContainerFilter::state() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerFilter.state)
  return _internal_state();
}
inline void ContainerFilter::unsafe_arena_set_allocated_state(
    ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.state_);
  }
  _impl_.state_ = reinterpret_cast<::runtime::v1alpha2::ContainerStateValue*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerFilter.state)
}
inline ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE ContainerFilter::release_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::ContainerStateValue* released = _impl_.state_;
  _impl_.state_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE ContainerFilter::unsafe_arena_release_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerFilter.state)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::ContainerStateValue* temp = _impl_.state_;
  _impl_.state_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NONNULL ContainerFilter::_internal_mutable_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.state_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerStateValue>(GetArena());
    _impl_.state_ = reinterpret_cast<::runtime::v1alpha2::ContainerStateValue*>(p);
  }
  return _impl_.state_;
}
inline ::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NONNULL ContainerFilter::mutable_state()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::ContainerStateValue* _msg = _internal_mutable_state();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerFilter.state)
  return _msg;
}
inline void ContainerFilter::set_allocated_state(::runtime::v1alpha2::ContainerStateValue* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.state_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.state_ = reinterpret_cast<::runtime::v1alpha2::ContainerStateValue*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerFilter.state)
}

// string pod_sandbox_id = 3;
inline void ContainerFilter::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& ContainerFilter::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerFilter.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerFilter::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerFilter.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerFilter::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerFilter.pod_sandbox_id)
  return _s;
}
inline const ::std::string& ContainerFilter::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void ContainerFilter::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerFilter::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerFilter::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerFilter.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerFilter::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerFilter.pod_sandbox_id)
}

// map<string, string> label_selector = 4;
inline int ContainerFilter::_internal_label_selector_size() const {
  return _internal_label_selector().size();
}
inline int ContainerFilter::label_selector_size() const {
  return _internal_label_selector_size();
}
inline void ContainerFilter::clear_label_selector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_selector_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerFilter::_internal_label_selector() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.label_selector_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerFilter::label_selector() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerFilter.label_selector)
  return _internal_label_selector();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerFilter::_internal_mutable_label_selector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.label_selector_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerFilter::mutable_label_selector()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerFilter.label_selector)
  return _internal_mutable_label_selector();
}

// -------------------------------------------------------------------

// ListContainersRequest

// .runtime.v1alpha2.ContainerFilter filter = 1;
inline bool ListContainersRequest::has_filter() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.filter_ != nullptr);
  return value;
}
inline void ListContainersRequest::clear_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ != nullptr) _impl_.filter_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ContainerFilter& ListContainersRequest::_internal_filter() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerFilter* p = _impl_.filter_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerFilter&>(::runtime::v1alpha2::_ContainerFilter_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerFilter& ListContainersRequest::filter() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListContainersRequest.filter)
  return _internal_filter();
}
inline void ListContainersRequest::unsafe_arena_set_allocated_filter(
    ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }
  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ContainerFilter*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ListContainersRequest.filter)
}
inline ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE ListContainersRequest::release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerFilter* released = _impl_.filter_;
  _impl_.filter_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE ListContainersRequest::unsafe_arena_release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ListContainersRequest.filter)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerFilter* temp = _impl_.filter_;
  _impl_.filter_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NONNULL ListContainersRequest::_internal_mutable_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerFilter>(GetArena());
    _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ContainerFilter*>(p);
  }
  return _impl_.filter_;
}
inline ::runtime::v1alpha2::ContainerFilter* PROTOBUF_NONNULL ListContainersRequest::mutable_filter()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerFilter* _msg = _internal_mutable_filter();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListContainersRequest.filter)
  return _msg;
}
inline void ListContainersRequest::set_allocated_filter(::runtime::v1alpha2::ContainerFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ContainerFilter*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ListContainersRequest.filter)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// Container

// string id = 1;
inline void Container::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& Container::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Container::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Container.id)
}
inline ::std::string* PROTOBUF_NONNULL Container::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Container.id)
  return _s;
}
inline const ::std::string& Container::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void Container::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Container::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Container::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Container.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void Container::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Container.id)
}

// string pod_sandbox_id = 2;
inline void Container::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& Container::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Container::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Container.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL Container::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Container.pod_sandbox_id)
  return _s;
}
inline const ::std::string& Container::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void Container::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Container::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Container::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Container.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void Container::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Container.pod_sandbox_id)
}

// .runtime.v1alpha2.ContainerMetadata metadata = 3;
inline bool Container::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void Container::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::runtime::v1alpha2::ContainerMetadata& Container::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerMetadata&>(::runtime::v1alpha2::_ContainerMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerMetadata& Container::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.metadata)
  return _internal_metadata();
}
inline void Container::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.Container.metadata)
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE Container::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::ContainerMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE Container::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Container.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::ContainerMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL Container::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL Container::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::ContainerMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Container.metadata)
  return _msg;
}
inline void Container::set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Container.metadata)
}

// .runtime.v1alpha2.ImageSpec image = 4;
inline bool Container::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void Container::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::runtime::v1alpha2::ImageSpec& Container::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& Container::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.image)
  return _internal_image();
}
inline void Container::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.Container.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE Container::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE Container::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Container.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL Container::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL Container::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Container.image)
  return _msg;
}
inline void Container::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Container.image)
}

// string image_ref = 5;
inline void Container::clear_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_ref_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& Container::image_ref() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.image_ref)
  return _internal_image_ref();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Container::set_image_ref(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.image_ref_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Container.image_ref)
}
inline ::std::string* PROTOBUF_NONNULL Container::mutable_image_ref()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_image_ref();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Container.image_ref)
  return _s;
}
inline const ::std::string& Container::_internal_image_ref() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.image_ref_.Get();
}
inline void Container::_internal_set_image_ref(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_ref_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Container::_internal_mutable_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.image_ref_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Container::release_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Container.image_ref)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.image_ref_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.image_ref_.Set("", GetArena());
  }
  return released;
}
inline void Container::set_allocated_image_ref(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.image_ref_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.image_ref_.IsDefault()) {
    _impl_.image_ref_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Container.image_ref)
}

// .runtime.v1alpha2.ContainerState state = 6;
inline void Container::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline ::runtime::v1alpha2::ContainerState Container::state() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.state)
  return _internal_state();
}
inline void Container::set_state(::runtime::v1alpha2::ContainerState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Container.state)
}
inline ::runtime::v1alpha2::ContainerState Container::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::ContainerState>(_impl_.state_);
}
inline void Container::_internal_set_state(::runtime::v1alpha2::ContainerState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// int64 created_at = 7;
inline void Container::clear_created_at() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline ::int64_t Container::created_at() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Container.created_at)
  return _internal_created_at();
}
inline void Container::set_created_at(::int64_t value) {
  _internal_set_created_at(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Container.created_at)
}
inline ::int64_t Container::_internal_created_at() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_at_;
}
inline void Container::_internal_set_created_at(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = value;
}

// map<string, string> labels = 8;
inline int Container::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int Container::labels_size() const {
  return _internal_labels_size();
}
inline void Container::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& Container::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& Container::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.Container.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL Container::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL Container::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.Container.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 9;
inline int Container::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int Container::annotations_size() const {
  return _internal_annotations_size();
}
inline void Container::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000100U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& Container::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& Container::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.Container.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL Container::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL Container::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.Container.annotations)
  return _internal_mutable_annotations();
}

// -------------------------------------------------------------------

// ListContainersResponse

// repeated .runtime.v1alpha2.Container containers = 1;
inline int ListContainersResponse::_internal_containers_size() const {
  return _internal_containers().size();
}
inline int ListContainersResponse::containers_size() const {
  return _internal_containers_size();
}
inline void ListContainersResponse::clear_containers() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.containers_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::Container* PROTOBUF_NONNULL ListContainersResponse::mutable_containers(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListContainersResponse.containers)
  return _internal_mutable_containers()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>* PROTOBUF_NONNULL ListContainersResponse::mutable_containers()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ListContainersResponse.containers)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_containers();
}
inline const ::runtime::v1alpha2::Container& ListContainersResponse::containers(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListContainersResponse.containers)
  return _internal_containers().Get(index);
}
inline ::runtime::v1alpha2::Container* PROTOBUF_NONNULL ListContainersResponse::add_containers()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::Container* _add =
      _internal_mutable_containers()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ListContainersResponse.containers)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>& ListContainersResponse::containers() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ListContainersResponse.containers)
  return _internal_containers();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>&
ListContainersResponse::_internal_containers() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.containers_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Container>* PROTOBUF_NONNULL
ListContainersResponse::_internal_mutable_containers() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.containers_;
}

// -------------------------------------------------------------------

// ContainerStatusRequest

// string container_id = 1;
inline void ContainerStatusRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ContainerStatusRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatusRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatusRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatusRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatusRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatusRequest.container_id)
  return _s;
}
inline const ::std::string& ContainerStatusRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void ContainerStatusRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatusRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatusRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatusRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatusRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatusRequest.container_id)
}

// bool verbose = 2;
inline void ContainerStatusRequest::clear_verbose() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline bool ContainerStatusRequest::verbose() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatusRequest.verbose)
  return _internal_verbose();
}
inline void ContainerStatusRequest::set_verbose(bool value) {
  _internal_set_verbose(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatusRequest.verbose)
}
inline bool ContainerStatusRequest::_internal_verbose() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.verbose_;
}
inline void ContainerStatusRequest::_internal_set_verbose(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ContainerStatus

// string id = 1;
inline void ContainerStatus::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& ContainerStatus::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatus::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.id)
  return _s;
}
inline const ::std::string& ContainerStatus::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void ContainerStatus::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatus::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatus::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.id)
}

// .runtime.v1alpha2.ContainerMetadata metadata = 2;
inline bool ContainerStatus::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void ContainerStatus::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline const ::runtime::v1alpha2::ContainerMetadata& ContainerStatus::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerMetadata&>(::runtime::v1alpha2::_ContainerMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerMetadata& ContainerStatus::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.metadata)
  return _internal_metadata();
}
inline void ContainerStatus::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStatus.metadata)
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE ContainerStatus::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::ContainerMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE ContainerStatus::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::ContainerMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL ContainerStatus::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  ::runtime::v1alpha2::ContainerMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.metadata)
  return _msg;
}
inline void ContainerStatus::set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.metadata)
}

// .runtime.v1alpha2.ContainerState state = 3;
inline void ContainerStatus::clear_state() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000400U);
}
inline ::runtime::v1alpha2::ContainerState ContainerStatus::state() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.state)
  return _internal_state();
}
inline void ContainerStatus::set_state(::runtime::v1alpha2::ContainerState value) {
  _internal_set_state(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000400U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.state)
}
inline ::runtime::v1alpha2::ContainerState ContainerStatus::_internal_state() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::runtime::v1alpha2::ContainerState>(_impl_.state_);
}
inline void ContainerStatus::_internal_set_state(::runtime::v1alpha2::ContainerState value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.state_ = value;
}

// int64 created_at = 4;
inline void ContainerStatus::clear_created_at() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000100U);
}
inline ::int64_t ContainerStatus::created_at() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.created_at)
  return _internal_created_at();
}
inline void ContainerStatus::set_created_at(::int64_t value) {
  _internal_set_created_at(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.created_at)
}
inline ::int64_t ContainerStatus::_internal_created_at() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_at_;
}
inline void ContainerStatus::_internal_set_created_at(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_at_ = value;
}

// int64 started_at = 5;
inline void ContainerStatus::clear_started_at() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.started_at_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000200U);
}
inline ::int64_t ContainerStatus::started_at() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.started_at)
  return _internal_started_at();
}
inline void ContainerStatus::set_started_at(::int64_t value) {
  _internal_set_started_at(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000200U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.started_at)
}
inline ::int64_t ContainerStatus::_internal_started_at() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.started_at_;
}
inline void ContainerStatus::_internal_set_started_at(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.started_at_ = value;
}

// int64 finished_at = 6;
inline void ContainerStatus::clear_finished_at() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.finished_at_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00001000U);
}
inline ::int64_t ContainerStatus::finished_at() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.finished_at)
  return _internal_finished_at();
}
inline void ContainerStatus::set_finished_at(::int64_t value) {
  _internal_set_finished_at(value);
  SetHasBit(_impl_._has_bits_[0], 0x00001000U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.finished_at)
}
inline ::int64_t ContainerStatus::_internal_finished_at() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.finished_at_;
}
inline void ContainerStatus::_internal_set_finished_at(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.finished_at_ = value;
}

// int32 exit_code = 7;
inline void ContainerStatus::clear_exit_code() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_code_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000800U);
}
inline ::int32_t ContainerStatus::exit_code() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.exit_code)
  return _internal_exit_code();
}
inline void ContainerStatus::set_exit_code(::int32_t value) {
  _internal_set_exit_code(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000800U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.exit_code)
}
inline ::int32_t ContainerStatus::_internal_exit_code() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exit_code_;
}
inline void ContainerStatus::_internal_set_exit_code(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_code_ = value;
}

// .runtime.v1alpha2.ImageSpec image = 8;
inline bool ContainerStatus::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void ContainerStatus::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline const ::runtime::v1alpha2::ImageSpec& ContainerStatus::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& ContainerStatus::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.image)
  return _internal_image();
}
inline void ContainerStatus::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStatus.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ContainerStatus::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ContainerStatus::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ContainerStatus::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.image)
  return _msg;
}
inline void ContainerStatus::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.image)
}

// string image_ref = 9;
inline void ContainerStatus::clear_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_ref_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& ContainerStatus::image_ref() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.image_ref)
  return _internal_image_ref();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatus::set_image_ref(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.image_ref_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.image_ref)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::mutable_image_ref()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_image_ref();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.image_ref)
  return _s;
}
inline const ::std::string& ContainerStatus::_internal_image_ref() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.image_ref_.Get();
}
inline void ContainerStatus::_internal_set_image_ref(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_ref_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.image_ref_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatus::release_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.image_ref)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.image_ref_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.image_ref_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatus::set_allocated_image_ref(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.image_ref_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.image_ref_.IsDefault()) {
    _impl_.image_ref_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.image_ref)
}

// string reason = 10;
inline void ContainerStatus::clear_reason() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.reason_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::std::string& ContainerStatus::reason() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.reason)
  return _internal_reason();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatus::set_reason(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.reason_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.reason)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::mutable_reason()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_reason();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.reason)
  return _s;
}
inline const ::std::string& ContainerStatus::_internal_reason() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.reason_.Get();
}
inline void ContainerStatus::_internal_set_reason(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.reason_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_reason() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.reason_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatus::release_reason() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.reason)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.reason_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.reason_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatus::set_allocated_reason(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.reason_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.reason_.IsDefault()) {
    _impl_.reason_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.reason)
}

// string message = 11;
inline void ContainerStatus::clear_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.message_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::std::string& ContainerStatus::message() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.message)
  return _internal_message();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatus::set_message(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.message_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.message)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::mutable_message()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_message();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.message)
  return _s;
}
inline const ::std::string& ContainerStatus::_internal_message() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.message_.Get();
}
inline void ContainerStatus::_internal_set_message(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.message_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.message_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatus::release_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.message)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.message_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.message_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatus::set_allocated_message(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.message_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.message_.IsDefault()) {
    _impl_.message_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.message)
}

// map<string, string> labels = 12;
inline int ContainerStatus::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int ContainerStatus::labels_size() const {
  return _internal_labels_size();
}
inline void ContainerStatus::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00002000U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatus::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatus::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerStatus.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatus::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00002000U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerStatus.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 13;
inline int ContainerStatus::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int ContainerStatus::annotations_size() const {
  return _internal_annotations_size();
}
inline void ContainerStatus::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00004000U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatus::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatus::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerStatus.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatus::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00004000U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerStatus.annotations)
  return _internal_mutable_annotations();
}

// repeated .runtime.v1alpha2.Mount mounts = 14;
inline int ContainerStatus::_internal_mounts_size() const {
  return _internal_mounts().size();
}
inline int ContainerStatus::mounts_size() const {
  return _internal_mounts_size();
}
inline void ContainerStatus::clear_mounts() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mounts_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL ContainerStatus::mutable_mounts(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.mounts)
  return _internal_mutable_mounts()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL ContainerStatus::mutable_mounts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ContainerStatus.mounts)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_mounts();
}
inline const ::runtime::v1alpha2::Mount& ContainerStatus::mounts(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.mounts)
  return _internal_mounts().Get(index);
}
inline ::runtime::v1alpha2::Mount* PROTOBUF_NONNULL ContainerStatus::add_mounts()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::Mount* _add =
      _internal_mutable_mounts()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ContainerStatus.mounts)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>& ContainerStatus::mounts() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ContainerStatus.mounts)
  return _internal_mounts();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>&
ContainerStatus::_internal_mounts() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mounts_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Mount>* PROTOBUF_NONNULL
ContainerStatus::_internal_mutable_mounts() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.mounts_;
}

// string log_path = 15;
inline void ContainerStatus::clear_log_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::std::string& ContainerStatus::log_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatus.log_path)
  return _internal_log_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatus::set_log_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  _impl_.log_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatus.log_path)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::mutable_log_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::std::string* _s = _internal_mutable_log_path();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatus.log_path)
  return _s;
}
inline const ::std::string& ContainerStatus::_internal_log_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.log_path_.Get();
}
inline void ContainerStatus::_internal_set_log_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.log_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatus::_internal_mutable_log_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.log_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatus::release_log_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatus.log_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000020U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  auto* released = _impl_.log_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.log_path_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatus::set_allocated_log_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  _impl_.log_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.log_path_.IsDefault()) {
    _impl_.log_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatus.log_path)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ContainerStatusResponse

// .runtime.v1alpha2.ContainerStatus status = 1;
inline bool ContainerStatusResponse::has_status() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.status_ != nullptr);
  return value;
}
inline void ContainerStatusResponse::clear_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.status_ != nullptr) _impl_.status_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ContainerStatus& ContainerStatusResponse::_internal_status() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerStatus* p = _impl_.status_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerStatus&>(::runtime::v1alpha2::_ContainerStatus_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerStatus& ContainerStatusResponse::status() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatusResponse.status)
  return _internal_status();
}
inline void ContainerStatusResponse::unsafe_arena_set_allocated_status(
    ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.status_);
  }
  _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::ContainerStatus*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStatusResponse.status)
}
inline ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE ContainerStatusResponse::release_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStatus* released = _impl_.status_;
  _impl_.status_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE ContainerStatusResponse::unsafe_arena_release_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatusResponse.status)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStatus* temp = _impl_.status_;
  _impl_.status_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NONNULL ContainerStatusResponse::_internal_mutable_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.status_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerStatus>(GetArena());
    _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::ContainerStatus*>(p);
  }
  return _impl_.status_;
}
inline ::runtime::v1alpha2::ContainerStatus* PROTOBUF_NONNULL ContainerStatusResponse::mutable_status()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStatus* _msg = _internal_mutable_status();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatusResponse.status)
  return _msg;
}
inline void ContainerStatusResponse::set_allocated_status(::runtime::v1alpha2::ContainerStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.status_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::ContainerStatus*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatusResponse.status)
}

// map<string, string> info = 2;
inline int ContainerStatusResponse::_internal_info_size() const {
  return _internal_info().size();
}
inline int ContainerStatusResponse::info_size() const {
  return _internal_info_size();
}
inline void ContainerStatusResponse::clear_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.info_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatusResponse::_internal_info() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.info_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatusResponse::info() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerStatusResponse.info)
  return _internal_info();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatusResponse::_internal_mutable_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.info_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatusResponse::mutable_info()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerStatusResponse.info)
  return _internal_mutable_info();
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// UpdateContainerResourcesRequest

// string container_id = 1;
inline void UpdateContainerResourcesRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& UpdateContainerResourcesRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.UpdateContainerResourcesRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void UpdateContainerResourcesRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.UpdateContainerResourcesRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL UpdateContainerResourcesRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.UpdateContainerResourcesRequest.container_id)
  return _s;
}
inline const ::std::string& UpdateContainerResourcesRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void UpdateContainerResourcesRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL UpdateContainerResourcesRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE UpdateContainerResourcesRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.UpdateContainerResourcesRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void UpdateContainerResourcesRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.UpdateContainerResourcesRequest.container_id)
}

// .runtime.v1alpha2.LinuxContainerResources linux = 2;
inline bool UpdateContainerResourcesRequest::has_linux() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.linux_ != nullptr);
  return value;
}
inline void UpdateContainerResourcesRequest::clear_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ != nullptr) _impl_.linux_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::LinuxContainerResources& UpdateContainerResourcesRequest::_internal_linux() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::LinuxContainerResources* p = _impl_.linux_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::LinuxContainerResources&>(::runtime::v1alpha2::_LinuxContainerResources_default_instance_);
}
inline const ::runtime::v1alpha2::LinuxContainerResources& UpdateContainerResourcesRequest::linux() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.UpdateContainerResourcesRequest.linux)
  return _internal_linux();
}
inline void UpdateContainerResourcesRequest::unsafe_arena_set_allocated_linux(
    ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }
  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerResources*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.UpdateContainerResourcesRequest.linux)
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE UpdateContainerResourcesRequest::release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxContainerResources* released = _impl_.linux_;
  _impl_.linux_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE UpdateContainerResourcesRequest::unsafe_arena_release_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.UpdateContainerResourcesRequest.linux)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxContainerResources* temp = _impl_.linux_;
  _impl_.linux_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL UpdateContainerResourcesRequest::_internal_mutable_linux() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.linux_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::LinuxContainerResources>(GetArena());
    _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerResources*>(p);
  }
  return _impl_.linux_;
}
inline ::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NONNULL UpdateContainerResourcesRequest::mutable_linux()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::LinuxContainerResources* _msg = _internal_mutable_linux();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.UpdateContainerResourcesRequest.linux)
  return _msg;
}
inline void UpdateContainerResourcesRequest::set_allocated_linux(::runtime::v1alpha2::LinuxContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.linux_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.linux_ = reinterpret_cast<::runtime::v1alpha2::LinuxContainerResources*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.UpdateContainerResourcesRequest.linux)
}

// .runtime.v1alpha2.WindowsContainerResources windows = 3;
inline bool UpdateContainerResourcesRequest::has_windows() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.windows_ != nullptr);
  return value;
}
inline void UpdateContainerResourcesRequest::clear_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.windows_ != nullptr) _impl_.windows_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::WindowsContainerResources& UpdateContainerResourcesRequest::_internal_windows() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::WindowsContainerResources* p = _impl_.windows_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::WindowsContainerResources&>(::runtime::v1alpha2::_WindowsContainerResources_default_instance_);
}
inline const ::runtime::v1alpha2::WindowsContainerResources& UpdateContainerResourcesRequest::windows() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.UpdateContainerResourcesRequest.windows)
  return _internal_windows();
}
inline void UpdateContainerResourcesRequest::unsafe_arena_set_allocated_windows(
    ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.windows_);
  }
  _impl_.windows_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerResources*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.UpdateContainerResourcesRequest.windows)
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE UpdateContainerResourcesRequest::release_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::WindowsContainerResources* released = _impl_.windows_;
  _impl_.windows_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE UpdateContainerResourcesRequest::unsafe_arena_release_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.UpdateContainerResourcesRequest.windows)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::WindowsContainerResources* temp = _impl_.windows_;
  _impl_.windows_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL UpdateContainerResourcesRequest::_internal_mutable_windows() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.windows_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::WindowsContainerResources>(GetArena());
    _impl_.windows_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerResources*>(p);
  }
  return _impl_.windows_;
}
inline ::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NONNULL UpdateContainerResourcesRequest::mutable_windows()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::WindowsContainerResources* _msg = _internal_mutable_windows();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.UpdateContainerResourcesRequest.windows)
  return _msg;
}
inline void UpdateContainerResourcesRequest::set_allocated_windows(::runtime::v1alpha2::WindowsContainerResources* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.windows_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.windows_ = reinterpret_cast<::runtime::v1alpha2::WindowsContainerResources*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.UpdateContainerResourcesRequest.windows)
}

// map<string, string> annotations = 4;
inline int UpdateContainerResourcesRequest::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int UpdateContainerResourcesRequest::annotations_size() const {
  return _internal_annotations_size();
}
inline void UpdateContainerResourcesRequest::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& UpdateContainerResourcesRequest::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& UpdateContainerResourcesRequest::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.UpdateContainerResourcesRequest.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL UpdateContainerResourcesRequest::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL UpdateContainerResourcesRequest::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.UpdateContainerResourcesRequest.annotations)
  return _internal_mutable_annotations();
}

// -------------------------------------------------------------------

// UpdateContainerResourcesResponse

// -------------------------------------------------------------------

// ExecSyncRequest

// string container_id = 1;
inline void ExecSyncRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& ExecSyncRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecSyncRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecSyncRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecSyncRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecSyncRequest.container_id)
  return _s;
}
inline const ::std::string& ExecSyncRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void ExecSyncRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecSyncRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ExecSyncRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void ExecSyncRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ExecSyncRequest.container_id)
}

// repeated string cmd = 2;
inline int ExecSyncRequest::_internal_cmd_size() const {
  return _internal_cmd().size();
}
inline int ExecSyncRequest::cmd_size() const {
  return _internal_cmd_size();
}
inline void ExecSyncRequest::clear_cmd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cmd_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncRequest::add_cmd()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_cmd()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.ExecSyncRequest.cmd)
  return _s;
}
inline const ::std::string& ExecSyncRequest::cmd(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecSyncRequest.cmd)
  return _internal_cmd().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncRequest::mutable_cmd(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecSyncRequest.cmd)
  return _internal_mutable_cmd()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ExecSyncRequest::set_cmd(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_cmd()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecSyncRequest.cmd)
}
template <typename Arg_, typename... Args_>
inline void ExecSyncRequest::add_cmd(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_cmd(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ExecSyncRequest.cmd)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ExecSyncRequest::cmd()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ExecSyncRequest.cmd)
  return _internal_cmd();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecSyncRequest::mutable_cmd() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ExecSyncRequest.cmd)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_cmd();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ExecSyncRequest::_internal_cmd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cmd_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecSyncRequest::_internal_mutable_cmd() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.cmd_;
}

// int64 timeout = 3;
inline void ExecSyncRequest::clear_timeout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timeout_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::int64_t ExecSyncRequest::timeout() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecSyncRequest.timeout)
  return _internal_timeout();
}
inline void ExecSyncRequest::set_timeout(::int64_t value) {
  _internal_set_timeout(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecSyncRequest.timeout)
}
inline ::int64_t ExecSyncRequest::_internal_timeout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.timeout_;
}
inline void ExecSyncRequest::_internal_set_timeout(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timeout_ = value;
}

// -------------------------------------------------------------------

// ExecSyncResponse

// bytes f_stdout = 1;
inline void ExecSyncResponse::clear_f_stdout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdout_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ExecSyncResponse::f_stdout() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecSyncResponse.f_stdout)
  return _internal_f_stdout();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecSyncResponse::set_f_stdout(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.f_stdout_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecSyncResponse.f_stdout)
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncResponse::mutable_f_stdout()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_f_stdout();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecSyncResponse.f_stdout)
  return _s;
}
inline const ::std::string& ExecSyncResponse::_internal_f_stdout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stdout_.Get();
}
inline void ExecSyncResponse::_internal_set_f_stdout(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdout_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncResponse::_internal_mutable_f_stdout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.f_stdout_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecSyncResponse::release_f_stdout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ExecSyncResponse.f_stdout)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.f_stdout_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.f_stdout_.Set("", GetArena());
  }
  return released;
}
inline void ExecSyncResponse::set_allocated_f_stdout(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.f_stdout_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.f_stdout_.IsDefault()) {
    _impl_.f_stdout_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ExecSyncResponse.f_stdout)
}

// bytes f_stderr = 2;
inline void ExecSyncResponse::clear_f_stderr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stderr_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& ExecSyncResponse::f_stderr() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecSyncResponse.f_stderr)
  return _internal_f_stderr();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecSyncResponse::set_f_stderr(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.f_stderr_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecSyncResponse.f_stderr)
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncResponse::mutable_f_stderr()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_f_stderr();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecSyncResponse.f_stderr)
  return _s;
}
inline const ::std::string& ExecSyncResponse::_internal_f_stderr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stderr_.Get();
}
inline void ExecSyncResponse::_internal_set_f_stderr(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stderr_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecSyncResponse::_internal_mutable_f_stderr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.f_stderr_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecSyncResponse::release_f_stderr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ExecSyncResponse.f_stderr)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.f_stderr_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.f_stderr_.Set("", GetArena());
  }
  return released;
}
inline void ExecSyncResponse::set_allocated_f_stderr(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.f_stderr_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.f_stderr_.IsDefault()) {
    _impl_.f_stderr_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ExecSyncResponse.f_stderr)
}

// int32 exit_code = 3;
inline void ExecSyncResponse::clear_exit_code() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_code_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::int32_t ExecSyncResponse::exit_code() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecSyncResponse.exit_code)
  return _internal_exit_code();
}
inline void ExecSyncResponse::set_exit_code(::int32_t value) {
  _internal_set_exit_code(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecSyncResponse.exit_code)
}
inline ::int32_t ExecSyncResponse::_internal_exit_code() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exit_code_;
}
inline void ExecSyncResponse::_internal_set_exit_code(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_code_ = value;
}

// -------------------------------------------------------------------

// ExecRequest

// string container_id = 1;
inline void ExecRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& ExecRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL ExecRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecRequest.container_id)
  return _s;
}
inline const ::std::string& ExecRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void ExecRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ExecRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void ExecRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ExecRequest.container_id)
}

// repeated string cmd = 2;
inline int ExecRequest::_internal_cmd_size() const {
  return _internal_cmd().size();
}
inline int ExecRequest::cmd_size() const {
  return _internal_cmd_size();
}
inline void ExecRequest::clear_cmd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cmd_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL ExecRequest::add_cmd()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_cmd()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.ExecRequest.cmd)
  return _s;
}
inline const ::std::string& ExecRequest::cmd(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecRequest.cmd)
  return _internal_cmd().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ExecRequest::mutable_cmd(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecRequest.cmd)
  return _internal_mutable_cmd()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ExecRequest::set_cmd(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_cmd()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecRequest.cmd)
}
template <typename Arg_, typename... Args_>
inline void ExecRequest::add_cmd(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_cmd(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ExecRequest.cmd)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ExecRequest::cmd()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ExecRequest.cmd)
  return _internal_cmd();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecRequest::mutable_cmd() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ExecRequest.cmd)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_cmd();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ExecRequest::_internal_cmd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cmd_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecRequest::_internal_mutable_cmd() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.cmd_;
}

// bool tty = 3;
inline void ExecRequest::clear_tty() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tty_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline bool ExecRequest::tty() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecRequest.tty)
  return _internal_tty();
}
inline void ExecRequest::set_tty(bool value) {
  _internal_set_tty(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecRequest.tty)
}
inline bool ExecRequest::_internal_tty() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tty_;
}
inline void ExecRequest::_internal_set_tty(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tty_ = value;
}

// bool f_stdin = 4;
inline void ExecRequest::clear_f_stdin() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdin_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline bool ExecRequest::f_stdin() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecRequest.f_stdin)
  return _internal_f_stdin();
}
inline void ExecRequest::set_f_stdin(bool value) {
  _internal_set_f_stdin(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecRequest.f_stdin)
}
inline bool ExecRequest::_internal_f_stdin() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stdin_;
}
inline void ExecRequest::_internal_set_f_stdin(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdin_ = value;
}

// bool f_stdout = 5;
inline void ExecRequest::clear_f_stdout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdout_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline bool ExecRequest::f_stdout() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecRequest.f_stdout)
  return _internal_f_stdout();
}
inline void ExecRequest::set_f_stdout(bool value) {
  _internal_set_f_stdout(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecRequest.f_stdout)
}
inline bool ExecRequest::_internal_f_stdout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stdout_;
}
inline void ExecRequest::_internal_set_f_stdout(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdout_ = value;
}

// bool f_stderr = 6;
inline void ExecRequest::clear_f_stderr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stderr_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline bool ExecRequest::f_stderr() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecRequest.f_stderr)
  return _internal_f_stderr();
}
inline void ExecRequest::set_f_stderr(bool value) {
  _internal_set_f_stderr(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecRequest.f_stderr)
}
inline bool ExecRequest::_internal_f_stderr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stderr_;
}
inline void ExecRequest::_internal_set_f_stderr(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stderr_ = value;
}

// -------------------------------------------------------------------

// ExecResponse

// string url = 1;
inline void ExecResponse::clear_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ExecResponse::url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ExecResponse.url)
  return _internal_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecResponse::set_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ExecResponse.url)
}
inline ::std::string* PROTOBUF_NONNULL ExecResponse::mutable_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_url();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ExecResponse.url)
  return _s;
}
inline const ::std::string& ExecResponse::_internal_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.url_.Get();
}
inline void ExecResponse::_internal_set_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecResponse::_internal_mutable_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecResponse::release_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ExecResponse.url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.url_.Set("", GetArena());
  }
  return released;
}
inline void ExecResponse::set_allocated_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.url_.IsDefault()) {
    _impl_.url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ExecResponse.url)
}

// -------------------------------------------------------------------

// AttachRequest

// string container_id = 1;
inline void AttachRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& AttachRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AttachRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AttachRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AttachRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL AttachRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AttachRequest.container_id)
  return _s;
}
inline const ::std::string& AttachRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void AttachRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AttachRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AttachRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AttachRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void AttachRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AttachRequest.container_id)
}

// bool f_stdin = 2;
inline void AttachRequest::clear_f_stdin() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdin_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline bool AttachRequest::f_stdin() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AttachRequest.f_stdin)
  return _internal_f_stdin();
}
inline void AttachRequest::set_f_stdin(bool value) {
  _internal_set_f_stdin(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AttachRequest.f_stdin)
}
inline bool AttachRequest::_internal_f_stdin() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stdin_;
}
inline void AttachRequest::_internal_set_f_stdin(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdin_ = value;
}

// bool tty = 3;
inline void AttachRequest::clear_tty() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tty_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline bool AttachRequest::tty() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AttachRequest.tty)
  return _internal_tty();
}
inline void AttachRequest::set_tty(bool value) {
  _internal_set_tty(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AttachRequest.tty)
}
inline bool AttachRequest::_internal_tty() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tty_;
}
inline void AttachRequest::_internal_set_tty(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tty_ = value;
}

// bool f_stdout = 4;
inline void AttachRequest::clear_f_stdout() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdout_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline bool AttachRequest::f_stdout() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AttachRequest.f_stdout)
  return _internal_f_stdout();
}
inline void AttachRequest::set_f_stdout(bool value) {
  _internal_set_f_stdout(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AttachRequest.f_stdout)
}
inline bool AttachRequest::_internal_f_stdout() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stdout_;
}
inline void AttachRequest::_internal_set_f_stdout(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stdout_ = value;
}

// bool f_stderr = 5;
inline void AttachRequest::clear_f_stderr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stderr_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline bool AttachRequest::f_stderr() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AttachRequest.f_stderr)
  return _internal_f_stderr();
}
inline void AttachRequest::set_f_stderr(bool value) {
  _internal_set_f_stderr(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AttachRequest.f_stderr)
}
inline bool AttachRequest::_internal_f_stderr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.f_stderr_;
}
inline void AttachRequest::_internal_set_f_stderr(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.f_stderr_ = value;
}

// -------------------------------------------------------------------

// AttachResponse

// string url = 1;
inline void AttachResponse::clear_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& AttachResponse::url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AttachResponse.url)
  return _internal_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AttachResponse::set_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AttachResponse.url)
}
inline ::std::string* PROTOBUF_NONNULL AttachResponse::mutable_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_url();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AttachResponse.url)
  return _s;
}
inline const ::std::string& AttachResponse::_internal_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.url_.Get();
}
inline void AttachResponse::_internal_set_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AttachResponse::_internal_mutable_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AttachResponse::release_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AttachResponse.url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.url_.Set("", GetArena());
  }
  return released;
}
inline void AttachResponse::set_allocated_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.url_.IsDefault()) {
    _impl_.url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AttachResponse.url)
}

// -------------------------------------------------------------------

// PortForwardRequest

// string pod_sandbox_id = 1;
inline void PortForwardRequest::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& PortForwardRequest::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortForwardRequest.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PortForwardRequest::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortForwardRequest.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL PortForwardRequest::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PortForwardRequest.pod_sandbox_id)
  return _s;
}
inline const ::std::string& PortForwardRequest::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void PortForwardRequest::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PortForwardRequest::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PortForwardRequest::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PortForwardRequest.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void PortForwardRequest::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PortForwardRequest.pod_sandbox_id)
}

// repeated int32 port = 2;
inline int PortForwardRequest::_internal_port_size() const {
  return _internal_port().size();
}
inline int PortForwardRequest::port_size() const {
  return _internal_port_size();
}
inline void PortForwardRequest::clear_port() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.port_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::int32_t PortForwardRequest::port(int index) const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortForwardRequest.port)
  return _internal_port().Get(index);
}
inline void PortForwardRequest::set_port(int index, ::int32_t value) {
  _internal_mutable_port()->Set(index, value);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortForwardRequest.port)
}
inline void PortForwardRequest::add_port(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_port()->Add(value);
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.PortForwardRequest.port)
}
inline const ::google::protobuf::RepeatedField<::int32_t>& PortForwardRequest::port() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.PortForwardRequest.port)
  return _internal_port();
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL PortForwardRequest::mutable_port()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.PortForwardRequest.port)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_port();
}
inline const ::google::protobuf::RepeatedField<::int32_t>&
PortForwardRequest::_internal_port() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.port_;
}
inline ::google::protobuf::RepeatedField<::int32_t>* PROTOBUF_NONNULL
PortForwardRequest::_internal_mutable_port() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.port_;
}

// -------------------------------------------------------------------

// PortForwardResponse

// string url = 1;
inline void PortForwardResponse::clear_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.url_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PortForwardResponse::url() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PortForwardResponse.url)
  return _internal_url();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PortForwardResponse::set_url(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.url_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PortForwardResponse.url)
}
inline ::std::string* PROTOBUF_NONNULL PortForwardResponse::mutable_url()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_url();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PortForwardResponse.url)
  return _s;
}
inline const ::std::string& PortForwardResponse::_internal_url() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.url_.Get();
}
inline void PortForwardResponse::_internal_set_url(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.url_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PortForwardResponse::_internal_mutable_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.url_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PortForwardResponse::release_url() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PortForwardResponse.url)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.url_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.url_.Set("", GetArena());
  }
  return released;
}
inline void PortForwardResponse::set_allocated_url(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.url_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.url_.IsDefault()) {
    _impl_.url_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PortForwardResponse.url)
}

// -------------------------------------------------------------------

// ImageFilter

// .runtime.v1alpha2.ImageSpec image = 1;
inline bool ImageFilter::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void ImageFilter::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ImageSpec& ImageFilter::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& ImageFilter::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ImageFilter.image)
  return _internal_image();
}
inline void ImageFilter::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ImageFilter.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ImageFilter::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ImageFilter::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ImageFilter.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ImageFilter::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ImageFilter::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ImageFilter.image)
  return _msg;
}
inline void ImageFilter::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ImageFilter.image)
}

// -------------------------------------------------------------------

// ListImagesRequest

// .runtime.v1alpha2.ImageFilter filter = 1;
inline bool ListImagesRequest::has_filter() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.filter_ != nullptr);
  return value;
}
inline void ListImagesRequest::clear_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ != nullptr) _impl_.filter_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ImageFilter& ListImagesRequest::_internal_filter() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageFilter* p = _impl_.filter_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageFilter&>(::runtime::v1alpha2::_ImageFilter_default_instance_);
}
inline const ::runtime::v1alpha2::ImageFilter& ListImagesRequest::filter() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListImagesRequest.filter)
  return _internal_filter();
}
inline void ListImagesRequest::unsafe_arena_set_allocated_filter(
    ::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }
  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ImageFilter*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ListImagesRequest.filter)
}
inline ::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE ListImagesRequest::release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageFilter* released = _impl_.filter_;
  _impl_.filter_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE ListImagesRequest::unsafe_arena_release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ListImagesRequest.filter)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageFilter* temp = _impl_.filter_;
  _impl_.filter_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageFilter* PROTOBUF_NONNULL ListImagesRequest::_internal_mutable_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageFilter>(GetArena());
    _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ImageFilter*>(p);
  }
  return _impl_.filter_;
}
inline ::runtime::v1alpha2::ImageFilter* PROTOBUF_NONNULL ListImagesRequest::mutable_filter()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageFilter* _msg = _internal_mutable_filter();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListImagesRequest.filter)
  return _msg;
}
inline void ListImagesRequest::set_allocated_filter(::runtime::v1alpha2::ImageFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ImageFilter*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ListImagesRequest.filter)
}

// -------------------------------------------------------------------

// Image

// string id = 1;
inline void Image::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& Image::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Image::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Image.id)
}
inline ::std::string* PROTOBUF_NONNULL Image::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Image.id)
  return _s;
}
inline const ::std::string& Image::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void Image::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Image::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Image::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Image.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void Image::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Image.id)
}

// repeated string repo_tags = 2;
inline int Image::_internal_repo_tags_size() const {
  return _internal_repo_tags().size();
}
inline int Image::repo_tags_size() const {
  return _internal_repo_tags_size();
}
inline void Image::clear_repo_tags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.repo_tags_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL Image::add_repo_tags()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_repo_tags()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.Image.repo_tags)
  return _s;
}
inline const ::std::string& Image::repo_tags(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.repo_tags)
  return _internal_repo_tags().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Image::mutable_repo_tags(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Image.repo_tags)
  return _internal_mutable_repo_tags()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Image::set_repo_tags(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_repo_tags()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Image.repo_tags)
}
template <typename Arg_, typename... Args_>
inline void Image::add_repo_tags(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_repo_tags(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.Image.repo_tags)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Image::repo_tags()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.Image.repo_tags)
  return _internal_repo_tags();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Image::mutable_repo_tags() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.Image.repo_tags)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_repo_tags();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Image::_internal_repo_tags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.repo_tags_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Image::_internal_mutable_repo_tags() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.repo_tags_;
}

// repeated string repo_digests = 3;
inline int Image::_internal_repo_digests_size() const {
  return _internal_repo_digests().size();
}
inline int Image::repo_digests_size() const {
  return _internal_repo_digests_size();
}
inline void Image::clear_repo_digests() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.repo_digests_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL Image::add_repo_digests()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_repo_digests()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:runtime.v1alpha2.Image.repo_digests)
  return _s;
}
inline const ::std::string& Image::repo_digests(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.repo_digests)
  return _internal_repo_digests().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Image::mutable_repo_digests(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Image.repo_digests)
  return _internal_mutable_repo_digests()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Image::set_repo_digests(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(*_internal_mutable_repo_digests()->Mutable(index), ::std::forward<Arg_>(value),
                        args... );
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Image.repo_digests)
}
template <typename Arg_, typename... Args_>
inline void Image::add_repo_digests(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_repo_digests(), ::std::forward<Arg_>(value),
      args... );
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.Image.repo_digests)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Image::repo_digests()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.Image.repo_digests)
  return _internal_repo_digests();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Image::mutable_repo_digests() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.Image.repo_digests)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_repo_digests();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Image::_internal_repo_digests() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.repo_digests_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Image::_internal_mutable_repo_digests() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.repo_digests_;
}

// uint64 size = 4;
inline void Image::clear_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline ::uint64_t Image::size() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.size)
  return _internal_size();
}
inline void Image::set_size(::uint64_t value) {
  _internal_set_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Image.size)
}
inline ::uint64_t Image::_internal_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.size_;
}
inline void Image::_internal_set_size(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = value;
}

// .runtime.v1alpha2.Int64Value uid = 5;
inline bool Image::has_uid() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.uid_ != nullptr);
  return value;
}
inline void Image::clear_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.uid_ != nullptr) _impl_.uid_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::runtime::v1alpha2::Int64Value& Image::_internal_uid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Int64Value* p = _impl_.uid_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Int64Value&>(::runtime::v1alpha2::_Int64Value_default_instance_);
}
inline const ::runtime::v1alpha2::Int64Value& Image::uid() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.uid)
  return _internal_uid();
}
inline void Image::unsafe_arena_set_allocated_uid(
    ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.uid_);
  }
  _impl_.uid_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.Image.uid)
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE Image::release_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::Int64Value* released = _impl_.uid_;
  _impl_.uid_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE Image::unsafe_arena_release_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Image.uid)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::Int64Value* temp = _impl_.uid_;
  _impl_.uid_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL Image::_internal_mutable_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.uid_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Int64Value>(GetArena());
    _impl_.uid_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(p);
  }
  return _impl_.uid_;
}
inline ::runtime::v1alpha2::Int64Value* PROTOBUF_NONNULL Image::mutable_uid()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::runtime::v1alpha2::Int64Value* _msg = _internal_mutable_uid();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Image.uid)
  return _msg;
}
inline void Image::set_allocated_uid(::runtime::v1alpha2::Int64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.uid_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.uid_ = reinterpret_cast<::runtime::v1alpha2::Int64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Image.uid)
}

// string username = 6;
inline void Image::clear_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.username_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::std::string& Image::username() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.username)
  return _internal_username();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Image::set_username(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.username_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.Image.username)
}
inline ::std::string* PROTOBUF_NONNULL Image::mutable_username()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_username();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Image.username)
  return _s;
}
inline const ::std::string& Image::_internal_username() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.username_.Get();
}
inline void Image::_internal_set_username(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.username_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Image::_internal_mutable_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.username_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Image::release_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Image.username)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.username_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.username_.Set("", GetArena());
  }
  return released;
}
inline void Image::set_allocated_username(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.username_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.username_.IsDefault()) {
    _impl_.username_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Image.username)
}

// .runtime.v1alpha2.ImageSpec spec = 7;
inline bool Image::has_spec() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  PROTOBUF_ASSUME(!value || _impl_.spec_ != nullptr);
  return value;
}
inline void Image::clear_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.spec_ != nullptr) _impl_.spec_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::runtime::v1alpha2::ImageSpec& Image::_internal_spec() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.spec_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& Image::spec() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.Image.spec)
  return _internal_spec();
}
inline void Image::unsafe_arena_set_allocated_spec(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.spec_);
  }
  _impl_.spec_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.Image.spec)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE Image::release_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.spec_;
  _impl_.spec_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE Image::unsafe_arena_release_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.Image.spec)

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.spec_;
  _impl_.spec_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL Image::_internal_mutable_spec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.spec_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.spec_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.spec_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL Image::mutable_spec()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_spec();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.Image.spec)
  return _msg;
}
inline void Image::set_allocated_spec(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.spec_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }

  _impl_.spec_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.Image.spec)
}

// -------------------------------------------------------------------

// ListImagesResponse

// repeated .runtime.v1alpha2.Image images = 1;
inline int ListImagesResponse::_internal_images_size() const {
  return _internal_images().size();
}
inline int ListImagesResponse::images_size() const {
  return _internal_images_size();
}
inline void ListImagesResponse::clear_images() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.images_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::Image* PROTOBUF_NONNULL ListImagesResponse::mutable_images(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListImagesResponse.images)
  return _internal_mutable_images()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>* PROTOBUF_NONNULL ListImagesResponse::mutable_images()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ListImagesResponse.images)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_images();
}
inline const ::runtime::v1alpha2::Image& ListImagesResponse::images(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListImagesResponse.images)
  return _internal_images().Get(index);
}
inline ::runtime::v1alpha2::Image* PROTOBUF_NONNULL ListImagesResponse::add_images()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::Image* _add =
      _internal_mutable_images()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ListImagesResponse.images)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>& ListImagesResponse::images() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ListImagesResponse.images)
  return _internal_images();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>&
ListImagesResponse::_internal_images() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.images_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::Image>* PROTOBUF_NONNULL
ListImagesResponse::_internal_mutable_images() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.images_;
}

// -------------------------------------------------------------------

// ImageStatusRequest

// .runtime.v1alpha2.ImageSpec image = 1;
inline bool ImageStatusRequest::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void ImageStatusRequest::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ImageSpec& ImageStatusRequest::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& ImageStatusRequest::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ImageStatusRequest.image)
  return _internal_image();
}
inline void ImageStatusRequest::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ImageStatusRequest.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ImageStatusRequest::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE ImageStatusRequest::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ImageStatusRequest.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ImageStatusRequest::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL ImageStatusRequest::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ImageStatusRequest.image)
  return _msg;
}
inline void ImageStatusRequest::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ImageStatusRequest.image)
}

// bool verbose = 2;
inline void ImageStatusRequest::clear_verbose() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline bool ImageStatusRequest::verbose() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ImageStatusRequest.verbose)
  return _internal_verbose();
}
inline void ImageStatusRequest::set_verbose(bool value) {
  _internal_set_verbose(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ImageStatusRequest.verbose)
}
inline bool ImageStatusRequest::_internal_verbose() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.verbose_;
}
inline void ImageStatusRequest::_internal_set_verbose(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ImageStatusResponse

// .runtime.v1alpha2.Image image = 1;
inline bool ImageStatusResponse::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void ImageStatusResponse::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::Image& ImageStatusResponse::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::Image* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::Image&>(::runtime::v1alpha2::_Image_default_instance_);
}
inline const ::runtime::v1alpha2::Image& ImageStatusResponse::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ImageStatusResponse.image)
  return _internal_image();
}
inline void ImageStatusResponse::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::Image* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::Image*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ImageStatusResponse.image)
}
inline ::runtime::v1alpha2::Image* PROTOBUF_NULLABLE ImageStatusResponse::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::Image* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::Image* PROTOBUF_NULLABLE ImageStatusResponse::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ImageStatusResponse.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::Image* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::Image* PROTOBUF_NONNULL ImageStatusResponse::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::Image>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::Image*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::Image* PROTOBUF_NONNULL ImageStatusResponse::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::Image* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ImageStatusResponse.image)
  return _msg;
}
inline void ImageStatusResponse::set_allocated_image(::runtime::v1alpha2::Image* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::Image*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ImageStatusResponse.image)
}

// map<string, string> info = 2;
inline int ImageStatusResponse::_internal_info_size() const {
  return _internal_info().size();
}
inline int ImageStatusResponse::info_size() const {
  return _internal_info_size();
}
inline void ImageStatusResponse::clear_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.info_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ImageStatusResponse::_internal_info() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.info_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ImageStatusResponse::info() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ImageStatusResponse.info)
  return _internal_info();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ImageStatusResponse::_internal_mutable_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.info_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ImageStatusResponse::mutable_info()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ImageStatusResponse.info)
  return _internal_mutable_info();
}

// -------------------------------------------------------------------

// AuthConfig

// string username = 1;
inline void AuthConfig::clear_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.username_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& AuthConfig::username() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AuthConfig.username)
  return _internal_username();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AuthConfig::set_username(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.username_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AuthConfig.username)
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::mutable_username()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_username();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AuthConfig.username)
  return _s;
}
inline const ::std::string& AuthConfig::_internal_username() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.username_.Get();
}
inline void AuthConfig::_internal_set_username(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.username_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::_internal_mutable_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.username_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AuthConfig::release_username() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AuthConfig.username)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.username_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.username_.Set("", GetArena());
  }
  return released;
}
inline void AuthConfig::set_allocated_username(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.username_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.username_.IsDefault()) {
    _impl_.username_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AuthConfig.username)
}

// string password = 2;
inline void AuthConfig::clear_password() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.password_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& AuthConfig::password() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AuthConfig.password)
  return _internal_password();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AuthConfig::set_password(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.password_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AuthConfig.password)
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::mutable_password()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_password();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AuthConfig.password)
  return _s;
}
inline const ::std::string& AuthConfig::_internal_password() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.password_.Get();
}
inline void AuthConfig::_internal_set_password(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.password_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::_internal_mutable_password() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.password_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AuthConfig::release_password() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AuthConfig.password)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.password_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.password_.Set("", GetArena());
  }
  return released;
}
inline void AuthConfig::set_allocated_password(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.password_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.password_.IsDefault()) {
    _impl_.password_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AuthConfig.password)
}

// string auth = 3;
inline void AuthConfig::clear_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.auth_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& AuthConfig::auth() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AuthConfig.auth)
  return _internal_auth();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AuthConfig::set_auth(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.auth_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AuthConfig.auth)
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::mutable_auth()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_auth();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AuthConfig.auth)
  return _s;
}
inline const ::std::string& AuthConfig::_internal_auth() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.auth_.Get();
}
inline void AuthConfig::_internal_set_auth(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.auth_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::_internal_mutable_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.auth_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AuthConfig::release_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AuthConfig.auth)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.auth_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.auth_.Set("", GetArena());
  }
  return released;
}
inline void AuthConfig::set_allocated_auth(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.auth_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.auth_.IsDefault()) {
    _impl_.auth_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AuthConfig.auth)
}

// string server_address = 4;
inline void AuthConfig::clear_server_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.server_address_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::std::string& AuthConfig::server_address() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AuthConfig.server_address)
  return _internal_server_address();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AuthConfig::set_server_address(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.server_address_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AuthConfig.server_address)
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::mutable_server_address()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_server_address();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AuthConfig.server_address)
  return _s;
}
inline const ::std::string& AuthConfig::_internal_server_address() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.server_address_.Get();
}
inline void AuthConfig::_internal_set_server_address(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.server_address_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::_internal_mutable_server_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.server_address_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AuthConfig::release_server_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AuthConfig.server_address)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.server_address_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.server_address_.Set("", GetArena());
  }
  return released;
}
inline void AuthConfig::set_allocated_server_address(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.server_address_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.server_address_.IsDefault()) {
    _impl_.server_address_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AuthConfig.server_address)
}

// string identity_token = 5;
inline void AuthConfig::clear_identity_token() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.identity_token_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline const ::std::string& AuthConfig::identity_token() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AuthConfig.identity_token)
  return _internal_identity_token();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AuthConfig::set_identity_token(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  _impl_.identity_token_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AuthConfig.identity_token)
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::mutable_identity_token()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::std::string* _s = _internal_mutable_identity_token();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AuthConfig.identity_token)
  return _s;
}
inline const ::std::string& AuthConfig::_internal_identity_token() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.identity_token_.Get();
}
inline void AuthConfig::_internal_set_identity_token(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.identity_token_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::_internal_mutable_identity_token() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.identity_token_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AuthConfig::release_identity_token() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AuthConfig.identity_token)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000010U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  auto* released = _impl_.identity_token_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.identity_token_.Set("", GetArena());
  }
  return released;
}
inline void AuthConfig::set_allocated_identity_token(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  _impl_.identity_token_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.identity_token_.IsDefault()) {
    _impl_.identity_token_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AuthConfig.identity_token)
}

// string registry_token = 6;
inline void AuthConfig::clear_registry_token() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.registry_token_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline const ::std::string& AuthConfig::registry_token() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.AuthConfig.registry_token)
  return _internal_registry_token();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void AuthConfig::set_registry_token(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  _impl_.registry_token_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.AuthConfig.registry_token)
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::mutable_registry_token()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::std::string* _s = _internal_mutable_registry_token();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.AuthConfig.registry_token)
  return _s;
}
inline const ::std::string& AuthConfig::_internal_registry_token() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.registry_token_.Get();
}
inline void AuthConfig::_internal_set_registry_token(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.registry_token_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL AuthConfig::_internal_mutable_registry_token() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.registry_token_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE AuthConfig::release_registry_token() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.AuthConfig.registry_token)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000020U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  auto* released = _impl_.registry_token_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.registry_token_.Set("", GetArena());
  }
  return released;
}
inline void AuthConfig::set_allocated_registry_token(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  _impl_.registry_token_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.registry_token_.IsDefault()) {
    _impl_.registry_token_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.AuthConfig.registry_token)
}

// -------------------------------------------------------------------

// PullImageRequest

// .runtime.v1alpha2.ImageSpec image = 1;
inline bool PullImageRequest::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void PullImageRequest::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ImageSpec& PullImageRequest::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& PullImageRequest::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PullImageRequest.image)
  return _internal_image();
}
inline void PullImageRequest::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PullImageRequest.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE PullImageRequest::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE PullImageRequest::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PullImageRequest.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL PullImageRequest::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL PullImageRequest::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PullImageRequest.image)
  return _msg;
}
inline void PullImageRequest::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PullImageRequest.image)
}

// .runtime.v1alpha2.AuthConfig auth = 2;
inline bool PullImageRequest::has_auth() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.auth_ != nullptr);
  return value;
}
inline void PullImageRequest::clear_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.auth_ != nullptr) _impl_.auth_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::AuthConfig& PullImageRequest::_internal_auth() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::AuthConfig* p = _impl_.auth_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::AuthConfig&>(::runtime::v1alpha2::_AuthConfig_default_instance_);
}
inline const ::runtime::v1alpha2::AuthConfig& PullImageRequest::auth() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PullImageRequest.auth)
  return _internal_auth();
}
inline void PullImageRequest::unsafe_arena_set_allocated_auth(
    ::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.auth_);
  }
  _impl_.auth_ = reinterpret_cast<::runtime::v1alpha2::AuthConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PullImageRequest.auth)
}
inline ::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE PullImageRequest::release_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::AuthConfig* released = _impl_.auth_;
  _impl_.auth_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE PullImageRequest::unsafe_arena_release_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PullImageRequest.auth)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::AuthConfig* temp = _impl_.auth_;
  _impl_.auth_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::AuthConfig* PROTOBUF_NONNULL PullImageRequest::_internal_mutable_auth() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.auth_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::AuthConfig>(GetArena());
    _impl_.auth_ = reinterpret_cast<::runtime::v1alpha2::AuthConfig*>(p);
  }
  return _impl_.auth_;
}
inline ::runtime::v1alpha2::AuthConfig* PROTOBUF_NONNULL PullImageRequest::mutable_auth()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::AuthConfig* _msg = _internal_mutable_auth();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PullImageRequest.auth)
  return _msg;
}
inline void PullImageRequest::set_allocated_auth(::runtime::v1alpha2::AuthConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.auth_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.auth_ = reinterpret_cast<::runtime::v1alpha2::AuthConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PullImageRequest.auth)
}

// .runtime.v1alpha2.PodSandboxConfig sandbox_config = 3;
inline bool PullImageRequest::has_sandbox_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.sandbox_config_ != nullptr);
  return value;
}
inline void PullImageRequest::clear_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.sandbox_config_ != nullptr) _impl_.sandbox_config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::PodSandboxConfig& PullImageRequest::_internal_sandbox_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::PodSandboxConfig* p = _impl_.sandbox_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::PodSandboxConfig&>(::runtime::v1alpha2::_PodSandboxConfig_default_instance_);
}
inline const ::runtime::v1alpha2::PodSandboxConfig& PullImageRequest::sandbox_config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PullImageRequest.sandbox_config)
  return _internal_sandbox_config();
}
inline void PullImageRequest::unsafe_arena_set_allocated_sandbox_config(
    ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.sandbox_config_);
  }
  _impl_.sandbox_config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.PullImageRequest.sandbox_config)
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE PullImageRequest::release_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxConfig* released = _impl_.sandbox_config_;
  _impl_.sandbox_config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE PullImageRequest::unsafe_arena_release_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PullImageRequest.sandbox_config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxConfig* temp = _impl_.sandbox_config_;
  _impl_.sandbox_config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL PullImageRequest::_internal_mutable_sandbox_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.sandbox_config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::PodSandboxConfig>(GetArena());
    _impl_.sandbox_config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(p);
  }
  return _impl_.sandbox_config_;
}
inline ::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NONNULL PullImageRequest::mutable_sandbox_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::PodSandboxConfig* _msg = _internal_mutable_sandbox_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PullImageRequest.sandbox_config)
  return _msg;
}
inline void PullImageRequest::set_allocated_sandbox_config(::runtime::v1alpha2::PodSandboxConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.sandbox_config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.sandbox_config_ = reinterpret_cast<::runtime::v1alpha2::PodSandboxConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PullImageRequest.sandbox_config)
}

// -------------------------------------------------------------------

// PullImageResponse

// string image_ref = 1;
inline void PullImageResponse::clear_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_ref_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& PullImageResponse::image_ref() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.PullImageResponse.image_ref)
  return _internal_image_ref();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void PullImageResponse::set_image_ref(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.image_ref_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.PullImageResponse.image_ref)
}
inline ::std::string* PROTOBUF_NONNULL PullImageResponse::mutable_image_ref()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_image_ref();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.PullImageResponse.image_ref)
  return _s;
}
inline const ::std::string& PullImageResponse::_internal_image_ref() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.image_ref_.Get();
}
inline void PullImageResponse::_internal_set_image_ref(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_ref_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL PullImageResponse::_internal_mutable_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.image_ref_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE PullImageResponse::release_image_ref() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.PullImageResponse.image_ref)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.image_ref_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.image_ref_.Set("", GetArena());
  }
  return released;
}
inline void PullImageResponse::set_allocated_image_ref(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.image_ref_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.image_ref_.IsDefault()) {
    _impl_.image_ref_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.PullImageResponse.image_ref)
}

// -------------------------------------------------------------------

// RemoveImageRequest

// .runtime.v1alpha2.ImageSpec image = 1;
inline bool RemoveImageRequest::has_image() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.image_ != nullptr);
  return value;
}
inline void RemoveImageRequest::clear_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ != nullptr) _impl_.image_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ImageSpec& RemoveImageRequest::_internal_image() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ImageSpec* p = _impl_.image_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ImageSpec&>(::runtime::v1alpha2::_ImageSpec_default_instance_);
}
inline const ::runtime::v1alpha2::ImageSpec& RemoveImageRequest::image() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RemoveImageRequest.image)
  return _internal_image();
}
inline void RemoveImageRequest::unsafe_arena_set_allocated_image(
    ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }
  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.RemoveImageRequest.image)
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE RemoveImageRequest::release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* released = _impl_.image_;
  _impl_.image_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE RemoveImageRequest::unsafe_arena_release_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RemoveImageRequest.image)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* temp = _impl_.image_;
  _impl_.image_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL RemoveImageRequest::_internal_mutable_image() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.image_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ImageSpec>(GetArena());
    _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(p);
  }
  return _impl_.image_;
}
inline ::runtime::v1alpha2::ImageSpec* PROTOBUF_NONNULL RemoveImageRequest::mutable_image()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ImageSpec* _msg = _internal_mutable_image();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RemoveImageRequest.image)
  return _msg;
}
inline void RemoveImageRequest::set_allocated_image(::runtime::v1alpha2::ImageSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.image_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.image_ = reinterpret_cast<::runtime::v1alpha2::ImageSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RemoveImageRequest.image)
}

// -------------------------------------------------------------------

// RemoveImageResponse

// -------------------------------------------------------------------

// NetworkConfig

// string pod_cidr = 1;
inline void NetworkConfig::clear_pod_cidr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_cidr_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& NetworkConfig::pod_cidr() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.NetworkConfig.pod_cidr)
  return _internal_pod_cidr();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void NetworkConfig::set_pod_cidr(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pod_cidr_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.NetworkConfig.pod_cidr)
}
inline ::std::string* PROTOBUF_NONNULL NetworkConfig::mutable_pod_cidr()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pod_cidr();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.NetworkConfig.pod_cidr)
  return _s;
}
inline const ::std::string& NetworkConfig::_internal_pod_cidr() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_cidr_.Get();
}
inline void NetworkConfig::_internal_set_pod_cidr(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_cidr_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL NetworkConfig::_internal_mutable_pod_cidr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_cidr_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE NetworkConfig::release_pod_cidr() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.NetworkConfig.pod_cidr)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pod_cidr_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_cidr_.Set("", GetArena());
  }
  return released;
}
inline void NetworkConfig::set_allocated_pod_cidr(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pod_cidr_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_cidr_.IsDefault()) {
    _impl_.pod_cidr_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.NetworkConfig.pod_cidr)
}

// -------------------------------------------------------------------

// RuntimeConfig

// .runtime.v1alpha2.NetworkConfig network_config = 1;
inline bool RuntimeConfig::has_network_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.network_config_ != nullptr);
  return value;
}
inline void RuntimeConfig::clear_network_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.network_config_ != nullptr) _impl_.network_config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::NetworkConfig& RuntimeConfig::_internal_network_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::NetworkConfig* p = _impl_.network_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::NetworkConfig&>(::runtime::v1alpha2::_NetworkConfig_default_instance_);
}
inline const ::runtime::v1alpha2::NetworkConfig& RuntimeConfig::network_config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RuntimeConfig.network_config)
  return _internal_network_config();
}
inline void RuntimeConfig::unsafe_arena_set_allocated_network_config(
    ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.network_config_);
  }
  _impl_.network_config_ = reinterpret_cast<::runtime::v1alpha2::NetworkConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.RuntimeConfig.network_config)
}
inline ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE RuntimeConfig::release_network_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::NetworkConfig* released = _impl_.network_config_;
  _impl_.network_config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE RuntimeConfig::unsafe_arena_release_network_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RuntimeConfig.network_config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::NetworkConfig* temp = _impl_.network_config_;
  _impl_.network_config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NONNULL RuntimeConfig::_internal_mutable_network_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.network_config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::NetworkConfig>(GetArena());
    _impl_.network_config_ = reinterpret_cast<::runtime::v1alpha2::NetworkConfig*>(p);
  }
  return _impl_.network_config_;
}
inline ::runtime::v1alpha2::NetworkConfig* PROTOBUF_NONNULL RuntimeConfig::mutable_network_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::NetworkConfig* _msg = _internal_mutable_network_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RuntimeConfig.network_config)
  return _msg;
}
inline void RuntimeConfig::set_allocated_network_config(::runtime::v1alpha2::NetworkConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.network_config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.network_config_ = reinterpret_cast<::runtime::v1alpha2::NetworkConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RuntimeConfig.network_config)
}

// -------------------------------------------------------------------

// UpdateRuntimeConfigRequest

// .runtime.v1alpha2.RuntimeConfig runtime_config = 1;
inline bool UpdateRuntimeConfigRequest::has_runtime_config() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.runtime_config_ != nullptr);
  return value;
}
inline void UpdateRuntimeConfigRequest::clear_runtime_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.runtime_config_ != nullptr) _impl_.runtime_config_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::RuntimeConfig& UpdateRuntimeConfigRequest::_internal_runtime_config() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::RuntimeConfig* p = _impl_.runtime_config_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::RuntimeConfig&>(::runtime::v1alpha2::_RuntimeConfig_default_instance_);
}
inline const ::runtime::v1alpha2::RuntimeConfig& UpdateRuntimeConfigRequest::runtime_config() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.UpdateRuntimeConfigRequest.runtime_config)
  return _internal_runtime_config();
}
inline void UpdateRuntimeConfigRequest::unsafe_arena_set_allocated_runtime_config(
    ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.runtime_config_);
  }
  _impl_.runtime_config_ = reinterpret_cast<::runtime::v1alpha2::RuntimeConfig*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.UpdateRuntimeConfigRequest.runtime_config)
}
inline ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE UpdateRuntimeConfigRequest::release_runtime_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::RuntimeConfig* released = _impl_.runtime_config_;
  _impl_.runtime_config_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE UpdateRuntimeConfigRequest::unsafe_arena_release_runtime_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.UpdateRuntimeConfigRequest.runtime_config)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::RuntimeConfig* temp = _impl_.runtime_config_;
  _impl_.runtime_config_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NONNULL UpdateRuntimeConfigRequest::_internal_mutable_runtime_config() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.runtime_config_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::RuntimeConfig>(GetArena());
    _impl_.runtime_config_ = reinterpret_cast<::runtime::v1alpha2::RuntimeConfig*>(p);
  }
  return _impl_.runtime_config_;
}
inline ::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NONNULL UpdateRuntimeConfigRequest::mutable_runtime_config()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::RuntimeConfig* _msg = _internal_mutable_runtime_config();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.UpdateRuntimeConfigRequest.runtime_config)
  return _msg;
}
inline void UpdateRuntimeConfigRequest::set_allocated_runtime_config(::runtime::v1alpha2::RuntimeConfig* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.runtime_config_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.runtime_config_ = reinterpret_cast<::runtime::v1alpha2::RuntimeConfig*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.UpdateRuntimeConfigRequest.runtime_config)
}

// -------------------------------------------------------------------

// UpdateRuntimeConfigResponse

// -------------------------------------------------------------------

// RuntimeCondition

// string type = 1;
inline void RuntimeCondition::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& RuntimeCondition::type() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RuntimeCondition.type)
  return _internal_type();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RuntimeCondition::set_type(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.type_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RuntimeCondition.type)
}
inline ::std::string* PROTOBUF_NONNULL RuntimeCondition::mutable_type()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_type();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RuntimeCondition.type)
  return _s;
}
inline const ::std::string& RuntimeCondition::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.type_.Get();
}
inline void RuntimeCondition::_internal_set_type(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RuntimeCondition::_internal_mutable_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.type_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RuntimeCondition::release_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RuntimeCondition.type)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.type_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.type_.Set("", GetArena());
  }
  return released;
}
inline void RuntimeCondition::set_allocated_type(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.type_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.type_.IsDefault()) {
    _impl_.type_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RuntimeCondition.type)
}

// bool status = 2;
inline void RuntimeCondition::clear_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.status_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline bool RuntimeCondition::status() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RuntimeCondition.status)
  return _internal_status();
}
inline void RuntimeCondition::set_status(bool value) {
  _internal_set_status(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RuntimeCondition.status)
}
inline bool RuntimeCondition::_internal_status() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.status_;
}
inline void RuntimeCondition::_internal_set_status(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.status_ = value;
}

// string reason = 3;
inline void RuntimeCondition::clear_reason() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.reason_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& RuntimeCondition::reason() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RuntimeCondition.reason)
  return _internal_reason();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RuntimeCondition::set_reason(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.reason_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RuntimeCondition.reason)
}
inline ::std::string* PROTOBUF_NONNULL RuntimeCondition::mutable_reason()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_reason();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RuntimeCondition.reason)
  return _s;
}
inline const ::std::string& RuntimeCondition::_internal_reason() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.reason_.Get();
}
inline void RuntimeCondition::_internal_set_reason(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.reason_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RuntimeCondition::_internal_mutable_reason() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.reason_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RuntimeCondition::release_reason() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RuntimeCondition.reason)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.reason_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.reason_.Set("", GetArena());
  }
  return released;
}
inline void RuntimeCondition::set_allocated_reason(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.reason_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.reason_.IsDefault()) {
    _impl_.reason_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RuntimeCondition.reason)
}

// string message = 4;
inline void RuntimeCondition::clear_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.message_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& RuntimeCondition::message() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RuntimeCondition.message)
  return _internal_message();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void RuntimeCondition::set_message(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.message_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.RuntimeCondition.message)
}
inline ::std::string* PROTOBUF_NONNULL RuntimeCondition::mutable_message()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_message();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RuntimeCondition.message)
  return _s;
}
inline const ::std::string& RuntimeCondition::_internal_message() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.message_.Get();
}
inline void RuntimeCondition::_internal_set_message(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.message_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL RuntimeCondition::_internal_mutable_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.message_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE RuntimeCondition::release_message() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.RuntimeCondition.message)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.message_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.message_.Set("", GetArena());
  }
  return released;
}
inline void RuntimeCondition::set_allocated_message(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.message_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.message_.IsDefault()) {
    _impl_.message_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.RuntimeCondition.message)
}

// -------------------------------------------------------------------

// RuntimeStatus

// repeated .runtime.v1alpha2.RuntimeCondition conditions = 1;
inline int RuntimeStatus::_internal_conditions_size() const {
  return _internal_conditions().size();
}
inline int RuntimeStatus::conditions_size() const {
  return _internal_conditions_size();
}
inline void RuntimeStatus::clear_conditions() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.conditions_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::RuntimeCondition* PROTOBUF_NONNULL RuntimeStatus::mutable_conditions(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.RuntimeStatus.conditions)
  return _internal_mutable_conditions()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>* PROTOBUF_NONNULL RuntimeStatus::mutable_conditions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.RuntimeStatus.conditions)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_conditions();
}
inline const ::runtime::v1alpha2::RuntimeCondition& RuntimeStatus::conditions(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.RuntimeStatus.conditions)
  return _internal_conditions().Get(index);
}
inline ::runtime::v1alpha2::RuntimeCondition* PROTOBUF_NONNULL RuntimeStatus::add_conditions()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::RuntimeCondition* _add =
      _internal_mutable_conditions()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.RuntimeStatus.conditions)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>& RuntimeStatus::conditions() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.RuntimeStatus.conditions)
  return _internal_conditions();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>&
RuntimeStatus::_internal_conditions() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.conditions_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::RuntimeCondition>* PROTOBUF_NONNULL
RuntimeStatus::_internal_mutable_conditions() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.conditions_;
}

// -------------------------------------------------------------------

// StatusRequest

// bool verbose = 1;
inline void StatusRequest::clear_verbose() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline bool StatusRequest::verbose() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.StatusRequest.verbose)
  return _internal_verbose();
}
inline void StatusRequest::set_verbose(bool value) {
  _internal_set_verbose(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.StatusRequest.verbose)
}
inline bool StatusRequest::_internal_verbose() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.verbose_;
}
inline void StatusRequest::_internal_set_verbose(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.verbose_ = value;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// StatusResponse

// .runtime.v1alpha2.RuntimeStatus status = 1;
inline bool StatusResponse::has_status() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.status_ != nullptr);
  return value;
}
inline void StatusResponse::clear_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.status_ != nullptr) _impl_.status_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::RuntimeStatus& StatusResponse::_internal_status() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::RuntimeStatus* p = _impl_.status_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::RuntimeStatus&>(::runtime::v1alpha2::_RuntimeStatus_default_instance_);
}
inline const ::runtime::v1alpha2::RuntimeStatus& StatusResponse::status() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.StatusResponse.status)
  return _internal_status();
}
inline void StatusResponse::unsafe_arena_set_allocated_status(
    ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.status_);
  }
  _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::RuntimeStatus*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.StatusResponse.status)
}
inline ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE StatusResponse::release_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::RuntimeStatus* released = _impl_.status_;
  _impl_.status_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE StatusResponse::unsafe_arena_release_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.StatusResponse.status)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::RuntimeStatus* temp = _impl_.status_;
  _impl_.status_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NONNULL StatusResponse::_internal_mutable_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.status_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::RuntimeStatus>(GetArena());
    _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::RuntimeStatus*>(p);
  }
  return _impl_.status_;
}
inline ::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NONNULL StatusResponse::mutable_status()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::RuntimeStatus* _msg = _internal_mutable_status();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.StatusResponse.status)
  return _msg;
}
inline void StatusResponse::set_allocated_status(::runtime::v1alpha2::RuntimeStatus* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.status_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.status_ = reinterpret_cast<::runtime::v1alpha2::RuntimeStatus*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.StatusResponse.status)
}

// map<string, string> info = 2;
inline int StatusResponse::_internal_info_size() const {
  return _internal_info().size();
}
inline int StatusResponse::info_size() const {
  return _internal_info_size();
}
inline void StatusResponse::clear_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.info_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& StatusResponse::_internal_info() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.info_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& StatusResponse::info() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.StatusResponse.info)
  return _internal_info();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL StatusResponse::_internal_mutable_info() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.info_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL StatusResponse::mutable_info()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.StatusResponse.info)
  return _internal_mutable_info();
}

// -------------------------------------------------------------------

// ImageFsInfoRequest

// -------------------------------------------------------------------

// UInt64Value

// uint64 value = 1;
inline void UInt64Value::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::uint64_t UInt64Value::value() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.UInt64Value.value)
  return _internal_value();
}
inline void UInt64Value::set_value(::uint64_t value) {
  _internal_set_value(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.UInt64Value.value)
}
inline ::uint64_t UInt64Value::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.value_;
}
inline void UInt64Value::_internal_set_value(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.value_ = value;
}

// -------------------------------------------------------------------

// FilesystemIdentifier

// string mountpoint = 1;
inline void FilesystemIdentifier::clear_mountpoint() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mountpoint_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& FilesystemIdentifier::mountpoint() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.FilesystemIdentifier.mountpoint)
  return _internal_mountpoint();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void FilesystemIdentifier::set_mountpoint(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.mountpoint_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.FilesystemIdentifier.mountpoint)
}
inline ::std::string* PROTOBUF_NONNULL FilesystemIdentifier::mutable_mountpoint()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_mountpoint();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.FilesystemIdentifier.mountpoint)
  return _s;
}
inline const ::std::string& FilesystemIdentifier::_internal_mountpoint() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mountpoint_.Get();
}
inline void FilesystemIdentifier::_internal_set_mountpoint(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mountpoint_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL FilesystemIdentifier::_internal_mutable_mountpoint() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.mountpoint_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE FilesystemIdentifier::release_mountpoint() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.FilesystemIdentifier.mountpoint)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.mountpoint_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.mountpoint_.Set("", GetArena());
  }
  return released;
}
inline void FilesystemIdentifier::set_allocated_mountpoint(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.mountpoint_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.mountpoint_.IsDefault()) {
    _impl_.mountpoint_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.FilesystemIdentifier.mountpoint)
}

// -------------------------------------------------------------------

// FilesystemUsage

// int64 timestamp = 1;
inline void FilesystemUsage::clear_timestamp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timestamp_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::int64_t FilesystemUsage::timestamp() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.FilesystemUsage.timestamp)
  return _internal_timestamp();
}
inline void FilesystemUsage::set_timestamp(::int64_t value) {
  _internal_set_timestamp(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.FilesystemUsage.timestamp)
}
inline ::int64_t FilesystemUsage::_internal_timestamp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.timestamp_;
}
inline void FilesystemUsage::_internal_set_timestamp(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timestamp_ = value;
}

// .runtime.v1alpha2.FilesystemIdentifier fs_id = 2;
inline bool FilesystemUsage::has_fs_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.fs_id_ != nullptr);
  return value;
}
inline void FilesystemUsage::clear_fs_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.fs_id_ != nullptr) _impl_.fs_id_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::FilesystemIdentifier& FilesystemUsage::_internal_fs_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::FilesystemIdentifier* p = _impl_.fs_id_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::FilesystemIdentifier&>(::runtime::v1alpha2::_FilesystemIdentifier_default_instance_);
}
inline const ::runtime::v1alpha2::FilesystemIdentifier& FilesystemUsage::fs_id() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.FilesystemUsage.fs_id)
  return _internal_fs_id();
}
inline void FilesystemUsage::unsafe_arena_set_allocated_fs_id(
    ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.fs_id_);
  }
  _impl_.fs_id_ = reinterpret_cast<::runtime::v1alpha2::FilesystemIdentifier*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.FilesystemUsage.fs_id)
}
inline ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE FilesystemUsage::release_fs_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::FilesystemIdentifier* released = _impl_.fs_id_;
  _impl_.fs_id_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE FilesystemUsage::unsafe_arena_release_fs_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.FilesystemUsage.fs_id)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::FilesystemIdentifier* temp = _impl_.fs_id_;
  _impl_.fs_id_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NONNULL FilesystemUsage::_internal_mutable_fs_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.fs_id_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::FilesystemIdentifier>(GetArena());
    _impl_.fs_id_ = reinterpret_cast<::runtime::v1alpha2::FilesystemIdentifier*>(p);
  }
  return _impl_.fs_id_;
}
inline ::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NONNULL FilesystemUsage::mutable_fs_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::FilesystemIdentifier* _msg = _internal_mutable_fs_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.FilesystemUsage.fs_id)
  return _msg;
}
inline void FilesystemUsage::set_allocated_fs_id(::runtime::v1alpha2::FilesystemIdentifier* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.fs_id_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.fs_id_ = reinterpret_cast<::runtime::v1alpha2::FilesystemIdentifier*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.FilesystemUsage.fs_id)
}

// .runtime.v1alpha2.UInt64Value used_bytes = 3;
inline bool FilesystemUsage::has_used_bytes() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.used_bytes_ != nullptr);
  return value;
}
inline void FilesystemUsage::clear_used_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.used_bytes_ != nullptr) _impl_.used_bytes_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::UInt64Value& FilesystemUsage::_internal_used_bytes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::UInt64Value* p = _impl_.used_bytes_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::UInt64Value&>(::runtime::v1alpha2::_UInt64Value_default_instance_);
}
inline const ::runtime::v1alpha2::UInt64Value& FilesystemUsage::used_bytes() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.FilesystemUsage.used_bytes)
  return _internal_used_bytes();
}
inline void FilesystemUsage::unsafe_arena_set_allocated_used_bytes(
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.used_bytes_);
  }
  _impl_.used_bytes_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.FilesystemUsage.used_bytes)
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE FilesystemUsage::release_used_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::UInt64Value* released = _impl_.used_bytes_;
  _impl_.used_bytes_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE FilesystemUsage::unsafe_arena_release_used_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.FilesystemUsage.used_bytes)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::UInt64Value* temp = _impl_.used_bytes_;
  _impl_.used_bytes_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL FilesystemUsage::_internal_mutable_used_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.used_bytes_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::UInt64Value>(GetArena());
    _impl_.used_bytes_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(p);
  }
  return _impl_.used_bytes_;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL FilesystemUsage::mutable_used_bytes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::UInt64Value* _msg = _internal_mutable_used_bytes();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.FilesystemUsage.used_bytes)
  return _msg;
}
inline void FilesystemUsage::set_allocated_used_bytes(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.used_bytes_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.used_bytes_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.FilesystemUsage.used_bytes)
}

// .runtime.v1alpha2.UInt64Value inodes_used = 4;
inline bool FilesystemUsage::has_inodes_used() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.inodes_used_ != nullptr);
  return value;
}
inline void FilesystemUsage::clear_inodes_used() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.inodes_used_ != nullptr) _impl_.inodes_used_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::UInt64Value& FilesystemUsage::_internal_inodes_used() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::UInt64Value* p = _impl_.inodes_used_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::UInt64Value&>(::runtime::v1alpha2::_UInt64Value_default_instance_);
}
inline const ::runtime::v1alpha2::UInt64Value& FilesystemUsage::inodes_used() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.FilesystemUsage.inodes_used)
  return _internal_inodes_used();
}
inline void FilesystemUsage::unsafe_arena_set_allocated_inodes_used(
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.inodes_used_);
  }
  _impl_.inodes_used_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.FilesystemUsage.inodes_used)
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE FilesystemUsage::release_inodes_used() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::UInt64Value* released = _impl_.inodes_used_;
  _impl_.inodes_used_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE FilesystemUsage::unsafe_arena_release_inodes_used() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.FilesystemUsage.inodes_used)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::UInt64Value* temp = _impl_.inodes_used_;
  _impl_.inodes_used_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL FilesystemUsage::_internal_mutable_inodes_used() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.inodes_used_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::UInt64Value>(GetArena());
    _impl_.inodes_used_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(p);
  }
  return _impl_.inodes_used_;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL FilesystemUsage::mutable_inodes_used()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::UInt64Value* _msg = _internal_mutable_inodes_used();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.FilesystemUsage.inodes_used)
  return _msg;
}
inline void FilesystemUsage::set_allocated_inodes_used(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.inodes_used_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.inodes_used_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.FilesystemUsage.inodes_used)
}

// -------------------------------------------------------------------

// ImageFsInfoResponse

// repeated .runtime.v1alpha2.FilesystemUsage image_filesystems = 1;
inline int ImageFsInfoResponse::_internal_image_filesystems_size() const {
  return _internal_image_filesystems().size();
}
inline int ImageFsInfoResponse::image_filesystems_size() const {
  return _internal_image_filesystems_size();
}
inline void ImageFsInfoResponse::clear_image_filesystems() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.image_filesystems_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL ImageFsInfoResponse::mutable_image_filesystems(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ImageFsInfoResponse.image_filesystems)
  return _internal_mutable_image_filesystems()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>* PROTOBUF_NONNULL ImageFsInfoResponse::mutable_image_filesystems()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ImageFsInfoResponse.image_filesystems)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_image_filesystems();
}
inline const ::runtime::v1alpha2::FilesystemUsage& ImageFsInfoResponse::image_filesystems(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ImageFsInfoResponse.image_filesystems)
  return _internal_image_filesystems().Get(index);
}
inline ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL ImageFsInfoResponse::add_image_filesystems()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::FilesystemUsage* _add =
      _internal_mutable_image_filesystems()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ImageFsInfoResponse.image_filesystems)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>& ImageFsInfoResponse::image_filesystems() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ImageFsInfoResponse.image_filesystems)
  return _internal_image_filesystems();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>&
ImageFsInfoResponse::_internal_image_filesystems() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.image_filesystems_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::FilesystemUsage>* PROTOBUF_NONNULL
ImageFsInfoResponse::_internal_mutable_image_filesystems() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.image_filesystems_;
}

// -------------------------------------------------------------------

// ContainerStatsRequest

// string container_id = 1;
inline void ContainerStatsRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ContainerStatsRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatsRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatsRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatsRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatsRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatsRequest.container_id)
  return _s;
}
inline const ::std::string& ContainerStatsRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void ContainerStatsRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatsRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatsRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatsRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatsRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatsRequest.container_id)
}

// -------------------------------------------------------------------

// ContainerStatsResponse

// .runtime.v1alpha2.ContainerStats stats = 1;
inline bool ContainerStatsResponse::has_stats() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.stats_ != nullptr);
  return value;
}
inline void ContainerStatsResponse::clear_stats() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.stats_ != nullptr) _impl_.stats_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ContainerStats& ContainerStatsResponse::_internal_stats() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerStats* p = _impl_.stats_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerStats&>(::runtime::v1alpha2::_ContainerStats_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerStats& ContainerStatsResponse::stats() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatsResponse.stats)
  return _internal_stats();
}
inline void ContainerStatsResponse::unsafe_arena_set_allocated_stats(
    ::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.stats_);
  }
  _impl_.stats_ = reinterpret_cast<::runtime::v1alpha2::ContainerStats*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStatsResponse.stats)
}
inline ::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE ContainerStatsResponse::release_stats() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStats* released = _impl_.stats_;
  _impl_.stats_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE ContainerStatsResponse::unsafe_arena_release_stats() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatsResponse.stats)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStats* temp = _impl_.stats_;
  _impl_.stats_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL ContainerStatsResponse::_internal_mutable_stats() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.stats_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerStats>(GetArena());
    _impl_.stats_ = reinterpret_cast<::runtime::v1alpha2::ContainerStats*>(p);
  }
  return _impl_.stats_;
}
inline ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL ContainerStatsResponse::mutable_stats()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStats* _msg = _internal_mutable_stats();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatsResponse.stats)
  return _msg;
}
inline void ContainerStatsResponse::set_allocated_stats(::runtime::v1alpha2::ContainerStats* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.stats_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.stats_ = reinterpret_cast<::runtime::v1alpha2::ContainerStats*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatsResponse.stats)
}

// -------------------------------------------------------------------

// ListContainerStatsRequest

// .runtime.v1alpha2.ContainerStatsFilter filter = 1;
inline bool ListContainerStatsRequest::has_filter() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.filter_ != nullptr);
  return value;
}
inline void ListContainerStatsRequest::clear_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ != nullptr) _impl_.filter_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ContainerStatsFilter& ListContainerStatsRequest::_internal_filter() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerStatsFilter* p = _impl_.filter_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerStatsFilter&>(::runtime::v1alpha2::_ContainerStatsFilter_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerStatsFilter& ListContainerStatsRequest::filter() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListContainerStatsRequest.filter)
  return _internal_filter();
}
inline void ListContainerStatsRequest::unsafe_arena_set_allocated_filter(
    ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }
  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ContainerStatsFilter*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ListContainerStatsRequest.filter)
}
inline ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE ListContainerStatsRequest::release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStatsFilter* released = _impl_.filter_;
  _impl_.filter_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE ListContainerStatsRequest::unsafe_arena_release_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ListContainerStatsRequest.filter)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStatsFilter* temp = _impl_.filter_;
  _impl_.filter_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NONNULL ListContainerStatsRequest::_internal_mutable_filter() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.filter_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerStatsFilter>(GetArena());
    _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ContainerStatsFilter*>(p);
  }
  return _impl_.filter_;
}
inline ::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NONNULL ListContainerStatsRequest::mutable_filter()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerStatsFilter* _msg = _internal_mutable_filter();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListContainerStatsRequest.filter)
  return _msg;
}
inline void ListContainerStatsRequest::set_allocated_filter(::runtime::v1alpha2::ContainerStatsFilter* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.filter_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.filter_ = reinterpret_cast<::runtime::v1alpha2::ContainerStatsFilter*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ListContainerStatsRequest.filter)
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ContainerStatsFilter

// string id = 1;
inline void ContainerStatsFilter::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ContainerStatsFilter::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatsFilter.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatsFilter::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatsFilter.id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatsFilter::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatsFilter.id)
  return _s;
}
inline const ::std::string& ContainerStatsFilter::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void ContainerStatsFilter::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatsFilter::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatsFilter::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatsFilter.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatsFilter::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatsFilter.id)
}

// string pod_sandbox_id = 2;
inline void ContainerStatsFilter::clear_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::std::string& ContainerStatsFilter::pod_sandbox_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStatsFilter.pod_sandbox_id)
  return _internal_pod_sandbox_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerStatsFilter::set_pod_sandbox_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pod_sandbox_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerStatsFilter.pod_sandbox_id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatsFilter::mutable_pod_sandbox_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pod_sandbox_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStatsFilter.pod_sandbox_id)
  return _s;
}
inline const ::std::string& ContainerStatsFilter::_internal_pod_sandbox_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pod_sandbox_id_.Get();
}
inline void ContainerStatsFilter::_internal_set_pod_sandbox_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pod_sandbox_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerStatsFilter::_internal_mutable_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pod_sandbox_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerStatsFilter::release_pod_sandbox_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStatsFilter.pod_sandbox_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pod_sandbox_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerStatsFilter::set_allocated_pod_sandbox_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pod_sandbox_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pod_sandbox_id_.IsDefault()) {
    _impl_.pod_sandbox_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStatsFilter.pod_sandbox_id)
}

// map<string, string> label_selector = 3;
inline int ContainerStatsFilter::_internal_label_selector_size() const {
  return _internal_label_selector().size();
}
inline int ContainerStatsFilter::label_selector_size() const {
  return _internal_label_selector_size();
}
inline void ContainerStatsFilter::clear_label_selector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_selector_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatsFilter::_internal_label_selector() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.label_selector_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerStatsFilter::label_selector() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerStatsFilter.label_selector)
  return _internal_label_selector();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatsFilter::_internal_mutable_label_selector() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.label_selector_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerStatsFilter::mutable_label_selector()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerStatsFilter.label_selector)
  return _internal_mutable_label_selector();
}

// -------------------------------------------------------------------

// ListContainerStatsResponse

// repeated .runtime.v1alpha2.ContainerStats stats = 1;
inline int ListContainerStatsResponse::_internal_stats_size() const {
  return _internal_stats().size();
}
inline int ListContainerStatsResponse::stats_size() const {
  return _internal_stats_size();
}
inline void ListContainerStatsResponse::clear_stats() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stats_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL ListContainerStatsResponse::mutable_stats(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ListContainerStatsResponse.stats)
  return _internal_mutable_stats()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>* PROTOBUF_NONNULL ListContainerStatsResponse::mutable_stats()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:runtime.v1alpha2.ListContainerStatsResponse.stats)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_stats();
}
inline const ::runtime::v1alpha2::ContainerStats& ListContainerStatsResponse::stats(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ListContainerStatsResponse.stats)
  return _internal_stats().Get(index);
}
inline ::runtime::v1alpha2::ContainerStats* PROTOBUF_NONNULL ListContainerStatsResponse::add_stats()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::runtime::v1alpha2::ContainerStats* _add =
      _internal_mutable_stats()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:runtime.v1alpha2.ListContainerStatsResponse.stats)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>& ListContainerStatsResponse::stats() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:runtime.v1alpha2.ListContainerStatsResponse.stats)
  return _internal_stats();
}
inline const ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>&
ListContainerStatsResponse::_internal_stats() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.stats_;
}
inline ::google::protobuf::RepeatedPtrField<::runtime::v1alpha2::ContainerStats>* PROTOBUF_NONNULL
ListContainerStatsResponse::_internal_mutable_stats() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.stats_;
}

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// -------------------------------------------------------------------

// ContainerAttributes

// string id = 1;
inline void ContainerAttributes::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ContainerAttributes::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerAttributes.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContainerAttributes::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ContainerAttributes.id)
}
inline ::std::string* PROTOBUF_NONNULL ContainerAttributes::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerAttributes.id)
  return _s;
}
inline const ::std::string& ContainerAttributes::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void ContainerAttributes::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContainerAttributes::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContainerAttributes::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerAttributes.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void ContainerAttributes::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerAttributes.id)
}

// .runtime.v1alpha2.ContainerMetadata metadata = 2;
inline bool ContainerAttributes::has_metadata() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.metadata_ != nullptr);
  return value;
}
inline void ContainerAttributes::clear_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ != nullptr) _impl_.metadata_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::ContainerMetadata& ContainerAttributes::_internal_metadata() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerMetadata* p = _impl_.metadata_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerMetadata&>(::runtime::v1alpha2::_ContainerMetadata_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerMetadata& ContainerAttributes::metadata() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerAttributes.metadata)
  return _internal_metadata();
}
inline void ContainerAttributes::unsafe_arena_set_allocated_metadata(
    ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }
  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerAttributes.metadata)
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE ContainerAttributes::release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::ContainerMetadata* released = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE ContainerAttributes::unsafe_arena_release_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerAttributes.metadata)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::ContainerMetadata* temp = _impl_.metadata_;
  _impl_.metadata_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL ContainerAttributes::_internal_mutable_metadata() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.metadata_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerMetadata>(GetArena());
    _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(p);
  }
  return _impl_.metadata_;
}
inline ::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NONNULL ContainerAttributes::mutable_metadata()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::ContainerMetadata* _msg = _internal_mutable_metadata();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerAttributes.metadata)
  return _msg;
}
inline void ContainerAttributes::set_allocated_metadata(::runtime::v1alpha2::ContainerMetadata* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.metadata_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.metadata_ = reinterpret_cast<::runtime::v1alpha2::ContainerMetadata*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerAttributes.metadata)
}

// map<string, string> labels = 3;
inline int ContainerAttributes::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int ContainerAttributes::labels_size() const {
  return _internal_labels_size();
}
inline void ContainerAttributes::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerAttributes::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerAttributes::labels() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerAttributes.labels)
  return _internal_labels();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerAttributes::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.labels_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerAttributes::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerAttributes.labels)
  return _internal_mutable_labels();
}

// map<string, string> annotations = 4;
inline int ContainerAttributes::_internal_annotations_size() const {
  return _internal_annotations().size();
}
inline int ContainerAttributes::annotations_size() const {
  return _internal_annotations_size();
}
inline void ContainerAttributes::clear_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotations_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerAttributes::_internal_annotations() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotations_.GetMap();
}
inline const ::google::protobuf::Map<::std::string, ::std::string>& ContainerAttributes::annotations() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_map:runtime.v1alpha2.ContainerAttributes.annotations)
  return _internal_annotations();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerAttributes::_internal_mutable_annotations() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.annotations_.MutableMap();
}
inline ::google::protobuf::Map<::std::string, ::std::string>* PROTOBUF_NONNULL ContainerAttributes::mutable_annotations()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_mutable_map:runtime.v1alpha2.ContainerAttributes.annotations)
  return _internal_mutable_annotations();
}

// -------------------------------------------------------------------

// ContainerStats

// .runtime.v1alpha2.ContainerAttributes attributes = 1;
inline bool ContainerStats::has_attributes() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.attributes_ != nullptr);
  return value;
}
inline void ContainerStats::clear_attributes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.attributes_ != nullptr) _impl_.attributes_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::ContainerAttributes& ContainerStats::_internal_attributes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::ContainerAttributes* p = _impl_.attributes_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::ContainerAttributes&>(::runtime::v1alpha2::_ContainerAttributes_default_instance_);
}
inline const ::runtime::v1alpha2::ContainerAttributes& ContainerStats::attributes() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStats.attributes)
  return _internal_attributes();
}
inline void ContainerStats::unsafe_arena_set_allocated_attributes(
    ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.attributes_);
  }
  _impl_.attributes_ = reinterpret_cast<::runtime::v1alpha2::ContainerAttributes*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStats.attributes)
}
inline ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE ContainerStats::release_attributes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerAttributes* released = _impl_.attributes_;
  _impl_.attributes_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE ContainerStats::unsafe_arena_release_attributes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStats.attributes)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerAttributes* temp = _impl_.attributes_;
  _impl_.attributes_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NONNULL ContainerStats::_internal_mutable_attributes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.attributes_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::ContainerAttributes>(GetArena());
    _impl_.attributes_ = reinterpret_cast<::runtime::v1alpha2::ContainerAttributes*>(p);
  }
  return _impl_.attributes_;
}
inline ::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NONNULL ContainerStats::mutable_attributes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::ContainerAttributes* _msg = _internal_mutable_attributes();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStats.attributes)
  return _msg;
}
inline void ContainerStats::set_allocated_attributes(::runtime::v1alpha2::ContainerAttributes* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.attributes_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.attributes_ = reinterpret_cast<::runtime::v1alpha2::ContainerAttributes*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStats.attributes)
}

// .runtime.v1alpha2.CpuUsage cpu = 2;
inline bool ContainerStats::has_cpu() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.cpu_ != nullptr);
  return value;
}
inline void ContainerStats::clear_cpu() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.cpu_ != nullptr) _impl_.cpu_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::runtime::v1alpha2::CpuUsage& ContainerStats::_internal_cpu() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::CpuUsage* p = _impl_.cpu_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::CpuUsage&>(::runtime::v1alpha2::_CpuUsage_default_instance_);
}
inline const ::runtime::v1alpha2::CpuUsage& ContainerStats::cpu() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStats.cpu)
  return _internal_cpu();
}
inline void ContainerStats::unsafe_arena_set_allocated_cpu(
    ::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.cpu_);
  }
  _impl_.cpu_ = reinterpret_cast<::runtime::v1alpha2::CpuUsage*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStats.cpu)
}
inline ::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE ContainerStats::release_cpu() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::CpuUsage* released = _impl_.cpu_;
  _impl_.cpu_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE ContainerStats::unsafe_arena_release_cpu() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStats.cpu)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::CpuUsage* temp = _impl_.cpu_;
  _impl_.cpu_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::CpuUsage* PROTOBUF_NONNULL ContainerStats::_internal_mutable_cpu() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.cpu_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::CpuUsage>(GetArena());
    _impl_.cpu_ = reinterpret_cast<::runtime::v1alpha2::CpuUsage*>(p);
  }
  return _impl_.cpu_;
}
inline ::runtime::v1alpha2::CpuUsage* PROTOBUF_NONNULL ContainerStats::mutable_cpu()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::runtime::v1alpha2::CpuUsage* _msg = _internal_mutable_cpu();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStats.cpu)
  return _msg;
}
inline void ContainerStats::set_allocated_cpu(::runtime::v1alpha2::CpuUsage* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.cpu_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.cpu_ = reinterpret_cast<::runtime::v1alpha2::CpuUsage*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStats.cpu)
}

// .runtime.v1alpha2.MemoryUsage memory = 3;
inline bool ContainerStats::has_memory() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.memory_ != nullptr);
  return value;
}
inline void ContainerStats::clear_memory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.memory_ != nullptr) _impl_.memory_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::runtime::v1alpha2::MemoryUsage& ContainerStats::_internal_memory() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::MemoryUsage* p = _impl_.memory_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::MemoryUsage&>(::runtime::v1alpha2::_MemoryUsage_default_instance_);
}
inline const ::runtime::v1alpha2::MemoryUsage& ContainerStats::memory() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStats.memory)
  return _internal_memory();
}
inline void ContainerStats::unsafe_arena_set_allocated_memory(
    ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.memory_);
  }
  _impl_.memory_ = reinterpret_cast<::runtime::v1alpha2::MemoryUsage*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStats.memory)
}
inline ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE ContainerStats::release_memory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::MemoryUsage* released = _impl_.memory_;
  _impl_.memory_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE ContainerStats::unsafe_arena_release_memory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStats.memory)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::MemoryUsage* temp = _impl_.memory_;
  _impl_.memory_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NONNULL ContainerStats::_internal_mutable_memory() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.memory_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::MemoryUsage>(GetArena());
    _impl_.memory_ = reinterpret_cast<::runtime::v1alpha2::MemoryUsage*>(p);
  }
  return _impl_.memory_;
}
inline ::runtime::v1alpha2::MemoryUsage* PROTOBUF_NONNULL ContainerStats::mutable_memory()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::runtime::v1alpha2::MemoryUsage* _msg = _internal_mutable_memory();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStats.memory)
  return _msg;
}
inline void ContainerStats::set_allocated_memory(::runtime::v1alpha2::MemoryUsage* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.memory_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.memory_ = reinterpret_cast<::runtime::v1alpha2::MemoryUsage*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStats.memory)
}

// .runtime.v1alpha2.FilesystemUsage writable_layer = 4;
inline bool ContainerStats::has_writable_layer() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.writable_layer_ != nullptr);
  return value;
}
inline void ContainerStats::clear_writable_layer() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.writable_layer_ != nullptr) _impl_.writable_layer_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline const ::runtime::v1alpha2::FilesystemUsage& ContainerStats::_internal_writable_layer() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::FilesystemUsage* p = _impl_.writable_layer_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::FilesystemUsage&>(::runtime::v1alpha2::_FilesystemUsage_default_instance_);
}
inline const ::runtime::v1alpha2::FilesystemUsage& ContainerStats::writable_layer() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ContainerStats.writable_layer)
  return _internal_writable_layer();
}
inline void ContainerStats::unsafe_arena_set_allocated_writable_layer(
    ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.writable_layer_);
  }
  _impl_.writable_layer_ = reinterpret_cast<::runtime::v1alpha2::FilesystemUsage*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.ContainerStats.writable_layer)
}
inline ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE ContainerStats::release_writable_layer() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::FilesystemUsage* released = _impl_.writable_layer_;
  _impl_.writable_layer_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE ContainerStats::unsafe_arena_release_writable_layer() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ContainerStats.writable_layer)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::FilesystemUsage* temp = _impl_.writable_layer_;
  _impl_.writable_layer_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL ContainerStats::_internal_mutable_writable_layer() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.writable_layer_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::FilesystemUsage>(GetArena());
    _impl_.writable_layer_ = reinterpret_cast<::runtime::v1alpha2::FilesystemUsage*>(p);
  }
  return _impl_.writable_layer_;
}
inline ::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NONNULL ContainerStats::mutable_writable_layer()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::runtime::v1alpha2::FilesystemUsage* _msg = _internal_mutable_writable_layer();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ContainerStats.writable_layer)
  return _msg;
}
inline void ContainerStats::set_allocated_writable_layer(::runtime::v1alpha2::FilesystemUsage* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.writable_layer_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.writable_layer_ = reinterpret_cast<::runtime::v1alpha2::FilesystemUsage*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ContainerStats.writable_layer)
}

// -------------------------------------------------------------------

// CpuUsage

// int64 timestamp = 1;
inline void CpuUsage::clear_timestamp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timestamp_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int64_t CpuUsage::timestamp() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.CpuUsage.timestamp)
  return _internal_timestamp();
}
inline void CpuUsage::set_timestamp(::int64_t value) {
  _internal_set_timestamp(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.CpuUsage.timestamp)
}
inline ::int64_t CpuUsage::_internal_timestamp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.timestamp_;
}
inline void CpuUsage::_internal_set_timestamp(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timestamp_ = value;
}

// .runtime.v1alpha2.UInt64Value usage_core_nano_seconds = 2;
inline bool CpuUsage::has_usage_core_nano_seconds() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.usage_core_nano_seconds_ != nullptr);
  return value;
}
inline void CpuUsage::clear_usage_core_nano_seconds() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.usage_core_nano_seconds_ != nullptr) _impl_.usage_core_nano_seconds_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::UInt64Value& CpuUsage::_internal_usage_core_nano_seconds() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::UInt64Value* p = _impl_.usage_core_nano_seconds_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::UInt64Value&>(::runtime::v1alpha2::_UInt64Value_default_instance_);
}
inline const ::runtime::v1alpha2::UInt64Value& CpuUsage::usage_core_nano_seconds() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.CpuUsage.usage_core_nano_seconds)
  return _internal_usage_core_nano_seconds();
}
inline void CpuUsage::unsafe_arena_set_allocated_usage_core_nano_seconds(
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.usage_core_nano_seconds_);
  }
  _impl_.usage_core_nano_seconds_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.CpuUsage.usage_core_nano_seconds)
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE CpuUsage::release_usage_core_nano_seconds() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::UInt64Value* released = _impl_.usage_core_nano_seconds_;
  _impl_.usage_core_nano_seconds_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE CpuUsage::unsafe_arena_release_usage_core_nano_seconds() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.CpuUsage.usage_core_nano_seconds)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::UInt64Value* temp = _impl_.usage_core_nano_seconds_;
  _impl_.usage_core_nano_seconds_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL CpuUsage::_internal_mutable_usage_core_nano_seconds() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.usage_core_nano_seconds_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::UInt64Value>(GetArena());
    _impl_.usage_core_nano_seconds_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(p);
  }
  return _impl_.usage_core_nano_seconds_;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL CpuUsage::mutable_usage_core_nano_seconds()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::UInt64Value* _msg = _internal_mutable_usage_core_nano_seconds();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.CpuUsage.usage_core_nano_seconds)
  return _msg;
}
inline void CpuUsage::set_allocated_usage_core_nano_seconds(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.usage_core_nano_seconds_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.usage_core_nano_seconds_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.CpuUsage.usage_core_nano_seconds)
}

// -------------------------------------------------------------------

// MemoryUsage

// int64 timestamp = 1;
inline void MemoryUsage::clear_timestamp() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timestamp_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int64_t MemoryUsage::timestamp() const {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.MemoryUsage.timestamp)
  return _internal_timestamp();
}
inline void MemoryUsage::set_timestamp(::int64_t value) {
  _internal_set_timestamp(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.MemoryUsage.timestamp)
}
inline ::int64_t MemoryUsage::_internal_timestamp() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.timestamp_;
}
inline void MemoryUsage::_internal_set_timestamp(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.timestamp_ = value;
}

// .runtime.v1alpha2.UInt64Value working_set_bytes = 2;
inline bool MemoryUsage::has_working_set_bytes() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.working_set_bytes_ != nullptr);
  return value;
}
inline void MemoryUsage::clear_working_set_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.working_set_bytes_ != nullptr) _impl_.working_set_bytes_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::runtime::v1alpha2::UInt64Value& MemoryUsage::_internal_working_set_bytes() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::runtime::v1alpha2::UInt64Value* p = _impl_.working_set_bytes_;
  return p != nullptr ? *p : reinterpret_cast<const ::runtime::v1alpha2::UInt64Value&>(::runtime::v1alpha2::_UInt64Value_default_instance_);
}
inline const ::runtime::v1alpha2::UInt64Value& MemoryUsage::working_set_bytes() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.MemoryUsage.working_set_bytes)
  return _internal_working_set_bytes();
}
inline void MemoryUsage::unsafe_arena_set_allocated_working_set_bytes(
    ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.working_set_bytes_);
  }
  _impl_.working_set_bytes_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:runtime.v1alpha2.MemoryUsage.working_set_bytes)
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE MemoryUsage::release_working_set_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::UInt64Value* released = _impl_.working_set_bytes_;
  _impl_.working_set_bytes_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE MemoryUsage::unsafe_arena_release_working_set_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.MemoryUsage.working_set_bytes)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::UInt64Value* temp = _impl_.working_set_bytes_;
  _impl_.working_set_bytes_ = nullptr;
  return temp;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL MemoryUsage::_internal_mutable_working_set_bytes() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.working_set_bytes_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::runtime::v1alpha2::UInt64Value>(GetArena());
    _impl_.working_set_bytes_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(p);
  }
  return _impl_.working_set_bytes_;
}
inline ::runtime::v1alpha2::UInt64Value* PROTOBUF_NONNULL MemoryUsage::mutable_working_set_bytes()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::runtime::v1alpha2::UInt64Value* _msg = _internal_mutable_working_set_bytes();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.MemoryUsage.working_set_bytes)
  return _msg;
}
inline void MemoryUsage::set_allocated_working_set_bytes(::runtime::v1alpha2::UInt64Value* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.working_set_bytes_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.working_set_bytes_ = reinterpret_cast<::runtime::v1alpha2::UInt64Value*>(value);
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.MemoryUsage.working_set_bytes)
}

// -------------------------------------------------------------------

// ReopenContainerLogRequest

// string container_id = 1;
inline void ReopenContainerLogRequest::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::std::string& ReopenContainerLogRequest::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:runtime.v1alpha2.ReopenContainerLogRequest.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ReopenContainerLogRequest::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:runtime.v1alpha2.ReopenContainerLogRequest.container_id)
}
inline ::std::string* PROTOBUF_NONNULL ReopenContainerLogRequest::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:runtime.v1alpha2.ReopenContainerLogRequest.container_id)
  return _s;
}
inline const ::std::string& ReopenContainerLogRequest::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void ReopenContainerLogRequest::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ReopenContainerLogRequest::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ReopenContainerLogRequest::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:runtime.v1alpha2.ReopenContainerLogRequest.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void ReopenContainerLogRequest::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:runtime.v1alpha2.ReopenContainerLogRequest.container_id)
}

// -------------------------------------------------------------------

// ReopenContainerLogResponse

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace v1alpha2
}  // namespace runtime


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::runtime::v1alpha2::SecurityProfile_ProfileType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::runtime::v1alpha2::SecurityProfile_ProfileType>() {
  return ::runtime::v1alpha2::SecurityProfile_ProfileType_descriptor();
}
template <>
struct is_proto_enum<::runtime::v1alpha2::Protocol> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::runtime::v1alpha2::Protocol>() {
  return ::runtime::v1alpha2::Protocol_descriptor();
}
template <>
struct is_proto_enum<::runtime::v1alpha2::MountPropagation> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::runtime::v1alpha2::MountPropagation>() {
  return ::runtime::v1alpha2::MountPropagation_descriptor();
}
template <>
struct is_proto_enum<::runtime::v1alpha2::NamespaceMode> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::runtime::v1alpha2::NamespaceMode>() {
  return ::runtime::v1alpha2::NamespaceMode_descriptor();
}
template <>
struct is_proto_enum<::runtime::v1alpha2::PodSandboxState> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::runtime::v1alpha2::PodSandboxState>() {
  return ::runtime::v1alpha2::PodSandboxState_descriptor();
}
template <>
struct is_proto_enum<::runtime::v1alpha2::ContainerState> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::runtime::v1alpha2::ContainerState>() {
  return ::runtime::v1alpha2::ContainerState_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"

#endif  // cri_2dv1alpha2_2eproto_2epb_2eh
